#!/bin/sh

# Script to build the VSW4 test suite
# Should be run from the root of unpacked tarball

#export TET_ROOT=`pwd`
#export PATH=$TET_ROOT/tet/bin:$TET_ROOT/xtest/bin:$PATH

if [ ! -x /opt/lsb-tet3-lite/bin/tcc ]
then
	printf "tcc not found, aborting\n"
	exit 1
fi

TET_ROOT=`pwd`
TET_PATH=/opt/lsb-tet3-lite
MANPATH=$MANPATH:$TET_PATH/man
PATH=$TET_PATH/bin:$TET_ROOT/xtest/bin:$PATH
export PATH MANPATH TET_ROOT TET_PATH

# Configure tetbuild.cfg and tetclean.cfg
case `uname -m` in
x86_64|ppc64|s390x)
  sed -e "s\\^TET_ROOT=\\TET_ROOT=$TET_ROOT\\" \
      -e "s\\/usr/X11R6/lib\\/usr/X11R6/lib64\\" \
      -e "s\\^TETBASE=\${TET_ROOT}/tet\\TETBASE=\${TET_PATH}\\" \
  xtest/tetbuild.cfg.skel > xtest/tetbuild.cfg
  ;;
*)
  sed -e "s\\^TET_ROOT=\\TET_ROOT=$TET_ROOT\\" \
      -e "s\\^TETBASE=\${TET_ROOT}/tet\\TETBASE=\${TET_PATH}\\" \
  xtest/tetbuild.cfg.skel > xtest/tetbuild.cfg
  ;;
esac

# Compile harness
make install

if [ $? -ne 0 ]; then
    echo Build of harness failed
    exit 1
fi

# Build testcases
cd $TET_ROOT/xtest

# Recreate the .Xdefaults and .Xdefaults-hostname
rm -f tset/CH10/gtdflt/.Xdefaults

# Generate Makefile and link.c in advance 
find tset -name "*.m" -print | while read src ; do
	if [ ! -z "$src" ]; then
        	DIR=`dirname $src`
		FILE=`basename $src`
		if [ ! -f $DIR/Makefile ] ; then
			cd $DIR; mmkf -o Makefile $FILE; cd $TET_ROOT/xtest
		fi
	fi
done

# use soft link instead of hard link
find tset -name Makefile -print | while read src ; do
        dst=$src.tmp
        sed -e "s\\\$(LN) ../Tests \$(LINKEXEC)\\ln -s ../Tests \$(LINKEXEC)\\" \
            -e "s\\\$(LN) ../Tests m\$(LINKEXEC)\\ln -s ../Tests m\$(LINKEXEC)\\" $src > $dst
        rm -f $src
        mv $dst $src
done

tcc -b -p
grep '|FAIL' `expr \`ls -d results/0* | tail -n 1\``/journal

if [ $? -ne 1 ]; then
  echo Build of testcases failed
  exit 1
fi

echo
echo
echo Build of testcases completed
echo 
echo Run the testcases by executing run_vsw4.sh
echo

