/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: lstdpths.m,v 1.5 92/06/11 16:00:03 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int *
 *   XListDepths(display, screen_number, count_return)
 *   Display	*display;
 *   int	screen_number;
 *   int	*count_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XListDepths	1
char    *TestName = "XListDepths";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XListDepths function
 */
static Display	*display;
static int	screen_number;
static int	*count_return;


static int 	ValueReturn;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	screen_number = DefaultScreen(Dsp);
	count_return = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XListDepths-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XListDepths returns an array which can be freed
 * with XFree, of all depths which are available on the screen
 * screen_number and whose size is returned through the
 * count_return argument.
 */
static void
t001()
{
int	 	number;
int		*dlst;
int		*dptr;
int		i;
XVisualInfo	vp;
Pixmap	pixm;
long	notmem[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	count_return = &number;
	startcall(display);
	if (isdeleted())
		return;
	dptr = dlst = XListDepths(display, screen_number, count_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	for(; dptr < &dlst[number] ; dptr++) {
		trace("Depth %d", *dptr);
		startcall(display);
		if (isdeleted())
			return;
		pixm = XCreatePixmap(display, DRW(display), 1 , 1, *dptr);
		endcall(display);
		if(geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		} else {
			CHECK;
			XFreePixmap(display, pixm);
		}
	}	

	if(notmember(dlst, number, notmem) == 0) {
		delete("notmember failed.");
		return;
	} else
		CHECK;

	for (i = 0; i < NM_LEN; i++) {
		startcall(display);
		if (isdeleted())
			return;
		/* notmem is a long, and is being cast to an int */
		pixm = XCreatePixmap(display, DRW(display), 1 , 1, (int)notmem[i]);
		endcall(display);
		if(geterr() == Success) {
			XFreePixmap(display, pixm);
			report("Got Success unexpectedly for depth %d.", (int)notmem[i]);
			FAIL;
		} else
			CHECK;
	}	

	XFree((char*)dlst);
	CHECKPASS(1+number+NM_LEN);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XListDepths-2.(A)
 * When screen_number is not a valid screen, then a call to
 * XListDepths does not set the count_return argument and
 * returns NULL.
 */
static void
t002()
{
int	number = -1;
int	*dlst;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	count_return = &number;
	screen_number = -1;
	startcall(display);
	if (isdeleted())
		return;
	dlst = XListDepths(display, screen_number, count_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(dlst != (int *) NULL) {
		report("%s() did not return NULL.", TestName);
		FAIL;
	} else
		CHECK;

	if( number != -1) {
		report("The count_return argument was modified.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XListDepths-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When sufficient memory cannot be allocated for the returned
 * array, then a call to XListDepths does not set the
 * count_return argument and returns NULL.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
