/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: nxtrqst.m,v 1.5 92/06/11 16:00:49 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   unsigned long
 *   XNextRequest(display)
 *   Display	*display;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XNextRequest	1
char    *TestName = "XNextRequest";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XNextRequest function
 */
static Display	*display;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XNextRequest-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XNextRequest returns the serial number of the
 * next request over the connection specified by the display
 * argument.
 */
static void
t001()
{
unsigned long	sno, nno;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	sno = XLastKnownRequestProcessed(display);
	startcall(display);
	if (isdeleted())
		return;
	nno = XNextRequest(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	report("Last processed serial number was %lu, next serial number is %lu.", sno, nno);
	if(sno == nno) {
		report("The next serial number (%lu) was not different to the current serial number.", nno);
		FAIL;
	} else
		CHECK;

	CHECKUNTESTED(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XNextRequest-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * Serial numbers are maintained separately for each display
 * connection.
 */
static void
t002()
{
unsigned long	nr1, nr2, nar1, nar2;
Display		*disp2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	disp2 = opendisplay();

	_startcall(display);
	nr1 = XNextRequest(display);
	_endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	} else
		CHECK;

	_startcall(disp2);
	nr2 = XNextRequest(disp2);
	_endcall(disp2);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	} else
		CHECK;

	XNoOp(display);

	_startcall(display);
	nar1 = XNextRequest(display);
	_endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	} else
		CHECK;

	_startcall(disp2);
	nar2 = XNextRequest(disp2);
	_endcall(disp2);

	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	} else
		CHECK;

	if(nr1 == nar1) {
		delete("The next serial number (%lu) was the same after a XNoOp request.", nr1);
		return;
	} else
		CHECK;


	if(nr2 != nar2) {
		report("The next serial number changed without a request.");
		FAIL;
	} else
		CHECK;

	CHECKUNTESTED(6);







	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
