/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: cnfgrwdw.m,v 1.11 92/06/11 16:04:01 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XConfigureWindow(display, w, value_mask, values)
 *   Display	*display;
 *   Window	w;
 *   unsigned int 	value_mask;
 *   XWindowChanges	*values;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XConfigureWindow	1
char    *TestName = "XConfigureWindow";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XConfigureWindow function
 */
static Display	*display;
static Window	w;
static unsigned int 	value_mask;
static XWindowChanges	*values;



/* The structure that is used as the default argument to the function. */
static XWindowChanges	winchng;

/*
 * Convenience routine that sets the 'w' argument to be simple window
 * with a background of W_FG at position (10,8) and * size 10x9.
 * The border width is 0.
 */
#define	OW_X	10
#define	OW_Y	8
#define	OW_WIDTH	23
#define	OW_HEIGHT	9

static	Window
onewin()
{
Window	base;
struct	area	area;

	base = defwin(display);
	setarea(&area, OW_X, OW_Y, OW_WIDTH, OW_HEIGHT);

	w = crechild(display, base, &area);
	XSetWindowBackground(display, w, W_FG);
	XClearWindow(display, w);

	return(base);
}

/*
 * A window with subwindows.
 */
static	char	*Tsub[] = {
	".",
	"top . (5, 5) 40x40",
	"sub1 top (1,3) 4x6",
	"sub2 top (8,3) 7x6",
	"sub3 top (3,14) 12x9",
	"sub4 top (28,10) 7x20",
	"sub5 top (14,25) 11x12",
};
#define	NTsub	(NELEM(Tsub))
#define	NTsubInf	(NTsub-1)	/* Number of inferiors excluding base */



extern	struct valname S_wingravity[];
extern	int 	NS_wingravity;

/* Values to resize onewin to (odd and even) */
#define	NEW_X	OW_X+5
#define	NEW_Y	OW_Y+5
#define	NEW_WIDTH	70
#define	NEW_HEIGHT	61

#define	BORDERW	1



/*
 * Flags to say that we received particular events.
 */
#define	WIN_CONFIG	001
#define	WIN_GRAVITY	002
#define	PAR_CONFIG	004
#define	PAR_GRAVITY	010

static	void	calcxy();

/*
 * Routine that does the event checking for the gravity notify
 * test.
 */
static
wingravevents(display, bt, evtype)
Display	*display;
struct	buildtree	*bt;
int 	evtype;
{
XEvent	ev;
XConfigureEvent	confgood;
XGravityEvent	gravgood;
XConfigureEvent	*cnp;
XGravityEvent	*gnp;
struct	buildtree	*btp;
struct	buildtree	*top;
int 	gotgrav = 0;
int 	i;
int 	pass = 0, fail = 0;

	top = btntobtp(bt, "top");

	/* Set up events */
	confgood.type = ConfigureNotify;
	confgood.serial = 0L;
	confgood.send_event = False;
	confgood.display = display;
	confgood.above = None;	/* XXX */
	confgood.override_redirect = False;

	confgood.x = top->x;
	confgood.y = top->y;
	confgood.width = NEW_WIDTH;
	confgood.height = NEW_HEIGHT;
	confgood.border_width = 0;

	gravgood.type = GravityNotify;
	gravgood.serial = 0L;
	gravgood.send_event = False;
	gravgood.display = display;

	while (getevent(display, &ev) > 0) {

		switch (ev.type) {
		case ConfigureNotify:
			cnp = (XConfigureEvent*)&ev;

			btp = btwtobtp(bt, cnp->window);
			if (btp == NULL) {
				report("Event received on unknown window");
				FAIL;
				continue;
			}
			trace("Event received for window '%s'", btp->name);

			if (gotgrav) {
				report("Configure event received after gravity event");
				FAIL;
			} else
				CHECK;

			/*
			 * Work out if this event occurred on the parent or the window.
			 */
			if (cnp->window != cnp->event) {
				if (!btp->parent || btp->parent->wid != cnp->event) {
					report("Event received on other than the parent window");
					FAIL;
				} else
					btp->uflags |= PAR_CONFIG;
			} else {
				btp->uflags |= WIN_CONFIG;
			}

			confgood.event = cnp->event;
			confgood.window = cnp->window;

			if (checkevent((XEvent*)&confgood, &ev) == 0)
				CHECK;
			else
				FAIL;

			break;
		case GravityNotify:
			gnp = (XGravityEvent*)&ev;
			gotgrav = 1;

			btp = btwtobtp(bt, gnp->window);
			if (btp == NULL) {
				report("Event received on unknown window");
				FAIL;
				continue;
			}
			trace("Event received for window '%s'", btp->name);

			/*
			 * Work out if this event occurred on the parent or the window.
			 */
			if (gnp->window != gnp->event) {
				if (!btp->parent || btp->parent->wid != gnp->event) {
					report("Event received on other than the parent window");
					FAIL;
				} else
					btp->uflags |= PAR_GRAVITY;
			} else {
				btp->uflags |= WIN_GRAVITY;
			}

			gravgood.event = gnp->event;
			gravgood.window = gnp->window;
			gravgood.x = btp->x;
			gravgood.y = btp->y;
			(void) calcxy(top, evtype, &gravgood.x, &gravgood.y);

			if (checkevent((XEvent*)&gravgood, &ev) == 0)
				CHECK;
			else
				FAIL;

			break;
		default:
			report("Unexpected event type %s", eventname(ev.type));
			FAIL;
		}
	}

	/*
	 * Since the previous loop is executed a variable number of times
	 * check that it was executed at least once and reset the pass count.
	 */
	if (pass > 0)
		pass = 1;
	else
		delete("No CHECK marks in wingravevents() loop");

	/*
	 * Check correct events on the top (resized) window.
	 */
	if (top->uflags & WIN_CONFIG)
		CHECK;
	else {
		report("Configure event not received on window 'top'");
		FAIL;
	}
	if (top->uflags & PAR_CONFIG)
		CHECK;
	else {
		report("Configure event not received on parent of window 'top'");
		FAIL;
	}
	if (top->uflags & (PAR_GRAVITY|WIN_GRAVITY)) {
		report("Gravity events unexpectedly received on window 'top'");
		FAIL;
	} else
		CHECK;

	for (i = 2; i < NTsub; i++) {
		if (evtype == NorthWestGravity || evtype == StaticGravity) {
			if (bt[i].uflags & WIN_GRAVITY) {
				report("Gravity event unexpectedly received on window '%s'", bt[i].name);
				FAIL;
			} else
				CHECK;
			if (bt[i].uflags & PAR_GRAVITY) {
				report("Gravity event unexpectedly received on parent of window '%s'", bt[i].name);
				FAIL;
			} else
				CHECK;

		} else {

			if (bt[i].uflags & WIN_GRAVITY)
				CHECK;
			else {
				report("Gravity event not received on window '%s'", bt[i].name);
				FAIL;
			}
			if (bt[i].uflags & PAR_GRAVITY)
				CHECK;
			else {
				report("Gravity event not received on parent of window '%s'", bt[i].name);
				FAIL;
			}
		}
		if (bt[i].uflags & (PAR_CONFIG|WIN_CONFIG)) {
			report("Configure event unexpectedly received on window '%s'", bt[i].name);
			FAIL;
		}
	}
	if (fail == 0 && pass != 14)
		delete("Path check error in wingravevents got %d, expecting 14", pass);

	if (fail == 0)
		return(True);
	else
		return(False);
}

static void
calcxy(top, evtype, x, y)
struct	buildtree	*top;
int 	evtype;
int 	*x;
int 	*y;
{
int 	dx = NEW_WIDTH-top->width;
int 	dy = NEW_HEIGHT-top->height;

	switch (evtype) {
	case NorthWestGravity:
		break;
	case NorthGravity:
		*x += dx/2;
		break;
	case NorthEastGravity:
		*x += dx;
		break;
	case WestGravity:
		*y += dy/2;
		break;
	case CenterGravity:
		*x += dx/2;
		*y += dy/2;
		break;
	case EastGravity:
		*x += dx;
		*y += dy/2;
		break;
	case SouthWestGravity:
		*y += dy;
		break;
	case SouthGravity:
		*x += dx/2;
		*y += dy;
		break;
	case SouthEastGravity:
		*x += dx;
		*y += dy;
		break;
	case StaticGravity:
		break;
	default:
		delete("Internal error -- Unknown gravity in calcxy");
	}
}



#define	ON_PARENT	0x01
#define	ON_WINDOW	0x02



/*
 * Window tree for the restacking operations.
 */
static char	*Tstack[] = {
	". borders",
	"bottom . (30,2) 60x34",
	"A . (10,5) 30x20",
	"B . (15, 10) 30x20",
	"mid . (20, 15) 30x20",
	"C . (25, 20) 30x20",
	"D . (30, 25) 30x20",
	"alone . (50,50) 20x20",	/* does not overlap with any window */
	"olmid . (3,32) 20x20",	/* Overlaps mid */
	"top . (5,80) 19x4",
};
#define	NTstack	(NELEM(Tstack))
#define	NTstackInf	(NELEM(Tstack)-1)
#define TstackTOS	(NTstackInf-1)	/* Top of stacking order number */

#ifdef GENERATE_PIXMAPS
#define	stackorder(a,b) (0)
#endif


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
	value_mask = 0;
	values = &winchng;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
}

/*
 * Assertion XConfigureWindow-1.(A)
 * When the CWX or the CWY bits are set in value_mask, then a
 * call to XConfigureWindow moves the window so that the
 * coordinates of the upper-left outer corner of the window are
 * x, y relative to the origin of the parent window.
 */
static void
t001()
{
Window	base;
struct	area	area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = onewin();

	setarea(&area, 20, 50, OW_WIDTH, OW_HEIGHT);
	values->x = area.x;
	values->y = area.y;

	value_mask = CWX|CWY;

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (checkarea(display, base, &area, W_FG, W_BG, CHECK_ALL))
		CHECK;
	else {
		report("Error in moving window");
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-2.(A)
 * When the CWWidth or CWHeight bits are set in value_mask,
 * then on a call to XConfigureWindow the width and height of
 * the inside size of the window are set to width and height.
 */
static void
t002()
{
Window	base;
struct	area	area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = onewin();

	setarea(&area, OW_X, OW_Y, 23, 47);
	values->width = area.width;
	values->height = area.height;

	value_mask = CWWidth|CWHeight;

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (checkarea(display, base, &area, W_FG, W_BG, CHECK_ALL))
		CHECK;
	else {
		report("Error in resizing window");
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-3.(A)
 * When a call to XConfigureWindow resizes the window, then
 * the subwindows of the window are repositioned according to
 * their win-gravity attribute and a GravityNotify event is
 * generated for each repositioned subwindow after the
 * ConfigureNotify event.
 */
static void
t003()
{
Window	base;
struct	buildtree	*bt;
struct	buildtree	*topbtp;
struct	valname 	*wgrav;
XSetWindowAttributes	setatts;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	values->width = NEW_WIDTH;
	values->height = NEW_HEIGHT;
	value_mask = CWWidth|CWHeight;

	for (wgrav = S_wingravity; wgrav < S_wingravity+NS_wingravity; wgrav++) {

		if (wgrav->val == UnmapGravity)
			continue;

		trace("-- Testing win-gravity %s", wgrav->name);

		base = defwin(display);
		bt = buildtree(display, base, Tsub, NTsub);

		topbtp = btntobtp(bt, "top");

		/* Select for all structure events on all windows */
		for (i = 0; i < NTsub; i++) {
			XSelectInput(display, bt[i].wid,
			 	SubstructureNotifyMask|StructureNotifyMask);
		}

		/*
		 * Set the win_gravity on windows below 'top'.  Also set the
		 * borderwidths to make things more interesting, esp on mono
		 * displays.
		 */
		setatts.win_gravity = wgrav->val;
		for (i = 2; i < NTsub; i++) {
			XChangeWindowAttributes(display, bt[i].wid, CWWinGravity, &setatts);
			XSetWindowBorderWidth(display, bt[i].wid, BORDERW);
			XSetWindowBorder(display, bt[i].wid, W_BG);	/* to show up */
		}

		w = topbtp->wid;
		startcall(display);
		if (isdeleted())
			return;
		XConfigureWindow(display, w, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(display, base);

#ifndef GENERATE_PIXMAPS
		if (wingravevents(display, bt, wgrav->val))
			CHECK;
#endif
	}

	/*
	 * In the GENERATE_PIXMAPS case there will be a path-check error here,
	 * this is intentional. (To prove that wingravevents() is used in
	 * the verification case)
	 */
	CHECKPASS(2*(NS_wingravity-1));	/* -1 for UnmapGravity */
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-4.(A)
 * When a call to XConfigureWindow resizes the window and the
 * win-gravity of a subwindow is UnmapGravity and the subwindow
 * is already mapped, then the subwindow is unmapped without
 * being moved and an UnmapNotify event is generated.
 */
static void
t004()
{
Window	base;
struct	buildtree	*bt;
struct	buildtree	*topbtp;
struct	buildtree	*btp;
XEvent	ev;
XUnmapEvent	good;
XUnmapEvent	*ump;
XSetWindowAttributes	setatts;
XWindowAttributes	atts;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/* Set up good unmap event struct */
	good.type = UnmapNotify;
	good.serial = 0L;
	good.send_event = False;
	good.display = display;
	good.from_configure = True;

	values->width = NEW_WIDTH;
	values->height = NEW_HEIGHT;
	value_mask = CWWidth|CWHeight;

	base = defwin(display);
	bt = buildtree(display, base, Tsub, NTsub);

	topbtp = btntobtp(bt, "top");

	/* Select for all structure events on all windows */
	for (i = 0; i < NTsub; i++) {
		XSelectInput(display, bt[i].wid,
			SubstructureNotifyMask|StructureNotifyMask);
	}

	/*
	 * Set the win_gravity on windows below 'top'.  Also set the
	 * borderwidths to make things more interesting, esp on mono
	 * displays.
	 */
	setatts.win_gravity = UnmapGravity;
	for (i = 2; i < NTsub; i++) {
		XChangeWindowAttributes(display, bt[i].wid, CWWinGravity, &setatts);
		XSetWindowBorderWidth(display, bt[i].wid, BORDERW);
	}

	w = topbtp->wid;
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (checkarea(display, w, (struct area *)0, W_FG, W_FG, CHECK_ALL))
		CHECK;
	else {
		report("UnmapGravity did not appear to remove subwindows");
		FAIL;
	}

	while (getevent(display, &ev) > 0) {

		if (ev.type != UnmapNotify)
			continue;

		ump = (XUnmapEvent*)&ev;

		btp = btwtobtp(bt, ump->window);
		if (btp == NULL) {
			report("Event received on unknown window");
			FAIL;
			continue;
		}
		trace("Event received for window '%s'", btp->name);

		/*
		 * Work out if this event occurred on the parent or the window.
		 */
		if (ump->window != ump->event) {
			if (!btp->parent || btp->parent->wid != ump->event) {
				report("Event received on other than the parent window");
				FAIL;
			} else
				btp->uflags |= ON_PARENT;
		} else {
			btp->uflags |= ON_WINDOW;
		}

		good.event = ump->event;
		good.window = ump->window;

		if (checkevent((XEvent*)&good, &ev) == 0)
			CHECK;
		else
			FAIL;
	}

	/*
	 * Go through the subwindows below top and check for events received
	 * and map_state and position unchanged.
	 */
	for (i = 2; i < NTsub; i++) {
		if (bt[i].uflags & ON_WINDOW)
			CHECK;
		else {
			report("Unmap event not received on window '%s'", bt[i].name);
			FAIL;
		}
		if (bt[i].uflags & ON_PARENT)
			CHECK;
		else {
			report("Unmap event not received on parent of window '%s'", bt[i].name);
			FAIL;
		}

		XGetWindowAttributes(display, bt[i].wid, &atts);
		if (atts.map_state != IsUnmapped) {
			report("map_state was %s, expecting IsUnmapped",
				mapstatename(atts.map_state));
			FAIL;
		} else
			CHECK;

		if (atts.x != bt[i].x || atts.y != bt[i].y) {
			report("Subwindow was moved after UnmapGravity used");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(1+6*(NTsub-2));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-5.(A)
 * If the server uses the window's bit-gravity attribute: When
 * a call to XConfigureWindow resizes the window, then the
 * contents of the window are repositioned or discarded
 * according to the bit-gravity attribute. Otherwise: When a
 * call to XConfigureWindow resizes the window, then the
 * contents of the window are discarded.
 */
static void
t005()
{
struct	valname	*bitgrav;
Window	base;
XSetWindowAttributes	setatts;
struct	area	area;
extern	struct	valname	S_bitgravity[];
extern	int 	NS_bitgravity;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	for (bitgrav = S_bitgravity; bitgrav < S_bitgravity+NS_bitgravity; bitgrav++) {

		trace("-- Trying bitgravity of %s", bitgrav->name);
		setarea(&area, OW_X, OW_Y, OW_WIDTH, OW_HEIGHT);

		base = onewin();
		XSetWindowBackground(display, w, W_BG);
		dset(display, w, W_FG);

		setatts.bit_gravity = bitgrav->val;
		XChangeWindowAttributes(display, w, CWBitGravity, &setatts);

		values->width = NEW_WIDTH;
		values->height = NEW_HEIGHT;
		value_mask = CWWidth|CWHeight;

		startcall(display);
		if (isdeleted())
			return;
		XConfigureWindow(display, w, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		switch (bitgrav->val) {
		case NorthGravity: case CenterGravity: case SouthGravity:
			area.x += (NEW_WIDTH-OW_WIDTH)/2;
			break;
		case NorthEastGravity: case EastGravity: case SouthEastGravity:
			area.x += (NEW_WIDTH-OW_WIDTH);
			break;
		}
		switch (bitgrav->val) {
		case CenterGravity: case WestGravity: case EastGravity:
			area.y += (NEW_HEIGHT-OW_HEIGHT)/2;
			break;
		case SouthGravity: case SouthEastGravity: case SouthWestGravity:
			area.y += (NEW_HEIGHT-OW_HEIGHT);
			break;
		}

		if (checkarea(display, base, (struct area *)0, W_BG, W_BG, CHECK_ALL|CHECK_DIFFER)) {

			/*
			 * The whole base window was clear, the server is not using
			 * bit-gravity, or else we are trying ForgetGravity.
			 */
			if (bitgrav->val != ForgetGravity)
				trace("server not using bit-gravity");
			CHECK;
		} else {
			/*
			 * For ForgetGravity then we must not get here.
			 */
			if (bitgrav->val == ForgetGravity) {
				report("Contents were not discarded with ForgetGravity");
				FAIL;
			} else if (checkarea(display, base, &area, W_FG, W_BG, CHECK_ALL)) {
				CHECK;
			} else {
				report("bits positioned incorrectly for bit-gravity of %s",
					bitgrav->name);
				FAIL;
			}
		}
	}

	CHECKPASS(NS_bitgravity);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-6.(A)
 * When the CWBorderWidth bit is set in value_mask, then the
 * width of the border is set to border_width pixels.
 */
static void
t006()
{
int 	i;
Window	base;
static	int 	bords[] = {
	1, 2, 3, 4, 5, 10, 17, 29, 34, 51, 90, 234
};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = onewin();

	/*
	 * Set the background and border to highlight the border.
	 */
	XSetWindowBackground(display, w, W_BG);
	XClearWindow(display, w);
	XSetWindowBorder(display, w, W_FG);

	for (i = 0; i < NELEM(bords); i++) {

		values->border_width = bords[i];
		value_mask = CWBorderWidth;

		startcall(display);
		if (isdeleted())
			return;
		XConfigureWindow(display, w, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(display, base);
	}

	CHECKPASS(NELEM(bords));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-7.(A)
 * When the CWSibling and CWStackMode bits are set in
 * value_mask and stack_mode is Above, then the window is
 * placed just above the sibling.
 */
static void
t007()
{
Window	base;
struct	buildtree	*bt;
struct	buildtree	*mid;
int 	sibso;
int 	winso;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	mid = btntobtp(bt, "mid");

	values->stack_mode = Above;
	values->sibling = mid->wid;
	value_mask = CWStackMode|CWSibling;

	for (i = 1; i < NTstack; i++) {

		if (bt+i == mid)
			continue;

		trace("Restacking '%s'", bt[i].name);
		w = bt[i].wid;

		startcall(display);
		if (isdeleted())
			return;
		XConfigureWindow(display, w, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		winso = stackorder(display, w);
		sibso = stackorder(display, mid->wid);
		if (winso != sibso+1) {
			report("Window not restacked to one above, got %d, expecting %d",
				winso, sibso+1);
			FAIL;
		} else
			CHECK;

		PIXCHECK(display, base);
	}

	CHECKPASS(2*(NTstackInf-1));

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-8.(A)
 * When the CWSibling and CWStackMode bits are set in
 * value_mask and stack_mode is Below, then the window is
 * placed just below the sibling.
 */
static void
t008()
{
Window	base;
struct	buildtree	*bt;
struct	buildtree	*mid;
int 	sibso;
int 	winso;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	mid = btntobtp(bt, "mid");

	values->stack_mode = Below;
	values->sibling = mid->wid;
	value_mask = CWStackMode|CWSibling;

	for (i = 1; i < NTstack; i++) {

		if (bt+i == mid)
			continue;

		trace("Restacking '%s'", bt[i].name);
		w = bt[i].wid;

		startcall(display);
		if (isdeleted())
			return;
		XConfigureWindow(display, w, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		winso = stackorder(display, w);
		sibso = stackorder(display, mid->wid);
		if (winso != sibso-1) {
			report("Window not restacked to one below, got %d, expecting %d",
				winso, sibso+1);
			FAIL;
		} else
			CHECK;

		PIXCHECK(display, base);
	}

	CHECKPASS(2*(NTstackInf-1));

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-9.(A)
 * When the CWSibling and CWStackMode bits are set in
 * value_mask, stack_mode is TopIf and the sibling occludes
 * the window, then the window is placed at the top of the
 * stack.
 */
static void
t009()
{
Window	base;
struct	buildtree	*bt;
struct	buildtree	*mid;
int 	oldso;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	mid = btntobtp(bt, "mid");

	values->stack_mode = TopIf;
	values->sibling = mid->wid;
	value_mask = CWStackMode|CWSibling;

	w = btntow(bt, "B");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != TstackTOS) {
		report("Window not at top of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	/* Now a window that is not occluded */
	w = btntow(bt, "D");
	oldso = stackorder(display, w);
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != oldso) {
		report("Window stacking order changed from %d to %d", oldso, winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-10.(A)
 * When the CWSibling and CWStackMode bits are set in
 * value_mask, stack_mode is BottomIf and the window occludes
 * the sibling, then the window is placed at the bottom of the
 * stack.
 */
static void
t010()
{
Window	base;
struct	buildtree	*bt;
struct	buildtree	*mid;
int 	oldso;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	mid = btntobtp(bt, "mid");

	values->stack_mode = BottomIf;
	values->sibling = mid->wid;
	value_mask = CWStackMode|CWSibling;

	w = btntow(bt, "D");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != 0) {
		report("Window not at bottom of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	/* Now a window that does not occlude */
	w = btntow(bt, "B");
	oldso = stackorder(display, w);
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != oldso) {
		report("Window stacking order changed from %d to %d", oldso, winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-11.(A)
 * When the CWSibling and CWStackMode bits are set in
 * value_mask, stack_mode is Opposite and the window occludes
 * the sibling, then the window is placed at the bottom of the
 * stack.
 */
static void
t011()
{
Window	base;
struct	buildtree	*bt;
struct	buildtree	*mid;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	mid = btntobtp(bt, "mid");

	values->stack_mode = Opposite;
	values->sibling = mid->wid;
	value_mask = CWStackMode|CWSibling;

	w = btntow(bt, "D");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != 0) {
		report("Window not at bottom of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-12.(A)
 * When the CWSibling and CWStackMode bits are set in
 * value_mask, stack_mode is Opposite and the sibling occludes
 * the window, then the window is placed at the top of the
 * stack.
 */
static void
t012()
{
Window	base;
struct	buildtree	*bt;
struct	buildtree	*mid;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	mid = btntobtp(bt, "mid");

	values->stack_mode = Opposite;
	values->sibling = mid->wid;
	value_mask = CWStackMode|CWSibling;

	/* A window that is occluded by sibling */
	w = btntow(bt, "B");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != TstackTOS) {
		report("Window not brought to top of stack (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-13.(A)
 * When the CWStackMode bit is set and the CWSibling bit is
 * not set in value_mask and stack_mode is Above, then the
 * window is placed at the top of the stack.
 */
static void
t013()
{
Window	base;
struct	buildtree	*bt;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	values->stack_mode = Above;
	value_mask = CWStackMode;

	w = btntow(bt, "mid");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != TstackTOS) {
		report("Window not at top of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-14.(A)
 * When the CWStackMode bit is set and the CWSibling bit is
 * not set in value_mask and stack_mode is Below, then the
 * window is placed at the bottom of the stack.
 */
static void
t014()
{
Window	base;
struct	buildtree	*bt;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	values->stack_mode = Below;
	value_mask = CWStackMode;

	w = btntow(bt, "mid");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != 0) {
		report("Window not at bottom of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-15.(A)
 * When the CWStackMode bit is set and the CWSibling bit is
 * not set in value_mask, stack_mode is TopIf and a sibling
 * occludes the window, then the window is placed at the top of
 * the stack.
 */
static void
t015()
{
Window	base;
struct	buildtree	*bt;
int 	oldso;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	values->stack_mode = TopIf;
	value_mask = CWStackMode;

	w = btntow(bt, "mid");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != TstackTOS) {
		report("Window not at top of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	w = btntow(bt, "alone");
	oldso = stackorder(display, w);
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != oldso) {
		report("Window stacking order changed from %d to %d", oldso, winso);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-16.(A)
 * When the CWStackMode bit is set and the CWSibling bit is
 * not set in value_mask, stack_mode is BottomIf and the
 * window occludes a sibling, then the window is placed at the
 * bottom of the stack.
 */
static void
t016()
{
Window	base;
struct	buildtree	*bt;
int 	oldso;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	values->stack_mode = BottomIf;
	value_mask = CWStackMode;

	w = btntow(bt, "mid");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != 0) {
		report("Window not at bottom of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	w = btntow(bt, "alone");
	oldso = stackorder(display, w);
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != oldso) {
		report("Window stacking order changed from %d to %d", oldso, winso);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-17.(A)
 * When the CWStackMode bit is set and the CWSibling bit is
 * not set in value_mask, stack_mode is Opposite and the
 * window occludes a sibling, then the window is placed at the
 * bottom of the stack.
 */
static void
t017()
{
Window	base;
struct	buildtree	*bt;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	values->stack_mode = Opposite;
	value_mask = CWStackMode;

	w = btntow(bt, "olmid");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != 0) {
		report("Window not at bottom of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-18.(A)
 * When the CWStackMode bit is set and the CWSibling bit is
 * not set in value_mask, stack_mode is Opposite and a sibling
 * occludes the window, then the window is placed at the top of
 * the stack.
 */
static void
t018()
{
Window	base;
struct	buildtree	*bt;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	values->stack_mode = Opposite;
	value_mask = CWStackMode;

	w = btntow(bt, "A");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != NTstackInf-1) {
		report("Window not at top of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-19.(A)
 * When the geometry of the window is changed as well as a
 * restacking operation performed, then the window's final
 * geometry is used to determine whether the window occludes or
 * is occluded by siblings.
 */
static void
t019()
{
Window	base;
struct	buildtree	*bt;
int 	winso;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	bt = buildtree(display, base, Tstack, NTstack);

	values->sibling = btntow(bt, "mid");
	values->stack_mode = Below;
	value_mask = CWStackMode;

	w = btntow(bt, "mid");
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	winso = stackorder(display, w);
	if (winso != 0) {
		report("Window not at top of stacking order (pos=%d)", winso);
		FAIL;
	} else
		CHECK;

	PIXCHECK(display, base);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-20.(-)
 * Values not specified are taken from the existing geometry
 * of the window.
 */
static void
t020()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XConfigureWindow-21.(A)
 * When the window is a root window, then a call to
 * XConfigureWindow has no effect.
 */
static void
t021()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	values->x = NEW_X;
	values->y = NEW_Y;
	values->width = NEW_WIDTH;
	values->height = NEW_HEIGHT;
	values->stack_mode = Above;
	value_mask = CWX|CWY|CWWidth|CWHeight|CWStackMode;

	w = DefaultRootWindow(display);

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (fail == 0)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-22.(A)
 * When the override-redirect attribute of the window is False
 * and some other client has selected SubstructureRedirectMask
 * on the parent window, then a ConfigureRequest event is
 * generated, and the window configuration is not changed.
 */
static void
t022()
{
Window	base;
XConfigureRequestEvent	good;
XSetWindowAttributes	setatts;
Display	*client2;
XEvent	ev;
XImage	*imp;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = onewin();

	setatts.override_redirect = False;
	XChangeWindowAttributes(display, w, CWOverrideRedirect, &setatts);

	client2 = opendisplay();
	XSelectInput(client2, base, SubstructureRedirectMask);
	XSync(client2, False);

	values->x = NEW_X;
	values->y = NEW_Y;
	values->width = NEW_WIDTH;
	values->height = NEW_HEIGHT;

	value_mask = CWX|CWY|CWWidth|CWHeight;

	good.type = ConfigureRequest;
	good.serial = 0L;
	good.send_event = False;
	good.display = client2;
	good.parent = base;
	good.window = w;
	good.x = NEW_X;
	good.y = NEW_Y;
	good.width = NEW_WIDTH;
	good.height = NEW_HEIGHT;
	good.border_width = 0;
	good.above = None;
	good.detail = Above;
	good.value_mask = value_mask;

	imp = savimage(display, base);

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XSync(client2, False);

	n = getevent(client2, &ev);
	if (n != 1) {
		report("Expecting 1 event");
		FAIL;
	} else
		CHECK;

	if (n > 0 && checkevent((XEvent*)&good, &ev) == 0)
		CHECK;
	else
		FAIL;

	if (compsavimage(display, base, imp))
		CHECK;
	else {
		report("Screen contents changed");
		FAIL;
	}

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-23.(A)
 * When another client has selected ResizeRedirectMask on the
 * window and the size is being changed, then a ResizeRequest
 * event is generated and the size is not changed.
 */
static void
t023()
{
Window	base;
XResizeRequestEvent	good;
XSetWindowAttributes	setatts;
Display	*client2;
XEvent	ev;
struct	area	area;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = onewin();

	setatts.override_redirect = False;
	XChangeWindowAttributes(display, w, CWOverrideRedirect, &setatts);

	client2 = opendisplay();
	XSelectInput(client2, w, ResizeRedirectMask);
	XSync(client2, False);

	values->x = NEW_X;
	values->y = NEW_Y;
	values->width = NEW_WIDTH;
	values->height = NEW_HEIGHT;

	value_mask = CWX|CWY|CWWidth|CWHeight;

	good.type = ResizeRequest;
	good.serial = 0L;
	good.send_event = False;
	good.display = client2;
	good.window = w;
	good.width = NEW_WIDTH;
	good.height = NEW_HEIGHT;

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XSync(client2, False);

	n = getevent(client2, &ev);
	if (n != 1) {
		report("Expecting 1 event");
		FAIL;
	} else
		CHECK;

	if (n > 0 && checkevent((XEvent*)&good, &ev) == 0)
		CHECK;
	else
		FAIL;

	/* Window moves, but does not change size */
	setarea(&area, NEW_X, NEW_Y, OW_WIDTH, OW_HEIGHT);
	if (checkarea(display, base, &area, W_FG, W_BG, CHECK_ALL))
		CHECK;
	else {
		report("New location and size of window not as expected");
		FAIL;
	}

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-24.(A)
 * When another client has selected ResizeRedirectMask on the
 * window and another client has selected
 * SubstructureRedirectMask on the parent window and the
 * override-redirect attribute of the window is False, then a
 * ConfigureRequest event is generated, and the window
 * configuration is not changed.
 */
static void
t024()
{
Window	base;
XConfigureRequestEvent	good;
XSetWindowAttributes	setatts;
Display	*client2;
Display	*client3;
XEvent	ev;
XImage	*imp;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = onewin();

	setatts.override_redirect = False;
	XChangeWindowAttributes(display, w, CWOverrideRedirect, &setatts);

	client2 = opendisplay();
	XSelectInput(client2, w, ResizeRedirectMask);
	XSync(client2, False);

	client3 = opendisplay();
	XSelectInput(client3, base, SubstructureRedirectMask);
	XSync(client3, False);

	values->x = NEW_X;
	values->y = NEW_Y;
	values->width = NEW_WIDTH;
	values->height = NEW_HEIGHT;

	value_mask = CWX|CWY|CWWidth|CWHeight;

	imp = savimage(display, base);

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XSync(client2, False);
	XSync(client3, False);

	n = getevent(client2, &ev);
	if (n != 0) {
		report("Got an event unexpectedly for client selecting ResizeRedirect");
		report("  Event type was %s", eventname(ev.type));
		FAIL;
	} else
		CHECK;

	good.type = ConfigureRequest;
	good.serial = 0L;
	good.send_event = False;
	good.display = client3;
	good.parent = base;
	good.window = w;
	good.x = NEW_X;
	good.y = NEW_Y;
	good.width = NEW_WIDTH;
	good.height = NEW_HEIGHT;
	good.border_width = 0;
	good.above = None;
	good.detail = Above;
	good.value_mask = value_mask;

	n = getevent(client3, &ev);
	if (n != 1) {
		report("Expecting one configure events");
		FAIL;
	} else
		CHECK;
	if (n > 0 && checkevent((XEvent*)&good, &ev) == 0)
		CHECK;
	else
		FAIL;

	if (compsavimage(display, base, imp))
		CHECK;
	else {
		report("Window changed when client was selecting SubstructureRedirect");
		FAIL;
	}
	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-25.(A)
 * When the configuration actually changes, then a
 * ConfigureNotify event is generated.
 */
static void
t025()
{
XConfigureEvent	good;
XEvent	ev;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	(void) onewin();

	XSelectInput(display, w, StructureNotifyMask);

	values->x = NEW_X;
	values->y = NEW_Y;
	values->width = NEW_WIDTH;
	values->height = NEW_HEIGHT;

	value_mask = CWX|CWY|CWWidth|CWHeight;

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	good.type = ConfigureNotify;
	good.serial = 0L;
	good.send_event = False;
	good.display = display;
	good.event = w;
	good.window = w;
	good.x = NEW_X;
	good.y = NEW_Y;
	good.width = NEW_WIDTH;
	good.height = NEW_HEIGHT;
	good.border_width = 0;
	good.above = None;
	good.override_redirect = False;

	n = getevent(display, &ev);
	if (n != 1) {
		report("Expecting 1 event, got %d", n);
		FAIL;
	} else
		CHECK;
	if (n > 0 && checkevent((XEvent*)&good, &ev) == 0)
		CHECK;
	else
		FAIL;

	/* Call again */
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if ((n = XPending(display)) == 0)
		CHECK;
	else {
		report("Received event when configuration was not changed");
		FAIL;
	}

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-26.(A)
 * When a call to XConfigureWindow changes the size of the
 * window, then Expose events are generated for regions that
 * are newly visible or for which the contents have been lost.
 */
static void
t026()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	(void) onewin();

	XSetWindowBackground(display, w, W_BG);
	XClearWindow(display, w);
	setforexpose(display, w);

	XSelectInput(display, w, ExposureMask);

	values->width = NEW_WIDTH;
	values->height = NEW_HEIGHT;
	value_mask = CWWidth|CWHeight;

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (exposecheck(display, w))
		CHECK;
	else {
		report("Correct expose events not received after resize");
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-27.(A)
 * When a call to XConfigureWindow uncovers part of any window
 * that was formerly obscured, then either Expose events are
 * generated or the contents are restored from backing store.
 */
static void
t027()
{
Window	base;
Display	*client2;
struct	buildtree	*bt;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	setforexpose(display, base);

	bt = buildtree(display, base, Tstack, NTstack);

	client2 = opendisplay();
	XSelectInput(client2, base, ExposureMask);
	XSync(client2, False);

	/*
	 * This loop will throw all the subwindows outside the borders of the
	 * parent window.
	 */
	values->x = W_STDWIDTH+9000;
	value_mask = CWX;
	for (i = 1; i < NTstack; i++) {
		w = bt[i].wid;
		startcall(display);
		if (isdeleted())
			return;
		XConfigureWindow(display, w, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	}
	XSync(client2, False);

	if (exposecheck(client2, base))
		CHECK;
	else {
		report("Neither Expose events or backing store processing");
		report("  could correctly restore the window contents.");
		FAIL;
	}
	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-28.(A)
 * When the window is moved without changing its size and the
 * window is mapped and the window is not obscured by
 * non-children, then the current window contents are not lost.
 */
static void
t028()
{
XImage	*imp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	(void) onewin();

	pattern(display, w);

	values->x = NEW_X;
	values->y = NEW_Y;
	value_mask = CWX|CWY;

	imp = savimage(display, w);

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (compsavimage(display, w, imp))
		CHECK;
	else {
		report("Window contents affected by moving window");
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-29.(D)
 *   Reason: There is no known reliable test method for this assertion
 * If backing-store is supported when mapped: When the window
 * is moved without changing its size and the window is mapped
 * and backing store exists, then the current window contents
 * are not lost.
 */
static void
t029()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XConfigureWindow-30.(A)
 * When CWSibling is set and CWStackMode is not set, then a
 * BadMatch error occurs.
 */
static void
t030()
{
Window	base;
Window	win;
struct	area	area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);

	setarea(&area, 2, 4, 9, 8);
	win = crechild(display, base, &area);
	w = crechild(display, base, &area);

	values->sibling = win;
	value_mask = CWSibling;

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL; /* already done */
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-31.(A)
 * When sibling is specified and it is not actually a sibling,
 * then a BadMatch error occurs.
 */
static void
t031()
{
Window	base;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = onewin();

	values->sibling = base;
	values->stack_mode = Above;
	value_mask = CWSibling|CWStackMode;

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL; /* again */
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-32.(A)
 * When width or height is zero, then a BadValue error occurs.
 */
static void
t032()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	(void) onewin();

	values->width = 0;
	values->height = 0;

	value_mask = CWWidth;
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadValue)
		CHECK;

	value_mask = CWHeight;
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadValue)
		CHECK;

	value_mask = CWWidth|CWHeight;
	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadValue)
		CHECK;

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConfigureWindow-33.(A)
 * When an attempt is made to set the border-width attribute
 * of an InputOnly window to a non-zero value, then a BadMatch
 * error occurs.
 */
static void
t033()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * It would be better for iponlywin to have a parent arg, how
	 * might this test interact with window managers.
	 */
	w = iponlywin(display);

	values->border_width = 21;
	value_mask = CWBorderWidth;

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadMatch)
		PASS;
	else
		FAIL; /* again */

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XConfigureWindow-34.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t034()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XConfigureWindow(display, w, value_mask, values);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	t021, 21,
	t022, 22,
	t023, 23,
	t024, 24,
	t025, 25,
	t026, 26,
	t027, 27,
	t028, 28,
	t029, 29,
	t030, 30,
	t031, 31,
	t032, 32,
	t033, 33,
	t034, 34,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
