/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stwdwbg.m,v 1.6 92/06/11 16:10:32 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetWindowBackground(display, w, background_pixel)
 *   Display *display;
 *   Window	w;
 *   unsigned long	background_pixel;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetWindowBackground	1
char    *TestName = "XSetWindowBackground";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XSetWindowBackground function
 */
static Display *display;
static Window	w;
static unsigned long	background_pixel;


static Window	parent;
static struct	area	ap;

static void
inittp()
{
	tpstartup();

	ap.x = 50;
	ap.y = 60;
	ap.width = 20;
	ap.height= 20;
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
	background_pixel = W_FG;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
}

/*
 * Assertion XSetWindowBackground-1.(A)
 * A call to XSetWindowBackground sets the background of the
 * window to the pixel value specified by background_pixel.
 */
static void
t001()
{
XEvent	event;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	parent = defdraw(display, VI_WIN);

	w = creunmapchild(display, parent, &ap);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackground(display, w, background_pixel);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSelectInput(display, w, ExposureMask);	
	XMapWindow(display, w);
	XWindowEvent(display,w,ExposureMask, &event); /* Await exposure */

	if(getpixel(display, w, 1, 1) != W_FG) {
		report("%s did not set the background pixel to W_FG.", TestName);
		FAIL;
	} else
		CHECK;
	
	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBackground-2.(A)
 * When the background is changed, then the window contents
 * are not changed.
 */
static void
t002()
{
Pixmap	pm;
XEvent event;
XVisualInfo	*vp;
XSetWindowAttributes	attributes;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	for(resetvinf(VI_WIN); nextvinf(&vp); ) {
		parent = makewin(display, vp);

		w = creunmapchild(display, parent, &ap);

		pm = maketile(display, w);
		XSetWindowBackgroundPixmap(display, w, pm);

		XSelectInput(display, w, ExposureMask);
		XMapWindow(display, w);
					/* Wait for window to be exposed */
		XWindowEvent(display,w,ExposureMask, &event);

		background_pixel = W_BG;
		startcall(display);
		if (isdeleted())
			return;
		XSetWindowBackground(display, w, background_pixel);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if ( isdeleted() )
			continue;

		if (checktile(display, w, NULL, 0, 0, pm))
			CHECK;
		else
			FAIL;

	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EMat4.mc */
/*
 * Assertion XSetWindowBackground-3.(A)
 * When the window argument is an InputOnly window then a
 * BadMatch error occurs.
 */
/* End of included file error/EMat4.mc */

/* Including from file error/EMat4.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	seterrdef();

	A_WINDOW = iponlywin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackground(display, w, background_pixel);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat4.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XSetWindowBackground-4.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackground(display, w, background_pixel);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
