/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stwdwbgpxm.m,v 1.7 92/06/11 16:10:39 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetWindowBackgroundPixmap(display, w, background_pixmap)
 *   Display *display;
 *   Window	w;
 *   Pixmap	background_pixmap;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetWindowBackgroundPixmap	1
char    *TestName = "XSetWindowBackgroundPixmap";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w
#define A_PIXMAP background_pixmap
#define A_DRAWABLE2 background_pixmap


/*
 * Arguments to the XSetWindowBackgroundPixmap function
 */
static Display *display;
static Window	w;
static Pixmap	background_pixmap;


static struct	area	ap;
static Window	parent;

static void	inittp()
{
	tpstartup();

	ap.x = 50;
	ap.y = 60;
	ap.width = 20;
	ap.height= 20;
}

static void
perform_map(display, w)
Display	*display;
Window	w;
{
	XEvent	event;
/* Await visibilty */
	XSelectInput(display, w , ExposureMask);
	XMapWindow(display, w);
	XWindowEvent(display, w, ExposureMask, &event);
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
	background_pixmap = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
	background_pixmap = ErrdefPixmap;
}

/*
 * Assertion XSetWindowBackgroundPixmap-1.(A)
 * A call to XSetWindowBackgroundPixmap sets the background
 * pixmap of the window to the pixmap specified by
 * background_pixmap.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	parent = defdraw(display, VI_WIN);

	w = creunmapchild(display, parent, &ap);
	XSetWindowBackground(display, w, W_FG);
	perform_map(display, w);

	background_pixmap = maketile(display, w);
	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackgroundPixmap(display, w, background_pixmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XUnmapWindow(display, w);
	perform_map(display, w);

	if (checktile( display, w, NULL, 0, 0, background_pixmap))
		CHECK;
	else {
		report("%s did not set the background pixmap correctly",
			TestName);
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBackgroundPixmap-2.(A)
 * When background_pixmap is ParentRelative, then each time
 * the background pixmap is required it is taken from the
 * parent window at the time that it is required, with the
 * background tile origin aligned with the origin of the parent
 * window.
 */
static void
t002()
{
Pixmap	pm;
unsigned int	border_width = 2;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	background_pixmap = ParentRelative;

	parent = defdraw(display, VI_WIN);
	w = creunmapchild(display, parent, &ap);
	XSetWindowBorderWidth(display, w, border_width);
	perform_map(display, w);
	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackgroundPixmap(display, w, background_pixmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	pm = maketile(display, parent);
	XSetWindowBackgroundPixmap(display, parent, pm);
	XClearWindow(display, w);

	if (checktile(display, w, (struct area *)0, -ap.x-border_width, -ap.y-border_width, pm))
		CHECK;
	else {
		report("ParentRelative check failed for 'maketile' background");
		FAIL;
	}

	pm = XCreatePixmap(display, parent, 10, 17, (int)getdepth(display, w));
	dclear(display, pm);
	pattern(display, pm);
	XSetWindowBackgroundPixmap(display, parent, pm);
	XClearWindow(display, w);
	if (checktile(display, w, (struct area *)0, -ap.x-border_width, -ap.y-border_width, pm))
		CHECK;
	else {
		report("ParentRelative check failed for 'pattern' background");
		FAIL;
	}

	XSetWindowBackground(display, parent, W_FG);
	XClearWindow(display, w);
	if (checkarea(display, w, (struct area *)0, W_FG, W_BG, CHECK_ALL))
		CHECK;
	else {
		report("ParentRelative check failed for plain background set to W_FG");
		FAIL;
	}

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBackgroundPixmap-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the background pixmap of a root window is set to
 * ParentRelative, then the default background is restored.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSetWindowBackgroundPixmap-4.(A)
 * When the background pixmap is set to None, then the window
 * has no defined background.
 */
static void
t004()
{
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		parent = makedrawable(display, vp);
		pattern(display, parent);

		w = mkwinchild(display, vp, &ap, False, parent, 0);

/* Set the background to ensure it gets unset */
		XSetWindowBackground(display, w, W_FG);

		background_pixmap = None;
		startcall(display);
		if (isdeleted())
			return;
		XSetWindowBackgroundPixmap(display, w, background_pixmap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		perform_map(display, w);

		if (isdeleted())
			continue;

		if( !checkpattern(display, parent, &ap))
		{	
			report("%s did not leave the child", TestName);
			report("window background transparent");
			FAIL;
		}
		else
			CHECK;
		
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBackgroundPixmap-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the background pixmap of a root window is set to None,
 * then the default background is restored.
 */
static void
t005()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* Including from file error/EPix.mc */
/*
 * Assertion XSetWindowBackgroundPixmap-6.(A)
 * When a pixmap argument does not name a valid Pixmap,
 * ParentRelative or None, then a BadPixmap error occurs.
 */
/* End of included file error/EPix.mc */

/* Including from file error/EPix.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();


	seterrdef();

	A_PIXMAP = (Pixmap) badpixm(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackgroundPixmap(display, w, background_pixmap);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadPixmap)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EPix.mc */

/*
 * Assertion XSetWindowBackgroundPixmap-7.(C)
 * If windows with depth other than one are supported: When
 * background_pixmap and the window do not have the same depth,
 * then a BadMatch error occurs.
 */
static void
t007()
{
Pixmap	pm;
XVisualInfo	*vp;
int 	found = 0;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	/* create parent before find vp because resetvinf will release Vinfop */
	parent = defdraw(display, VI_WIN);

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		if (vp->depth != 1) {
			found = 1;
			break;
		}
	}

	if (!found) {
		unsupported("Only windows with depth one are supported");
		return;
	}

	pm = XCreatePixmap(display, DRW(display), 2, 2, 1);
	background_pixmap = pm;

	w = mkwinchild(display, vp, &ap, False, parent, 2);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackgroundPixmap(display, w, background_pixmap);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	XFreePixmap(display, pm);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBackgroundPixmap-8.(C)
 * If multiple screens are supported: When background_pixmap
 * and the window are not created for the same screen, then a
 * BadMatch error occurs.
 */
static void
t008()
{
XVisualInfo	*vp;
Pixmap	pm;
int 	*depths;
int 	count;
int 	found = 0;
int 	i;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	if (config.alt_screen == -1) {
		unsupported("No alternate screen supported");
		return;
	}

	depths = XListDepths(display, config.alt_screen, &count);

	/* create parent before find vp because resetvinf will release Vinfop */
	parent = defdraw(display, VI_WIN);

	for (resetvinf(VI_WIN); (!found && nextvinf(&vp)); ) {
		for (i = 0; i < count; i++) {
			if (depths[i] == vp->depth)
			{
				found = 1;
				break;
			}	
		}
	}

	if (!found) {
		unsupported("Pixmaps of same depth as a window not supported on alt screen");
		return;
	}

	w = creunmapchild(display, parent, &ap);

	pm = XCreatePixmap(display, RootWindow(display, config.alt_screen),
		2, 3, vp->depth);
	background_pixmap = pm;

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackgroundPixmap(display, w, background_pixmap);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	XFreePixmap(display, pm);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBackgroundPixmap-9.(C)
 * If multiple window depths are supported: When
 * background_pixmap is ParentRelative and the window and the
 * parent window do not have the same depth, then a BadMatch
 * error occurs.
 */
static void
t009()
{
XVisualInfo	*vp;
XVisualInfo	*vp2 = 0;
int 	found = 0;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		if (vp2 == 0) {
			vp2 = vp;
		} else if (vp->depth != vp2->depth) {
			found = 1;
			break;
		}
	}

	if (!found) {
		unsupported("Only one depth of window is supported");
		return;
	}

	parent = makedrawable(display, vp2);
	w      = mkwinchild(display, vp, &ap, False, parent, 1);

	background_pixmap = ParentRelative;
	startcall(display);
	if (isdeleted())
		return;
	(void)XSetWindowBackgroundPixmap(display, w, background_pixmap);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EMat4.mc */
/*
 * Assertion XSetWindowBackgroundPixmap-10.(A)
 * When the window argument is an InputOnly window then a
 * BadMatch error occurs.
 */
/* End of included file error/EMat4.mc */

/* Including from file error/EMat4.mc */
static void
t010()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	seterrdef();

	A_WINDOW = iponlywin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackgroundPixmap(display, w, background_pixmap);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat4.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XSetWindowBackgroundPixmap-11.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t011()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBackgroundPixmap(display, w, background_pixmap);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
