/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stwdwbrdrp.m,v 1.6 92/06/11 16:10:55 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetWindowBorderPixmap(display, w, border_pixmap)
 *   Display	*display;
 *   Window	w;
 *   Pixmap	border_pixmap;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetWindowBorderPixmap	1
char    *TestName = "XSetWindowBorderPixmap";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w
#define A_PIXMAP border_pixmap
#define A_DRAWABLE2 border_pixmap


/*
 * Arguments to the XSetWindowBorderPixmap function
 */
static Display	*display;
static Window	w;
static Pixmap	border_pixmap;


static struct	area	ap;
static Window	parent;

static void
inittp()
{
	tpstartup();

	ap.x = 50;
	ap.y = 60;
	ap.width = 20;
	ap.height= 20;
}

static void
perform_map(display, w)
Display	*display;
Window	w;
{
	XEvent	event;
/* Await visibilty */
	XSelectInput(display, w , ExposureMask);
	XMapWindow(display, w);
	XWindowEvent(display, w, ExposureMask, &event);
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
	border_pixmap = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
	border_pixmap = ErrdefPixmap;
}

/*
 * Assertion XSetWindowBorderPixmap-1.(A)
 * A call to XSetWindowBorderPixmap sets the border pixmap of
 * the window to the pixmap specified by border_pixmap.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	parent = defdraw(display, VI_WIN);

	w = creunmapchild(display, parent, &ap);

	XSetWindowBorderWidth(display, w, 5);
	XSetWindowBorder(display, w, W_FG);

	perform_map(display, w);

	border_pixmap = maketile(display, w);
	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorderPixmap(display, w, border_pixmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XUnmapWindow(display, w);
	perform_map(display,w);

	PIXCHECK(display, parent);
	
	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBorderPixmap-2.(A)
 * When the border pixmap is changed, then the border is
 * repainted.
 */
static void
t002()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

/* Create windows */
	parent = defdraw(display, VI_WIN);
	w = creunmapchild(display, parent, &ap);
/* Set up child window border for test */
	XSetWindowBorderWidth(display, w, 5);
	XSetWindowBorder(display, w, W_FG);

	perform_map(display, w);

	border_pixmap = maketile(display,w);
	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorderPixmap(display, w, border_pixmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	
	PIXCHECK(display, parent);

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBorderPixmap-3.(A)
 * When border_pixmap is CopyFromParent, then the
 * border-pixmap attribute is copied from the parent window.
 */
static void
t003()
{
Pixmap	pm;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		parent = makedrawable(display, vp);

		w = parent;
		border_pixmap = maketile(display, parent);
		startcall(display);
		if (isdeleted())
			return;
		XSetWindowBorderPixmap(display, w, border_pixmap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		w = mkwinchild(display, vp, &ap, False, parent, 5);
		perform_map(display, w);

		border_pixmap = CopyFromParent;
		startcall(display);
		if (isdeleted())
			return;
		XSetWindowBorderPixmap(display, w, border_pixmap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(display, parent);
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBorderPixmap-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When border_pixmap is CopyFromParent and the window is a
 * root window, then the default border pixmap is restored.
 */
static void
t004()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* Including from file error/EPix.mc */
/*
 * Assertion XSetWindowBorderPixmap-5.(A)
 * When a pixmap argument does not name a valid Pixmap or
 * CopyFromParent, then a BadPixmap error occurs.
 */
/* End of included file error/EPix.mc */

/* Including from file error/EPix.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();


	seterrdef();

	A_PIXMAP = (Pixmap) badpixm(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorderPixmap(display, w, border_pixmap);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadPixmap)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EPix.mc */

/*
 * Assertion XSetWindowBorderPixmap-6.(C)
 * If windows with depth other than one are supported: When
 * border_pixmap and the window do not have the same depth,
 * then a BadMatch error occurs.
 */
static void
t006()
{
Pixmap	pm;
XVisualInfo	*vp;
int 	found = 0;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	/* create parent before find vp because resetvinf will release Vinfop */
	parent = defdraw(display, VI_WIN);

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		if (vp->depth != 1) {
			found = 1;
			break;
		}
	}

	if (!found) {
		unsupported("Only windows with depth one are supported");
		return;
	}

	pm = XCreatePixmap(display, DRW(display), 2, 2, 1);
	border_pixmap = pm;

	w = mkwinchild(display, vp, &ap, False, parent, 2);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorderPixmap(display, w, border_pixmap);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	XFreePixmap(display, pm);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBorderPixmap-7.(C)
 * If multiple window depths are supported: When border_pixmap
 * is CopyFromParent, and the window does not have the same
 * depth as the parent window, then a BadMatch error occurs.
 */
static void
t007()
{
XVisualInfo	*vp;
XVisualInfo	*vp2 = 0;
int 	found = 0;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		if (vp2 == 0) {
			vp2 = vp;
		} else if (vp->depth != vp2->depth) {
			found = 1;
			break;
		}
	}

	if (!found) {
		unsupported("Only one depth of window is supported");
		return;
	}

	parent = makedrawable(display, vp2);
	w      = mkwinchild(display, vp, &ap, False, parent, 1);

	border_pixmap = CopyFromParent;
	startcall(display);
	if (isdeleted())
		return;
	(void)XSetWindowBorderPixmap(display, w, border_pixmap);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EMat4.mc */
/*
 * Assertion XSetWindowBorderPixmap-8.(A)
 * When the window argument is an InputOnly window then a
 * BadMatch error occurs.
 */
/* End of included file error/EMat4.mc */

/* Including from file error/EMat4.mc */
static void
t008()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	seterrdef();

	A_WINDOW = iponlywin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorderPixmap(display, w, border_pixmap);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat4.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XSetWindowBorderPixmap-9.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t009()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorderPixmap(display, w, border_pixmap);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
