/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtatmnm.m,v 1.5 92/06/11 16:12:47 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char *
 *   XGetAtomName(display, atom)
 *   Display *display;
 *   Atom atom;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetAtomName	1
char    *TestName = "XGetAtomName";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_ATOM atom


/*
 * Arguments to the XGetAtomName function
 */
static Display *display;
static Atom atom;


#include "Xatom.h"

static struct xga_struct {
	char *name;
	Atom atom;
} xga_list[] = {
	 "PRIMARY", XA_PRIMARY, 
	 "CUT_BUFFER0", XA_CUT_BUFFER0, 
	 "RECTANGLE", XA_RECTANGLE,
	 "COPYRIGHT", XA_COPYRIGHT,
};
static int xga_nlist = NELEM(xga_list);


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	atom = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	atom = ErrdefAtom;
}

/*
 * Assertion XGetAtomName-1.(A)
 * A call to XGetAtomName returns the name, which can be
 * freeed with XFree, associated with the specified atom.
 */
static void
t001()
{
char *ret_str;
int l;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* For some predefined atoms: */
	for(l=0; l<xga_nlist; l++) {

/* 	Call xname to obtain the name associated with the atom. */
		atom = xga_list[l].atom;
		trace("checking atom %d (%s)", atom, atomname(atom));
		startcall(display);
		if (isdeleted())
			return;
		ret_str = XGetAtomName(display, atom);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (ret_str == NULL) {
			FAIL;
			report("%s returned a null string with atom name %s",
				TestName, xga_list[l].name);
			continue;
		} else
			CHECK;

/* 	Verify the strings returned were as expected. */
		if (strcmp(xga_list[l].name, ret_str) != 0 ) {
			FAIL;
			report("%s returned an unexpected string");
			report("Expected: '%s'", xga_list[l].name);
			report("Returned: '%s'", ret_str);
		} else
			CHECK;
		XFree(ret_str);
	}

	CHECKPASS(2*xga_nlist);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAto.mc */
/*
 * Assertion XGetAtomName-2.(A)
 * When an atom argument does not name a valid Atom, then a
 * BadAtom error occurs.
 */
/* End of included file error/EAto.mc */

/* Including from file error/EAto.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_ATOM = ~0L;
	startcall(display);
	if (isdeleted())
		return;
	XGetAtomName(display, atom);
	endcall(display);
	if (geterr() != BadAtom) {
		report("Got %s, Expecting BadAtom", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadAtom)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAto.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
