/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: allcclr.m,v 1.7 92/06/11 16:14:27 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XAllocColor(display, colormap, screen_in_out)
 *   Display *display;
 *   Colormap colormap;
 *   XColor *screen_in_out;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XAllocColor	1
char    *TestName = "XAllocColor";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XAllocColor function
 */
static Display *display;
static Colormap colormap;
static XColor *screen_in_out;


static int 	StatusReturn;

XColor dummy;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
	screen_in_out = &dummy;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XAllocColor-1.(A)
 * A call to XAllocColor allocates a read-only entry in the
 * colormap argument corresponding to the closest RGB value
 * that is supported by the hardware to the value specified by
 * the red, green, and blue components in the XColor
 * structure named by the screen_in_out argument, returns that
 * value in the red, green, and blue components in the XColor
 * structure named by the screen_in_out argument and returns
 * the colourmap entry in the pixel component in the XColor
 * structure named by the screen_in_out argument.
 */
static void
t001()
{
Display	*disp2;
XVisualInfo *vp;
Status status;
XColor screencol, testcol;
unsigned long vmask;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals reported as valid.");
		return;
	}

	if( (disp2 = opendisplay()) == (Display *) 0) {
		delete("Could not open display");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting AllocColor() for class %s", displayclassname(vp->class));
		colormap = makecolmap(display, vp -> visual, AllocNone);

		testcol.red = 0xf3f3;
		testcol.green = 0xe4e4;
		testcol.blue = 0xd5d5;
		
		trace("Source colour :  r %u g %u b %u", testcol.red, testcol.green, testcol.blue);
		screen_in_out = &testcol;
		display = disp2;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocColor(display, colormap, screen_in_out);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if( status == (Status) 0) {
			report("XAllocColor() failed to return non-zero.");
			FAIL;
			continue;
		}

		screencol = testcol;
		trace("Screen: pixel %lu, r %u g %u b %u", screencol.pixel , screencol.red, screencol.green, screencol.blue);
		trace("Test : pixel %lu, r %u g %u b %u", testcol.pixel, testcol.red, testcol.green, testcol.blue);

		screen_in_out = &testcol;
		display = Dsp; 
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocColor(display, colormap, screen_in_out);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	
		if(status == (Status) 0) {
			report("XAllocColor() failed to return non-zero.");
			FAIL;
			continue;	
		}

		if( screencol.pixel != testcol.pixel ) {
			report("XAllocColor() return pixel value %lu instead of %lu.", testcol.pixel, screencol.pixel);
			FAIL;
		} else
			CHECK;

		if( (screencol.red != testcol.red) || (screencol.green != testcol.green) || (screencol.blue != testcol.blue) ) {
			report("XAllocColor() return RGB values r %u g %u b %u instead of r %u g %u b %u.", 
				testcol.red, testcol.green, testcol.blue, screencol.red, screencol.green, screencol.blue);
			FAIL;
		} else
			CHECK;
	
		startcall(display);
		XStoreColor(display, colormap, &testcol);
		endcall(display);

		if (geterr() == Success) {
			report("Got success writing to a read-only cell");
			tet_result(TET_FAIL);
		} else
			CHECK;

	}

	CHECKPASS(3 * nsupvis());


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocColor-2.(A)
 * A call to XAllocColor does not use or affect the flags
 * component of the XColor structure named by the screen_in_out
 * argument.
 */
static void
t002()
{
XVisualInfo *vp;
Status status;
XColor screencol, testcol;
unsigned long vmask;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals reported as valid.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting AllocColor() for class %s", displayclassname(vp->class));
		colormap = makecolmap(display, vp -> visual, AllocNone);

		testcol.red = 255 << 8;
		testcol.green = 63 << 8;
		testcol.blue = 31 << 8;
		testcol.flags = DoGreen;		

		screen_in_out = &testcol;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocColor(display, colormap, screen_in_out);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if( status == (Status) 0) {
			report("XAllocColor() failed to return non-zero.");
			FAIL;
			continue;
		}

		if( DoGreen != testcol.flags ) {
			report("XAllocColor() altered flags value %lu to %lu.",
					 DoGreen, testcol.flags);
			FAIL;
		} else
			CHECK;

		screencol = testcol;
		trace("Screen: pixel %lu, r %u g %u b %u", screencol.pixel , screencol.red, screencol.green, screencol.blue);
		trace("Test : pixel %lu, r %u g %u b %u", testcol.pixel, testcol.red, testcol.green, testcol.blue);

		testcol.red = 255 << 8;
		testcol.green = 63 << 8;
		testcol.blue = 31 << 8;
		testcol.flags = DoRed | DoBlue;

		screen_in_out = &testcol;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocColor(display, colormap, screen_in_out);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	
		if(status == (Status) 0) {
			report("XAllocColor() failed to return non-zero.");
			FAIL;
			continue;	
		}

		if( (DoRed | DoBlue) != testcol.flags ) {
			report("XAllocColor() altered flags value %lu to %lu.",
					 (DoRed | DoBlue), testcol.flags);
			FAIL;
		} else
			CHECK;

		if( screencol.pixel != testcol.pixel ) {
			report("XAllocColor() return pixel value %lu instead of %lu.", testcol.pixel, screencol.pixel);
			FAIL;
		} else
			CHECK;

		if( (screencol.red != testcol.red) || (screencol.green != testcol.green) || (screencol.blue != testcol.blue) ) {
			report("XAllocColor() return RGB values r %u g %u b %u instead of r %u g %u b %u.", 
				testcol.red, testcol.green, testcol.blue, screencol.red, screencol.green, screencol.blue);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(4 * nsupvis());



	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocColor-3.(A)
 * When a call to XAllocColor has allocated a read-only entry
 * in a colourmap, then a subsequent call to XAllocColor with
 * identical red, green and blue components in the XColor
 * structure named by the screen_in_out argument returns an
 * identical pixel component in the XColor structure.
 */
static void
t003()
{
XVisualInfo *vp;
Status status;
XColor screencol, testcol;
unsigned long vmask;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals reported as valid.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting AllocColor() for class %s", displayclassname(vp->class));
		colormap = makecolmap(display, vp -> visual, AllocNone);

		testcol.red = 255 << 8;
		testcol.green = 127 << 8;
		testcol.blue = 63 << 8;

		screen_in_out = &testcol;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocColor(display, colormap, screen_in_out);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if( status == (Status) 0) {
			report("XAllocColor() failed to return non-zero.");
			FAIL;
			continue;
		}

		screencol = testcol;
		trace("Screen: pixel %lu, r %u g %u b %u", screencol.pixel , screencol.red, screencol.green, screencol.blue);
		trace("Test : pixel %lu, r %u g %u b %u", testcol.pixel, testcol.red, testcol.green, testcol.blue);

		screen_in_out = &testcol;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocColor(display, colormap, screen_in_out);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	
		if(status == (Status) 0) {
			report("XAllocColor() failed to return non-zero.");
			FAIL;
			continue;	
		}

		if( screencol.pixel != testcol.pixel ) {
			report("XAllocColor() return pixel value %lu instead of %lu.", testcol.pixel, screencol.pixel);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECol.mc */
/*
 * Assertion XAllocColor-4.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XAllocColor(display, colormap, screen_in_out);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/* Including from file error/EAll.mc */
/*
 * Assertion XAllocColor-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
