/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: allcclrpln.m,v 1.7 92/06/11 16:14:43 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XAllocColorPlanes(display, colormap, contig, pixels_return, ncolors, nreds, ngreens, nblues, rmask_return, gmask_return, bmask_return)
 *   Display *display;
 *   Colormap colormap;
 *   Bool contig;
 *   unsigned long *pixels_return;
 *   int ncolors;
 *   int nreds;
 *   int ngreens;
 *   int nblues;
 *   unsigned long *rmask_return;
 *   unsigned long *gmask_return;
 *   unsigned long *bmask_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XAllocColorPlanes	1
char    *TestName = "XAllocColorPlanes";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XAllocColorPlanes function
 */
static Display *display;
static Colormap colormap;
static Bool contig;
static unsigned long *pixels_return;
static int ncolors;
static int nreds;
static int ngreens;
static int nblues;
static unsigned long *rmask_return;
static unsigned long *gmask_return;
static unsigned long *bmask_return;


static int 	StatusReturn;

static unsigned long dummy;

static int contiguous(mask)
unsigned long mask;
{
	if(!mask)
		return(0L);

	while((mask&1) == 0L)
		mask>>=1;
	mask++;
	return( bitcount(mask) == 1);
}

static int maskshift(mask)
unsigned long mask;
{
	int	i = 0;

	if(!mask)
		return(0);

	while((mask&1) == 0L) {
		mask>>=1;
		i++;
	}
	return(i);
}



/* Value list for use in test t007 */
static int 	contigvallist[] = {
	True,
	False,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
	contig = 0;
	pixels_return = &dummy;
	ncolors = 1;
	nreds = 0;
	ngreens = 0;
	nblues = 0;
	rmask_return = &dummy;
	gmask_return = &dummy;
	bmask_return = &dummy;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XAllocColorPlanes-1.(A)
 * A call to XAllocColorPlanes allocates
 * ncolors*2pow(nreds+ngreens+nblues) read/write colourmap
 * entries whose indices in the colourmap are obtained by ORing
 * each pixel value returned in the pixels_return argument with
 * zero or more of the planes obtained by ORing the masks
 * returned in the rmask_return, gmask_return and bmask_return
 * arguments.
 */
static void
t001()
{
XVisualInfo	*vp;
unsigned long	vmask = (1<<DirectColor);
unsigned long	*pixm, *rr, *gr, *br;
unsigned long	rm, gm, bm, pm, rgbmask, pixel, r, g, b;
XColor 		color, qcolor;
Status		status;
int		cells;
int		rshift, gshift, bshift;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		tet_result(TET_UNSUPPORTED);
		return;
	}

	resetsupvis(vmask);
	nextsupvis(&vp);
		
	colormap = makecolmap(display, vp->visual, AllocNone);
	ncolors = 1;
	nreds = bitcount(vp->red_mask);
	ngreens = bitcount(vp->green_mask);
	nblues = bitcount(vp->blue_mask);
	cells = ncolors * (1 << (nreds+ngreens+nblues));
	contig = False;
	pixels_return = &pm;
	rmask_return = &rm;
	gmask_return = &gm;
	bmask_return = &bm;

	startcall(display);
	if (isdeleted())
		return;
	status = XAllocColorPlanes(display, colormap, contig, pixels_return, ncolors, nreds, ngreens, nblues, rmask_return, gmask_return, bmask_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(status == (Status) 0) {
		report("%s failed to return non-zero", TestName);
		FAIL;
	} else {
		trace("%d cells allocated in the colormap", cells);
		CHECK;
	}

	rgbmask = rm | gm | bm ;
	rshift = maskshift(rm);
	gshift = maskshift(gm);
	bshift = maskshift(bm);
	trace("rgbmask is %x", rgbmask);
	trace("red shift is %d", rshift);
	trace("green shift is %d", gshift);
	trace("blue shift is %d", bshift);

	/* 
	 * The following code works on the assumption that for each subfield, 
	 * all planes in the subfield mask are allocated (so are contiguous).
	 *
	 * (It does not assume the subfields are adjacent)
	 */

	for(r = 0; r<(1<<nreds); r++) {
		pixel = (r<<rshift) | pm;
		debug(1, "pixel value %x", pixel);
		color.pixel = pixel;
		color.flags =DoRed;
		color.red = 0xffff;

		startcall(display);
		XStoreColor(display, colormap, &color);
		endcall(display);

		if(geterr() != Success) {
			report("XStoreColor() failed with pixel value %lu", pixel);
			FAIL;
		} else {
			qcolor.pixel = pixel;
			startcall(display);
			XQueryColor(display, colormap, &qcolor);
			endcall(display);
		if(geterr() != Success) {
			report("XQueryColor() failed with pixel value %lu", pixel);
			FAIL;
		} else
			CHECK;
		}
	}

	for(g = 0; g<(1<<ngreens); g++) {
		pixel = (g<<gshift) | pm;
		debug(1, "pixel value %x", pixel);
		color.pixel = pixel;
		color.flags =DoGreen;
		color.green = 0xffff;

		startcall(display);
		XStoreColor(display, colormap, &color);
		endcall(display);

		if(geterr() != Success) {
			report("XStoreColor() failed with pixel value %lu", pixel);
			FAIL;
		} else {
			qcolor.pixel = pixel;
			startcall(display);
			XQueryColor(display, colormap, &qcolor);
			endcall(display);
			if(geterr() != Success) {
				report("XQueryColor() failed with pixel value %lu", pixel);
				FAIL;
			} else
				CHECK;
		}
	}

	for(b = 0; b<(1<<nblues); b++) {
		pixel = (b<<bshift) | pm;
		debug(1, "pixel value %x", pixel);
		color.pixel = pixel;
		color.flags =DoBlue;
		color.blue = 0xffff;

		startcall(display);
		XStoreColor(display, colormap, &color);
		endcall(display);

		if(geterr() != Success) {
			report("XStoreColor() failed with pixel value %lu", pixel);
			FAIL;
		} else {
			qcolor.pixel = pixel;
			startcall(display);
			XQueryColor(display, colormap, &qcolor);
			endcall(display);
			if(geterr() != Success) {
				report("XQueryColor() failed with pixel value %lu", pixel);
				FAIL;
			} else
				CHECK;
		}
	}

	CHECKPASS(1 + (1<<nreds) + (1<<ngreens) + (1<<nblues));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocColorPlanes-2.(A)
 * A call to XAllocColorPlanes sets nreds bits to one in the
 * mask named by rmask_return, ngreens bits to one in the mask
 * named by gmask_return, and nblues bits to one in the mask
 * named by bmask_return and returns ncolors pixels values in
 * the pixels_return argument such that no mask has a bit set
 * in common with any other mask or with any of the pixels.
 */
static void
t002()
{
XVisualInfo	*vp;
unsigned long	vmask = (1<<DirectColor);
unsigned long	*pixm, *rr, *gr, *br;
unsigned long	rm, gm, bm, pm, rgbmask;
unsigned long	pm_array[3], i;
Status		status;
int		cells, rnreds, rngreens, rnblues;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		tet_result(TET_UNSUPPORTED);
		return;
	}

	resetsupvis(vmask);
	nextsupvis(&vp);
		
	colormap = makecolmap(display, vp->visual, AllocNone);
	ncolors = 1;
	nreds = bitcount(vp->red_mask);
	ngreens = bitcount(vp->green_mask);
	nblues = bitcount(vp->blue_mask);
	cells = ncolors * (1 << (nreds+ngreens+nblues));
	contig = False;
	pixels_return = &pm;
	rmask_return = &rm;
	gmask_return = &gm;
	bmask_return = &bm;

	startcall(display);
	if (isdeleted())
		return;
	status = XAllocColorPlanes(display, colormap, contig, pixels_return, ncolors, nreds, ngreens, nblues, rmask_return, gmask_return, bmask_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(status == (Status) 0) {
		report("%s failed to return non-zero", TestName);
		FAIL;
	} else {
		trace("%d cells allocated in the colormap", cells);
		CHECK;
	}

	rnreds = bitcount(rm);
	rngreens = bitcount(gm);
	rnblues = bitcount(bm);

	if( (rnreds != nreds) || (rngreens != ngreens) || (rnblues != nblues) ) {
		report("Request for %d reds %d greens %d blues returned %d reds %d greens %d blues.",
			  nreds, ngreens, nblues, rnreds, rngreens, rnblues);
		FAIL;
	} else
		CHECK;

	rgbmask = rm | gm | bm ;
	
	if( bitcount(rgbmask) != rnreds + rnblues + rngreens) {
		report("The returned colourplane  masks had at least one shared bit.");
		FAIL;
	} else
		CHECK;

	if( (rgbmask & pm) != 0L) {
		report("A pixel shared at least one bit with a colourplane.");
		FAIL;
	} else
		CHECK;

	/*
	 * Repeat requesting a number of colours and fewer planes.
	 */
	colormap = makecolmap(display, vp->visual, AllocNone);

	ncolors = 3;
	/* Make sure that this visual is big enough to support so many colours */
	if (ncolors > bitcount(vp->blue_mask))
		ncolors = bitcount(vp->blue_mask);
	if (ncolors > bitcount(vp->green_mask))
		ncolors = bitcount(vp->green_mask);
	if (ncolors > bitcount(vp->red_mask))
		ncolors = bitcount(vp->red_mask);
	debug(1, "ncolors = %d", ncolors);

	nreds = 1;
	ngreens = 1;
	nblues = 1;
	cells = ncolors * (1 << (nreds+ngreens+nblues));
	contig = False;
	pixels_return = &pm_array[0];
	rmask_return = &rm;
	gmask_return = &gm;
	bmask_return = &bm;

	startcall(display);
	if (isdeleted())
		return;
	status = XAllocColorPlanes(display, colormap, contig, pixels_return, ncolors, nreds, ngreens, nblues, rmask_return, gmask_return, bmask_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(status == (Status) 0) {
		report("%s failed to return non-zero", TestName);
		FAIL;
	} else {
		trace("%d cells allocated in the colormap", cells);
		CHECK;
	}

	rnreds = bitcount(rm);
	rngreens = bitcount(gm);
	rnblues = bitcount(bm);

	if( (rnreds != nreds) || (rngreens != ngreens) || (rnblues != nblues) ) {
		report("Request for %d reds %d greens %d blues returned %d reds %d greens %d blues.",
			  nreds, ngreens, nblues, rnreds, rngreens, rnblues);
		FAIL;
	} else
		CHECK;

	rgbmask = rm | gm | bm ;
	
	if( bitcount(rgbmask) != rnreds + rnblues + rngreens) {
		report("The returned colourplane  masks had at least one shared bit.");
		FAIL;
	} else
		CHECK;

	for (i = 0; i < ncolors; i++) {
		pm = pm_array[i];
		trace("pixel value of 0x%lx", pm);
		if( (rgbmask & pm) != 0L) {
			report("pixel value %ld shared at least one bit with a colourplane.", pm);
			FAIL;
		} else
			CHECK;
		if ((pm & ~DEPTHMASK(vp->depth)) != 0L) {
			report("pixel value 0x%lx contained bits outside the depth", pm);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(7+2*ncolors);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocColorPlanes-3.(A)
 * If the visual class DirectColor is supported: When the
 * visual type of the colormap argument is DirectColor, then
 * rmask_return lies within the red pixel subfield and the
 * gmask_return lies within the green pixel subfield and the
 * bmask_return lies within the blue pixel subfield.
 */
static void
t003()
{
XVisualInfo	*vp;
unsigned long	vmask = (1<<DirectColor);
unsigned long	*pixm, *rr, *gr, *br;
unsigned long	rm, gm, bm, pm, rgbmask, pixel, i;
Status		status;
int		cells;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		tet_result(TET_UNSUPPORTED);
		return;
	}

	resetsupvis(vmask);
	nextsupvis(&vp);
		
	colormap = makecolmap(display, vp->visual, AllocNone);
	ncolors = 1;
	nreds = bitcount(vp->red_mask);
	ngreens = bitcount(vp->green_mask);
	nblues = bitcount(vp->blue_mask);
	cells = ncolors * (1 << (nreds+ngreens+nblues));
	contig = False;
	pixels_return = &pm;
	rmask_return = &rm;
	gmask_return = &gm;
	bmask_return = &bm;

	startcall(display);
	if (isdeleted())
		return;
	status = XAllocColorPlanes(display, colormap, contig, pixels_return, ncolors, nreds, ngreens, nblues, rmask_return, gmask_return, bmask_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(status == (Status) 0) {
		report("%s failed to return non-zero", TestName);
		FAIL;
	} else {
		trace("%d cells allocated in the colormap", cells);
		CHECK;
	}

	/*
	 * We can check the mask returned matches the pixel subfield exactly
	 * since we requested all planes to be allocated.
	 */
	if( (vp->red_mask != rm) ) {
		report("Returned red mask not contained in red pixel subfield");
		FAIL;
	} else
		CHECK;
	if( (vp->green_mask != gm) ) {
		report("Returned green mask not contained in green pixel subfield");
		FAIL;
	} else
		CHECK;
	if( (vp->blue_mask != bm) ) {
		report("Returned blue mask not contained in blue pixel subfield");
		FAIL;
	} else
		CHECK;

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocColorPlanes-4.(A)
 * When the contig argument is True, then rmask_return,
 * gmask_return and bmask_return each contains a contiguous set
 * of bits set to one.
 */
static void
t004()
{
XVisualInfo	*vp;
unsigned long	vmask = (1<<DirectColor);
unsigned long	*pixm, *rr, *gr, *br;
unsigned long	rm, gm, bm, pm, rgbmask, pixel, i;
XColor 		color;
Status		status;
int		cells;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		tet_result(TET_UNSUPPORTED);
		return;
	}

	resetsupvis(vmask);
	nextsupvis(&vp);
		
	colormap = makecolmap(display, vp->visual, AllocNone);
	ncolors = 1;
	nreds = bitcount(vp->red_mask);
	ngreens = bitcount(vp->green_mask);
	nblues = bitcount(vp->blue_mask);
	cells = ncolors * (1 << (nreds+ngreens+nblues));
	contig = True;
	pixels_return = &pm;
	rmask_return = &rm;
	gmask_return = &gm;
	bmask_return = &bm;

	/* 
	 * Just ask for a small number of planes if more was possible.
	 * Originally this test requested all planes - but of course this
	 * gave little scope for contig to make any difference.
	 */
	if(nreds > 2)
		nreds = 2;
	if(ngreens > 2)
		ngreens = 2;
	if(nblues > 2)
		nblues = 2;
	trace("test with %d reds %d greens %d blues", nreds, ngreens, nblues);

	startcall(display);
	if (isdeleted())
		return;
	status = XAllocColorPlanes(display, colormap, contig, pixels_return, ncolors, nreds, ngreens, nblues, rmask_return, gmask_return, bmask_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(status == (Status) 0) {
		report("%s failed to return non-zero", TestName);
		FAIL;
	} else {
		trace("%d cells allocated in the colormap", cells);
		CHECK;
	}

	if((contiguous(rm) && contiguous(gm) && contiguous(bm)) == 0) {
		report("Returned colourplane masks were not contiguous");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECol.mc */
/*
 * Assertion XAllocColorPlanes-5.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XAllocColorPlanes(display, colormap, contig, pixels_return, ncolors, nreds, ngreens, nblues, rmask_return, gmask_return, bmask_return);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/*
 * Assertion XAllocColorPlanes-6.(A)
 * When the ncolors argument is zero, then a BadValue error
 * occurs.
 */
static void
t006()
{
XVisualInfo *vp;
Visual *visual;
unsigned long vmask = (1<<DirectColor);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		tet_result(TET_UNSUPPORTED);
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		colormap = makecolmap(display, vp->visual, AllocNone);
		ncolors = 0;
		startcall(display);
		if (isdeleted())
			return;
		StatusReturn = XAllocColorPlanes(display, colormap, contig, pixels_return, ncolors, nreds, ngreens, nblues, rmask_return, gmask_return, bmask_return);
		endcall(display);
		if (StatusReturn != 0) {
			report("Status returned was %d", StatusReturn);
			FAIL;
		}
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadValue)
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocColorPlanes-7.(A)
 * When the value of contig is other than True or False, then
 * a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t007()
{
#undef	VALUE_ARG
#define	VALUE_ARG contig
#undef	VALUE_LIST
#define	VALUE_LIST contigvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		StatusReturn = XAllocColorPlanes(display, colormap, contig, pixels_return, ncolors, nreds, ngreens, nblues, rmask_return, gmask_return, bmask_return);
		endcall(display);
		if (StatusReturn != 0) {
			report("Status returned was %d", StatusReturn);
			FAIL;
		}
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
