/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: allcnmdclr.m,v 1.7 92/06/11 16:14:53 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XAllocNamedColor(display, colormap, color_name, screen_def_return, exact_def_return)
 *   Display *display;
 *   Colormap colormap;
 *   char *color_name;
 *   XColor *screen_def_return;
 *   XColor *exact_def_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XAllocNamedColor	1
char    *TestName = "XAllocNamedColor";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XAllocNamedColor function
 */
static Display *display;
static Colormap colormap;
static char *color_name;
static XColor *screen_def_return;
static XColor *exact_def_return;


static int 	StatusReturn;

XColor dummycol;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
	color_name = "";
	screen_def_return = &dummycol;
	exact_def_return = &dummycol;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XAllocNamedColor-1.(A)
 * A call to XAllocNamedColor allocates a read-only entry in
 * the colormap argument corresponding to the named colour
 * specified by the color_name argument and returns the exact
 * RGB values in the red, green, and blue components in the
 * XColor structure named by the exact_def_return argument,
 * returns the closest colour supported by the screen with
 * which the colormap argument is associated in the red, green,
 * and blue components in the XColor structure named by the
 * screen_def_return argument and returns the colourmap entry
 * in the pixel component in the XColor structure named by the
 * screen_def_return argument.
 */
static void
t001()
{
XVisualInfo *vp;
char *goodname;
Status status;
XColor screencol, exactcol, testcol;
unsigned long vmask;
unsigned short trunc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals reported as valid.");
		return;
	}

	if( (goodname = tet_getvar("XT_GOOD_COLORNAME")) == (char *) 0) {
		delete("XT_GOOD_COLORNAME is not defined.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting XAllocNamedColor() for class %s, color %s", displayclassname(vp->class), goodname);
		colormap = makecolmap(display, vp -> visual, AllocNone);
		color_name = goodname;		
		screen_def_return = &screencol;
		exact_def_return = &exactcol;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocNamedColor(display, colormap, color_name, screen_def_return, exact_def_return);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if( status == (Status) 0) {
			report("%s failed to return non-zero.", TestName);
			FAIL;
			continue;
		} else
			CHECK;

		testcol = screencol;
		trace("Screen: pixel %lu, r %u g %u b %u", screencol.pixel , screencol.red, screencol.green, screencol.blue);
		trace("Exact : pixel %lu, r %u g %u b %u", exactcol.pixel, exactcol.red, exactcol.green, exactcol.blue);
		trace("Test : pixel %lu, r %u g %u b %u", testcol.pixel, testcol.red, testcol.green, testcol.blue);

		status = XAllocColor(display, colormap, &testcol);
		if(status == (Status) 0) {
			report("XAllocColor() failed to return non-zero.");
			FAIL;
			continue;
		} else
			CHECK;

		trace("Exact : pixel %lu, r %u g %u b %u", exactcol.pixel, exactcol.red, exactcol.green, exactcol.blue);

		if( screencol.pixel != testcol.pixel ) {
			report("%s return pixel value %lu instead of %lu.", 
				TestName, screencol.pixel, testcol.pixel);
			FAIL;
		} else
			CHECK;

		if((screencol.red != testcol.red) ||
		   (screencol.green != testcol.green) ||
	 	   (screencol.blue != testcol.blue) ) {
			report("%s return RGB values r %u g %u b %u instead of r %u g %u b %u.", 
				TestName,
				screencol.red, screencol.green, screencol.blue,
				testcol.red, testcol.green, testcol.blue);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(4 * nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocNamedColor-2.(A)
 * When the color_name argument refers to a colour in the
 * colour database, then XAllocNamedColor returns non-zero.
 */
static void
t002()
{
XVisualInfo *vp;
char *goodname;
Status status;
XColor screencol, exactcol, testcol;
unsigned long vmask;
unsigned short trunc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals reported as valid.");
		return;
	}

	if( (goodname = tet_getvar("XT_GOOD_COLORNAME")) == (char *) 0) {
		delete("XT_GOOD_COLORNAME is not defined.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting XAllocNamedColor() for class %s, color %s", displayclassname(vp->class), goodname);
		colormap = makecolmap(display, vp -> visual, AllocNone);
		color_name = goodname;		
		screen_def_return = &screencol;
		exact_def_return = &exactcol;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocNamedColor(display, colormap, color_name, screen_def_return, exact_def_return);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if( status == (Status) 0) {
			report("%s failed to return non-zero with color %s.", 
				TestName, goodname);
			FAIL;
		} else 
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocNamedColor-3.(A)
 * When the color_name argument does not refer to a colour in
 * the colour database, then XAllocNamedColor returns zero.
 */
static void
t003()
{
XVisualInfo *vp;
char *badname;
Status status;
XColor screencol, exactcol, testcol;
unsigned long vmask;
unsigned short trunc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals reported as valid.");
		return;
	}

	if( (badname = tet_getvar("XT_BAD_COLORNAME")) == (char *) 0) {
		delete("XT_BAD_COLORNAME is not defined.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting XAllocNamedColor() for class %s, color %s", displayclassname(vp->class), badname);
		colormap = makecolmap(display, vp -> visual, AllocNone);
		color_name = badname;		
		screen_def_return = &screencol;
		exact_def_return = &exactcol;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocNamedColor(display, colormap, color_name, screen_def_return, exact_def_return);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if( status != (Status) 0) {
			report("%s failed to return zero with color %s.", 
				TestName, badname);
			FAIL;
		} else 
			CHECK;
	}

	CHECKPASS(nsupvis());


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllocNamedColor-4.(A)
 * Upper and lower case characters in the color_name argument
 * refer to the same colour.
 */
static void
t004()
{
XVisualInfo *vp;
char *cp, *goodname, *casename;
Status status;
XColor screencol, exactcol, alscreencol, alexactcol;
unsigned long vmask;
unsigned short trunc;
int i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals reported as valid.");
		return;
	}

	if( (goodname = tet_getvar("XT_GOOD_COLORNAME")) == (char *) 0) {
		delete("XT_GOOD_COLORNAME is not defined.");
		return;
	}

	casename = (char *) malloc( strlen(goodname) + 1);
	strcpy(casename, goodname);

	for(i=0, cp=casename; *cp; i++)
		if(i&1)
			*cp++ = tolower(*cp);
		else
			*cp++ = toupper(*cp);


	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		trace("Attempting XAllocNamedColor() for class %s, color %s", displayclassname(vp->class), goodname);
		colormap = makecolmap(display, vp -> visual, AllocNone);
		color_name = goodname;		
		screen_def_return = &screencol;
		exact_def_return = &exactcol;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocNamedColor(display, colormap, color_name, screen_def_return, exact_def_return);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if( status == (Status) 0) {
			report("%s failed to return non-zero with color %s.", 
				TestName, goodname);
			FAIL;
			continue;
		} else
			CHECK;
			
		color_name = casename;
		trace("Testing colourname %s", color_name);
		screen_def_return = &alscreencol;
		exact_def_return = &alexactcol;
		startcall(display);
		if (isdeleted())
			return;
		status = XAllocNamedColor(display, colormap, color_name, screen_def_return, exact_def_return);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if( status == (Status) 0) {
			report("%s failed to return non-zero with color %s.", 
				TestName, casename);
			FAIL;
			continue;
		} else 
			CHECK;

		if(screencol.pixel != alscreencol.pixel) {
			report("%s for name %s", TestName, casename);
			report("returned pixel value %lu", alscreencol.pixel);
			report("%s for name %s", TestName, goodname);
			report("returned pixel value %lu", screencol.pixel);
			FAIL;
		} else
			CHECK;

		if((exactcol.red != alexactcol.red) ||
		   (exactcol.green != alexactcol.green) ||
		   (exactcol.blue != alexactcol.blue)) {
			report("%s for name %s", TestName, casename);
			report("returned exact RGB values r %u g %u b %u",
			  alexactcol.red, alexactcol.green, alexactcol.blue);
			report("%s for name %s", TestName, goodname);
			report("returned exact RGB values r %u g %u b %u",
			  exactcol.red, exactcol.green, exactcol.blue);
			FAIL;
		} else
			CHECK;

		if((screencol.red != alscreencol.red) ||
		   (screencol.green != alscreencol.green) ||
		   (screencol.blue != alscreencol.blue)) {
			report("%s for name %s", TestName, casename);
			report("returned closest RGB values r %u g %u b %u",
			  alscreencol.red, alscreencol.green, alscreencol.blue);
			report("%s for name %s", TestName, goodname);
			report("returned closest RGB values r %u g %u b %u",
			  screencol.red, screencol.green, screencol.blue);
			FAIL;
		} else
			CHECK;
	}

	free(casename);
	CHECKPASS(5 * nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECol.mc */
/*
 * Assertion XAllocNamedColor-5.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XAllocNamedColor(display, colormap, color_name, screen_def_return, exact_def_return);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/* Including from file error/EAll.mc */
/*
 * Assertion XAllocNamedColor-6.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
