/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: chnggc.m,v 1.12 92/06/11 16:15:03 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XChangeGC(display, gc, valuemask, values)
 *   Display		*display;
 *   GC		gc;
 *   unsigned 	long valuemask;
 *   XGCValues	*values;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XChangeGC	1
char    *TestName = "XChangeGC";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_GC gc


/*
 * Arguments to the XChangeGC function
 */
static Display		*display;
static GC		gc;
static unsigned 	long valuemask;
static XGCValues	*values;


XGCValues	srcgcv = { GXxor, AllPlanes, 0, 1, 0, LineSolid, CapButt,
			   JoinMiter, FillSolid, EvenOddRule, ArcPieSlice,
			   None, };

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	gc = 0;
	valuemask = GCTile;
	values = &srcgcv;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	gc = ErrdefGC;
}

/*
 * Assertion XChangeGC-1.(A)
 * A call to XChangeGC changes the components specified by the
 * valuemask argument in the specified GC to the values in the
 * values argument.
 */
static void
t001()
{
XVisualInfo	*vp;
Window		win;
XRectangle	rect;
Pixmap		pmap;
XGCValues	gcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);
	gc = makegc(display, win);
	
	gcv.function = GXxor;
	valuemask = GCFunction;
	values = &gcv;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawPoint(display, win, gc, 0, 0);
	XSetForeground(display, gc, W_FG ^ W_BG);
	XDrawPoint(display, win, gc, 0, 0);

	if( ! checkpixel(display, win, 0, 0, W_BG) ) {
		report("The GC function component was not set to GXxor");
		report("by a call to XChangeGC");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-2.(A)
 * When a call to XChangeGC changes the clip-mask component of
 * gc, then any previous XSetClipRectangles request on the
 * specified GC is overridden.
 */
static void
t002()
{
XVisualInfo *vp;
XGCValues   srcgcv;
GC gc;
Window win;
XImage	*im;
XRectangle *rectangles;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);

	srcgcv.foreground = W_FG;
	srcgcv.line_width = 10;
	srcgcv.clip_mask = None;

	/* Create a GC. */
	gc = XCreateGC(display, win, (GCClipMask | GCLineWidth | GCForeground),
								 &srcgcv);

	/* Draw line from (0, 0) to (100, 0) using XDrawLine. */
	XDrawLine(display, win, gc, 0, 0, 100, 0);

	/* Save image on drawable using XGetImage. */
	im = savimage(display, win);

	/* Clear drawable. */
	dclear(display, win);

	/* Change the clip_mask component of the GC using XSetClipRectangles. */
	XSetClipRectangles(display, gc, 0, 0, rectangles, 0, Unsorted);

	/* Draw line from (0, 0) to (100, 0) using XDrawLine. */
	XDrawLine(display, win, gc, 0, 0, 100, 0);

	/* Verify using XGetImage that the image on the drawable is altered. */
	if(diffsavimage(display, win, im)) {
		delete("XSetClipRectangles did not set clip_mask component of GC");
		return;
	} else
		CHECK;

	/* Change the GC clip_mask component using XChangeGC to original. */
	srcgcv.clip_mask = None;
	values = &srcgcv;
	valuemask = GCClipMask;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* Draw line from (0, 0) to (100, 0) using XDrawLine. */
	XDrawLine(display, win, gc, 0, 0, 100, 0);

	/* Verify using XGetImage that the image is restored. */
	if( ! compsavimage(display, win, im)) {
		report("XChangeGC with GCClipMask did not override call to XSetClipRectangles");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-3.(A)
 * When a call to XChangeGC changes the dash-offset or dashes
 * component of gc, then any previous XSetDashes request on the
 * specified GC is overridden.
 */
static void
t003()
{
XVisualInfo *vp;
XGCValues   srcgcv;
GC gc;
Window win;
XImage	*im;
static char	dashes[] = {20, 10};
static char	odashes[] = {10, 10};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);

	srcgcv.foreground = W_FG;
	srcgcv.background = W_BG;
	srcgcv.line_style = LineDoubleDash;
	srcgcv.line_width = 10;
	srcgcv.dashes = 10;
	srcgcv.dash_offset = 0;

	/* Create a GC. */
	gc = XCreateGC(display, win, (GCLineWidth | GCLineStyle | GCDashOffset |
			GCDashList | GCForeground | GCBackground), &srcgcv);

	/* Draw dashed line from (0, 0) to (100, 0) using XDrawLine. */
	XDrawLine(display, win, gc, 0, 0, 100, 0);

	/* Save image on drawable using XGetImage. */
	im = savimage(display, win);

	/* Part 1. Verify cancelling when GCDashList is used. */
	/* Change the dashes component of the GC */
	XSetDashes(display, gc, 0, dashes, sizeof(dashes)/sizeof(char));

	/* Draw dashed line from (0, 0) to (100, 0) using XDrawLine. */
	XDrawLine(display, win, gc, 0, 0, 100, 0);

	/* Verify using XGetImage that XSetDashes altered image on drawable. */
	if(diffsavimage(display, win, im)) {
		delete("XSetDashes did not set dashes component of GC");
		return;
	} else
		CHECK;

	/* Change the GC dashes component using XChangeGC to original value. */
	srcgcv.dashes = 10;
	values = &srcgcv;
	valuemask = GCDashList;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* Draw dashed line from (0, 0) to (100, 0) using XDrawLine. */
	XDrawLine(display, win, gc, 0, 0, 100, 0);

	/* Verify using XGetImage that the image is as before XSetDashes. */
	if( ! compsavimage(display, win, im)) {
		report("XChangeGC with GCDashList did not override call to XSetDashes");
		FAIL;
	} else
		CHECK;

	/* Part 2. Verify cancelling when GCDashOffset is used. */
	/* Change the dash_offset component of the GC using XSetDashes. */
	XSetDashes(display, gc, 5, odashes, sizeof(odashes)/sizeof(char));

	/* Draw dashed line from (0, 0) to (100, 0) using XDrawLine. */
	XDrawLine(display, win, gc, 0, 0, 100, 0);

	/* Verify using XGetImage that XSetDashes altered image on drawable. */
	if(diffsavimage(display, win, im)) {
		delete("XSetDashes did not set dash_offset component of GC");
		return;
	} else
		CHECK;

	/* Change the GC dash_offset component using XChangeGC to original value. */
	srcgcv.dash_offset = 0;
	values = &srcgcv;
	valuemask = GCDashOffset;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* Draw dashed line from (0, 0) to (100, 0) using XDrawLine. */
	XDrawLine(display, win, gc, 0, 0, 100, 0);

	/* Verify using XGetImage that the image is as before XSetDashes. */
	if( ! compsavimage(display, win, im)) {
		report("XChangeGC with GCDashOffset did not override call to XSetDashes");
		FAIL;
	} else
		CHECK;

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EGC.mc */
/*
 * Assertion XChangeGC-4.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/*
 * Assertion XChangeGC-5.(A)
 * When the font does not name a valid font, and the GCFont
 * bit is set in valuemask, then a BadFont error occurs.
 */
static void
t005()
{
Window w;
Font font;
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.font = badfont(Dsp);

	gc = makegc(display, DRW(display));
	values = &vals;
	valuemask = GCFont;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadFont) {
		report("Got %s, Expecting BadFont", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadFont) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-6.(A)
 * When the tile does not name a valid pixmap, and the GCTile
 * bit is set in valuemask, then a BadPixmap error occurs.
 */
static void
t006()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.tile = badpixm(display);

	gc = makegc(display, DRW(display));
	values = &vals;
	valuemask = GCTile;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadPixmap) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-7.(A)
 * When the stipple does not name a valid pixmap, and the
 * GCStipple bit is set in valuemask, then a BadPixmap error
 * occurs.
 */
static void
t007()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.stipple = badpixm(display);

	gc = makegc(display, DRW(display));
	values = &vals;
	valuemask = GCStipple;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadPixmap) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-8.(A)
 * When the clip-mask does not name a valid pixmap, or None,
 * and the GCClipMask bit is set in valuemask, then a
 * BadPixmap error occurs.
 */
static void
t008()
{
Window w;
Pixmap pm;
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.clip_mask = badpixm(display);

	gc = makegc(display, DRW(display));
	values = &vals;
	valuemask = GCClipMask;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadPixmap)
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-9.(C)
 * When the graphics context and the tile pixmap do not have
 * the same depth, and the GCTile bit is set in valuemask,
 * then a BadMatch error occurs.
 */
static void
t009()
{
int		count;
XGCValues 	vals;
XVisualInfo	*vp;
Drawable	d;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	count = 0;
	vals.tile = XCreatePixmap(display, DRW(display), 1,1,1);
	valuemask = GCTile;
	values = &vals;
	for(resetvinf(VI_WIN); nextvinf(&vp);)
		if(vp->depth != 1) {
			trace("Testing a window of depth %d",vp->depth);
			count++;
			d = makewin(display, vp);
			gc = makegc(display, d);
			startcall(display);
			if (isdeleted())
				return;
			XChangeGC(display, gc, valuemask, values);
			gcflush(A_DISPLAY, A_GC);
			endcall(display);
			if (geterr() != BadMatch) {
				report("Got %s, Expecting BadMatch", errorname(geterr()));
				FAIL;
			}
			if(geterr() == BadMatch)
				CHECK;
			else
				FAIL;
		}

	for(resetvinf(VI_PIX); nextvinf(&vp);)
		if(vp->depth != 1) {
			trace("Testing a window of depth %d",vp->depth);
			count++;
			d = makepixm(display, vp);
			gc = makegc(display, d);
			startcall(display);
			if (isdeleted())
				return;
			XChangeGC(display, gc, valuemask, values);
			gcflush(A_DISPLAY, A_GC);
			endcall(display);
			if (geterr() != BadMatch) {
				report("Got %s, Expecting BadMatch", errorname(geterr()));
				FAIL;
			}
			if(geterr() == BadMatch)
				CHECK;
			else
				FAIL;
		}


	if(count == 0) {
		tet_result(TET_UNSUPPORTED);
		report("Only depth one drawables are supported.");
		return;
	} else
		CHECKPASS(count);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-10.(A)
 * When the graphics context and the tile pixmap were not
 * created for the same root, and the GCTile bit is set in
 * valuemask, then a BadMatch error occurs.
 */
static void
t010()
{
char    *altroot;
int     scr_num;
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	altroot = tet_getvar("XT_ALT_SCREEN");
	if (altroot == NULL) {
		delete("XT_ALT_SCREEN not set");
		return;
	}
	if (*altroot == 'U') {
		report("Only one screen supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	scr_num = atoi(altroot);
	if (scr_num == DefaultScreen(display)) {
		delete("The Alternate root was the same as the one under test");
		return;
	}
	if (scr_num >= ScreenCount(display)) {
		delete("Screen given in XT_ALT_SCREEN could not be accessed");
		return;
	}

	/*
	 * Create a 1x1 depth 1 pixmap on other screen
	 * and use it to create a gc
	 */
	
	vals.tile =  XCreatePixmap(display, RootWindow(display, scr_num), 1, 1, 1);
	gc = makegc(display, DRW(display));
	values = &vals;
	valuemask = GCTile;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-11.(C)
 * When the stipple pixmap does not have depth one, and the
 * GCStipple bit is set in valuemask, then a BadMatch error
 * occurs.
 */
static void
t011()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vals.stipple = nondepth1pixmap(display, DRW(display))) == (Pixmap) 0) {
		tet_result(TET_UNSUPPORTED);
		report("Only depth 1 pixmaps are supported.");
		return;
	} else
		CHECK;

	gc = makegc(display, DRW(display));
	values = &vals;
	valuemask = GCStipple;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}
	
	if(geterr() == BadMatch)
		CHECK;
	else
		FAIL;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-12.(C)
 * When the graphics context and the stipple pixmap were not
 * created for the same root, and the GCStipple bit is set in
 * valuemask, then a BadMatch error occurs.
 */
static void
t012()
{
char    *altroot;
int     scr_num;
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	altroot = tet_getvar("XT_ALT_SCREEN");
	if (altroot == NULL) {
		delete("XT_ALT_SCREEN not set");
		return;
	}
	if (*altroot == 'U') {
		report("Only one screen supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	scr_num = atoi(altroot);
	if (scr_num == DefaultScreen(display)) {
		delete("The Alternate root was the same as the one under test");
		return;
	}
	if (scr_num >= ScreenCount(display)) {
		delete("Screen given in XT_ALT_SCREEN could not be accessed");
		return;
	}

	/*
	 * Create a 1x1 depth 1 pixmap on other screen
	 * and use it to create a gc
	 */
	
	vals.stipple = XCreatePixmap(display, RootWindow(display, scr_num), 1, 1, 1);
	gc = makegc(display, DRW(display));
	values = &vals;
	valuemask = GCStipple;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-13.(C)
 * When the clip-mask is set to a pixmap, and the clip-mask
 * does not have depth one, and the GCClipMask bit is set in
 * valuemask, then a BadMatch error occurs.
 */
static void
t013()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vals.clip_mask = nondepth1pixmap(display, DRW(display))) == (Pixmap) 0) {
		tet_result(TET_UNSUPPORTED);
		report("Only depth 1 pixmaps are supported.");
		return;
	} else
		CHECK;

	gc = makegc(display, DRW(display));
	values = &vals;
	valuemask = GCClipMask;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadMatch)
		CHECK;
	else
		FAIL;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-14.(C)
 * When the clip-mask is set to a pixmap, and the graphics
 * context and the clip-mask were not created for the same
 * root, and the GCClipMask bit is set in valuemask, then a
 * BadMatch error occurs.
 */
static void
t014()
{
char    *altroot;
int     scr_num;
XGCValues	vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	altroot = tet_getvar("XT_ALT_SCREEN");
	if (altroot == NULL) {
		delete("XT_ALT_SCREEN not set");
		return;
	}
	if (*altroot == 'U') {
		report("Only one screen supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	scr_num = atoi(altroot);
	if (scr_num == DefaultScreen(display)) {
		delete("The Alternate root was the same as the one under test");
		return;
	}
	if (scr_num >= ScreenCount(display)) {
		delete("Screen given in XT_ALT_SCREEN could not be accessed");
		return;
	}

	/*
	 * Create a 1x1 depth 1 pixmap on other screen
	 * and use it to create a gc
	 */

	vals.clip_mask = XCreatePixmap(display, RootWindow(display, scr_num), 1, 1, 1);
	gc = makegc(display, DRW(display));
	values = &vals;
	valuemask = GCClipMask;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-15.(A)
 * When the valuemask argument is other than a bitwise OR of
 * none or any of GCFunction, GCPlaneMask, GCForeground,
 * GCBackground, GCLineWidth, GCLineStyle, GCCapStyle,
 * GCJoinStyle, GCFillStyle, GCFillRule, GCTile, GCStipple,
 * GCTileStipXOrigin, GCTileStipYOrigin, GCFont,
 * GCSubWindowMode, GCGraphicsExposures, GCClipXOrigin,
 * GCClipYOrigin, GCClipMask, GCDashOffset, GCDashList, or
 * GCArcMode, then a BadValue error occurs.
 */
static void
t015()
{
Window w;
XGCValues srcgcv;
int i;
unsigned char *p;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	i = sizeof(XGCValues);
	p = (unsigned char *)&srcgcv;
	while( i-- > 0 )
		*p++ = 0xff;

	gc = makegc(display, DRW(display));
	values = &srcgcv;
	srcgcv.foreground = 1;
	srcgcv.function = ~0;
	valuemask = GCFunction | GCForeground | ~(GCFunction | GCPlaneMask | GCForeground | GCBackground |
		    GCLineWidth | GCLineStyle | GCCapStyle | GCJoinStyle |
		    GCFillStyle | GCFillRule | GCTile | GCStipple |
		    GCTileStipXOrigin | GCTileStipYOrigin | GCFont | GCSubwindowMode |
		    GCGraphicsExposures | GCClipXOrigin | GCClipYOrigin | GCClipMask |
		    GCDashOffset | GCDashList | GCArcMode);
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadValue)
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-16.(A)
 * When the function is other than GXclear, GXand,
 * GXandReverse, GXcopy, GXandInverted, GXnoop, GXxor,
 * GXor, GXnor, GXequiv, GXinvert, GXorReverse,
 * GXcopyInverted, GXorInverted, GXnand or GXset, and the
 * GCFunction bit is set in valuemask, then a BadValue error
 * occurs.
 */
static void
t016()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	srcgcv.function = GXclear + GXand + GXandReverse + GXcopy + 
			GXandInverted + GXnoop + GXxor + GXor + GXnor + 
			GXequiv + GXinvert + GXorReverse +
			GXcopyInverted + GXorInverted + GXnand + GXset + 1;
	values = &srcgcv;
	valuemask = GCFunction;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-17.(A)
 * When the line_style is other than LineSolid,
 * LineDoubleDash or LineOnOffDash, and the GCLineStyle bit is
 * set in valuemask, then a BadValue error occurs.
 */
static void
t017()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	srcgcv.line_style = LineSolid + LineDoubleDash + LineOnOffDash + 1;
	values = &srcgcv;
	valuemask = GCLineStyle;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-18.(A)
 * When the cap_style is other than CapNotLast, CapButt,
 * CapRound or CapProjecting, and the GCCapStyle bit is set in
 * valuemask, then a BadValue error occurs.
 */
static void
t018()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	srcgcv.cap_style = CapNotLast + CapButt + CapRound + CapProjecting + 1;
	values = &srcgcv;
	valuemask = GCCapStyle;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-19.(A)
 * When the join_style is other than JoinMiter, JoinRound or
 * JoinBevel, and the GCJoinStyle bit is set in valuemask,
 * then a BadValue error occurs.
 */
static void
t019()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	srcgcv.join_style = JoinMiter + JoinRound + JoinBevel + 1;
	values = &srcgcv;
	valuemask = GCJoinStyle;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-20.(A)
 * When the fill_style is other than FillSolid, FillTiled,
 * FillStippled or FillOpaqueStippled, and the GCFillStyle bit
 * is set in valuemask, then a BadValue error occurs.
 */
static void
t020()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	srcgcv.fill_style = FillSolid + FillTiled + FillStippled + FillOpaqueStippled + 1;
	values = &srcgcv;
	valuemask = GCFillStyle;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-21.(A)
 * When the fill_rule is other than EvenOddRule or WindingRule,
 * and the GCFillRule bit is set in valuemask, then a BadValue
 * error occurs.
 */
static void
t021()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	srcgcv.fill_rule = EvenOddRule + WindingRule + 1;
	values = &srcgcv;
	valuemask = GCFillRule;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-22.(A)
 * When arc_mode is other than ArcChord or ArcPieSlice, and
 * the GCArcMode bit is set in valuemask, then a BadValue
 * error occurs.
 */
static void
t022()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	srcgcv.arc_mode = ArcChord + ArcPieSlice + 1;
	values = &srcgcv;
	valuemask = GCArcMode;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-23.(A)
 * When the subwindow_mode is other than ClipByChildren or
 * IncludeInferiors, and the GCSubwindowMode bit is set in
 * valuemask, then a BadValue error occurs.
 */
static void
t023()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	srcgcv.subwindow_mode = ClipByChildren + IncludeInferiors + 1;
	values = &srcgcv;
	valuemask = GCSubwindowMode;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-24.(A)
 * When graphics_exposure is other than True or False, and
 * the GCGraphicsExposures bit is set in valuemask, then a
 * BadValue error occurs.
 */
static void
t024()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display,DRW(display));
	srcgcv.graphics_exposures = (int) False + (int) True + 1;
	values = &srcgcv;
	valuemask = GCGraphicsExposures;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeGC-25.(A)
 * When dashes is set to zero, and the GCDashList bit is set
 * in valuemask, then a BadValue error occurs.
 */
static void
t025()
{
XGCValues srcgcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	srcgcv.dashes = (char) 0;
	values = &srcgcv;
	valuemask = GCDashList;
	startcall(display);
	if (isdeleted())
		return;
	XChangeGC(display, gc, valuemask, values);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	t021, 21,
	t022, 22,
	t023, 23,
	t024, 24,
	t025, 25,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
