/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: crtclrmp.m,v 1.12 92/06/11 16:15:27 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Colormap
 *   XCreateColormap(display, win, visual, alloc)
 *   Display	*display;
 *   Window	win;
 *   Visual	*visual;
 *   int 	alloc;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCreateColormap	1
char    *TestName = "XCreateColormap";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW win
#define A_DRAWABLE win


/*
 * Arguments to the XCreateColormap function
 */
static Display	*display;
static Window	win;
static Visual	*visual;
static int 	alloc;


Visual dummyvisual;


/* Value list for use in test t010 */
static int 	allocvallist[] = {
	AllocNone,
	AllocAll,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	win = DRW(display);
	visual = &dummyvisual;
	alloc = AllocNone;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XCreateColormap-1.(A)
 * A call to XCreateColormap creates a colourmap of the type
 * visual on the screen for which the window argument was
 * created and returns a colourmap ID.
 */
static void
t001()
{
XVisualInfo *vp;
Colormap	cmap;
Display		*d2;
Window		w2;
int i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	alloc = AllocNone;
	for (resetvinf(VI_WIN); nextvinf(&vp); ) {

		win = makewin(display, vp);
		visual = vp->visual;
		startcall(display);
		if (isdeleted())
			return;
		cmap = XCreateColormap(display, win, visual, alloc);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		/*
		 * The only way that we can really check that we have a valid colourmap
		 * is to try and use it.  So try to install it.  Of course it
		 * could be that the install routine is broken but the point is that
		 * something is wrong.  Print out both routine names to alert the tester
		 * about possible sources of the problem.
		 */
		CATCH_ERROR(display);
		XInstallColormap(display, cmap);
		RESTORE_ERROR(display);
		if (GET_ERROR(display) != Success) {
			report("Result of a XCreateColormap could not be used with XInstallColormap");
			FAIL;
		} else {
			CATCH_ERROR(display);
			XUninstallColormap(display, cmap);
			RESTORE_ERROR(display);

			if (GET_ERROR(display) != Success) {
				report("Result of a XCreateColormap could not be used with XUninstallColormap");
				FAIL;
			} else 
				CHECK;
		}
	}
	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateColormap-2.(D)
 *   Reason: The statement in the X11 specification is not specific enough to write a test
 * If any of the visual classes StaticGray, StaticColor or
 * TrueColor are supported: When XCreateColormap is called with
 * a visual class that is one of StaticGray, StaticColor or
 * TrueColor and alloc is AllocNone, then the entries have
 * defined values specific to the visual that are
 * implementation defined.
 */
static void
t002()
{
Colormap	cmap;
XVisualInfo	*vp;
unsigned long vmask = ((1L<<TrueColor) | (1L<<StaticColor) | (1L<<StaticGray));
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		unsupported("TrueColor, StaticColor and StaticGray are not supported");
		return;
	}

	alloc = AllocNone;
	for ( resetsupvis(vmask); nextsupvis(&vp); ) {

		visual = vp->visual;
		startcall(display);
		if (isdeleted())
			return;
		cmap = XCreateColormap(display, win, visual, alloc);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (geterr() == Success)
			CHECK;
	}

	CHECKUNTESTED(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateColormap-3.(C)
 * If any of the visual classes PseudoColor, GrayScale or
 * DirectColor are supported: When XCreateColormap is called
 * with a visual class that is one of PseudoColor, GrayScale
 * or DirectColor and alloc is AllocNone, then the colourmap
 * has no allocated entries.
 */
static void
t003()
{
Colormap	cmap;
XVisualInfo	*vi;
Status 		status;
unsigned long	*pix;
unsigned long	vmask;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


#define RW_COLORMAP_MASK ((1L<<DirectColor) | (1L<<PseudoColor) | (1L<<GrayScale))

	if( (vmask = visualsupported(display, RW_COLORMAP_MASK)) == 0L) {
		unsupported("DirectColor, PseudoColor and GrayScale are not supported");
		return;
	}

	alloc = AllocNone;
	for(resetsupvis(vmask); nextsupvis(&vi); ) {

		visual = vi->visual;
		startcall(display);
		if (isdeleted())
			return;
		cmap = XCreateColormap(display, win, visual, alloc);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		n = maxsize(vi);
		pix = (unsigned long *)malloc(n * sizeof(unsigned long));

		if (pix == NULL) {
			delete("malloc failed for pix array");
			return;
		}

		trace("allocate %d colour cells", n);
		status = XAllocColorCells(display, cmap, False, (unsigned long *)0, 0, pix, n);
		free((void*)pix);
		if (status == False) {
			report("Unable to allocate all colour cells");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateColormap-4.(A)
 * If any of the visual classes PseudoColor, GrayScale or
 * DirectColor are supported: When XCreateColormap is called
 * with a visual class that is one of PseudoColor, GrayScale
 * or DirectColor and the argument alloc is AllocAll, then the
 * entire colourmap is allocated read-write.
 */
static void
t004()
{
XVisualInfo	*vi;
Colormap	cmap;
Status	status;
unsigned long	pix;
unsigned long 	vmask;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

#define RW_COLORMAP_MASK ((1L<<DirectColor) | (1L<<PseudoColor) | (1L<<GrayScale))

	if( (vmask = visualsupported(display, RW_COLORMAP_MASK)) == 0L) {
		unsupported("DirectColor, PseudoColor and GrayScale are not supported");
		return;
	}

	alloc = AllocAll;
	for (resetsupvis(vmask); nextsupvis(&vi); ) {

		visual = vi->visual;
		startcall(display);
		if (isdeleted())
			return;
		cmap = XCreateColormap(display, win, visual, alloc);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		/*
		 * If the entire colourmap is allocated then it should not be
		 * possible to allocate any more entries.
		 */

		status = XAllocColorCells(display, cmap, False, (unsigned long*)0, 0, &pix, 1);

		if (status != (Status) 0) {
			report("There was an unallocated colour cell");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateColormap-5.(C)
 * If either of the visual classes GrayScale or PseudoColor
 * are supported: When the visual class is GrayScale or
 * PseudoColor, and the argument alloc is AllocAll, then a
 * call to XCreateColormap allocates N read-write colourmap
 * entries with pixel values from zero to N - 1, where N is the
 * number of colourmap entries value in the specified visual.
 */
static void
t005()
{
Colormap	cmap;
XVisualInfo	*vi;
XColor		col;
unsigned long	pix;
unsigned long		vmask 	= (1L<<PseudoColor) | (1L<<GrayScale);
int 		n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		unsupported("PseudoColor and GrayScale are not supported");
		return;
	}

	col.red = 31<<8;
	col.green = 63<<8;
	col.blue = 127<<8;
	col.flags = DoRed|DoGreen|DoBlue;

	alloc = AllocAll;
	for(resetsupvis(vmask); nextsupvis(&vi); ) {

		visual = vi->visual;
		startcall(display);
		if (isdeleted())
			return;
		cmap = XCreateColormap(display, win, visual, alloc);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if(geterr() != Success)
			continue;

		n = maxsize(vi);
		for(pix = 0L; pix < n; pix++) {
			col.pixel = pix;

			CATCH_ERROR(display);
			XStoreColor(display, cmap, &col);
			RESTORE_ERROR(display);
	
			if(GET_ERROR(display) != Success) {
				report("Pixel %lu could not be written in the colourmap.", pix);
				FAIL;
			}
		}

		if(pix == n)
			CHECK;
	}
	
	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateColormap-6.(C)
 * If the visual class DirectColor is supported: When the
 * visual class is DirectColor and the argument alloc is
 * AllocAll, then a call to XCreateColormap allocates
 * 2pow(nred+ngreen+nblue) read-write colourmap entries with
 * pixel values obtained by ORing zero or more of the planes
 * obtained by ORing the red_mask, green_mask and blue_mask
 * values in the visual argument, where nreds, ngreens and
 * nblues are the number of bits in the respective masks.
 */
static void
t006()
{
#define  NUM_COLS 4096
Colormap	cmap;
XVisualInfo	*vi;
XColor		colarr[NUM_COLS];
unsigned long	plane, count;
unsigned long		vmask = (1L<<DirectColor), b, planemask;
int 		n, i, j, bitsum;
int		bitpos[sizeof(long) * 8];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	if( (vmask = visualsupported(display, vmask)) == 0L) {
		unsupported("DirectColor is not supported");
		return;
	}

	for(i=0; i<NUM_COLS; i++) {
		colarr[i].red = 31<<8;
		colarr[i].green = 63<<8;
		colarr[i].blue = 127<<8;
		colarr[i].flags = DoRed|DoGreen|DoBlue;
	}		

	resetsupvis(vmask); 
	nextsupvis(&vi);

	alloc = AllocAll;
	visual = vi->visual;
	startcall(display);
	if (isdeleted())
		return;
	cmap = XCreateColormap(display, win, visual, alloc);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(geterr() == Success) {
		bitsum = (bitcount(vi->red_mask) + bitcount(vi->green_mask) + bitcount(vi->blue_mask));
		n = 1<< bitsum;
		for(count = 0L; count < n; count+=NUM_COLS) { 
			for (i=0; i<NUM_COLS; i++)
				colarr[i].pixel = count+i;
			CATCH_ERROR(display);
			XStoreColors(display, cmap, colarr,
				(count+NUM_COLS <= n) ? NUM_COLS : n-count);
			RESTORE_ERROR(display);
	
			if(GET_ERROR(display) != Success) {
				report("Pixel in range [%lu-%lu] could not be written in the colourmap.", count,count+NUM_COLS);
				FAIL;
			} 

		}
	}

	if (count >= n)
		CHECK;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateColormap-7.(C)
 * If any of the visual classes PseudoColor, GrayScale or
 * DirectColor are supported: When XCreateColormap is called
 * with a visual class that is one of PseudoColor, GrayScale
 * or DirectColor and the alloc argument is AllocAll, then
 * none of the entries can be freed with XFreeColors().
 */
static void
t007()
{
XVisualInfo	*vi;
Colormap	cmap;
Status	status;
unsigned long	pix;
unsigned long 	vmask = ((1L<<DirectColor) | (1L<<PseudoColor) | (1L<<GrayScale));
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		unsupported("DirectColor, PseudoColor and GrayScale are not supported");
		return;
	}

	alloc = AllocAll;
	for (resetsupvis(vmask); nextsupvis(&vi); ) {

		visual = vi->visual;
		startcall(display);
		if (isdeleted())
			return;
		cmap = XCreateColormap(display, win, visual, alloc);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		status = XAllocColorCells(display, cmap, False, (unsigned long*)0, 0, &pix, 1);

		pix = 0;

		CATCH_ERROR(display);
		XFreeColors(display, cmap, &pix, 1, 0);
		RESTORE_ERROR(display);
		if( GET_ERROR(display) == Success) {
			delete("XFreeColors() did not give an error deallocating a cell.");
			FAIL;
		} else
			CHECK;

		status = XAllocColorCells(display, cmap, False, (unsigned long*)0, 0, &pix, 1);

		if (status != (Status) 0) {
			report("A cell was deallocated by XFreeColors().");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(2*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateColormap-8.(C)
 * If any of the visual classes StaticGray, StaticColor or
 * TrueColor are supported: When XCreateColormap is called with
 * a visual that is one of StaticGray, StaticColor or
 * TrueColor, and the argument alloc is other than AllocNone,
 * then a BadMatch error occurs.
 */
static void
t008()
{
Colormap	cmap;
XVisualInfo	*vp;
unsigned long vmask = (1L<<TrueColor) | (1L<<StaticColor) | (1L<<StaticGray);
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, vmask)) == 0L) {
		unsupported("TrueColor, StaticColor and StaticGray are not supported");
		return;
	}

	alloc = AllocAll;
	for ( resetsupvis(vmask); nextsupvis(&vp); ) {
		visual = vp->visual;
		startcall(display);
		if (isdeleted())
			return;
		cmap = XCreateColormap(display, win, visual, alloc);
		endcall(display);
		if (geterr() != BadMatch) {
			report("Got %s, Expecting BadMatch", errorname(geterr()));
			FAIL;
		}
		if (geterr() == BadMatch)
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateColormap-9.(D)
 *   Reason: There is no known reliable test method for this assertion
 * If there is a visual class that is not supported on the
 * screen: When XCreateColormap is called with a visual that is
 * not supported on the screen for which the window argument
 * was created, then a BadMatch error occurs.
 */
static void
t009()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XCreateColormap-10.(A)
 * When the value of alloc is other than AllocNone or AllocAll,
 * then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t010()
{
#undef	VALUE_ARG
#define	VALUE_ARG alloc
#undef	VALUE_LIST
#define	VALUE_LIST allocvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XCreateColormap(display, win, visual, alloc);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/*
 * Assertion XCreateColormap-11.(A)
 * When XCreateColormap is called with an invalid visual, then
 * a BadValue error occurs.
 */
static void
t011()
{
Visual	vi;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	alloc = AllocNone;
	badvis(&vi);
	visual = &vi;
	startcall(display);
	if (isdeleted())
		return;
	XCreateColormap(display, win, visual, alloc);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadValue)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XCreateColormap-12.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t012()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XCreateColormap(display, win, visual, alloc);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* Including from file error/EAll.mc */
/*
 * Assertion XCreateColormap-13.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t013()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
