/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: frpxmp.m,v 1.6 92/06/29 18:58:16 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XFreePixmap(display, pixmap)
 *   Display *display;
 *   Pixmap  pixmap;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XFreePixmap	1
char    *TestName = "XFreePixmap";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_PIXMAP pixmap
#define A_DRAWABLE pixmap


/*
 * Arguments to the XFreePixmap function
 */
static Display *display;
static Pixmap  pixmap;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	pixmap = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	pixmap = ErrdefPixmap;
}

/*
 * Assertion XFreePixmap-1.(A)
 * A call to XFreePixmap removes the association between the
 * pixmap ID pixmap and the specified pixmap.
 */
static void
t001()
{
XGCValues	gcv;
XVisualInfo	*vp;
GC		gc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(resetvinf(VI_PIX); nextvinf(&vp); ) {
		pixmap = XCreatePixmap(display, DRW(display), 1, 1, vp->depth);
		gc = makegc(display, pixmap);
		startcall(display);
		if (isdeleted())
			return;
		XFreePixmap(display, pixmap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		
		startcall(Dsp);
		XDrawPoint(display, pixmap, gc, 0, 0);
		endcall(Dsp);
		if(geterr() != BadDrawable) {
			report("Got %s instead of BadDrawable when drawing on a freed pixmap. ", errorname(geterr()));
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreePixmap-2.(A)
 * The storage allocated to the pixmap is not recovered until
 * all references to it have been removed.
 */
static void
t002()
{
Window		win;
XVisualInfo	*vp;
XGCValues	gcv;
GC		gc, gc2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(resetvinf(VI_WIN); nextvinf(&vp);) {
		win = makewin(display, vp);	
		pixmap = XCreatePixmap(display, DRW(display), W_STDWIDTH, W_STDHEIGHT, vp->depth);
		dset(display, pixmap, W_BG);
		pattern(display, pixmap);
	
		gcv.fill_style = FillTiled;
		gcv.tile = pixmap;
		gcv.foreground = W_FG;
		gcv.background = W_BG;

                /*
                 * Create the GC with the window of the same depth because
                 * the root window could be of a different depth.
                 */
		gc = XCreateGC(display, win, GCFillStyle|GCTile|GCForeground|GCBackground, &gcv);
		startcall(display);
		if (isdeleted())
			return;
		XFreePixmap(display, pixmap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	
		XFillRectangle(display, win, gc, 0, 0, W_STDWIDTH+1, W_STDHEIGHT+1);		
		
		if( checkpattern(display, win, (struct area *) 0 ) != True) {
			report("Tiled pattern on window was not correct after");
			report("tile component in GC was freed by XFreePixmap");
			FAIL;
		} else
			CHECK;
	
	}
	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EPix.mc */
/*
 * Assertion XFreePixmap-3.(A)
 * When a pixmap argument does not name a valid Pixmap, then a
 * BadPixmap error occurs.
 */
/* End of included file error/EPix.mc */

/* Including from file error/EPix.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	seterrdef();

	A_PIXMAP = (Pixmap) badpixm(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XFreePixmap(display, pixmap);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadPixmap)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EPix.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
