/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: qrybstsz.m,v 1.7 92/06/11 16:16:54 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight)
 *   Display *display;
 *   int class;
 *   Drawable which_screen;
 *   unsigned int width;
 *   unsigned int height;
 *   unsigned int *rwidth;
 *   unsigned int *rheight;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XQueryBestSize	1
char    *TestName = "XQueryBestSize";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE which_screen


/*
 * Arguments to the XQueryBestSize function
 */
static Display *display;
static int class;
static Drawable which_screen;
static unsigned int width;
static unsigned int height;
static unsigned int *rwidth;
static unsigned int *rheight;


static int 	StatusReturn;

unsigned int cheight, cwidth;


/* Value list for use in test t007 */
static int 	classvallist[] = {
	TileShape,
	CursorShape,
	StippleShape,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	class = 0;
	which_screen = 0;
	width = 0;
	height = 0;
	rwidth = &cwidth;
	rheight = &cheight;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	which_screen = ErrdefDrawable;
}

/*
 * Assertion XQueryBestSize-1.(A)
 * When the class argument is CursorShape, then a call to
 * XQueryBestSize returns the largest cursor height and width
 * that can be fully displayed on the specified screen.
 */
static void
t001()
{
Status qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	which_screen = DRW(display);
	class = CursorShape;

	width = 10000;
	height = 10000;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	width = cwidth;
	height = cheight;

	cwidth = cheight = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	if(width != cwidth) {
		report("XQueryBestSize returned best width %d", cwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != cheight) {
		report("XQueryBestSize returned best height %d", cheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;

	width = 0;
	height = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	width = cwidth;
	height = cheight;

	cwidth = cheight = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	if(width != cwidth) {
		report("XQueryBestSize returned best width %d", cwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != cheight) {
		report("XQueryBestSize returned best height %d", cheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;

	CHECKPASS(8);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XQueryBestSize-2.(A)
 * When the class argument is TileShape, then a call to
 * XQueryBestSize returns the height and width of tile that are
 * closest to the arguments height and width that can be tiled
 * fastest on the specified screen.
 */
static void
t002()
{
Status qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	which_screen = DRW(display);
	class = TileShape;

	width = 10000;
	height = 10000;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	width = cwidth;
	height = cheight;

	cwidth = cheight = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	if(width != cwidth) {
		report("XQueryBestSize returned best width %d", cwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != cheight) {
		report("XQueryBestSize returned best height %d", cheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;

	width = 0;
	height = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	width = cwidth;
	height = cheight;

	cwidth = cheight = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	if(width != cwidth) {
		report("XQueryBestSize returned best width %d", cwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != cheight) {
		report("XQueryBestSize returned best height %d", cheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;

	CHECKPASS(8);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XQueryBestSize-3.(A)
 * When the class argument is StippleShape, then a call to
 * XQueryBestSize returns the height and width of stipple that
 * are closest to the arguments height and width that can be
 * stippled fastest on the specified screen.
 */
static void
t003()
{
Status qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	which_screen = DRW(display);
	class = StippleShape;

	width = 10000;
	height = 10000;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	width = cwidth;
	height = cheight;

	cwidth = cheight = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	if(width != cwidth) {
		report("XQueryBestSize returned best width %d", cwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != cheight) {
		report("XQueryBestSize returned best height %d", cheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;

	width = 0;
	height = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	width = cwidth;
	height = cheight;

	cwidth = cheight = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestSize returned wrong value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	if(width != cwidth) {
		report("XQueryBestSize returned best width %d", cwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != cheight) {
		report("XQueryBestSize returned best height %d", cheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;

	CHECKPASS(8);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XQueryBestSize-4.(A)
 * When the class argument is TileShape, and the which_screen
 * argument specifies an InputOnly window, then a BadMatch
 * error occurs.
 */
static void
t004()
{
int qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	which_screen = iponlywin(display);
	class = TileShape;
	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}
	trace("Status returned is %d", qstat);

	if(qstat == 0)
		CHECK;
	else
		FAIL;

	if(geterr() == BadMatch)
		CHECK;
	else
		FAIL;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XQueryBestSize-5.(A)
 * When the class argument is StippleShape, and the
 * which_screen argument specifies an InputOnly window, then a
 * BadMatch error occurs.
 */
static void
t005()
{
int qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	which_screen = iponlywin(display);
	class = StippleShape;
	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}
	trace("Status returned is %d", qstat);

	if(qstat == 0)
		CHECK;
	else
		FAIL;

	if(geterr() == BadMatch)
		CHECK;
	else
		FAIL;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EDra.mc */
/*
 * Assertion XQueryBestSize-6.(A)
 * When a drawable argument does not name a valid Drawable,
 * then a BadDrawable error occurs.
 */
/* End of included file error/EDra.mc */

/* Including from file error/EDra.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_DRAWABLE = (Drawable)badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadDrawable) {
		report("Got %s, Expecting BadDrawable", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadDrawable)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EDra.mc */

/*
 * Assertion XQueryBestSize-7.(A)
 * When the value of class is other than TileShape,
 * CursorShape or StippleShape, then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t007()
{
#undef	VALUE_ARG
#define	VALUE_ARG class
#undef	VALUE_LIST
#define	VALUE_LIST classvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		StatusReturn = XQueryBestSize(display, class, which_screen, width, height, rwidth, rheight);
		endcall(display);
		if (StatusReturn != 0) {
			report("Status returned was %d", StatusReturn);
			FAIL;
		}
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
