/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: qrybsttl.m,v 1.4 92/06/11 16:17:02 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XQueryBestTile(display, drawable, width, height, rwidth, rheight)
 *   Display *display;
 *   Drawable drawable;
 *   unsigned int width;
 *   unsigned int height;
 *   unsigned int *rwidth;
 *   unsigned int *rheight;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XQueryBestTile	1
char    *TestName = "XQueryBestTile";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE drawable


/*
 * Arguments to the XQueryBestTile function
 */
static Display *display;
static Drawable drawable;
static unsigned int width;
static unsigned int height;
static unsigned int *rwidth;
static unsigned int *rheight;


static int 	StatusReturn;

unsigned int bwidth, bheight;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	drawable = 0;
	width = 0;
	height = 0;
	rwidth = &bwidth;
	rheight = &bheight;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	drawable = ErrdefDrawable;
}

/*
 * Assertion XQueryBestTile-1.(A)
 * A call to XQueryBestTile returns the height and width of
 * tile that are closest to the arguments height and width that
 * can be tiled fastest on the specified screen.
 */
static void
t001()
{
Status qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	drawable = DRW(display);

	width = 17;
	height = 13;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestTile(display, drawable, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestTile returned wrong value %d", qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best width (for 17)  = %d", bwidth);
	trace("Best height (for 13) = %d", bheight);

	width = bwidth;
	height = bheight;

	bwidth = bheight = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestTile(display, drawable, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestTile returned wrong value %d", qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best width (for %d)  = %d", width, bwidth);
	trace("Best height (for %d) = %d", height, bheight);

	if(width != bwidth) {
		report("XQueryBestTile returned best width %d", bwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != bheight) {
		report("XQueryBestTile returned best height %d", bheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;

	width = 0;
	height = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestTile(display, drawable, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestTile returned wrong value %d", qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best width (for 0)  = %d", bwidth);
	trace("Best height (for 0) = %d", bheight);

	width = bwidth;
	height = bheight;

	bwidth = bheight = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestTile(display, drawable, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("XQueryBestTile returned wrong value %d", qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best width (for %d)  = %d", width, bwidth);
	trace("Best height (for %d) = %d", height, bheight);

	if(width != bwidth) {
		report("XQueryBestTile returned best width %d", bwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != bheight) {
		report("XQueryBestTile returned best height %d", bheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;

	CHECKPASS(8);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EDra.mc */
/*
 * Assertion XQueryBestTile-2.(A)
 * When a drawable argument does not name a valid Drawable,
 * then a BadDrawable error occurs.
 */
/* End of included file error/EDra.mc */

/* Including from file error/EDra.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_DRAWABLE = (Drawable)badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XQueryBestTile(display, drawable, width, height, rwidth, rheight);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadDrawable) {
		report("Got %s, Expecting BadDrawable", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadDrawable)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EDra.mc */

/* Including from file error/EMat1.mc */
/*
 * Assertion XQueryBestTile-3.(A)
 * When a drawable argument is an InputOnly window then a
 * BadMatch error occurs.
 */
/* End of included file error/EMat1.mc */

/* Including from file error/EMat1.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_DRAWABLE = (Drawable)iponlywin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XQueryBestTile(display, drawable, width, height, rwidth, rheight);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat1.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
