/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: qryclrs.m,v 1.10 92/06/11 16:17:16 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XQueryColors(display, colormap, defs_in_out, ncolors)
 *   Display	*display;
 *   Colormap colormap;
 *   XColor	*defs_in_out;
 *   int	ncolors;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XQueryColors	1
char    *TestName = "XQueryColors";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XQueryColors function
 */
static Display	*display;
static Colormap colormap;
static XColor	*defs_in_out;
static int	ncolors;


XColor	dummycol;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
	defs_in_out = &dummycol;
	ncolors = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XQueryColors-1.(A)
 * A call to XQueryColors obtains the RGB values of the
 * colourmap entries specified by the pixel components of the
 * XColor structures named by the defs_in_out argument, and
 * returns the RGB values in the red, green, and blue
 * components and sets the flags components to the bitwise OR
 * of DoRed, DoGreen, and DoBlue.
 */
static void
t001()
{
XVisualInfo 	*vp;
XColor		*cellptr, *defptr, *refptr, *acptr;
int		size;
unsigned long	i;
unsigned long 	vmask = 0L;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vmask = visualsupported(display, vmask)) != 0L)
		for(resetsupvis(vmask); nextsupvis(&vp); ) {
			size = vp->colormap_size;
			if (vp->class == DirectColor)
				size = maxsize(vp);
			colormap = makecolmap(display, vp->visual, AllocNone);

			defptr = (XColor *) malloc( size * sizeof(XColor));
			refptr = (XColor *) malloc( size * sizeof(XColor));
			
			if( (defptr == (XColor *) 0) || (refptr == (XColor *) 0)) {
				delete("malloc() failed to allocate space for the colour array.");
				return;
			}

			for(i=0, cellptr = defptr; i<size; i++, cellptr++) {  /* May or may not allocate the entire colourmap */
				cellptr->pixel = ~0L;
				cellptr->red = i<<8;
				cellptr->green = i<<8;
				cellptr->blue =  i<<8;
				cellptr->flags = 0;

				if(XAllocColor(display, colormap, cellptr) == False) {
					delete("XAllocColor() failed.");
					return;
				}

				refptr[i] = defptr[i];
			} 
			if(i == size)
				CHECK;

			ncolors = size;
			defs_in_out = defptr;
			startcall(display);
			if (isdeleted())
				return;
			XQueryColors(display, colormap, defs_in_out, ncolors);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			if(geterr() == Success)
				for(i=0, cellptr = defptr, acptr = refptr; i<size; i++, cellptr++, acptr++) {
					if(cellptr->flags != (DoRed|DoGreen|DoBlue)) {
						report("XQueryColors() did not set the flags of cell %u to (DoRed|DoGreen|DoBlue)", cellptr->pixel);
						FAIL;
					}

					if((cellptr->pixel != acptr->pixel) || (cellptr->red != acptr->red) ||
							(cellptr->green != acptr->green) || (cellptr->blue != acptr->blue)){
						report("XQueryColors() returned pixel %u  r %u g %u b %u instead of pixel %u r %u g %u b %u",
							cellptr->pixel, cellptr->red, cellptr->green, cellptr->blue,
							acptr->pixel, acptr->red, acptr->green, acptr->blue);
						FAIL;
					} 
				}
			else
				FAIL;	

			if(i == size)
				CHECK;

			free(defptr);
			free(refptr);
	}

	CHECKPASS(2*nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECol.mc */
/*
 * Assertion XQueryColors-2.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	XQueryColors(display, colormap, defs_in_out, ncolors);
	endcall(display);
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/*
 * Assertion XQueryColors-3.(A)
 * When the pixel component in one or more of the ncolors
 * XColor structures named by the defs_in_out argument is not a
 * valid entry in the colormap argument, then a BadValue error
 * occurs which will report any one of the invalid pixel
 * values.
 */
static void
t003()
{
XVisualInfo 	*vp;
XColor		tcol;
XColor		qcol[3];
unsigned long	vmask;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals reported as valid.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vp); ) {
		colormap = makecolmap(display, vp->visual, AllocNone);
		XAllocColor(display, colormap, &tcol);
		defs_in_out = qcol;
		qcol[0].pixel = tcol.pixel;
		qcol[1].pixel = (unsigned long)-1L;
		qcol[2].pixel = (unsigned long)-2L;
		ncolors = 3;
		startcall(display);
		if (isdeleted())
			return;
		XQueryColors(display, colormap, defs_in_out, ncolors);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}
		if(geterr() == BadValue) {
			if((getbadvalue() != (unsigned long)-1) && (getbadvalue() != (unsigned long)-2)) {
				report("BadValue reported was neither -1 or -2");
				FAIL;
			} else
				CHECK;
		}

	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
