/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stclpmsk.m,v 1.8 92/06/11 16:17:40 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetClipMask(display, gc, pixmap)
 *   Display *display;
 *   GC gc;
 *   Pixmap pixmap;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetClipMask	1
char    *TestName = "XSetClipMask";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_GC gc
#define A_PIXMAP pixmap
#define A_DRAWABLE pixmap


/*
 * Arguments to the XSetClipMask function
 */
static Display *display;
static GC gc;
static Pixmap pixmap;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	gc = 0;
	pixmap = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	gc = ErrdefGC;
	pixmap = ErrdefPixmap;
}

/*
 * Assertion XSetClipMask-1.(A)
 * A call to XSetClipMask sets the clip_mask component of the
 * specified GC to the value of the pixmap argument.
 */
static void
t001()
{
XVisualInfo	*vp;
Window		win;
XGCValues	values;
Pixmap		pmap;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp); /* background_pixel = W_BG */
	pmap = XCreatePixmap( display, win, 1, 1, 1); 	

	values.foreground = W_FG;

	gc = XCreateGC(display, win, GCForeground, &values);

	dset(display, pmap, 0);

	pixmap = pmap;
	startcall(display);
	if (isdeleted())
		return;
	XSetClipMask(display, gc, pixmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawPoint(display,win,gc,0,0);

	if( ! checkpixel(display, win, 0,0, W_BG)) {
		delete("Pixel at (0,0) was not set to background.");
		return;
	} else 
		CHECK;

	XFreePixmap(display, pmap);

	pmap = XCreatePixmap( display, win, 1, 1, 1); 	

	dset(display, pmap, 1);

	pixmap = pmap;
	startcall(display);
	if (isdeleted())
		return;
	XSetClipMask(display, gc, pixmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawPoint(display,win,gc,0,0);

	if( ! checkpixel(display, win, 0,0, W_FG)) {
		report("Pixel at (0,0) was not set to foreground.");
		FAIL;
	} else 
		CHECK;

	XFreePixmap(display, pmap);
	XFreeGC(display, gc);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetClipMask-2.(A)
 * When the pixmap argument is None, then the clip_mask
 * component of the specified gc has no effect on any
 * subsequent graphics operation.
 */
static void
t002()
{
XVisualInfo	*vp;
Window		win;
XGCValues	values;
Pixmap		pmap;
GC		pgc;
struct area	ar;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp); /* background_pixel = W_BG */

	pmap = XCreatePixmap( display, win, W_STDWIDTH, W_STDHEIGHT,1); 	

	dset(display, pmap, 0);
	values.foreground = 1;
	pgc = XCreateGC(display, pmap, GCForeground, &values);
	XFillRectangle(display, pmap, pgc, 0,0, W_STDWIDTH/2,W_STDHEIGHT/2);

	values.foreground = W_FG;
	values.clip_mask = pmap;
	gc = XCreateGC(display, win, GCForeground | GCClipMask, &values);
	XFillRectangle(display, win, gc, 0,0, W_STDWIDTH, W_STDHEIGHT);


	ar.x = ar.y = 0;
	ar.width = W_STDWIDTH / 2;
	ar.height = W_STDHEIGHT / 2;

	if( checkarea(display, win, &ar, W_FG, W_BG, 0) == False) {
		delete("check area failed - clipping is wrong.");
		return;
	} else
		CHECK;
	
	pixmap = None;
	startcall(display);
	if (isdeleted())
		return;
	XSetClipMask(display, gc, pixmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XFillRectangle(display, win, gc, 0,0, W_STDWIDTH, W_STDHEIGHT);

	if( checkarea(display, win, (struct area *) 0, W_FG, W_BG, CHECK_IN) == False) {
		report("Window was clipped with clip_mask = None.");
		FAIL;
	} else
		CHECK;


	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetClipMask-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EGC.mc */
/*
 * Assertion XSetClipMask-4.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetClipMask(display, gc, pixmap);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/*
 * Assertion XSetClipMask-5.(A)
 * When the pixmap argument is a pixmap and the gc and pixmap
 * arguments were not created for the same root, then a
 * BadMatch error occurs.
 */
static void
t005()
{
XVisualInfo     vi;
Pixmap  errpm;
char    *altroot;
int     scr_num;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	altroot = tet_getvar("XT_ALT_SCREEN");
	if (altroot == NULL) {
		delete("XT_ALT_SCREEN not set");
		return;
	}
	if (*altroot == 'U') {
		report("Only one root window supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	scr_num = atoi(altroot);
	if (scr_num == DefaultScreen(display)) {
		delete("The alternate root window was the same as the one under test");
	        return;
	}
	if (scr_num >= ScreenCount(display)) {
		delete("Screen given in XT_ALT_SCREEN could not be accessed");
		return;
	}

	vi.visual = NULL;
	vi.screen = DefaultScreen(display);
	vi.depth = 1;
	pixmap = makepixm(display, &vi);

	errpm = XCreatePixmap(display, RootWindow(display, scr_num), 1, 1, 1);
	A_GC = makegc(display, errpm);

	startcall(display);
	if (isdeleted())
		return;
	XSetClipMask(display, gc, pixmap);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	XFreePixmap(display, errpm);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetClipMask-6.(A)
 * When the pixmap argument does not have depth 1, then a
 * BadMatch error occurs.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((pixmap = nondepth1pixmap(display, DRW(display))) == (Pixmap) 0) {
		report("Only depth 1 pixmaps are supported.");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	gc = XCreateGC(display, DRW(display), 0L, 0);

	startcall(display);
	if (isdeleted())
		return;
	XSetClipMask(display, gc, pixmap);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	XFreeGC(display, gc);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EPix.mc */
/*
 * Assertion XSetClipMask-7.(A)
 * When a pixmap argument does not name a valid Pixmap, then a
 * BadPixmap error occurs.
 */
/* End of included file error/EPix.mc */

/* Including from file error/EPix.mc */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	seterrdef();

	A_PIXMAP = (Pixmap) badpixm(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetClipMask(display, gc, pixmap);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadPixmap)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EPix.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
