/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stclporgn.m,v 1.6 92/06/11 16:17:48 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetClipOrigin(display, gc, clip_x_origin, clip_y_origin)
 *   Display *display;
 *   GC gc;
 *   int clip_x_origin;
 *   int clip_y_origin;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetClipOrigin	1
char    *TestName = "XSetClipOrigin";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_GC gc


/*
 * Arguments to the XSetClipOrigin function
 */
static Display *display;
static GC gc;
static int clip_x_origin;
static int clip_y_origin;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	gc = 0;
	clip_x_origin = -1;
	clip_y_origin = -1;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	gc = ErrdefGC;
}

/*
 * Assertion XSetClipOrigin-1.(A)
 * A call to XSetClipOrigin sets the clip_x_origin and
 * clip_y_origin components of the specified GC to the values
 * of the clip_x_origin and clip_y_origin arguments.
 */
static void
t001()
{
XVisualInfo	*vp;
Window		win;
XGCValues	values;
Pixmap		pmap;
GC		pgc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);
	pmap = XCreatePixmap( display, win, 1, 1, 1); 	

	values.foreground = W_FG;
	values.background = W_BG;
	values.clip_mask = pmap;
	values.clip_x_origin = 0;
	values.clip_y_origin = 0;

	pgc = XCreateGC(display, pmap, GCForeground | GCBackground, &values);
	XFillRectangle(display, pmap, pgc, 0,0, 1,1);

	gc = XCreateGC(display, win, GCForeground | GCBackground | GCClipMask | GCClipXOrigin | GCClipYOrigin, &values);
	XDrawPoint(display,win,gc,2,2);

	if( ! checkpixel(display, win, 2, 2, W_BG)) {
		delete("Pixel at (2,2) was not set to background.");
		return;
	} else 
		CHECK;

	clip_x_origin = 2;
	clip_y_origin = 2;
	startcall(display);
	if (isdeleted())
		return;
	XSetClipOrigin(display, gc, clip_x_origin, clip_y_origin);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawPoint(display,win,gc,2,2);

	if( ! checkpixel(display, win, 2, 2, W_FG)) {
		report("Pixel at (2,2) was not set to foreground.");
		FAIL;
	} else 
		CHECK;
	
	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetClipOrigin-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EGC.mc */
/*
 * Assertion XSetClipOrigin-3.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetClipOrigin(display, gc, clip_x_origin, clip_y_origin);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
