/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stdshs.m,v 1.7 92/06/11 16:18:04 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetDashes(display, gc, dash_offset, dash_list, n)
 *   Display *display;
 *   GC gc;
 *   int dash_offset;
 *   char *dash_list;
 *   int n;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetDashes	1
char    *TestName = "XSetDashes";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_GC gc


/*
 * Arguments to the XSetDashes function
 */
static Display *display;
static GC gc;
static int dash_offset;
static char *dash_list;
static int n;


char dashes[] = { 2, 1};
char odashes[] = {1, 3, 2};
char edashes[] = {1, 3, 2, 1, 3, 2};
char errdashes[] = {1, 0, 1};

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	gc = 0;
	dash_offset = 0;
	dash_list = dashes;
	n = sizeof(dashes)/sizeof(char);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	gc = ErrdefGC;
}

/*
 * Assertion XSetDashes-1.(A)
 * A call to XSetDashes sets the dash_offset and dash_list
 * attributes for dashed line styles in the specified GC to the
 * values of the dash_offset and dash_list arguments.
 */
static void
t001()
{
XVisualInfo *vp;
XGCValues values;
GC gc;
Window win;
int twidth;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(twidth = 0; twidth < 2; twidth++) {
		resetvinf(VI_WIN);
		nextvinf(&vp);
		win = makewin(display, vp);

		values.foreground = W_FG;
		values.background = W_BG;
		values.line_style = LineDoubleDash;
		values.line_width = twidth;
		values.dashes = 1; /* [1, 1] */


		gc = XCreateGC(display, win, (GCLineWidth |GCLineStyle | GCDashList | GCForeground | GCBackground), &values);

		XDrawPoint(display, win, gc, 1, 0);
		XDrawLine(display, win, gc, 0, 0, 2, 0);

		if( ! checkpixel(display, win, 1, 0, W_BG)) {
			delete("Pixel at (1, 0) was not set to background (width %d)", twidth);
			return;
		} else
			CHECK;

		dash_offset = 0;
		dash_list= dashes;
		n = sizeof(dashes)/sizeof(char);
		startcall(display);
		if (isdeleted())
			return;
		XSetDashes(display, gc, dash_offset, dash_list, n);
		gcflush(A_DISPLAY, A_GC);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		XDrawLine(display, win, gc, 0, 0, 2, 0);

		if( ! checkpixel(display, win, 1, 0, W_FG)) {
			report("Pixel at (1, 0) was not set to foreground (width %d)", twidth);			
			FAIL;
		} else
			CHECK;

	}
	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetDashes-2.(A)
 * The initial and alternate elements of the dash_list
 * argument specify the lengths of the even dashes and the
 * second and alternate elements specify the lengths of the odd
 * dashes.
 */
static void
t002()
{
XVisualInfo *vp;
GC gc;
XGCValues values;
Window win;
int twidth;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	for(twidth=0; twidth < 2; twidth++) {
		resetvinf(VI_WIN);
		nextvinf(&vp);
		win = makewin(display, vp);
	
		values.foreground = W_FG;
		values.background = W_BG;
		values.line_style = LineDoubleDash;
		values.line_width = twidth;
	
		gc = XCreateGC(display, win, (GCLineWidth | GCLineStyle | GCForeground | GCBackground ), &values);
	
		dash_offset = 0;
		dash_list = dashes;
		n = sizeof(dashes)/sizeof(char);
	
		startcall(display);
		if (isdeleted())
			return;
		XSetDashes(display, gc, dash_offset, dash_list, n);
		gcflush(A_DISPLAY, A_GC);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		XDrawLine(display, win, gc, 0, 0, 4, 0);
	
		if( (checkpixels(display, win, 0, 0, 1, 0, 2, W_FG) == True) &&
		    (checkpixel(display, win, 2, 0, W_BG) == True) &&
		    (checkpixel(display, win, 3, 0, W_FG) == True) )
			CHECK;
		else {
			report("Horizontal dashing incorrect for width %d.", twidth);
			FAIL;
		}

		XDrawLine(display, win, gc, 0, 1, 0, 5);
	
		if(  (checkpixels(display, win, 0, 1, 0, 1, 2, W_FG) == True) &&
		     (checkpixel(display, win, 0, 3, W_BG) == True) &&
		     (checkpixel(display, win, 0, 4, W_FG) == True) )
			 CHECK;
		 else {
			 report("Vertical dashing incorrect for width %d.", twidth);
			 FAIL;
		 }

	}
	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetDashes-3.(A)
 * When an odd-length list is specified, then it is
 * concatenated with itself to produce an even-length list.
 */
static void
t003()
{
XVisualInfo *vp;
XGCValues values;
XImage *image;
Window win;
int i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);

	values.foreground = W_FG;
	values.background = W_BG;
	values.line_style = LineDoubleDash;
	gc = XCreateGC(display, win, (GCLineStyle | GCForeground | GCBackground), &values);

	dash_offset = 0;
	dash_list = odashes;
	n = sizeof(odashes)/sizeof(char);
	startcall(display);
	if (isdeleted())
		return;
	XSetDashes(display, gc, dash_offset, dash_list, n);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawLine(display, win, gc, 0, 0, 10, 0);


	dash_offset = 0;
	dash_list = edashes;
	n = sizeof(edashes)/sizeof(char);
	startcall(display);
	if (isdeleted())
		return;
	XSetDashes(display, gc, dash_offset, dash_list, n);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawLine(display, win, gc, 0, 1, 10, 1);

	image = XGetImage(display, win, 0, 0, 10, 2, AllPlanes, ZPixmap);

	for(i=0; i<10; i++)
		if(XGetPixel(image, i, 0) != XGetPixel(image, i, 1)) {
			report("Pixel at (%d,0) was not the same as pixel at (%d,1).", i, i);
			FAIL;
		} else
			CHECK;

	CHECKPASS(10);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetDashes-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EGC.mc */
/*
 * Assertion XSetDashes-5.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetDashes(display, gc, dash_offset, dash_list, n);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/*
 * Assertion XSetDashes-6.(A)
 * When the specified dash list is empty, then a BadValue
 * error occurs.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = XCreateGC(display, DRW(display), 0L, (XGCValues*) 0);

	dash_offset = 0;
	dash_list = (char *)NULL;
	n = 0;
	startcall(display);
	if (isdeleted())
		return;
	XSetDashes(display, gc, dash_offset, dash_list, n);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadValue)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetDashes-7.(A)
 * When an element of the dash list is 0, then a BadValue
 * error occurs.
 */
static void
t007()
{
int screen;
GC gc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	screen = DefaultScreen(display);
	gc = XCreateGC(display, RootWindow(display, screen), 0L, (XGCValues*) 0);
	dash_offset = 0;
	dash_list = errdashes;
	n = sizeof(errdashes)/sizeof(char);
	startcall(display);
	if (isdeleted())
		return;
	XSetDashes(display, gc, dash_offset, dash_list, n);
	gcflush(A_DISPLAY, A_GC);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadValue)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
