/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: clrwdw.m,v 1.4 92/06/11 16:20:48 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XClearWindow(display, w)
 *   Display	*display;
 *   Window	w;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XClearWindow	1
char    *TestName = "XClearWindow";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XClearWindow function
 */
static Display	*display;
static Window	w;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
}

/*
 * Assertion XClearWindow-1.(A)
 * A call to XClearWindow paints the entire area in the window
 * specified by the w argument with the window's background
 * pixel or pixmap.
 */
static void
t001()
{
Pixmap  pm;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		w = makewin(display, vp);

		XSetWindowBackground(display, w, W_FG);
		startcall(display);
		if (isdeleted())
			return;
		XClearWindow(display, w);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (checkarea(display, w, (struct area *)0, W_FG, W_BG, 0) == False) {
			report("Clearing to W_FG failed");
			tet_result(TET_FAIL);
		} else {
			CHECK;
		}

		XSetWindowBackground(display, w, W_BG);
		startcall(display);
		if (isdeleted())
			return;
		XClearWindow(display, w);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (checkarea(display, w, (struct area *)0, W_BG, W_BG, 0) == False) {
			report("Clearing to W_BG failed");
			tet_result(TET_FAIL);
		} else
			CHECK;

		pm = maketile(display, w);
		XSetWindowBackgroundPixmap(display, w, pm);
		startcall(display);
		if (isdeleted())
			return;
		XClearWindow(display, w);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		/*
		 * Check the tiled area first and then the outside.
		 */
		if (checktile(display, w, (struct area*)0, 0, 0, pm) == False) {
			report("Failed with background pixmap");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(3*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XClearWindow-2.(A)
 * On a call to XClearWindow the subwindow-mode is
 * ClipByChildren.
 */
static void
t002()
{
struct  area    carea;
Pixmap  pm;
Window  child;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		w = makewin(display, vp);
		carea.x = 10;
		carea.y = 0;
		carea.width = carea.height = 0;
		child = crechild(display, w, &carea);

		pm = maketile(display, w);
		XSetWindowBackgroundPixmap(display, w, pm);
		startcall(display);
		if (isdeleted())
			return;
		XClearWindow(display, w);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		carea.x = carea.y = carea.width = carea.height = 0;
		if (checkarea(display, child, &carea, W_BG, W_BG, CHECK_ALL) == False) {
			report("Child window was drawn upon by clear window");
			FAIL;
		} else {
			CHECK;
		}
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XClearWindow-3.(-)
 * When the window has a defined background tile, then the
 * rectangle is tiled with a plane-mask of all ones and the
 * GXCopy function.
 */
static void
t003()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XClearWindow-4.(A)
 * When the window has a background_pixmap of None, then the
 * contents of the window are not changed.
 */
static void
t004()
{
struct  area    area;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		w = makewin(display, vp);
		pattern(display, w);
		XSetWindowBackground(display, w, W_FG);
		XSetWindowBackgroundPixmap(display, w, None);
		startcall(display);
		if (isdeleted())
			return;
		XClearWindow(display, w);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (checkpattern(display, w, (struct area *)0) == False) {
			report("Window was changed when background was None");
			FAIL;
		} else {
			CHECK;
		}
	}
	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XClearWindow-5.(A)
 * When the window has a background_pixmap of ParentRelative
 * and the parent has a background_pixmap of None, then the
 * contents of the window are not changed.
 */
static void
t005()
{
struct  area    area;
Window  child;
Window  win;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		win = makewin(display, vp);
		area.x = area.y = area.width = area.height = 0;
		child = crechild(display, win, &area);
		pattern(display, child);

		/*
		 * Set parent to None, child to Parent Relative.
		 */
		XSetWindowBackgroundPixmap(display, win, None);
		XSetWindowBackgroundPixmap(display, child, ParentRelative);

		w = child;
		startcall(display);
		if (isdeleted())
			return;
		XClearWindow(display, w);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (checkpattern(display, child, (struct area *)0) == False) {
			report("Window was changed when background was ParentRelative");
			FAIL;
		} else {
			CHECK;
		}
	}
	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EMat1.mc */
/*
 * Assertion XClearWindow-6.(A)
 * When a drawable argument is an InputOnly window then a
 * BadMatch error occurs.
 */
/* End of included file error/EMat1.mc */

/* Including from file error/EMat1.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_DRAWABLE = (Drawable)iponlywin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XClearWindow(display, w);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat1.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XClearWindow-7.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XClearWindow(display, w);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
