/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: crtpxmpcrs.m,v 1.14 92/06/11 16:22:14 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Cursor
 *   XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y)
 *   Display *display;
 *   Pixmap source;
 *   Pixmap mask;
 *   XColor *foreground_color;
 *   XColor *background_color;
 *   unsigned int x;
 *   unsigned int y;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCreatePixmapCursor	1
char    *TestName = "XCreatePixmapCursor";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_PIXMAP source
#define A_DRAWABLE source
#define A_PIXMAP2 mask
#define A_DRAWABLE2 mask


/*
 * Arguments to the XCreatePixmapCursor function
 */
static Display *display;
static Pixmap source;
static Pixmap mask;
static XColor *foreground_color;
static XColor *background_color;
static unsigned int x;
static unsigned int y;


static XVisualInfo depth1;

/*
 * mkcolor() -	return a pointer to a color structure.
 *		flag indicates whether or not color is foreground
 */
static XColor *
mkcolor(flag)
{
	static	XColor	fore;
	static	XColor	back;
	static	int	first = 1;

	if (first)
	{
		first = 0;

		fore.pixel = BlackPixel(display, DefaultScreen(display));
		XQueryColor(display, DefaultColormap(display, DefaultScreen(display)), &fore);
		back.pixel = WhitePixel(display, DefaultScreen(display));
		XQueryColor(display, DefaultColormap(display, DefaultScreen(display)), &back);
	}
	return(flag ? &fore : &back);
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	source = 0;
	mask = 0;
	foreground_color = mkcolor(1);
	background_color = mkcolor(0);
	x = 0;
	y = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	source = ErrdefPixmap;
	mask = ErrdefPixmap;
}

/*
 * Assertion XCreatePixmapCursor-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XCreatePixmapCursor creates a Cursor from the
 * source and mask pixmaps with colours defined by
 * foreground_color and background_color and hotspot position
 * given by x and y and returns the cursor ID.
 */
static void
t001()
{
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* Create source pixmap. */
	source = makepixm(display, &depth1);

/* Create mask pixmap. */
	mask = makepixm(display, &depth1);

/* Call XCreatePixmapCursor with foreground colour W_FG and */
/* background colour W_BG and hotspot at (0,0). */
	startcall(display);
	if (isdeleted())
		return;
	qstat = XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XCreatePixmapCursor returns non-zero. */
	if (qstat == (Cursor) 0) {
		report("Returned incorrect value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;
	
	CHECKUNTESTED(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When mask is None, then all pixels of the source are
 * displayed.
 */
static void
t002()
{
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* Create source pixmap. */
	source = makepixm(display, &depth1);

	mask = None;
/* Call XCreatePixmapCursor with mask None. */
	startcall(display);
	if (isdeleted())
		return;
	qstat = XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XCreatePixmapCursor returns non-zero. */
	if (qstat == (Cursor) 0) {
		report("Returned incorrect value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;
	
	CHECKUNTESTED(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When mask is other than None and a given bit in the mask
 * bitmap is 1, then the corresponding pixel of source is
 * displayed.
 */
static void
t003()
{
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* Create source pixmap. */
	source = makepixm(display, &depth1);
/* Create mask pixmap. */
	mask = makepixm(display, &depth1);

/* Call XCreatePixmapCursor with mask other than none and */
/* mask with at least one bit set. */
	startcall(display);
	if (isdeleted())
		return;
	qstat = XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XCreatePixmapCursor returns non-zero. */
	if (qstat == (Cursor) 0) {
		report("Returned incorrect value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;
	
	CHECKUNTESTED(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When mask is other than None and a given bit in the mask
 * bitmap is 0, then the corresponding pixel of source is not
 * displayed.
 */
static void
t004()
{
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* Create source pixmap. */
	source = makepixm(display, &depth1);
/* Create mask pixmap. */
	mask = makepixm(display, &depth1);
	dset(display, (Drawable) mask, (unsigned long) 0);

/* Call XCreatePixmapCursor with mask other than none and */
/* mask with at least one bit not set. */
	startcall(display);
	if (isdeleted())
		return;
	qstat = XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XCreatePixmapCursor returns non-zero. */
	if (qstat == (Cursor) 0) {
		report("Returned incorrect value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;
	
	CHECKUNTESTED(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a bit in the source bitmap is 1, then foreground_color
 * is used.
 */
static void
t005()
{
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* Create source pixmap. */
	source = makepixm(display, &depth1);
/* Create mask pixmap. */
	mask = makepixm(display, &depth1);

/* Call XCreatePixmapCursor with source bitmap containing */
/* at least one bit set to 1. */
	startcall(display);
	if (isdeleted())
		return;
	qstat = XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XCreatePixmapCursor returns non-zero. */
	if (qstat == (Cursor) 0) {
		report("Returned incorrect value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;
	
	CHECKUNTESTED(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-6.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a bit in the source bitmap is 0, then background_color
 * is used.
 */
static void
t006()
{
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* Create source pixmap. */
	source = makepixm(display, &depth1);
	dset(display, (Drawable) source, 0);
/* Create mask pixmap. */
	mask = makepixm(display, &depth1);

/* Call XCreatePixmapCursor with source bitmap containing */
/* at least one bit set to 0. */
	startcall(display);
	if (isdeleted())
		return;
	qstat = XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XCreatePixmapCursor returns non-zero. */
	if (qstat == (Cursor) 0) {
		report("Returned incorrect value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;
	
	CHECKUNTESTED(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-7.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a Cursor is created by a call to XCreatePixmapCursor,
 * and the source and/or mask argument is freed by a subsequent
 * call to XFreePixmap, then the cursor is unaffected.
 */
static void
t007()
{
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create source pixmap. */
	source = XCreatePixmap(display, DRW(display), 2, 2, 1);
/* Create mask pixmap. */
	mask = XCreatePixmap(display, DRW(display), 2, 2, 1);

/* Call XCreatePixmapCursor with mask other than none and */
/* mask with at least one bit set. */
	startcall(display);
	if (isdeleted())
		return;
	qstat = XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XCreatePixmapCursor returns non-zero. */
	if (qstat == (Cursor) 0) {
		report("Returned incorrect value %ld", (long) qstat);
		FAIL;
	} else
		CHECK;
	
/* Call XFreePixmap with source pixmap. */
	XFreePixmap(display, source);

/* Call XFreePixmap with mask pixmap. */
	XFreePixmap(display, mask);
	
	CHECKUNTESTED(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-8.(C)
 * If multiple window depths are supported: When source does
 * not have depth one, then a BadMatch error occurs.
 */
static void
t008()
{
XVisualInfo *vp;
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* If only one depth supported report UNSUPPORTED. */
	for (resetvinf(VI_PIX); nextvinf(&vp); ) {
		if (vp->depth != 1) {
			depth1.depth = vp->depth;
			break;
		}
	}
	if (depth1.depth == 1) {
                unsupported("Only one depth of window is supported");
		return;
	}
/* Create source pixmap of depth not equal to one. */
	source = makepixm(display, &depth1);
/* Create mask pixmap. */
	depth1.depth = 1;
	mask = makepixm(display, &depth1);

/* Call XCreatePixmapCursor with source pixmap. */
	startcall(display);
	if (isdeleted())
		return;
	XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

/* Verify that BadMatch error occurred. */
	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-9.(A)
 * When mask is not None and mask does not have depth one,
 * then a BadMatch error occurs.
 */
static void
t009()
{
XVisualInfo *vp;
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* If only one depth supported report UNSUPPORTED. */
	for (resetvinf(VI_PIX); nextvinf(&vp); ) {
		if (vp->depth != 1) {
			depth1.depth = vp->depth;
			break;
		}
	}
	if (depth1.depth == 1) {
		report("Only one depth supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}
/* Create mask pixmap of depth not equal to one. */
	mask = makepixm(display, &depth1);
/* Create source pixmap. */
	depth1.depth = 1;
	source = makepixm(display, &depth1);

/* Call XCreatePixmapCursor with mask pixmap. */
	startcall(display);
	if (isdeleted())
		return;
	XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

/* Verify that BadMatch error occurred. */
	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-10.(A)
 * When mask is not None and both source and mask are not the
 * same size, then a BadMatch error occurs.
 */
static void
t010()
{
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* Create source pixmap of size 1x1. */
	source = XCreatePixmap(display, DRW(display), 1, 1, 1);
	regid(display, (union regtypes *)&source, REG_PIXMAP);
/* Create mask pixmap of size 2x2. */
	mask = XCreatePixmap(display, DRW(display), 2, 2, 1);
	regid(display, (union regtypes *)&mask, REG_PIXMAP);

/* Call XCreatePixmapCursor with mask pixmap. */
	startcall(display);
	if (isdeleted())
		return;
	XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

/* Verify that BadMatch error occurred. */
	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreatePixmapCursor-11.(A)
 * When the hotspot position is not a point within the source
 * pixmap, then a BadMatch error occurs.
 */
static void
t011()
{
Cursor qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	depth1.depth = 1;
/* Create source pixmap. */
	source = makepixm(display, &depth1);
/* Create mask pixmap. */
	mask = makepixm(display, &depth1);

	x = 2 * W_STDWIDTH + 1;
	y = 2 * W_STDHEIGHT + 1;

/* Call XCreatePixmapCursor with hotspot outside the source pixmap. */
	startcall(display);
	if (isdeleted())
		return;
	XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

/* Verify that BadMatch error occurred. */
	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XCreatePixmapCursor-12.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t012()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EPix.mc */
/*
 * Assertion XCreatePixmapCursor-13.(A)
 * When a pixmap argument does not name a valid Pixmap, then a
 * BadPixmap error occurs.
 */
/* End of included file error/EPix.mc */

/* Including from file error/EPix.mc */
static void
t013()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	seterrdef();

	A_PIXMAP = (Pixmap) badpixm(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XCreatePixmapCursor(display, source, mask, foreground_color, background_color, x, y);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadPixmap)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EPix.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
