/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: drwimgstr.m,v 1.10 92/06/11 16:31:16 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XDrawImageString(display, d, gc, x, y, string, length)
 *   Display	*display;
 *   Drawable d;
 *   GC		gc;
 *   int 	x;
 *   int 	y;
 *   char	*string;
 *   int 	length;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XDrawImageString	1
char    *TestName = "XDrawImageString";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE d
#define A_GC gc


/*
 * Arguments to the XDrawImageString function
 */
static Display	*display;
static Drawable d;
static GC		gc;
static int 	x;
static int 	y;
static char	*string;
static int 	length;



/* Needed because function not used */
#define	A_DRAW	A_DRAWABLE

static	Font	Xtfonts[XT_NFONTS];

static void
localstartup()
{
	fontstartup();
	if(Dsp) {
		openfonts(Xtfonts, XT_NFONTS);
		setgcfont(Xtfonts[1]);
	}
}


#define	PLANEFG	0x55555555
static	int 	planelist[] = {
	0, 0x1, 0x2, 0x23, 0xf8, 0x765, 0x3987, 0x129078, 0x23567193};



#if !defined(T_XDrawImageString) && !defined(T_XCopyPlane) && !defined(T_XPutImage) && !defined(T_XCopyArea)
#define NEED_STIPPLE
#endif



int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	d = 0;
	gc = 0;
	x = 3;
	y = 30;
	string = "A bCdElMnO";
	length = strlen(string);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	d = ErrdefDrawable;
	gc = ErrdefGC;
}

/*
 * Assertion XDrawImageString-1.(A)
 * A call to XDrawImageString first fills the destination
 * rectangle with the background pixel value and next draws a
 * string of 8-bit characters, selected from the gc's font,
 * using the foreground pixel value.
 */
static void
t001()
{
XVisualInfo	*vp;
int 	fn;
unsigned int 	width;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		d = makewin(display, vp);
		gc = makegc(display, d);

		XSetForeground(display, gc, W_BG);
		XSetBackground(display, gc, W_FG);

		getsize(display, d, &width, (unsigned int*)0);

		for (fn = 0; fn < XT_NFONTS; fn++) {

			XSetFont(display, gc, Xtfonts[fn]);
			if (fn == 4 || fn == 6)
				x = width-4;
			else
				x = 3;

			startcall(display);
			if (isdeleted())
				return;
			XDrawImageString(display, d, gc, x, y, string, length);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(display, d);
			dclear(display, d);
		}
	}

	CHECKPASS(XT_NFONTS*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-2.(-)
 * The upper-left corner of the filled rectangle is at [ x, y
 * - font-ascent], with width equal to the sum of the
 * per-character widths, and with height font-ascent+
 * font-descent, where font-ascent and font-descent are the
 * logical ascent and descent of the font.
 */
static void
t002()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawImageString-3.(A)
 * The origin of the string is at [ x, y].
 */
static void
t003()
{
XVisualInfo	*vp;
unsigned int 	width, height;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	string = "v";
	length = 1;

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		d = makewin(display, vp);
		gc = makegc(display, d);

		getsize(display, d, &width, &height);

		for (x = 0; x < width; x += 20) {
			for (y = 0; y < width; y += 20) {
				startcall(display);
				if (isdeleted())
					return;
				XDrawImageString(display, d, gc, x, y, string, length);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
			}
		}

		PIXCHECK(display, d);
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-4.(A)
 * The effective function is GXcopy, and the effective
 * fill_style is FillSolid.
 */
static void
t004()
{
XVisualInfo	*vp;
XImage	*sav;
Pixmap	tile;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		d = makewin(display, vp);
		gc = makegc(display, d);

		XSetForeground(display, gc, W_BG);
		XSetBackground(display, gc, W_FG);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		sav = savimage(display, d);
		dclear(display, d);

		/* Make pixmap with same depth as drawable under test */
		tile = makepixm(display, vp);
		pattern(display, tile);

		XSetFunction(display, gc, GXxor);
		XSetFillStyle(display, gc, FillTiled);
		XSetTile(display, gc, tile);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (compsavimage(display, d, sav))
			CHECK;
		else {
			report("Effective function and fill style test failed");
			FAIL;
		}
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-5.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the font is defined with 2-byte matrix indexing, then
 * each byte is used as a byte2 with a byte1 of zero.
 */
static void
t005()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/* Including from file gc/plane-mas.mc */
/*
 * Assertion XDrawImageString-6.(A)
 * The value for plane_mask is truncated to the depth of the
 * GC.
 */
static void
t006()
{
XVisualInfo	*vp;
int 	depthmask;
int 	*ip;
unsigned long	pix;
unsigned long	expected;
int 	pmx = -1, pmy;
int 	ntested;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	ntested = 0;
	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		if (pmx == -1) {
			/*
			 * Get a point that is set in the drawable.
			 */
			startcall(display);
			if (isdeleted())
				return;
			XDrawImageString(display, d, gc, x, y, string, length);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			setfuncpixel(A_DISPLAY, A_DRAW, &pmx, &pmy);
			dclear(A_DISPLAY, A_DRAW);
		}

		depthmask = DEPTHMASK(vp->depth);

		XSetForeground(A_DISPLAY, A_GC, PLANEFG);

		for (ip = planelist; ip < &planelist[NELEM(planelist)]; ip++) {
			if (*ip <= depthmask)
				continue;

			trace("plane-mask 0x%x", *ip);
			XSetPlaneMask(A_DISPLAY, A_GC, *ip);
			ntested++;

			dclear(A_DISPLAY, A_DRAW);
			startcall(display);
			if (isdeleted())
				return;
			XDrawImageString(display, d, gc, x, y, string, length);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			expected = (*ip & PLANEFG) & depthmask;
			pix = getpixel(A_DISPLAY, A_DRAW, pmx, pmy);

			if (pix == expected)
				CHECK;
			else {
				report("got pixel 0x%x, expecting 0x%x", pix, expected);
				FAIL;
			}
		}

	}

	CHECKPASS(ntested);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-7.(A)
 * The plane_mask specifies which planes of the destination
 * are to be modified, one bit per plane, with bits being
 * assigned to planes from the least significant bit of the
 * word to the most significant bit.
 */
static void
t007()
{
XVisualInfo	*vp;
int 	depthmask;
int 	*ip;
unsigned long	pix;
unsigned long	expected;
int 	pmx = -1, pmy;
int 	ntested = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		if (pmx == -1) {
			/*
			 * Get a point that is set in the drawable.
			 */
			startcall(display);
			if (isdeleted())
				return;
			XDrawImageString(display, d, gc, x, y, string, length);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			setfuncpixel(A_DISPLAY, A_DRAW, &pmx, &pmy);
			dclear(A_DISPLAY, A_DRAW);
		}

		depthmask = DEPTHMASK(vp->depth);

		XSetForeground(A_DISPLAY, A_GC, PLANEFG);
		for (ip = planelist; ip < &planelist[NELEM(planelist)]; ip++) {
			if (*ip > depthmask)
				break;

			trace("plane-mask 0x%x", *ip);
			XSetPlaneMask(A_DISPLAY, A_GC, *ip);
			ntested++;

			dclear(A_DISPLAY, A_DRAW);
			startcall(display);
			if (isdeleted())
				return;
			XDrawImageString(display, d, gc, x, y, string, length);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			expected = *ip & PLANEFG;
			pix = getpixel(A_DISPLAY, A_DRAW, pmx, pmy);

			if (pix == expected)
				CHECK;
			else {
				report("got pixel 0x%x, expecting 0x%x", pix, expected);
				FAIL;
			}
		}

	}

	CHECKPASS(ntested);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/plane-mas.mc */

/* Including from file gc/font.mc */
/* End of included file gc/font.mc */

/* Including from file gc/subwindow.mc */
/*
 * Assertion XDrawImageString-8.(A)
 * When subwindow_mode is ClipByChildren, then the
 * destination window and source (if it is a window) are
 * clipped by all viewable InputOutput children.
 */
static void
t008()
{
XVisualInfo	*vp;
Window	subwins[10];
unsigned int 	swmwidth, swmheight;
struct	area	area;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		XSetSubwindowMode(A_DISPLAY, A_GC, ClipByChildren);

		/*
		 * Place subwindows in strips across the window
		 * leaving spaces between them.
		 */
		getsize(A_DISPLAY, A_DRAW, &swmwidth, &swmheight);
		swmwidth /= 2*NELEM(subwins);

		for (i = 0; i < NELEM(subwins); i++) {
			setarea(&area, 2*i*swmwidth, 0, swmwidth, 0);
			subwins[i] = crechild(A_DISPLAY, A_DRAW, &area);
		}

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		for (i = 0; i < NELEM(subwins); i++) {
			if (checkclear(A_DISPLAY, subwins[i]))
				CHECK;
			else {
				report("Area of subwindow was not clear");
				FAIL;
			}
		}
	}

	CHECKPASS(NELEM(subwins)*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-9.(A)
 * When subwindow_mode is IncludeInferiors, then neither the
 * destination window nor source (if it is a window) is clipped
 * by inferiors.
 */
static void
t009()
{
XVisualInfo	*vp;
Window	subwins[5];
XImage	*swmsav;
unsigned int 	swmwidth, swmheight;
struct	area	area;
int 	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		swmsav = savimage(A_DISPLAY, A_DRAW);
		dclear(A_DISPLAY, A_DRAW);

		XSetSubwindowMode(A_DISPLAY, A_GC, IncludeInferiors);

		/*
		 * Place subwindows in strips across the window
		 * leaving spaces between them.
		 */
		getsize(A_DISPLAY, A_DRAW, &swmwidth, &swmheight);
		swmwidth /= 2*NELEM(subwins);

		for (i = 0; i < NELEM(subwins); i++) {
			setarea(&area, 2*i*swmwidth, 0, swmwidth, 0);
			subwins[i] = crechild(A_DISPLAY, A_DRAW, &area);
			for (j = 0; j < swmheight; j += 10) {
				setarea(&area, 0, j, swmwidth, 6);
				(void) crechild(A_DISPLAY, subwins[i], &area);
			}
		}

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (compsavimage(A_DISPLAY, A_DRAW, swmsav))
			CHECK;
		else {
			report("Inferiors affected result with IncludeInferiors");
			FAIL;
		}

		/*
		 * As a special test due to its importance, test that drawing
		 * on to the root window has the same effect.  Move the test
		 * window to the root window origin.
		 */
		{
		Drawable savdraw;

			dclear(A_DISPLAY, A_DRAW);
			XSetWindowBorderWidth(A_DISPLAY, A_DRAW, 0);
			XMoveWindow(A_DISPLAY, A_DRAW, 0, 0);

			savdraw = A_DRAW;
			A_DRAW = DRW(A_DISPLAY);

			startcall(display);
			if (isdeleted())
				return;
			XDrawImageString(display, d, gc, x, y, string, length);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			A_DRAW = savdraw;

			if (compsavimage(A_DISPLAY, A_DRAW, swmsav))
				CHECK;
			else {
				report("Drawing on root window with IncludeInferiors gave incorrect results");
				FAIL;
			}
		}
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-10.(A)
 * When subwindow_mode is IncludeInferiors on a window of one
 * depth with mapped inferiors of differing depth, then no
 * errors occur.
 */
static void
t010()
{
XVisualInfo	*vp1, *vp2;
XImage	*swmsav;
int 	found;
XSetWindowAttributes w_attr;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vp2 = NULL;
	found = 0;

	for (resetvinf(VI_WIN); nextvinf(&vp1); ) {
		if (vp2 == NULL)
			vp2 = vp1;
		if (vp1->depth != vp2->depth) {
			found++;
			break;
		}
	}

	if (!found) {
		report("Only one depth visual supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	A_DRAW = makewin(A_DISPLAY, vp1);
	A_GC = makegc(A_DISPLAY, A_DRAW);

	startcall(display);
	if (isdeleted())
		return;
	XDrawImageString(display, d, gc, x, y, string, length);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	swmsav = savimage(A_DISPLAY, A_DRAW);
	dclear(A_DISPLAY, A_DRAW);

        /*
         * Must explicitly set the colormap and border_pixel parameters
         * to ensure that a BadMatch error doesn't get generated, since
         * it is entire possible that the depth differs from that of the
         * parent window.
         */
        w_attr.colormap = makecolmap(A_DISPLAY, vp2->visual, AllocNone);
        w_attr.border_pixel = 0;
	XCreateWindow(A_DISPLAY
		, A_DRAW
		, 0
		, 0
		, 300
		, 300
		, 0
		, vp2->depth
		, InputOutput
		, vp2->visual
		, CWColormap | CWBorderPixel 
		, &w_attr
		);
	XSetWindowBackground(A_DISPLAY, A_DRAW, W_BG);
	XMapWindow(A_DISPLAY, A_DRAW);

	/* Wait for Expose? */
	/* No.... we're not a top level window so no wmgr interference */
		
	XSetSubwindowMode(A_DISPLAY, A_GC, IncludeInferiors);

	startcall(display);
	if (isdeleted())
		return;
	XDrawImageString(display, d, gc, x, y, string, length);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (compsavimage(A_DISPLAY, A_DRAW, swmsav))
		trace("Drawing through to different depth window produces same results");
	else
		trace("Drawing through to different depth window produces different results");
		
	if (geterr() == Success)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/subwindow.mc */

/* Including from file gc/clip-x-or.mc */
/*
 * Assertion XDrawImageString-11.(A)
 * The clip origin coordinates clip-x-origin and clip-y-origin
 * are interpreted relative to the origin of the destination
 * drawable specified in the graphics operation.
 */
static void
t011()
{
XVisualInfo	*vp;
Pixmap	cmopixmap;
struct	area	area;
unsigned int 	cmowidth;
unsigned int 	cmoheight;
int 	divsize;
int 	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	divsize = 3;

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		/*
		 * Get size of the pixmap.  It is divsize smaller on all sides
		 * than the window.
		 */
		getsize(A_DISPLAY, A_DRAW, &cmowidth, &cmoheight);
		cmowidth /= divsize;
		cmoheight /= divsize;
		debug(2, "clip-mask height=%d, width=%d", cmowidth, cmoheight);

		/*
		 * Create a pixmap that is about divsize^2 of the area of
		 * the window.
		 */
		cmopixmap = XCreatePixmap(A_DISPLAY, A_DRAW, cmowidth, cmoheight, 1);
		dset(A_DISPLAY, cmopixmap, 1L);

		XSetClipMask(A_DISPLAY, A_GC, cmopixmap);
		XFreePixmap(A_DISPLAY, cmopixmap);

		for (i = 0; i < divsize; i++) {
			for (j = 0; j < divsize; j++) {
				setarea(&area, i*cmowidth, j*cmoheight, cmowidth, cmoheight);
				debug(2, "Origin at (%d,%d)", area.x, area.y);
				XSetClipOrigin(A_DISPLAY, A_GC, area.x, area.y);
				startcall(display);
				if (isdeleted())
					return;
				XDrawImageString(display, d, gc, x, y, string, length);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				if (checkarea(A_DISPLAY,A_DRAW,&area,W_BG,W_BG,CHECK_OUT))
					CHECK;
				else {
					report("Drawing occurred outside clip_mask");
					FAIL;
				}
				PIXCHECK(A_DISPLAY, A_DRAW);
				dclear(A_DISPLAY, A_DRAW);
			}
		}
	}

	CHECKPASS(2*divsize*divsize*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/clip-x-or.mc */

/* Including from file gc/clip-y-or.mc */
/* End of included file gc/clip-y-or.mc */

/* Including from file gc/clip-mask.mc */
/*
 * Assertion XDrawImageString-12.(A)
 * When the clip_mask is set to None, then the pixels are
 * always drawn regardless of the clip origin.
 */
static void
t012()
{
XVisualInfo	*vp;
XImage	*cmsav;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		cmsav = savimage(A_DISPLAY, A_DRAW);
		dclear(A_DISPLAY, A_DRAW);

		/*
		 * Not very interesting, because this should be the default
		 * anyway.
		 */
		XSetClipMask(A_DISPLAY, A_GC, None);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (compsavimage(A_DISPLAY, A_DRAW, cmsav))
			CHECK;
		else {
			report("Clip mask of None changed graphics");
			FAIL;
		}
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-13.(A)
 * When pixels are outside the area covered by the clip_mask,
 * then they are not drawn.
 */
static void
t013()
{
XVisualInfo	*vp;
Pixmap	cmpm;
struct	area	area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		setarea(&area, 0, 0, 5, 5);
		cmpm = XCreatePixmap(A_DISPLAY, A_DRAW, area.width, area.height, 1);
		dset(A_DISPLAY, cmpm, 1L);
		XSetClipMask(A_DISPLAY, A_GC, cmpm);
		XFreePixmap(A_DISPLAY, cmpm);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (checkarea(A_DISPLAY, A_DRAW, &area, 0L, W_BG, CHECK_OUT))
			CHECK;
		else {
			report("Drawing outside clip mask");
			FAIL;
		}
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-14.(A)
 * When pixels have a clip_mask bit set to 1, and they would
 * be drawn, then they are drawn.
 */
static void
t014()
{
#define stipple_width 24
#define stipple_height 11
static unsigned char stipple_bits[] = {
   0xff, 0x0f, 0x00, 0x3f, 0xf0, 0xff, 0xcf, 0xff, 0x03, 0xf7, 0x0f, 0xfc,
   0xff, 0xf1, 0x0f, 0x78, 0x7e, 0xf0, 0x80, 0x8f, 0x1f, 0x2a, 0xf0, 0xe1,
   0x80, 0x07, 0x1e, 0xaa, 0xff, 0xe0, 0x80, 0xff, 0x0f};
XVisualInfo	*vp;
Pixmap	cmpixmap;
Pixmap	cmstip;
XImage	*cmsav;
GC		cmgc;
unsigned int 	cmwidth, cmheight;
int 	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		cmsav = savimage(A_DISPLAY, A_DRAW);
		if (isdeleted())
			return;
		/* Must clear the drawable - otherwise there is no way 
		of proving clip mask of all 1's allowed drawing */
		dclear(A_DISPLAY, A_DRAW);

		getsize(A_DISPLAY, A_DRAW, &cmwidth, &cmheight);
		cmpixmap = XCreatePixmap(A_DISPLAY, A_DRAW, cmwidth, cmheight, 1);
		if (isdeleted())
			return;
		dset(A_DISPLAY, cmpixmap, 1L);

		XSetClipMask(A_DISPLAY, A_GC, cmpixmap);

		trace("Clip mask of all 1's");
		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (compsavimage(A_DISPLAY, A_DRAW, cmsav))
			CHECK;
		else {
			report("fail on clip_mask of all 1's");
			FAIL;
		}
		dclear(A_DISPLAY, A_DRAW);

		cmstip = XCreateBitmapFromData(A_DISPLAY, A_DRAW, (char*)stipple_bits,
			stipple_width, stipple_height);
		cmgc = makegc(A_DISPLAY, cmstip);
		if (isdeleted())
			return;

		for (i = 0; i < cmwidth; i += stipple_width) {
			for (j = 0; j < cmheight; j += stipple_height) {
				XCopyArea(A_DISPLAY, cmstip, cmpixmap, cmgc, 0, 0,
					stipple_width, stipple_height, i, j);
			}
		}
		XSetClipMask(A_DISPLAY, A_GC, cmpixmap);

		trace("Clip mask with pattern");
		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(A_DISPLAY, A_DRAW);

		XFreePixmap(A_DISPLAY, cmpixmap);
		XFreePixmap(A_DISPLAY, cmstip);
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-15.(A)
 * When pixels have a clip_mask bit set to 0, then they are
 * not drawn.
 */
static void
t015()
{
XVisualInfo	*vp;
Pixmap	cmpixmap;
unsigned int 	cmwidth, cmheight;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		getsize(A_DISPLAY, A_DRAW, &cmwidth, &cmheight);
		cmpixmap = XCreatePixmap(A_DISPLAY, A_DRAW, cmwidth, cmheight, 1);
		if (isdeleted())
			return;
		dset(A_DISPLAY, cmpixmap, 0L);

		XSetClipMask(A_DISPLAY, A_GC, cmpixmap);

		trace("Clip mask of all 0's");
		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (checkclear(A_DISPLAY, A_DRAW))
			CHECK;
		else {
			report("Pixels were drawn with clip_mask of all zeros");
			FAIL;
		}

		XFreePixmap(A_DISPLAY, cmpixmap);
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/clip-mask.mc */

/* Including from file gc/foregroun.mc */
/*
 * Assertion XDrawImageString-16.(A)
 * The foregound component of the gc is used to determine the
 * foreground pixel value.
 */
static void
t016()
{
static unsigned int fglist[] = {
	0, 0x1, 0x2, 0x3, 0x4, 0x6, 0x7, 0x10, 0x33, 0x81,
	0xa3, 0xff, 256, 300, 1000,
	0x111111, 0x400200, 0x777777,
	};
XVisualInfo	*vp;
long	fgind;
unsigned long	fg;
int 	nchecks = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		for (fgind = 0; fgind < sizeof(fglist)/sizeof(int); fgind++) {

			fg = fglist[fgind];
			if (fg > DEPTHMASK(vp->depth))
				break;

			nchecks++;
			trace("Testing fg pixel of %d", fg);
			XSetForeground(A_DISPLAY, A_GC, fg);

			startcall(display);
			if (isdeleted())
				return;
			XDrawImageString(display, d, gc, x, y, string, length);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAW);

			dclear(A_DISPLAY, A_DRAW);
		}
	}

	CHECKPASS(nchecks);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-17.(A)
 * The value for foreground is truncated to the depth of the
 * GC.
 */
static void
t017()
{
XVisualInfo	*vp;
XImage	*imp;
unsigned long	pix;
unsigned long	fgfg;
int 	found;
int 	i_x, i_y;
unsigned int 	i_width, i_height;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		XSetForeground(A_DISPLAY, A_GC, ~0L);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		imp = savimage(A_DISPLAY, A_DRAW);
		getsize(A_DISPLAY, A_DRAW, &i_width, &i_height);
		found = 0;
		/*
		 * Look for a non background pixel, if there is one then this
		 * should be the foreground truncated to the depth.
		 */
		for (i_y = 0; i_y < i_height; i_y++) {
			for (i_x = 0; i_x < i_width; i_x++) {
				pix = XGetPixel(imp, i_x, i_y);
				if (pix != W_BG) {
					found = 1;
					if (pix == DEPTHMASK(vp->depth)) {
						CHECK;
					} else {
						report("Pixel was set to %d, expecting %d", pix, DEPTHMASK(vp->depth));
						FAIL;
					}
					i_y = i_height;	/* So that we leave loop XXX */
					break;
				}
			}
		}
		if (found == 0) {
			if (W_BG == (~0 & DEPTHMASK(vp->depth))) {
				report("WARNING: W_BG != 0; this should not be the case");
				CHECK;
			} else {
				report("After setting foreground to (~0)");
				report("no non-background pixel was drawn");
				FAIL;
			}
		}

		dclear(A_DISPLAY, A_DRAW);

		/* Set all bits other than the bottom one */
		fgfg = (~0)^1;
		XSetForeground(A_DISPLAY, A_GC, fgfg);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		imp = savimage(A_DISPLAY, A_DRAW);
		getsize(A_DISPLAY, A_DRAW, &i_width, &i_height);
		found = 0;
		/*
		 * Look for a non background pixel, if there is one then this
		 * should be the foreground truncated to the depth.
		 */
		for (i_y = 0; i_y < i_height; i_y++) {
			for (i_x = 0; i_x < i_width; i_x++) {
				pix = XGetPixel(imp, i_x, i_y);
				if (pix != W_BG) {
					found = 1;
					if (pix == (fgfg & DEPTHMASK(vp->depth))) {
						CHECK;
					} else {
						report("Pixel was set to %d, expecting %d", pix, (fgfg & DEPTHMASK(vp->depth)));
						FAIL;
					}
					i_y = i_height;	/* So that we leave loop XXX */
					break;
				}
			}
		}
		if (found == 0) {
			/*
			 * This should be the case on monochrome displays
			 */
			if (W_BG == (fgfg & DEPTHMASK(vp->depth))) {
				CHECK;
			} else {
				report("After setting foreground to (~0)^1");
				report("no non-background pixel was drawn");
				FAIL;
			}
		}
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/foregroun.mc */

/* Including from file gc/backgroun.mc */
/*
 * Assertion XDrawImageString-18.(A)
 * The background component of the gc is used to determine the
 * background pixel value.
 */
static void
t018()
{
static unsigned long bglist[] = {
	0, 0x1, 0x2, 0x3, 0x4, 0x6, 0x7, 0x10, 0x33, 0x81,
	0xa3, 0xff, 256, 300, 1000,
	0x111111, 0x400200, 0x777777,
	};
XVisualInfo	*vp;
#ifdef NEED_STIPPLE
Pixmap	stip;
char	*stipbit = "\0";
#endif
long	bgind;
unsigned long	bg;
int 	nchecks = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

#ifdef NEED_STIPPLE
	stip = XCreateBitmapFromData(A_DISPLAY, DRW(A_DISPLAY), stipbit, 1, 1);
#endif

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		for (bgind = 0; bgind < NELEM(bglist); bgind++) {

			bg = bglist[bgind];
			if (bg > DEPTHMASK(vp->depth))
				break;

			trace("Testing bg pixel of %d", bg);
			nchecks++;

#ifdef NEED_STIPPLE
			/*
			 * Set a stipple to all zero's and set the fill style
			 * to be FillOpaqueStippled.  This will mean that everything
			 * should be drawn in the background colour.
			 * (different for XCopyPlane, which doesn't use fill style,
			 * nor does XPutImage or XCopyArea).
			 */
			XSetStipple(A_DISPLAY, A_GC, stip);
			XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
#endif
/* now set the background to bg and do the call. If it's XCopyPlane/XPutImage then
 * we have cleared the src drawable/image to zero so we will always find 0 in the
 * selected plane so we will fill with bg. If we're XDrawImageString then
 * we do the equivalent of the background fill anyway,  if we're
 * XCopyArea then we have to do a dset of src with bg otherwise we'll have
 * used FillOpaqueStippled and a stipple of a lone zero, only, to get the
 * same effect of a background fill.
 */
			XSetBackground(A_DISPLAY, A_GC, bg);

			startcall(display);
			if (isdeleted())
				return;
			XDrawImageString(display, d, gc, x, y, string, length);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAW);

			dclear(A_DISPLAY, A_DRAW);
		}
	}

#ifdef NEED_STIPPLE
	XFreePixmap(A_DISPLAY, stip);
#endif

	CHECKPASS(nchecks);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawImageString-19.(A)
 * The value for background is truncated to the depth of the
 * GC.
 */
static void
t019()
{
XVisualInfo	*vp;
XImage	*imp;
#ifdef NEED_STIPPLE
Pixmap	stip;
char	*stipbit = "\0\0\0\0";
#endif
unsigned long	pix;
int 	found;
int 	i_x, i_y;
unsigned int 	i_width, i_height;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

#ifdef NEED_STIPPLE
	stip = XCreateBitmapFromData(A_DISPLAY, DRW(A_DISPLAY), stipbit, 1, 1);
#endif

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

#ifdef NEED_STIPPLE
		/*
		 * Set a stipple to all zero's and set the fill style
		 * to be FillOpaqueStippled.  This will mean that everything
		 * should be drawn in the background colour.
		 */
		XSetStipple(A_DISPLAY, A_GC, stip);
		XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
#endif

/* now set the background to ~0 and do the call. If it's XCopyPlane/XPutImage then
 * we have cleared the src drawable/image to zero so we will always find 0 in the
 * selected plane so we will fill with ~0. If we're XDrawImageString then
 * we do the equivalent of the background fill anyway, if we're XCopyArea
 * then we need to fill the src will all 1's, otherwise we'll have
 * used FillOpaqueStippled and a stipple of a lone zero, only, to get the
 * same effect of a background fill.
 */
		XSetBackground(A_DISPLAY, A_GC, ~0L);

		startcall(display);
		if (isdeleted())
			return;
		XDrawImageString(display, d, gc, x, y, string, length);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		imp = savimage(A_DISPLAY, A_DRAW);
		getsize(A_DISPLAY, A_DRAW, &i_width, &i_height);
		found = 0;
		for (i_y = 0; i_y < i_height; i_y++) {
			for (i_x = 0; i_x < i_width; i_x++) {
				pix = XGetPixel(imp, i_x, i_y);
				if (pix != W_BG) {
					found = 1;
					if (pix == DEPTHMASK(vp->depth)) {
						CHECK;
					} else {
						report("Pixel was set to %d, expecting %d", pix, DEPTHMASK(vp->depth));
#ifdef NEED_STIPPLE
						report("FillOpaqueStippled is also used in this test, and could affect the result");
#endif
						FAIL;
					}
					i_y = i_height;	/* So that we leave loop XXX */
					break;
				}
			}
		}
		if (found == 0) {
			report("Nothing was drawn");
#ifdef NEED_STIPPLE
			report("FillOpaqueStippled is also used in this test, and could affect the result");
#endif
			FAIL;
		}
	}
#ifdef NEED_STIPPLE
	XFreePixmap(A_DISPLAY, stip);
#endif

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/backgroun.mc */

/* Including from file error/EDra.mc */
/*
 * Assertion XDrawImageString-20.(A)
 * When a drawable argument does not name a valid Drawable,
 * then a BadDrawable error occurs.
 */
/* End of included file error/EDra.mc */

/* Including from file error/EDra.mc */
static void
t020()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_DRAWABLE = (Drawable)badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XDrawImageString(display, d, gc, x, y, string, length);
	endcall(display);
	if (geterr() != BadDrawable) {
		report("Got %s, Expecting BadDrawable", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadDrawable)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EDra.mc */

/* Including from file error/EGC.mc */
/*
 * Assertion XDrawImageString-21.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t021()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XDrawImageString(display, d, gc, x, y, string, length);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/* Including from file error/EMat1.mc */
/*
 * Assertion XDrawImageString-22.(A)
 * When a drawable argument is an InputOnly window then a
 * BadMatch error occurs.
 */
/* End of included file error/EMat1.mc */

/* Including from file error/EMat1.mc */
static void
t022()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_DRAWABLE = (Drawable)iponlywin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XDrawImageString(display, d, gc, x, y, string, length);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat1.mc */

/* Including from file error/EMat2.mc */
/*
 * Assertion XDrawImageString-23.(A)
 * When the graphics context and the drawable do not have the
 * same depth, then a BadMatch error occurs.
 */
/* End of included file error/EMat2.mc */

/* Including from file error/EMat2.mc */
static void
t023()
{
XVisualInfo	*vp;
Drawable	errpm;
int 	founddepth = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_PIX); nextvinf(&vp); ) {
		if (vp->depth != 1) {
			founddepth++;

			errpm = makewin(A_DISPLAY, vp);
			A_GC = makegc(A_DISPLAY, errpm);

			break;
		}
	}

	if (!founddepth) {
		report("Only one depth supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	vp->depth = 1;
	A_DRAWABLE = makewin(A_DISPLAY, vp);

	startcall(display);
	if (isdeleted())
		return;
	XDrawImageString(display, d, gc, x, y, string, length);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat2.mc */

/* Including from file error/EMat3.mc */
/*
 * Assertion XDrawImageString-24.(A)
 * When the graphics context and the drawable were not created
 * for the same root, then a BadMatch error occurs.
 */
/* End of included file error/EMat3.mc */

/* Including from file error/EMat3.mc */
static void
t024()
{
XVisualInfo	vi;
Pixmap	errpm;
int 	scr_num;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

        if (config.alt_screen == -1) {
                unsupported("No alternate root supported");
                return;
        }

	scr_num = config.alt_screen;
	if (scr_num == DefaultScreen(A_DISPLAY)) {
		delete("The alternate root was the same as the one under test");
		return;
	}
	if (scr_num >= ScreenCount(A_DISPLAY)) {
		delete("The alternate root could not be accessed");
		return;
	}

	vi.visual = NULL;
	vi.screen = DefaultScreen(A_DISPLAY);	/* XXX */
	vi.depth = 1;
	A_DRAWABLE = makepixm(A_DISPLAY, &vi);

	/*
	 * Create a 1x1 depth 1 pixmap on other screen
	 * and use it to create a gc
	 */
	errpm = XCreatePixmap(A_DISPLAY, RootWindow(A_DISPLAY, scr_num), 1, 1, 1);
	A_GC = makegc(A_DISPLAY, errpm);

	startcall(display);
	if (isdeleted())
		return;
	XDrawImageString(display, d, gc, x, y, string, length);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	XFreePixmap(A_DISPLAY, errpm);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat3.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	t021, 21,
	t022, 22,
	t023, 23,
	t024, 24,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = localstartup;
void	(*tet_cleanup)() = fontcleanup;
