/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: drwln.m,v 1.4 92/06/11 16:32:07 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XDrawLine(display, d, gc, x1, y1, x2, y2)
 *   Display		*display;
 *   Drawable	d;
 *   GC		gc;
 *   int 	x1;
 *   int 	y1;
 *   int 	x2;
 *   int 	y2;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XDrawLine	1
char    *TestName = "XDrawLine";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE d
#define A_GC gc


/*
 * Arguments to the XDrawLine function
 */
static Display		*display;
static Drawable	d;
static GC		gc;
static int 	x1;
static int 	y1;
static int 	x2;
static int 	y2;


static void
setfordash()
{
	x1 = 20; y1 = 20;
	x2 = 70; y2 = 20;
}
static void
drawline(ax1, ay1, ax2, ay2)
int 	ax1, ay1, ax2, ay2;
{
int 	pass = 0, fail = 0;

	x1 = ax1; y1 = ay1;
	x2 = ax2; y2 = ay2;
	startcall(display);
	if (isdeleted())
		return;
	XDrawLine(display, d, gc, x1, y1, x2, y2);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
}
	


#define A_DRAW	A_DRAWABLE

static	int 	f_pix_x = -1;
static	int 	f_pix_y = -1;

static void
functest(functype)
int 	functype;
{
XVisualInfo	*vp;
unsigned long	srcpix;
unsigned long	destpix;
unsigned long	expected;
unsigned long	actualpix;
unsigned long	maxpix;
int 	pass = 0, fail = 0;

	trace("Function %s", gcfunctionname(functype));

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		/*
		 * If this is the first call find a pixel that is drawn
		 * that can be used for all the tests.
		 */
		if (f_pix_x == -1) {
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			setfuncpixel(A_DISPLAY, A_DRAW, &f_pix_x, &f_pix_y);
			dclear(A_DISPLAY, A_DRAW);
			if (f_pix_x == -1) {
				report("Nothing was drawn with a gc function of GXcopy");
				/*
				 * If we were actually testing GXcopy then the test has
				 * failed (in a way), so say FAIL.
				 */
				if (functype == GXcopy)
					FAIL;
				else
					delete("Setup error in functest");
				return;
			}
		}

		XSetFunction(A_DISPLAY, A_GC, functype);

		/* Try out a few more values without taking too long */
		maxpix = 3;

		for (destpix = 0; destpix <= maxpix; destpix++) {
			for (srcpix = 0; srcpix <= 1; srcpix++) {

				trace("src=%d, dest=%d", srcpix, destpix);

				/*
				 * The test on depth shouldn't be necessary, however a common
				 * bug exists where pixels are not truncated on mono screens.
				 * The bug is diagnosed in another test so causing failures here
				 * as well would be confusing.
				 */
				if (vp->depth == 1)
					dset(A_DISPLAY, A_DRAW, destpix&1);
				else
					dset(A_DISPLAY, A_DRAW, destpix);
				XSetForeground(A_DISPLAY, A_GC, srcpix);
				startcall(display);
				if (isdeleted())
					return;
				XDrawLine(display, d, gc, x1, y1, x2, y2);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}

				switch (functype) {
				case GXclear:
					expected = 0;
					break;
				case GXand:
					expected = srcpix & destpix;
					break;
				case GXandReverse:
					expected = srcpix & ~destpix;
					break;
				case GXcopy:
					expected = srcpix;
					break;
				case GXandInverted:
					expected = ~srcpix & destpix;
					break;
				case GXnoop:
					expected = destpix;
					break;
				case GXxor:
					expected = srcpix ^ destpix;
					break;
				case GXor:
					expected = srcpix | destpix;
					break;
				case GXnor:
					expected = ~srcpix & ~destpix;
					break;
				case GXequiv:
					expected = ~srcpix ^ destpix;
					break;
				case GXinvert:
					expected = ~destpix;
					break;
				case GXorReverse:
					expected = srcpix | ~destpix;
					break;
				case GXcopyInverted:
					expected = ~srcpix;
					break;
				case GXorInverted:
					expected = ~srcpix | destpix;
					break;
				case GXnand:
					expected = ~srcpix | ~destpix;
					break;
				case GXset:
					expected = ~0;
					break;
				default:
					report("Bad value given to functest");
					tet_result(TET_UNRESOLVED);
					break;
				}

				expected &= DEPTHMASK(vp->depth);
				actualpix = getpixel(A_DISPLAY, A_DRAW, f_pix_x, f_pix_y);

				debug(3, "src %d, dst %d, expect %d", srcpix, destpix,expected);

				if (actualpix == expected)
					CHECK;
				else {
					report("%s fail expected %d, got %d", gcfunctionname(functype), expected, actualpix);
					FAIL;
				}
			}
		}

	}

	CHECKPASS(2*(maxpix+1)*nvinf());
}


#define	PLANEFG	0x55555555
static	int 	planelist[] = {
	0, 0x1, 0x2, 0x23, 0xf8, 0x765, 0x3987, 0x129078, 0x23567193};

static	int 	linestyles[] = {LineSolid, LineOnOffDash, LineDoubleDash};
static	int 	capstyles[] = {CapNotLast, CapButt, CapProjecting, CapRound};

static char	lsdashes[] = {
	14,14};

#ifndef tile_width
#define tile_width 19
#define tile_height 19
static unsigned char tile_bits[] = {
   0xff, 0xff, 0x07, 0x80, 0x0f, 0x00, 0x67, 0x30, 0x07, 0x9b, 0xcf, 0x06,
   0xe8, 0xbf, 0x00, 0x94, 0x4f, 0x01, 0x54, 0x50, 0x01, 0x2a, 0xa7, 0x02,
   0xaa, 0xaf, 0x02, 0xaf, 0xaf, 0x07, 0xaa, 0xaf, 0x02, 0x2a, 0xa7, 0x02,
   0x54, 0x50, 0x01, 0x94, 0x4f, 0x01, 0xe8, 0xbf, 0x00, 0x98, 0xcf, 0x00,
   0x63, 0x30, 0x06, 0x80, 0x0f, 0x00, 0xff, 0xf8, 0x07};
#endif
#ifdef stipple_width
#define stipple_width 24
#define stipple_height 11
static unsigned char stipple_bits[] = {
   0xff, 0x0f, 0x00, 0x3f, 0xf0, 0xff, 0xcf, 0xff, 0x03, 0xf7, 0x0f, 0xfc,
   0xff, 0xf1, 0x0f, 0x78, 0x7e, 0xf0, 0x80, 0x8f, 0x1f, 0x2a, 0xf0, 0xe1,
   0x80, 0x07, 0x1e, 0xaa, 0xff, 0xe0, 0x80, 0xff, 0x0f};
#endif


#define	CAP_X1 15
#define	CAP_Y1 15
#define	CAP_X2 63
#define	CAP_Y2 42



#ifndef tile_width
#define tile_width 19
#define tile_height 19
static unsigned char tile_bits[] = {
   0xff, 0xff, 0x07, 0x80, 0x0f, 0x00, 0x67, 0x30, 0x07, 0x9b, 0xcf, 0x06,
   0xe8, 0xbf, 0x00, 0x94, 0x4f, 0x01, 0x54, 0x50, 0x01, 0x2a, 0xa7, 0x02,
   0xaa, 0xaf, 0x02, 0xaf, 0xaf, 0x07, 0xaa, 0xaf, 0x02, 0x2a, 0xa7, 0x02,
   0x54, 0x50, 0x01, 0x94, 0x4f, 0x01, 0xe8, 0xbf, 0x00, 0x98, 0xcf, 0x00,
   0x63, 0x30, 0x06, 0x80, 0x0f, 0x00, 0xff, 0xf8, 0x07};
#endif
#ifndef stipple_width
#define stipple_width 24
#define stipple_height 11
static unsigned char stipple_bits[] = {
   0xff, 0x0f, 0x00, 0x3f, 0xf0, 0xff, 0xcf, 0xff, 0x03, 0xf7, 0x0f, 0xfc,
   0xff, 0xf1, 0x0f, 0x78, 0x7e, 0xf0, 0x80, 0x8f, 0x1f, 0x2a, 0xf0, 0xe1,
   0x80, 0x07, 0x1e, 0xaa, 0xff, 0xe0, 0x80, 0xff, 0x0f};
#endif



#if !defined(T_XDrawImageString) && !defined(T_XCopyPlane) && !defined(T_XPutImage) && !defined(T_XCopyArea)
#define NEED_STIPPLE
#endif



static	char dolist[] = {
	8, 1, 2, 3, 1, 1, 1, 1, 1, 11, 2, 8};

static char	dashlist[] = {
	1, 5, 3, 9, 15, 1, 1, 2, 2, 3, 3,
	2, 3, 1, 5, 1, 7, 1, 2, 1};

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	d = 0;
	gc = 0;
	x1 = 10;
	y1 = 10;
	x2 = 31;
	y2 = 23;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	d = ErrdefDrawable;
	gc = ErrdefGC;
}

/*
 * Assertion XDrawLine-1.(A)
 * A call to XDrawLine draws a line between the pair of points
 * (x1,y1) and (x2,y2) in the drawable d.
 */
static void
t001()
{
XVisualInfo *vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {

		d = makewin(display, vp);
		gc = makegc(display, d);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(display, d);
	}
	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-2.(A)
 * A call to XDrawLine does not perform joining at coincident
 * endpoints.
 */
static void
t002()
{
XVisualInfo *vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {

		x1 = 10; y1 = 10;
		x2 = 31; y2 = 23;

		d = makewin(display, vp);
		gc = makegc(display, d);
		XSetLineAttributes(display, gc, 6, LineSolid, CapButt, JoinMiter);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		x1 = x2;
		y1 = y2;
		x2 = 10;
		y2 = 25;


		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(display, d);
	}
	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-3.(A)
 * A call to XDrawLine does not draw a pixel more than once.
 */
static void
t003()
{
XVisualInfo *vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {

		d = makewin(display, vp);
		gc = makegc(display, d);

		XSetFunction(display, gc, GXxor);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(display, d);
	}
	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-4.(A)
 * When lines intersect, then the intersecting pixels are
 * drawn multiple times.
 */
static void
t004()
{
XVisualInfo *vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {

		x1 = 10; y1 = 10;
		x2 = 31; y2 = 23;

		d = makewin(display, vp);
		gc = makegc(display, d);
		XSetLineAttributes(display, gc, 6, LineSolid, CapButt, JoinMiter);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		XSetFunction(display, gc, GXxor);

		x1 = 10; y1 = 30;
		x2 = 30; y2 = 10;

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(display, d);
	}
	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file gc/function.mc */
/*
 * Assertion XDrawLine-5.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXclear, then the destination becomes (dst AND (NOT
 * plane_mask)).
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXclear);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-6.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXand, then the destination becomes ((src AND dst) AND
 * plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXand);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-7.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXandReverse, then the destination becomes ((src AND (NOT
 * dst)) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXandReverse);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-8.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXcopy, then the destination becomes (src AND plane_mask)
 * OR (dst AND (NOT plane_mask)).
 */
static void
t008()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXcopy);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-9.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXandInverted, then the destination becomes (((NOT src) AND
 * dst) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t009()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXandInverted);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-10.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXnoop, then the destination becomes (dst AND plane_mask)
 * OR (dst AND (NOT plane_mask)).
 */
static void
t010()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXnoop);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-11.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXxor, then the destination becomes ((src XOR dst) AND
 * plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t011()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXxor);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-12.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is GXor,
 * then the destination becomes ((src OR dst) AND plane_mask)
 * OR (dst AND (NOT plane_mask)).
 */
static void
t012()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXor);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-13.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXnor, then the destination becomes (((NOT src) AND (NOT
 * dst)) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t013()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXnor);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-14.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXequiv, then the destination becomes (((NOT src) XOR dst)
 * AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t014()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXequiv);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-15.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXinvert, then the destination becomes ((NOT dst) AND
 * plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t015()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXinvert);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-16.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXorReverse, then the destination becomes ((src OR (NOT
 * dst)) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t016()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXorReverse);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-17.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXcopyInverted, then the destination becomes ((NOT src) AND
 * plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t017()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXcopyInverted);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-18.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXorInverted, then the destination becomes (((NOT src) OR
 * dst) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t018()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXorInverted);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-19.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXnand, then the destination becomes (((NOT src) OR (NOT
 * dst)) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t019()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXnand);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-20.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXset, then the destination becomes ( plane_mask) OR (dst
 * AND (NOT plane_mask)).
 */
static void
t020()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXset);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/function.mc */

/* Including from file gc/plane-mas.mc */
/*
 * Assertion XDrawLine-21.(A)
 * The value for plane_mask is truncated to the depth of the
 * GC.
 */
static void
t021()
{
XVisualInfo	*vp;
int 	depthmask;
int 	*ip;
unsigned long	pix;
unsigned long	expected;
int 	pmx = -1, pmy;
int 	ntested;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	ntested = 0;
	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		if (pmx == -1) {
			/*
			 * Get a point that is set in the drawable.
			 */
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			setfuncpixel(A_DISPLAY, A_DRAW, &pmx, &pmy);
			dclear(A_DISPLAY, A_DRAW);
		}

		depthmask = DEPTHMASK(vp->depth);

		XSetForeground(A_DISPLAY, A_GC, PLANEFG);

		for (ip = planelist; ip < &planelist[NELEM(planelist)]; ip++) {
			if (*ip <= depthmask)
				continue;

			trace("plane-mask 0x%x", *ip);
			XSetPlaneMask(A_DISPLAY, A_GC, *ip);
			ntested++;

			dclear(A_DISPLAY, A_DRAW);
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			expected = (*ip & PLANEFG) & depthmask;
			pix = getpixel(A_DISPLAY, A_DRAW, pmx, pmy);

			if (pix == expected)
				CHECK;
			else {
				report("got pixel 0x%x, expecting 0x%x", pix, expected);
				FAIL;
			}
		}

	}

	CHECKPASS(ntested);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-22.(A)
 * The plane_mask specifies which planes of the destination
 * are to be modified, one bit per plane, with bits being
 * assigned to planes from the least significant bit of the
 * word to the most significant bit.
 */
static void
t022()
{
XVisualInfo	*vp;
int 	depthmask;
int 	*ip;
unsigned long	pix;
unsigned long	expected;
int 	pmx = -1, pmy;
int 	ntested = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		if (pmx == -1) {
			/*
			 * Get a point that is set in the drawable.
			 */
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			setfuncpixel(A_DISPLAY, A_DRAW, &pmx, &pmy);
			dclear(A_DISPLAY, A_DRAW);
		}

		depthmask = DEPTHMASK(vp->depth);

		XSetForeground(A_DISPLAY, A_GC, PLANEFG);
		for (ip = planelist; ip < &planelist[NELEM(planelist)]; ip++) {
			if (*ip > depthmask)
				break;

			trace("plane-mask 0x%x", *ip);
			XSetPlaneMask(A_DISPLAY, A_GC, *ip);
			ntested++;

			dclear(A_DISPLAY, A_DRAW);
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			expected = *ip & PLANEFG;
			pix = getpixel(A_DISPLAY, A_DRAW, pmx, pmy);

			if (pix == expected)
				CHECK;
			else {
				report("got pixel 0x%x, expecting 0x%x", pix, expected);
				FAIL;
			}
		}

	}

	CHECKPASS(ntested);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/plane-mas.mc */

/* Including from file gc/line-widt.mc */
/*
 * Assertion XDrawLine-23.(A)
 * When line_width is greater than or equal to one, and the
 * center of a pixel is fully inside the boundary, then the
 * pixel is drawn.
 */
static void
t023()
{
XVisualInfo	*vp;
static	struct	linedata {
	unsigned int 	width;
	int 	x1;
	int 	y1;
	int 	x2;
	int 	y2;
} linedata[] = {
	{1, 15, 10, 50, 30},
	{5, 50, 4, 43, 21},
	{8, 70, 6, 55, 27},
	{2, 13, 13, 15, 24},
	{15, 10, 51, 49, 60},
	};
struct	linedata	*lp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

#define NLINEDATA (sizeof(linedata)/sizeof(linedata[0]))

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		for (lp = linedata; lp < &linedata[NLINEDATA]; lp++) {

			setwidth(A_DISPLAY, A_GC, lp->width);
			drawline(lp->x1, lp->y1, lp->x2, lp->y2);
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-24.(A)
 * When line_width is greater than or equal to one, and the
 * center of the pixel is exactly on the boundary, and the
 * boundary is not horizontal, and the interior is immediately
 * to its right (x increasing direction), then the pixel is
 * drawn.
 */
static void
t024()
{
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)10);
		drawline(6, 13, 14, 7);

		/*
		 * Pixel at 13,14 should be unset whereas the one at 7,6 should
		 * be set.
		 */
		if (checkpixel(A_DISPLAY, A_DRAWABLE, 13, 14, W_BG))
			CHECK;
		else {
			report("pixel at 13,14 was set");
			FAIL;
		}
		if (checkpixel(A_DISPLAY, A_DRAWABLE, 7, 6, W_FG))
			CHECK;
		else {
			report("pixel at 7,6 was not set");
			FAIL;
		}
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-25.(A)
 * When line_width is greater than or equal to one, and the
 * center of the pixel is exactly on the boundary, and the
 * boundary is horizontal, and the interior or the boundary is
 * immediately below (y increasing direction), and the interior
 * or the boundary is immediately to the right (x increasing
 * direction), then the pixel is drawn.
 */
static void
t025()
{
XVisualInfo	*vp;
XImage	*lwimp;
unsigned int 	lwwidth = 8;
int 	begx = 10, begy = 10, len = 60;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, lwwidth);

		drawline(begx, begy, begx+len, begy);

		lwimp = savimage(A_DISPLAY, A_DRAWABLE);
		for (i = 0; i < len; i++) {
			if (XGetPixel(lwimp, begx+i, begy-lwwidth/2) == W_FG)
				CHECK;
			else {
				report("Pixel with interior below was not drawn");
				FAIL;
			}
		}
		for (i = 0; i < len; i++) {
			if (XGetPixel(lwimp, begx+i, begy+lwwidth/2) == W_BG)
				CHECK;
			else {
				report("Pixel with interior above was drawn");
				FAIL;
			}
		}

		for (i = 0; i < lwwidth; i++) {
			if (XGetPixel(lwimp, 10, begy-lwwidth/2+i) == W_FG)
				CHECK;
			else {
				report("Pixel with interior to the right was not drawn");
				FAIL;
			}
		}
		for (i = 0; i < lwwidth; i++) {
			if (XGetPixel(lwimp, 70, begy-lwwidth/2+i) == W_BG)
				CHECK;
			else {
				report("Pixel with interior to the left was drawn");
				FAIL;
			}
		}

	}

	CHECKPASS(nvinf()*(2*len+2*lwwidth));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-26.(B)
 *   Reason: The statement in the X11 specification is not specific enough to write a test
 * When line_width is zero, then a one pixel wide line is
 * drawn using an unspecified, device-dependent algorithm.
 */
static void
t026()
{
XVisualInfo	*vp;
struct	area	area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)0);
		drawline(10, 10, 70, 10);

		setarea(&area, 10, 10, 61, 1);
		if (!checkarea(A_DISPLAY, A_DRAWABLE,
				&area, W_FG, W_BG, CHECK_ALL)) {
			/*
			 * The spec doesn't even require that horizontal lines be drawn
			 * properly.  So we can only warn about suprises.
			 */
			trace("Horizontal thin line was not drawn one pixel in width as expected");
		}
	}

	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-27.(A)
 * When line_width is zero, and a line is drawn unclipped from
 * [x1, y1] to [x2, y2] and another line is drawn unclipped
 * from [x1+dx, y1+dy] to [x2+dx, y2+dy], and a point [x, y] is
 * touched by drawing the first line, then point [x+dx, y+dy]
 * is touched by drawing the second line.
 */
static void
t027()
{
XVisualInfo	*vp;
XImage	*lwim1, *lwim2;
int 	x1, x2, y1, y2;
int 	lwx, lwy;
unsigned int 	lwwidth = 70;
unsigned int 	lwheight = 50;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		x1 = 10; y1 = 10;
		x2 = 63; y2 = 43;

		setwidth(A_DISPLAY, A_GC, (unsigned)0);

		drawline(x1, y1, x2, y2);
		lwim1 = XGetImage(A_DISPLAY, A_DRAWABLE, x1, y1, lwwidth, lwheight, AllPlanes, ZPixmap);

		if (isdeleted())
			return;

		x1 += 12;
		x2 += 12;

		y1 += 17;
		y2 += 17;

		dclear(A_DISPLAY, A_DRAWABLE);
		drawline(x1, y1, x2, y2);
		lwim2 = XGetImage(A_DISPLAY, A_DRAWABLE, x1, y1, lwwidth, lwheight, AllPlanes, ZPixmap);

		if (isdeleted())
			return;

		for (lwy = 0; lwy < lwheight; lwy++) {
			for (lwx = 0; lwx < lwwidth; lwx++) {
				if (XGetPixel(lwim1, lwx, lwy) != XGetPixel(lwim2, lwx, lwy)) {
					report("Thin line differed after displacement");
					report("Point is %d,%d", lwx, lwy);
					FAIL;

					lwy = lwheight;
					break;
				}
			}
		}
		CHECK;
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-28.(-)
 * When line_width is zero, and a line is drawn unclipped from
 * [x1, y1] to [x2, y2] and another line is drawn unclipped
 * from [x1+dx, y1+dy] to [x2+dx, y2+dy], and a point [x, y] is
 * not touched by drawing the first line, then point [x+dx,
 * y+dy] is not touched by drawing the second line.
 */
static void
t028()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-29.(-)
 * When a line is clipped, and the point is inside the
 * clipping region, and the point would be touched by the
 * unclipped line, then the pixel is drawn.
 */
static void
t029()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* End of included file gc/line-widt.mc */

/* Including from file gc/line-styl.mc */
/*
 * Assertion XDrawLine-30.(-)
 * When the line_style is LineSolid, then the full path of
 * the line is drawn.
 */
static void
t030()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-31.(A)
 * When the line_style is LineOnOffDash, then cap_style
 * applies to all internal ends of the individual dashes,
 * except CapNotLast is treated as CapButt.
 */
static void
t031()
{
XVisualInfo	*vp;
int 	i;
int 	caps;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();
		for(lw=4; lw <= 5; lw++) { /* odd and even line widths */
			setwidth(A_DISPLAY, A_GC, lw);
			XSetDashes(A_DISPLAY, A_GC, 0, lsdashes, 2);
			setlinestyle(A_DISPLAY, A_GC, LineOnOffDash);

			for (i = 0; i < NELEM(capstyles); i++) {

				caps = capstyles[i];
				trace("LineOnOffDash with %s width %u", capstylename(caps), lw);
				setcapstyle(A_DISPLAY, A_GC, caps);
				startcall(display);
				if (isdeleted())
					return;
				XDrawLine(display, d, gc, x1, y1, x2, y2);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				PIXCHECK(A_DISPLAY, A_DRAWABLE);
				dclear(A_DISPLAY, A_DRAWABLE);
			}
			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}

	CHECKPASS(2*NELEM(capstyles)*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-32.(-)
 * When line_style is LineSolid and fill_style is FillSolid,
 * then on a call to XDrawLine the source pixel for the drawing
 * operation is foreground.
 */
static void
t032()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-33.(-)
 * When line_style is LineSolid and fill_style is FillTiled,
 * then on a call to XDrawLine the source for the drawing
 * operation is tile.
 */
static void
t033()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-34.(-)
 * When line_style is LineSolid and fill_style is
 * FillOpaqueStippled, then on a call to XDrawLine the source
 * for the drawing operation is a tile with the same width and
 * height as stipple, but with background everywhere stipple
 * has a zero and with foreground everywhere stipple has a one.
 */
static void
t034()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-35.(-)
 * When line_style is LineSolid and fill_style is FillStippled,
 * then on a call to XDrawLine the source for the drawing
 * operation is foreground masked by the stipple pattern tiled
 * in a single plane.
 */
static void
t035()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-36.(-)
 * When line_style is LineOnOffDash and the dash is odd, then
 * nothing is drawn.
 */
static void
t036()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-37.(-)
 * When line_style is LineOnOffDash and the dash is even and
 * fill_style is FillSolid, then on a call to XDrawLine the
 * source pixel for the drawing operation is foreground.
 */
static void
t037()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-38.(A)
 * When line_style is LineOnOffDash and the dash is even and
 * fill_style is FillTiled, then on a call to XDrawLine the
 * source for the drawing operation is tile.
 */
static void
t038()
{
XVisualInfo	*vp;
Pixmap	fstile;
unsigned long	fsfg, fsbg;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();

		/* 1 */
		fsfg = W_FG;
		fsbg = W_BG;
		if (vp->depth > 1) {
			fsfg ^= 0x2;
			fsbg ^= 0x2;
		}
		fstile = XCreatePixmapFromBitmapData(A_DISPLAY, A_DRAWABLE
			, (char*)tile_bits, tile_width, tile_height
			, fsfg, fsbg
			, vp->depth
			);
		XSetTile(A_DISPLAY, A_GC, fstile);

		XSetFillStyle(A_DISPLAY, A_GC, FillTiled);
		setlinestyle(A_DISPLAY, A_GC, LineOnOffDash);

		for(lw=5; lw <= 6; lw++) { /* odd and even widths */
			setwidth(A_DISPLAY, A_GC, lw);

			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAWABLE);

			dclear(A_DISPLAY, A_DRAWABLE);
		}
		XFreePixmap(A_DISPLAY, fstile);
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-39.(A)
 * When line_style is LineOnOffDash and the dash is even and
 * fill_style is FillOpaqueStippled, then on a call to
 * XDrawLine the source for the drawing operation is a tile
 * with the same width and height as stipple, but with
 * background everywhere stipple has a zero and with foreground
 * everywhere stipple has a one.
 */
static void
t039()
{
XVisualInfo	*vp;
Pixmap	fsstip;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();

		fsstip = XCreateBitmapFromData(A_DISPLAY, A_DRAWABLE
			, (char*)stipple_bits, stipple_width, stipple_height
			);
		XSetStipple(A_DISPLAY, A_GC, fsstip);

		XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
		setlinestyle(A_DISPLAY, A_GC, LineOnOffDash);
		for(lw=5; lw <= 6; lw++) { /* odd and even widths */
			setwidth(A_DISPLAY, A_GC, lw);

			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAWABLE);

			dclear(A_DISPLAY, A_DRAWABLE);
		}
		XFreePixmap(A_DISPLAY, fsstip);
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-40.(A)
 * When line_style is LineOnOffDash and the dash is even and
 * fill_style is FillStippled, then on a call to XDrawLine the
 * source for the drawing operation is foreground masked by the
 * stipple pattern tiled in a single plane.
 */
static void
t040()
{
XVisualInfo	*vp;
Pixmap	fsstip;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();

		fsstip = XCreateBitmapFromData(A_DISPLAY, A_DRAWABLE
			, (char*)stipple_bits, stipple_width, stipple_height
			);
		XSetStipple(A_DISPLAY, A_GC, fsstip);

		XSetFillStyle(A_DISPLAY, A_GC, FillStippled);
		setlinestyle(A_DISPLAY, A_GC, LineOnOffDash);
		for(lw=5; lw <= 6; lw++) { /* odd and even widths */
			setwidth(A_DISPLAY, A_GC, lw);

			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAWABLE);

			dclear(A_DISPLAY, A_DRAWABLE);
		}
		XFreePixmap(A_DISPLAY, fsstip);
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-41.(A)
 * When line_style is LineDoubleDash and the dash is even and
 * fill_style is FillSolid, then on a call to XDrawLine the
 * source pixel for the drawing operation is foreground.
 */
static void
t041()
{
XVisualInfo	*vp;
XImage	*lsimp;
unsigned int 	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		for(lw=4; lw <= 5; lw++) { /* odd and even widths */
			/*
			 * By reversing the foreground and background pixels we should be
			 * able to fill in the whole line. First save the whole line.
			 */
			trace("Draw with LineSolid, width %u, and save result", lw);
			XSetLineAttributes(A_DISPLAY, A_GC, lw, LineSolid, CapButt, JoinMiter);
			XSetForeground(A_DISPLAY, A_GC, W_FG|W_BG);	/* Draw with OR of fg&bg */
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			XSetForeground(A_DISPLAY, A_GC, W_FG);

			lsimp = savimage(A_DISPLAY, A_DRAWABLE);
			dclear(A_DISPLAY, A_DRAWABLE);

			/* Now call with DoubleDash */
			XSetDashes(A_DISPLAY, A_GC, 0, lsdashes, 2);
			XSetLineAttributes(A_DISPLAY, A_GC, lw, LineDoubleDash, CapButt, JoinMiter);
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			/* Reverse fg&bg and or in line */
			XSetFunction(A_DISPLAY, A_GC, GXor);
			XSetForeground(A_DISPLAY, A_GC, W_BG);
			XSetBackground(A_DISPLAY, A_GC, W_FG);
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			XSetForeground(A_DISPLAY, A_GC, W_FG);
			XSetBackground(A_DISPLAY, A_GC, W_BG);

			trace("compare to result of drawing lines with LineDoubleDash");
			if (compsavimage(A_DISPLAY, A_DRAWABLE, lsimp))
				CHECK;
			else {
				report("Reversing fg and bg did not complete line correctly");
				FAIL;
			}
			/* assume we're not allowed to draw nothing, complain if so */
			if (!checkarea(A_DISPLAY, A_DRAWABLE,
				(struct area *)0, W_BG, W_BG, CHECK_IN | CHECK_DIFFER))
				CHECK;
			else {
				report("%s didn't draw anything when ORing together even and odd dashes", TestName);
				FAIL;
			}
			dclear(A_DISPLAY, A_DRAWABLE);
		}

	}

	/*
	 * Now do direct checking on vertical or horizontal lines
	 */
	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();
		XSetDashes(A_DISPLAY, A_GC, 0, lsdashes, 2);

		for(lw=4; lw <=5; lw++) { /* odd/even widths */
			trace("Direct check of LineDoubleDash on vertical/horizontal lines width %u", lw);
			XSetLineAttributes(A_DISPLAY, A_GC,
				lw, LineDoubleDash, CapButt, JoinMiter);
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			PIXCHECK(A_DISPLAY, A_DRAWABLE);
			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}

	CHECKPASS(6*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-42.(A)
 * When line_style is LineDoubleDash and the dash is even and
 * fill_style is FillTiled, then on a call to XDrawLine the
 * source for the drawing operation is tile.
 */
static void
t042()
{
XVisualInfo	*vp;
Pixmap	fstile;
unsigned long	fsfg, fsbg;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();

		/* 1 */
		fsfg = W_FG;
		fsbg = W_BG;
		if (vp->depth > 1) {
			fsfg ^= 0x2;
			fsbg ^= 0x2;
		}
		fstile = XCreatePixmapFromBitmapData(A_DISPLAY, A_DRAWABLE
			, (char*)tile_bits, tile_width, tile_height
			, fsfg, fsbg
			, vp->depth
			);
		XSetTile(A_DISPLAY, A_GC, fstile);

		XSetFillStyle(A_DISPLAY, A_GC, FillTiled);
		setlinestyle(A_DISPLAY, A_GC, LineDoubleDash);
		for(lw=4; lw <= 5; lw++) { /* odd and even widths */
			setwidth(A_DISPLAY, A_GC, lw);

			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAWABLE);
			dclear(A_DISPLAY, A_DRAWABLE);
		}
		XFreePixmap(A_DISPLAY, fstile);
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-43.(A)
 * When line_style is LineDoubleDash and the dash is even and
 * fill_style is FillOpaqueStippled, then on a call to
 * XDrawLine the source for the drawing operation is a tile
 * with the same width and height as stipple, but with
 * background everywhere stipple has a zero and with foreground
 * everywhere stipple has a one.
 */
static void
t043()
{
XVisualInfo	*vp;
Pixmap	fsstip;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();

		fsstip = XCreateBitmapFromData(A_DISPLAY, A_DRAWABLE
			, (char*)stipple_bits, stipple_width, stipple_height
			);
		XSetStipple(A_DISPLAY, A_GC, fsstip);

		XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
		setlinestyle(A_DISPLAY, A_GC, LineDoubleDash);
		for(lw=4; lw <= 5; lw++) { /* odd and even widths */
			setwidth(A_DISPLAY, A_GC, lw);

			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAWABLE);
			dclear(A_DISPLAY, A_DRAWABLE);
		}
		XFreePixmap(A_DISPLAY, fsstip);
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-44.(A)
 * When line_style is LineDoubleDash and the dash is even and
 * fill_style is FillStippled, then on a call to XDrawLine the
 * source for the drawing operation is foreground masked by the
 * stipple pattern tiled in a single plane.
 */
static void
t044()
{
XVisualInfo	*vp;
Pixmap	fsstip;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();
		XSetDashes(A_DISPLAY, A_GC, 0, lsdashes, 2);

		fsstip = XCreateBitmapFromData(A_DISPLAY, A_DRAWABLE
			, (char*)stipple_bits, stipple_width, stipple_height
			);
		XSetStipple(A_DISPLAY, A_GC, fsstip);

		XSetFillStyle(A_DISPLAY, A_GC, FillStippled);
		setlinestyle(A_DISPLAY, A_GC, LineDoubleDash);
		for(lw=4; lw <= 5; lw++) { /* odd and even widths */
			setwidth(A_DISPLAY, A_GC, lw);

			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAWABLE);
			dclear(A_DISPLAY, A_DRAWABLE);
		}
		XFreePixmap(A_DISPLAY, fsstip);
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-45.(-)
 * When line-style is LineDoubleDash and the dash is odd and
 * fill_style is FillSolid, then on a call to XDrawLine the
 * source for the drawing operation is background.
 */
static void
t045()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-46.(-)
 * When line-style is LineDoubleDash and the dash is odd and
 * fill_style is FillTiled, then on a call to XDrawLine the
 * source for the drawing operation is tile.
 */
static void
t046()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-47.(-)
 * When line-style is LineDoubleDash and the dash is odd and
 * fill_style is FillOpaqueStippled, then on a call to
 * XDrawLine the source for the drawing operation is a tile
 * with the same width and height as stipple, but with
 * background everywhere stipple has a zero and with foreground
 * everywhere stipple has a one.
 */
static void
t047()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-48.(-)
 * When line-style is LineDoubleDash and the dash is odd and
 * fill_style is FillStippled, then on a call to XDrawLine the
 * source for the drawing operation is background masked by the
 * stipple pattern tiled in a single plane.
 */
static void
t048()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-49.(A)
 * A call to XDrawLine does not draw each pixel of a
 * particular line more than once.
 */
static void
t049()
{
XVisualInfo	*vp;
XImage	*lsimp;
int 	i;
int 	ls;
unsigned int 	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();
		XSetDashes(A_DISPLAY, A_GC, 0, lsdashes, 2);

		for(lw=4; lw <= 5; lw++) { /* odd and even widths */
			setwidth(A_DISPLAY, A_GC, lw);

			for (i = 0; i < NELEM(linestyles); i++) {
				XSetFunction(A_DISPLAY, A_GC, GXcopy);

				ls = linestyles[i];

				trace("Drawing line style %s width %u", linestylename(ls), lw);
				setlinestyle(A_DISPLAY, A_GC, ls);
				startcall(display);
				if (isdeleted())
					return;
				XDrawLine(display, d, gc, x1, y1, x2, y2);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				lsimp = savimage(A_DISPLAY, A_DRAWABLE);
				dclear(A_DISPLAY, A_DRAWABLE);

				XSetFunction(A_DISPLAY, A_GC, GXxor);
				startcall(display);
				if (isdeleted())
					return;
				XDrawLine(display, d, gc, x1, y1, x2, y2);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				if (compsavimage(A_DISPLAY, A_DRAWABLE, lsimp))
					CHECK;
				else {
					report("Pixels drawn more than once for %s", linestylename(ls));
					FAIL;
				}
				/* assume we're not allowed to draw nothing, complain if so */
				if (!checkarea(A_DISPLAY, A_DRAWABLE,
					(struct area *)0, W_BG, W_BG, CHECK_IN | CHECK_DIFFER))
					CHECK;
				else {
					report("%s didn't draw anything in Xor mode", TestName);
					FAIL;
				}
				dclear(A_DISPLAY, A_DRAWABLE);
			}
			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}

	CHECKPASS(4*NELEM(linestyles)*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/line-styl.mc */

/* Including from file gc/cap-style.mc */
/*
 * Assertion XDrawLine-50.(A)
 * When line_width is greater than or equal to one, and
 * line_style is LineSolid, and a line is drawn from [x1, y1]
 * to [x2, y2], and a line is drawn from [x2, y2] to [x1, y1],
 * then the same pixels are drawn in each case.
 */
static void
t050()
{
XVisualInfo	*vp;
struct	linedata {
	int 	x1;
	int 	y1;
	int 	x2;
	int 	y2;
};
static struct	linedata linedata[] = {
	{1, 1, 70, 10},
	{3, 5, 10, 50},
	{50, 25, 10, 20},
	{45, 18, 34, 80},
	{1, 77, 38, 22},
	{88, 12, 54, 23},
	};
struct	linedata	*lp;
int 	ndata = NELEM(linedata);
XImage	*lsimp;
unsigned int 	lswidth;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		lswidth = 1;
		for (lp = linedata; lp < &linedata[ndata]; lp++) {
			setwidth(A_DISPLAY, A_GC, lswidth++);
			drawline(lp->x1, lp->y1, lp->x2, lp->y2);
		}

		lsimp = savimage(A_DISPLAY, A_DRAWABLE);
		dclear(A_DISPLAY, A_DRAWABLE);

		/* Draw the lines reversed */
		lswidth = 1;
		for (lp = linedata; lp < &linedata[ndata]; lp++) {
			setwidth(A_DISPLAY, A_GC, lswidth++);
			drawline(lp->x2, lp->y2, lp->x1, lp->y1);
		}

		/* Compare the results */
		if (compsavimage(A_DISPLAY, A_DRAWABLE, lsimp))
			CHECK;
		else {
			report("Lines not same when drawn in opposite direction");
			FAIL;
		}

		/* assume we're not allowed to draw nothing, complain if so */
		if (!checkarea(A_DISPLAY, A_DRAWABLE,
			(struct area *)0, W_BG, W_BG, CHECK_IN | CHECK_DIFFER))
			CHECK;
		else {
			report("%s didn't draw anything in opposite direction", TestName);
			FAIL;
		}
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-51.(A)
 * When the cap_style is CapNotLast and the line_width is
 * greater than zero (> 0), then this is equivalent to CapButt.
 */
static void
t051()
{
XVisualInfo	*vp;
XImage	*capsav;
unsigned int 	capw;
static unsigned int 	capwidths[] = {
	1, 2, 5, 6, 10, 11, 40, 41};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		for (capw = 0; capw < NELEM(capwidths); capw++) {
			setwidth(A_DISPLAY, A_GC, capwidths[capw]);

			setcapstyle(A_DISPLAY, A_GC, CapNotLast);
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			capsav = savimage(A_DISPLAY, A_DRAWABLE);
			dclear(A_DISPLAY, A_DRAWABLE);

			setcapstyle(A_DISPLAY, A_GC, CapButt);
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			if (compsavimage(A_DISPLAY, A_DRAWABLE, capsav))
				CHECK;
			else {
				report("CapNotLast was not equivalent to CapButt for linewidth %d", capwidths[capw]);
				FAIL;
			}
			/* assume we're not allowed to draw nothing, complain if so */
			if (!checkarea(A_DISPLAY, A_DRAWABLE,
				(struct area *)0, W_BG, W_BG, CHECK_IN | CHECK_DIFFER))
				CHECK;
			else {
				report("%s didn't draw anything with CapButt", TestName);
				FAIL;
			}
			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}
	CHECKPASS(2*nvinf()*NELEM(capwidths));

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-52.(A)
 * When the cap_style is CapNotLast and the line_width is zero
 * (0), then this is equivalent to CapButt except that the
 * final endpoint is not drawn.
 */
static void
t052()
{
XVisualInfo	*vp;
int 	capx1, capy1, capx2, capy2;
struct	area	area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	capx1 = 10; capy1 = 10;
	capx2 = 70; capy2 = 30;

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)0);

		setcapstyle(A_DISPLAY, A_GC, CapButt);
		drawline(capx1, capy1, capx2, capy2);

		XSetFunction(A_DISPLAY, A_GC, GXxor);
		setcapstyle(A_DISPLAY, A_GC, CapNotLast);
		drawline(capx1, capy1, capx2, capy2);

		/* Only the final endpoint should be set */
		setarea(&area, capx2, capy2, 1, 1);
		if (checkarea(A_DISPLAY, A_DRAWABLE, &area, W_FG, W_BG, CHECK_ALL))
			CHECK;
		else {
			report("Cap style incorrect for thin line and CapNotLast");
			FAIL;
		}
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-53.(A)
 * When the cap_style is CapButt, then the line is square at
 * the endpoint (perpendicular to the slope of the line) with
 * no projection beyond.
 */
static void
t053()
{
XVisualInfo	*vp;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		for(lw=8; lw <= 9; lw++) { /* odd or even line widths */
			setwidth(A_DISPLAY, A_GC, lw);
			setcapstyle(A_DISPLAY, A_GC, CapButt);

			drawline(15, 15, 50, 33);

			PIXCHECK(A_DISPLAY, A_DRAWABLE);

			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-54.(A)
 * When the cap_style is CapRound and line-width is zero, then
 * this is equivalent to CapButt.
 */
static void
t054()
{
XVisualInfo	*vp;
XImage	*savimp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)0);
		setcapstyle(A_DISPLAY, A_GC, CapRound);

		drawline(15, 15, 52, 35);
		savimp = savimage(A_DISPLAY, A_DRAWABLE);

		dclear(A_DISPLAY, A_DRAWABLE);
		setcapstyle(A_DISPLAY, A_GC, CapButt);
		drawline(15, 15, 52, 35);

		if (compsavimage(A_DISPLAY, A_DRAWABLE, savimp))
			CHECK;
		else {
			report("CapRound not equivalent to CapButt for thin line");
			FAIL;
		}

	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-55.(A)
 * When the cap_style is CapRound and line-width is not zero,
 * then the line has a circular arc, with the diamater equal to
 * the line_width, centred on the endpoint.
 */
static void
t055()
{
XVisualInfo	*vp;
int 	i;
static 	unsigned int 	capwidths[] = {
	1, 2, 4, 5, 10, 11, 40,41};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setcapstyle(A_DISPLAY, A_GC, CapRound);

		for (i = 0; i < NELEM(capwidths); i++) {
			trace("Trying CapRound with width of %d", capwidths[i]);
			setwidth(A_DISPLAY, A_GC, capwidths[i]);
			drawline(CAP_X1, CAP_Y1, CAP_X2, CAP_Y2);

			PIXCHECK(A_DISPLAY, A_DRAWABLE);

			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}

	CHECKPASS(NELEM(capwidths)*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-56.(A)
 * When the cap_style is CapProjecting and line-width is zero,
 * then this is equivalent to CapButt.
 */
static void
t056()
{
XVisualInfo	*vp;
XImage	*savimp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)0);
		setcapstyle(A_DISPLAY, A_GC, CapProjecting);

		drawline(CAP_X1, CAP_Y1, CAP_X2, CAP_Y2);
		savimp = savimage(A_DISPLAY, A_DRAWABLE);

		dclear(A_DISPLAY, A_DRAWABLE);
		setcapstyle(A_DISPLAY, A_GC, CapButt);
		drawline(CAP_X1, CAP_Y1, CAP_X2, CAP_Y2);

		if (compsavimage(A_DISPLAY, A_DRAWABLE, savimp))
			CHECK;
		else {
			report("CapProjecting not equivalent to CapButt for thin line");
			FAIL;
		}
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-57.(A)
 * When the cap_style is CapProjecting and line-width is not
 * zero, then the line is square at the end, but the path
 * continues beyond the endpoint for a distance equal to half
 * the line-width.
 */
static void
t057()
{
XVisualInfo	*vp;
struct	area	area;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setcapstyle(A_DISPLAY, A_GC, CapProjecting);

		for(lw=11; lw <= 12; lw++) { /* odd and even linewidths */
			setwidth(A_DISPLAY, A_GC, lw);
			drawline(20, 20, 70, 20);

			setarea(&area, 20-lw/2, 20-lw/2, (70-20)+lw, lw);
			if (checkarea(A_DISPLAY, A_DRAWABLE, &area, W_FG, W_BG, CHECK_ALL))
				CHECK;
			else {
				report("CapProjecting on horizontal line failed (width = %u)", lw);
				FAIL;
			}

			dclear(A_DISPLAY, A_DRAWABLE);
			drawline(CAP_X1, CAP_Y1, CAP_X2, CAP_Y2);

			PIXCHECK(A_DISPLAY, A_DRAWABLE);

			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}

	CHECKPASS(4*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-58.(A)
 * When a line has coincident endpoints (x1=x2, y1=y2), and
 * the cap_style is applied to both endpoints and the
 * line_width is equal to zero and the cap_style is CapNotLast,
 * then the results are device-dependent, but the desired
 * effect is that nothing is drawn.
 */
static void
t058()
{
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setcapstyle(A_DISPLAY, A_GC, CapNotLast);

		setwidth(A_DISPLAY, A_GC, (unsigned)0);
		drawline(20, 20, 20, 20);

		if (checkarea(A_DISPLAY, A_DRAWABLE, (struct area *)0, W_BG, W_BG,
				CHECK_DIFFER|CHECK_ALL)) {

			trace("zero width, length line - Nothing was drawn (desired effect)");
		} else {
			trace("zero width, length line - Something was drawn (not the desired effect)");
		}

		CHECK;
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-59.(A)
 * When a line has coincident endpoints (x1=x2, y1=y2), and
 * the cap_style is applied to both endpoints and the
 * line_width is equal to zero and the cap_style is CapRound,
 * then the results are the same as for CapButt with line_width
 * equal to zero.
 */
static void
t059()
{
XVisualInfo	*vp;
XImage	*savimp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)0);
		setcapstyle(A_DISPLAY, A_GC, CapRound);

		drawline(15, 15, 15, 15);
		savimp = savimage(A_DISPLAY, A_DRAWABLE);

		dclear(A_DISPLAY, A_DRAWABLE);
		setcapstyle(A_DISPLAY, A_GC, CapButt);
		drawline(15, 15, 15, 15);

		if (compsavimage(A_DISPLAY, A_DRAWABLE, savimp))
			CHECK;
		else {
			report("CapRound not equivalent to CapButt for zero length thin line");
			FAIL;
		}

	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-60.(A)
 * When a line has coincident endpoints (x1=x2, y1=y2), and
 * the cap_style is applied to both endpoints and the
 * line_width is equal to zero and the cap_style is
 * CapProjecting, then the results are the same as for CapButt
 * with line_width equal to zero.
 */
static void
t060()
{
XVisualInfo	*vp;
XImage	*savimp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)0);
		setcapstyle(A_DISPLAY, A_GC, CapProjecting);

		drawline(15, 15, 15, 15);
		savimp = savimage(A_DISPLAY, A_DRAWABLE);

		dclear(A_DISPLAY, A_DRAWABLE);
		setcapstyle(A_DISPLAY, A_GC, CapButt);
		drawline(15, 15, 15, 15);

		if (compsavimage(A_DISPLAY, A_DRAWABLE, savimp))
			CHECK;
		else {
			report("CapProjecting not equivalent to CapButt for zero length thin line");
			FAIL;
		}

	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-61.(A)
 * When a line has coincident endpoints (x1=x2, y1=y2), and
 * the cap_style is applied to both endpoints and the
 * line_width is not equal to zero and the cap_style is CapButt,
 * then nothing is drawn.
 */
static void
t061()
{
XVisualInfo	*vp;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		for(lw=10; lw <= 11; lw++) { /* odd and even widths */
			setwidth(A_DISPLAY, A_GC, lw);
			setcapstyle(A_DISPLAY, A_GC, CapButt);

			drawline(20, 20, 20, 20);

			if (checkarea(A_DISPLAY, A_DRAWABLE, (struct area *)0, W_BG, W_BG, CHECK_ALL))
				CHECK;
			else {
				report("Something was drawn with zero length line and CapButt (width = %u)", lw);
				FAIL;
			}
			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-62.(A)
 * When a line has coincident endpoints (x1=x2, y1=y2), and
 * the cap_style is applied to both endpoints and the
 * line_width is not equal to zero and the cap_style is
 * CapRound, then the closed path is a circle, centered at the
 * endpoint, and with the diameter equal to the line-width.
 */
static void
t062()
{
XVisualInfo	*vp;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		for(lw=4; lw <= 5; lw++) { /* odd and even line widths */
			setwidth(A_DISPLAY, A_GC, lw);
			setcapstyle(A_DISPLAY, A_GC, CapRound);

			drawline(20, 20, 20, 20);

			PIXCHECK(A_DISPLAY, A_DRAWABLE);

			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-63.(A)
 * When a line has coincident endpoints (x1=x2, y1=y2), and
 * the cap_style is applied to both endpoints and the
 * line_width is not equal to zero and the cap_style is
 * CapProjecting, then the closed path is a square, aligned
 * with the coordinate axes, centered at the endpoint, and with
 * the sides equal to the line-width.
 */
static void
t063()
{
XVisualInfo	*vp;
struct	area	area;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		for(lw=21; lw <= 22; lw++) { /* odd and even line widths */
			setwidth(A_DISPLAY, A_GC, lw);
			setcapstyle(A_DISPLAY, A_GC, CapProjecting);
			drawline(40, 40, 40, 40);

			setarea(&area, 40-lw/2, 40-lw/2, lw, lw);
			if (checkarea(A_DISPLAY, A_DRAWABLE, &area, W_FG, W_BG, CHECK_ALL))
				CHECK;
			else {
				report("CapProjecting with zero length line did not draw a square (width = %u)", lw);
				FAIL;
			}
			dclear(A_DISPLAY, A_DRAWABLE);
		}
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-64.(-)
 * When a cap_style is applied at an end point, then the phase
 * of the dash pattern is reset to the dash_offset.
 */
static void
t064()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* End of included file gc/cap-style.mc */

/* Including from file gc/fill-styl.mc */
/*
 * Assertion XDrawLine-65.(A)
 * When fill_style is FillSolid, then on a call to XDrawLine
 * the source pixel for the drawing operation is foreground.
 */
static void
t065()
{
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		XSetFillStyle(A_DISPLAY, A_GC, FillSolid);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(A_DISPLAY, A_DRAWABLE);
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-66.(A)
 * When fill-style is FillTiled, then on a call to XDrawLine
 * the source for the drawing operation is tile.
 */
static void
t066()
{
XVisualInfo	*vp;
XImage	*fssav;
Pixmap	fstile, fstile2;
unsigned long	fsfg, fsbg;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)8);

		/* 1 */
		fsfg = W_FG;
		fsbg = W_BG;
		if (vp->depth > 1) {
			fsfg ^= 0x2;
			fsbg ^= 0x2;
		}
		fstile = XCreatePixmapFromBitmapData(A_DISPLAY, A_DRAWABLE
			, (char*)tile_bits, tile_width, tile_height
			, fsfg, fsbg
			, vp->depth
			);
		XSetTile(A_DISPLAY, A_GC, fstile);

		XSetFillStyle(A_DISPLAY, A_GC, FillTiled);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		/* 2 */
		dclear(A_DISPLAY, A_DRAWABLE);

		XSetFillStyle(A_DISPLAY, A_GC, FillSolid);
		XSetForeground(A_DISPLAY, A_GC, fsfg^fsbg);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		fssav = savimage(A_DISPLAY, A_DRAWABLE);

		/*
		 * Draw with foreground and background as before.
		 */
		dclear(A_DISPLAY, A_DRAWABLE);
		XSetFillStyle(A_DISPLAY, A_GC, FillTiled);
		XSetTile(A_DISPLAY, A_GC, fstile);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		/*
		 * Draw with background and foreground reversed.
		 * xor it in.
		 */
		fstile2 = XCreatePixmapFromBitmapData(A_DISPLAY, A_DRAWABLE
			, (char*)tile_bits, tile_width, tile_height
			, fsbg, fsfg
			, vp->depth
			);
		XSetFillStyle(A_DISPLAY, A_GC, FillTiled);
		XSetTile(A_DISPLAY, A_GC, fstile2);
		XSetFunction(A_DISPLAY, A_GC, GXxor);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (compsavimage(A_DISPLAY, A_DRAWABLE, fssav))
			CHECK;
		else {
			report("Reversing tile fg and bg did not draw solid line");
			FAIL;
		}
		XFreePixmap(A_DISPLAY, fstile);
		XFreePixmap(A_DISPLAY, fstile2);
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-67.(A)
 * When fill_style is FillOpaqueStippled, then on a call to
 * XDrawLine the source for the drawing operation is a tile
 * with the same width and height as stipple, but with
 * background everywhere stipple has a zero and with foreground
 * everywhere stipple has a one.
 */
static void
t067()
{
XVisualInfo	*vp;
XImage	*fssav;
Pixmap	fsstip;
unsigned long	fsfg, fsbg;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)8);

		/* 1 */
		fsfg = W_FG;
		fsbg = W_BG;
		if (vp->depth > 1) {
			fsfg ^= 0x2;
			fsbg ^= 0x2;
			XSetForeground(A_DISPLAY, A_GC, fsfg);
			XSetBackground(A_DISPLAY, A_GC, fsbg);
		}
		fsstip = XCreateBitmapFromData(A_DISPLAY, A_DRAWABLE
			, (char*)stipple_bits, stipple_width, stipple_height
			);
		XSetStipple(A_DISPLAY, A_GC, fsstip);

		XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		/* 2 */
		dclear(A_DISPLAY, A_DRAWABLE);

		XSetFillStyle(A_DISPLAY, A_GC, FillSolid);
		XSetForeground(A_DISPLAY, A_GC, fsfg^fsbg);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		fssav = savimage(A_DISPLAY, A_DRAWABLE);
		XSetForeground(A_DISPLAY, A_GC, fsfg);

		/*
		 * Draw with foreground and background as before.
		 */
		dclear(A_DISPLAY, A_DRAWABLE);
		XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
		XSetStipple(A_DISPLAY, A_GC, fsstip);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		/*
		 * Draw with background and foreground reversed.
		 * xor it in.
		 */
		XSetForeground(A_DISPLAY, A_GC, fsbg);
		XSetBackground(A_DISPLAY, A_GC, fsfg);
		XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
		XSetStipple(A_DISPLAY, A_GC, fsstip);
		XSetFunction(A_DISPLAY, A_GC, GXxor);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (compsavimage(A_DISPLAY, A_DRAWABLE, fssav))
			CHECK;
		else {
			report("Reversing fg and bg did not draw solid line");
			FAIL;
		}
		XFreePixmap(A_DISPLAY, fsstip);
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-68.(A)
 * When fill_style is FillStippled, then on a call to
 * XDrawLine the source for the drawing operation is foreground
 * masked by the stipple pattern tiled in a single plane.
 */
static void
t068()
{
XVisualInfo	*vp;
Pixmap	fsstip;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setwidth(A_DISPLAY, A_GC, (unsigned)8);

		/* 1 */
		fsstip = XCreateBitmapFromData(A_DISPLAY, A_DRAWABLE
			, (char*)stipple_bits, stipple_width, stipple_height
			);
		XSetStipple(A_DISPLAY, A_GC, fsstip);
		XSetFillStyle(A_DISPLAY, A_GC, FillStippled);
		/* To show it has no effect */
		XSetBackground(A_DISPLAY, A_GC, W_FG);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		XFreePixmap(A_DISPLAY, fsstip);

	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/fill-styl.mc */

/* Including from file gc/subwindow.mc */
/*
 * Assertion XDrawLine-69.(A)
 * When subwindow_mode is ClipByChildren, then the
 * destination window and source (if it is a window) are
 * clipped by all viewable InputOutput children.
 */
static void
t069()
{
XVisualInfo	*vp;
Window	subwins[10];
unsigned int 	swmwidth, swmheight;
struct	area	area;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		XSetSubwindowMode(A_DISPLAY, A_GC, ClipByChildren);

		/*
		 * Place subwindows in strips across the window
		 * leaving spaces between them.
		 */
		getsize(A_DISPLAY, A_DRAW, &swmwidth, &swmheight);
		swmwidth /= 2*NELEM(subwins);

		for (i = 0; i < NELEM(subwins); i++) {
			setarea(&area, 2*i*swmwidth, 0, swmwidth, 0);
			subwins[i] = crechild(A_DISPLAY, A_DRAW, &area);
		}

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		for (i = 0; i < NELEM(subwins); i++) {
			if (checkclear(A_DISPLAY, subwins[i]))
				CHECK;
			else {
				report("Area of subwindow was not clear");
				FAIL;
			}
		}
	}

	CHECKPASS(NELEM(subwins)*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-70.(A)
 * When subwindow_mode is IncludeInferiors, then neither the
 * destination window nor source (if it is a window) is clipped
 * by inferiors.
 */
static void
t070()
{
XVisualInfo	*vp;
Window	subwins[5];
XImage	*swmsav;
unsigned int 	swmwidth, swmheight;
struct	area	area;
int 	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		swmsav = savimage(A_DISPLAY, A_DRAW);
		dclear(A_DISPLAY, A_DRAW);

		XSetSubwindowMode(A_DISPLAY, A_GC, IncludeInferiors);

		/*
		 * Place subwindows in strips across the window
		 * leaving spaces between them.
		 */
		getsize(A_DISPLAY, A_DRAW, &swmwidth, &swmheight);
		swmwidth /= 2*NELEM(subwins);

		for (i = 0; i < NELEM(subwins); i++) {
			setarea(&area, 2*i*swmwidth, 0, swmwidth, 0);
			subwins[i] = crechild(A_DISPLAY, A_DRAW, &area);
			for (j = 0; j < swmheight; j += 10) {
				setarea(&area, 0, j, swmwidth, 6);
				(void) crechild(A_DISPLAY, subwins[i], &area);
			}
		}

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (compsavimage(A_DISPLAY, A_DRAW, swmsav))
			CHECK;
		else {
			report("Inferiors affected result with IncludeInferiors");
			FAIL;
		}

		/*
		 * As a special test due to its importance, test that drawing
		 * on to the root window has the same effect.  Move the test
		 * window to the root window origin.
		 */
		{
		Drawable savdraw;

			dclear(A_DISPLAY, A_DRAW);
			XSetWindowBorderWidth(A_DISPLAY, A_DRAW, 0);
			XMoveWindow(A_DISPLAY, A_DRAW, 0, 0);

			savdraw = A_DRAW;
			A_DRAW = DRW(A_DISPLAY);

			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			A_DRAW = savdraw;

			if (compsavimage(A_DISPLAY, A_DRAW, swmsav))
				CHECK;
			else {
				report("Drawing on root window with IncludeInferiors gave incorrect results");
				FAIL;
			}
		}
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-71.(A)
 * When subwindow_mode is IncludeInferiors on a window of one
 * depth with mapped inferiors of differing depth, then no
 * errors occur.
 */
static void
t071()
{
XVisualInfo	*vp1, *vp2;
XImage	*swmsav;
int 	found;
XSetWindowAttributes w_attr;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vp2 = NULL;
	found = 0;

	for (resetvinf(VI_WIN); nextvinf(&vp1); ) {
		if (vp2 == NULL)
			vp2 = vp1;
		if (vp1->depth != vp2->depth) {
			found++;
			break;
		}
	}

	if (!found) {
		report("Only one depth visual supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	A_DRAW = makewin(A_DISPLAY, vp1);
	A_GC = makegc(A_DISPLAY, A_DRAW);

	startcall(display);
	if (isdeleted())
		return;
	XDrawLine(display, d, gc, x1, y1, x2, y2);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	swmsav = savimage(A_DISPLAY, A_DRAW);
	dclear(A_DISPLAY, A_DRAW);

        /*
         * Must explicitly set the colormap and border_pixel parameters
         * to ensure that a BadMatch error doesn't get generated, since
         * it is entire possible that the depth differs from that of the
         * parent window.
         */
        w_attr.colormap = makecolmap(A_DISPLAY, vp2->visual, AllocNone);
        w_attr.border_pixel = 0;
	XCreateWindow(A_DISPLAY
		, A_DRAW
		, 0
		, 0
		, 300
		, 300
		, 0
		, vp2->depth
		, InputOutput
		, vp2->visual
		, CWColormap | CWBorderPixel 
		, &w_attr
		);
	XSetWindowBackground(A_DISPLAY, A_DRAW, W_BG);
	XMapWindow(A_DISPLAY, A_DRAW);

	/* Wait for Expose? */
	/* No.... we're not a top level window so no wmgr interference */
		
	XSetSubwindowMode(A_DISPLAY, A_GC, IncludeInferiors);

	startcall(display);
	if (isdeleted())
		return;
	XDrawLine(display, d, gc, x1, y1, x2, y2);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (compsavimage(A_DISPLAY, A_DRAW, swmsav))
		trace("Drawing through to different depth window produces same results");
	else
		trace("Drawing through to different depth window produces different results");
		
	if (geterr() == Success)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/subwindow.mc */

/* Including from file gc/clip-x-or.mc */
/*
 * Assertion XDrawLine-72.(A)
 * The clip origin coordinates clip-x-origin and clip-y-origin
 * are interpreted relative to the origin of the destination
 * drawable specified in the graphics operation.
 */
static void
t072()
{
XVisualInfo	*vp;
Pixmap	cmopixmap;
struct	area	area;
unsigned int 	cmowidth;
unsigned int 	cmoheight;
int 	divsize;
int 	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	divsize = 3;

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		/*
		 * Get size of the pixmap.  It is divsize smaller on all sides
		 * than the window.
		 */
		getsize(A_DISPLAY, A_DRAW, &cmowidth, &cmoheight);
		cmowidth /= divsize;
		cmoheight /= divsize;
		debug(2, "clip-mask height=%d, width=%d", cmowidth, cmoheight);

		/*
		 * Create a pixmap that is about divsize^2 of the area of
		 * the window.
		 */
		cmopixmap = XCreatePixmap(A_DISPLAY, A_DRAW, cmowidth, cmoheight, 1);
		dset(A_DISPLAY, cmopixmap, 1L);

		XSetClipMask(A_DISPLAY, A_GC, cmopixmap);
		XFreePixmap(A_DISPLAY, cmopixmap);

		for (i = 0; i < divsize; i++) {
			for (j = 0; j < divsize; j++) {
				setarea(&area, i*cmowidth, j*cmoheight, cmowidth, cmoheight);
				debug(2, "Origin at (%d,%d)", area.x, area.y);
				XSetClipOrigin(A_DISPLAY, A_GC, area.x, area.y);
				startcall(display);
				if (isdeleted())
					return;
				XDrawLine(display, d, gc, x1, y1, x2, y2);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				if (checkarea(A_DISPLAY,A_DRAW,&area,W_BG,W_BG,CHECK_OUT))
					CHECK;
				else {
					report("Drawing occurred outside clip_mask");
					FAIL;
				}
				PIXCHECK(A_DISPLAY, A_DRAW);
				dclear(A_DISPLAY, A_DRAW);
			}
		}
	}

	CHECKPASS(2*divsize*divsize*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/clip-x-or.mc */

/* Including from file gc/clip-y-or.mc */
/* End of included file gc/clip-y-or.mc */

/* Including from file gc/clip-mask.mc */
/*
 * Assertion XDrawLine-73.(A)
 * When the clip_mask is set to None, then the pixels are
 * always drawn regardless of the clip origin.
 */
static void
t073()
{
XVisualInfo	*vp;
XImage	*cmsav;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		cmsav = savimage(A_DISPLAY, A_DRAW);
		dclear(A_DISPLAY, A_DRAW);

		/*
		 * Not very interesting, because this should be the default
		 * anyway.
		 */
		XSetClipMask(A_DISPLAY, A_GC, None);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (compsavimage(A_DISPLAY, A_DRAW, cmsav))
			CHECK;
		else {
			report("Clip mask of None changed graphics");
			FAIL;
		}
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-74.(A)
 * When pixels are outside the area covered by the clip_mask,
 * then they are not drawn.
 */
static void
t074()
{
XVisualInfo	*vp;
Pixmap	cmpm;
struct	area	area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		setarea(&area, 0, 0, 5, 5);
		cmpm = XCreatePixmap(A_DISPLAY, A_DRAW, area.width, area.height, 1);
		dset(A_DISPLAY, cmpm, 1L);
		XSetClipMask(A_DISPLAY, A_GC, cmpm);
		XFreePixmap(A_DISPLAY, cmpm);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (checkarea(A_DISPLAY, A_DRAW, &area, 0L, W_BG, CHECK_OUT))
			CHECK;
		else {
			report("Drawing outside clip mask");
			FAIL;
		}
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-75.(A)
 * When pixels have a clip_mask bit set to 1, and they would
 * be drawn, then they are drawn.
 */
static void
t075()
{
#define stipple_width 24
#define stipple_height 11
static unsigned char stipple_bits[] = {
   0xff, 0x0f, 0x00, 0x3f, 0xf0, 0xff, 0xcf, 0xff, 0x03, 0xf7, 0x0f, 0xfc,
   0xff, 0xf1, 0x0f, 0x78, 0x7e, 0xf0, 0x80, 0x8f, 0x1f, 0x2a, 0xf0, 0xe1,
   0x80, 0x07, 0x1e, 0xaa, 0xff, 0xe0, 0x80, 0xff, 0x0f};
XVisualInfo	*vp;
Pixmap	cmpixmap;
Pixmap	cmstip;
XImage	*cmsav;
GC		cmgc;
unsigned int 	cmwidth, cmheight;
int 	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		cmsav = savimage(A_DISPLAY, A_DRAW);
		if (isdeleted())
			return;
		/* Must clear the drawable - otherwise there is no way 
		of proving clip mask of all 1's allowed drawing */
		dclear(A_DISPLAY, A_DRAW);

		getsize(A_DISPLAY, A_DRAW, &cmwidth, &cmheight);
		cmpixmap = XCreatePixmap(A_DISPLAY, A_DRAW, cmwidth, cmheight, 1);
		if (isdeleted())
			return;
		dset(A_DISPLAY, cmpixmap, 1L);

		XSetClipMask(A_DISPLAY, A_GC, cmpixmap);

		trace("Clip mask of all 1's");
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (compsavimage(A_DISPLAY, A_DRAW, cmsav))
			CHECK;
		else {
			report("fail on clip_mask of all 1's");
			FAIL;
		}
		dclear(A_DISPLAY, A_DRAW);

		cmstip = XCreateBitmapFromData(A_DISPLAY, A_DRAW, (char*)stipple_bits,
			stipple_width, stipple_height);
		cmgc = makegc(A_DISPLAY, cmstip);
		if (isdeleted())
			return;

		for (i = 0; i < cmwidth; i += stipple_width) {
			for (j = 0; j < cmheight; j += stipple_height) {
				XCopyArea(A_DISPLAY, cmstip, cmpixmap, cmgc, 0, 0,
					stipple_width, stipple_height, i, j);
			}
		}
		XSetClipMask(A_DISPLAY, A_GC, cmpixmap);

		trace("Clip mask with pattern");
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(A_DISPLAY, A_DRAW);

		XFreePixmap(A_DISPLAY, cmpixmap);
		XFreePixmap(A_DISPLAY, cmstip);
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-76.(A)
 * When pixels have a clip_mask bit set to 0, then they are
 * not drawn.
 */
static void
t076()
{
XVisualInfo	*vp;
Pixmap	cmpixmap;
unsigned int 	cmwidth, cmheight;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		getsize(A_DISPLAY, A_DRAW, &cmwidth, &cmheight);
		cmpixmap = XCreatePixmap(A_DISPLAY, A_DRAW, cmwidth, cmheight, 1);
		if (isdeleted())
			return;
		dset(A_DISPLAY, cmpixmap, 0L);

		XSetClipMask(A_DISPLAY, A_GC, cmpixmap);

		trace("Clip mask of all 0's");
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (checkclear(A_DISPLAY, A_DRAW))
			CHECK;
		else {
			report("Pixels were drawn with clip_mask of all zeros");
			FAIL;
		}

		XFreePixmap(A_DISPLAY, cmpixmap);
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/clip-mask.mc */

/* Including from file gc/foregroun.mc */
/*
 * Assertion XDrawLine-77.(A)
 * The foregound component of the gc is used to determine the
 * foreground pixel value.
 */
static void
t077()
{
static unsigned int fglist[] = {
	0, 0x1, 0x2, 0x3, 0x4, 0x6, 0x7, 0x10, 0x33, 0x81,
	0xa3, 0xff, 256, 300, 1000,
	0x111111, 0x400200, 0x777777,
	};
XVisualInfo	*vp;
long	fgind;
unsigned long	fg;
int 	nchecks = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		for (fgind = 0; fgind < sizeof(fglist)/sizeof(int); fgind++) {

			fg = fglist[fgind];
			if (fg > DEPTHMASK(vp->depth))
				break;

			nchecks++;
			trace("Testing fg pixel of %d", fg);
			XSetForeground(A_DISPLAY, A_GC, fg);

			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAW);

			dclear(A_DISPLAY, A_DRAW);
		}
	}

	CHECKPASS(nchecks);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-78.(A)
 * The value for foreground is truncated to the depth of the
 * GC.
 */
static void
t078()
{
XVisualInfo	*vp;
XImage	*imp;
unsigned long	pix;
unsigned long	fgfg;
int 	found;
int 	i_x, i_y;
unsigned int 	i_width, i_height;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		XSetForeground(A_DISPLAY, A_GC, ~0L);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		imp = savimage(A_DISPLAY, A_DRAW);
		getsize(A_DISPLAY, A_DRAW, &i_width, &i_height);
		found = 0;
		/*
		 * Look for a non background pixel, if there is one then this
		 * should be the foreground truncated to the depth.
		 */
		for (i_y = 0; i_y < i_height; i_y++) {
			for (i_x = 0; i_x < i_width; i_x++) {
				pix = XGetPixel(imp, i_x, i_y);
				if (pix != W_BG) {
					found = 1;
					if (pix == DEPTHMASK(vp->depth)) {
						CHECK;
					} else {
						report("Pixel was set to %d, expecting %d", pix, DEPTHMASK(vp->depth));
						FAIL;
					}
					i_y = i_height;	/* So that we leave loop XXX */
					break;
				}
			}
		}
		if (found == 0) {
			if (W_BG == (~0 & DEPTHMASK(vp->depth))) {
				report("WARNING: W_BG != 0; this should not be the case");
				CHECK;
			} else {
				report("After setting foreground to (~0)");
				report("no non-background pixel was drawn");
				FAIL;
			}
		}

		dclear(A_DISPLAY, A_DRAW);

		/* Set all bits other than the bottom one */
		fgfg = (~0)^1;
		XSetForeground(A_DISPLAY, A_GC, fgfg);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		imp = savimage(A_DISPLAY, A_DRAW);
		getsize(A_DISPLAY, A_DRAW, &i_width, &i_height);
		found = 0;
		/*
		 * Look for a non background pixel, if there is one then this
		 * should be the foreground truncated to the depth.
		 */
		for (i_y = 0; i_y < i_height; i_y++) {
			for (i_x = 0; i_x < i_width; i_x++) {
				pix = XGetPixel(imp, i_x, i_y);
				if (pix != W_BG) {
					found = 1;
					if (pix == (fgfg & DEPTHMASK(vp->depth))) {
						CHECK;
					} else {
						report("Pixel was set to %d, expecting %d", pix, (fgfg & DEPTHMASK(vp->depth)));
						FAIL;
					}
					i_y = i_height;	/* So that we leave loop XXX */
					break;
				}
			}
		}
		if (found == 0) {
			/*
			 * This should be the case on monochrome displays
			 */
			if (W_BG == (fgfg & DEPTHMASK(vp->depth))) {
				CHECK;
			} else {
				report("After setting foreground to (~0)^1");
				report("no non-background pixel was drawn");
				FAIL;
			}
		}
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/foregroun.mc */

/* Including from file gc/backgroun.mc */
/*
 * Assertion XDrawLine-79.(A)
 * The background component of the gc is used to determine the
 * background pixel value.
 */
static void
t079()
{
static unsigned long bglist[] = {
	0, 0x1, 0x2, 0x3, 0x4, 0x6, 0x7, 0x10, 0x33, 0x81,
	0xa3, 0xff, 256, 300, 1000,
	0x111111, 0x400200, 0x777777,
	};
XVisualInfo	*vp;
#ifdef NEED_STIPPLE
Pixmap	stip;
char	*stipbit = "\0";
#endif
long	bgind;
unsigned long	bg;
int 	nchecks = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

#ifdef NEED_STIPPLE
	stip = XCreateBitmapFromData(A_DISPLAY, DRW(A_DISPLAY), stipbit, 1, 1);
#endif

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

		for (bgind = 0; bgind < NELEM(bglist); bgind++) {

			bg = bglist[bgind];
			if (bg > DEPTHMASK(vp->depth))
				break;

			trace("Testing bg pixel of %d", bg);
			nchecks++;

#ifdef NEED_STIPPLE
			/*
			 * Set a stipple to all zero's and set the fill style
			 * to be FillOpaqueStippled.  This will mean that everything
			 * should be drawn in the background colour.
			 * (different for XCopyPlane, which doesn't use fill style,
			 * nor does XPutImage or XCopyArea).
			 */
			XSetStipple(A_DISPLAY, A_GC, stip);
			XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
#endif
/* now set the background to bg and do the call. If it's XCopyPlane/XPutImage then
 * we have cleared the src drawable/image to zero so we will always find 0 in the
 * selected plane so we will fill with bg. If we're XDrawImageString then
 * we do the equivalent of the background fill anyway,  if we're
 * XCopyArea then we have to do a dset of src with bg otherwise we'll have
 * used FillOpaqueStippled and a stipple of a lone zero, only, to get the
 * same effect of a background fill.
 */
			XSetBackground(A_DISPLAY, A_GC, bg);

			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAW);

			dclear(A_DISPLAY, A_DRAW);
		}
	}

#ifdef NEED_STIPPLE
	XFreePixmap(A_DISPLAY, stip);
#endif

	CHECKPASS(nchecks);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-80.(A)
 * The value for background is truncated to the depth of the
 * GC.
 */
static void
t080()
{
XVisualInfo	*vp;
XImage	*imp;
#ifdef NEED_STIPPLE
Pixmap	stip;
char	*stipbit = "\0\0\0\0";
#endif
unsigned long	pix;
int 	found;
int 	i_x, i_y;
unsigned int 	i_width, i_height;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

#ifdef NEED_STIPPLE
	stip = XCreateBitmapFromData(A_DISPLAY, DRW(A_DISPLAY), stipbit, 1, 1);
#endif

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);

#ifdef NEED_STIPPLE
		/*
		 * Set a stipple to all zero's and set the fill style
		 * to be FillOpaqueStippled.  This will mean that everything
		 * should be drawn in the background colour.
		 */
		XSetStipple(A_DISPLAY, A_GC, stip);
		XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
#endif

/* now set the background to ~0 and do the call. If it's XCopyPlane/XPutImage then
 * we have cleared the src drawable/image to zero so we will always find 0 in the
 * selected plane so we will fill with ~0. If we're XDrawImageString then
 * we do the equivalent of the background fill anyway, if we're XCopyArea
 * then we need to fill the src will all 1's, otherwise we'll have
 * used FillOpaqueStippled and a stipple of a lone zero, only, to get the
 * same effect of a background fill.
 */
		XSetBackground(A_DISPLAY, A_GC, ~0L);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		imp = savimage(A_DISPLAY, A_DRAW);
		getsize(A_DISPLAY, A_DRAW, &i_width, &i_height);
		found = 0;
		for (i_y = 0; i_y < i_height; i_y++) {
			for (i_x = 0; i_x < i_width; i_x++) {
				pix = XGetPixel(imp, i_x, i_y);
				if (pix != W_BG) {
					found = 1;
					if (pix == DEPTHMASK(vp->depth)) {
						CHECK;
					} else {
						report("Pixel was set to %d, expecting %d", pix, DEPTHMASK(vp->depth));
#ifdef NEED_STIPPLE
						report("FillOpaqueStippled is also used in this test, and could affect the result");
#endif
						FAIL;
					}
					i_y = i_height;	/* So that we leave loop XXX */
					break;
				}
			}
		}
		if (found == 0) {
			report("Nothing was drawn");
#ifdef NEED_STIPPLE
			report("FillOpaqueStippled is also used in this test, and could affect the result");
#endif
			FAIL;
		}
	}
#ifdef NEED_STIPPLE
	XFreePixmap(A_DISPLAY, stip);
#endif

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/backgroun.mc */

/* Including from file gc/tile.mc */
/* End of included file gc/tile.mc */

/* Including from file gc/stipple.mc */
/* End of included file gc/stipple.mc */

/* Including from file gc/ts-x-orig.mc */
/*
 * Assertion XDrawLine-81.(A)
 * The tile/stipple origin coordinates ts-x-origin and
 * ts-y-origin are interpreted relative to the origin of the
 * destination drawable specified in the graphics operation.
 */
static void
t081()
{
XVisualInfo	*vp;
Pixmap	tsostip;
Pixmap	tsotile;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		tsostip = XCreateBitmapFromData(A_DISPLAY, A_DRAWABLE
			, (char*)stipple_bits, stipple_width, stipple_height
			);
		XSetStipple(A_DISPLAY, A_GC, tsostip);
		XFreePixmap(A_DISPLAY, tsostip);
		XSetFillStyle(A_DISPLAY, A_GC, FillStippled);

		XSetTSOrigin(A_DISPLAY, A_GC, 9, 17);
		dclear(A_DISPLAY, A_DRAWABLE);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		XSetTSOrigin(A_DISPLAY, A_GC, 0xd8f2, 0x4321);
		dclear(A_DISPLAY, A_DRAWABLE);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		XSetTSOrigin(A_DISPLAY, A_GC, -3, 7);
		dclear(A_DISPLAY, A_DRAWABLE);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		/* Now Tiles */
		tsotile = XCreatePixmapFromBitmapData(A_DISPLAY, A_DRAWABLE
			, (char*)tile_bits, tile_width, tile_height
			, W_FG, W_BG
			, vp->depth
			);
		XSetTile(A_DISPLAY, A_GC, tsotile);
		XFreePixmap(A_DISPLAY, tsotile);
		XSetFillStyle(A_DISPLAY, A_GC, FillTiled);

		XSetTSOrigin(A_DISPLAY, A_GC, 9, 17);
		dclear(A_DISPLAY, A_DRAWABLE);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		XSetTSOrigin(A_DISPLAY, A_GC, 0xd8f2, 0x4321);
		dclear(A_DISPLAY, A_DRAWABLE);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		XSetTSOrigin(A_DISPLAY, A_GC, -3, 7);
		dclear(A_DISPLAY, A_DRAWABLE);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);
	}

	CHECKPASS(6*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/ts-x-orig.mc */

/* Including from file gc/ts-y-orig.mc */
/* End of included file gc/ts-y-orig.mc */

/* Including from file gc/dash-offs.mc */
/*
 * Assertion XDrawLine-82.(A)
 * The dash_offset defines the phase of the pattern,
 * specifying how many pixels into the dash-list the pattern
 * begins in any single graphics request.
 */
static void
t082()
{
static	unsigned short	dashoff[] = {
	0x0, 0x2, 0x7, 0x99, 0xfe78};
XVisualInfo	*vp;
XImage	*dosav;
int 	i;
int 	combinedlen = 0;
unsigned int	lw;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * Calculate total length of list needed to check circularity.
	 */
	for (i = 0; i < NELEM(dolist); i++)
		combinedlen += dolist[i];

	if (NELEM(dolist) & 1)	/* Odd length */
		combinedlen *= 2;

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();
		setlinestyle(A_DISPLAY, A_GC, LineOnOffDash);
		for(lw=10; lw <= 11; lw++) { /* both odd and even widths */
			setwidth(A_DISPLAY, A_GC, lw);

			for (i = 0; i < NELEM(dashoff); i++) {
				trace("dash offset of %d", dashoff[i]);
				XSetDashes(A_DISPLAY, A_GC, dashoff[i], dolist, NELEM(dolist));
				
				dclear(A_DISPLAY, A_DRAWABLE);
				startcall(display);
				if (isdeleted())
					return;
				XDrawLine(display, d, gc, x1, y1, x2, y2);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}

				PIXCHECK(A_DISPLAY, A_DRAWABLE);
			}

			XSetDashes(A_DISPLAY, A_GC, 0, dolist, NELEM(dolist));
			dclear(A_DISPLAY, A_DRAWABLE);
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			dosav = savimage(A_DISPLAY, A_DRAWABLE);

			trace("dash offset of the combined length of dash list %d", 
									combinedlen);
			XSetDashes(A_DISPLAY, A_GC, combinedlen, dolist, NELEM(dolist));
			dclear(A_DISPLAY, A_DRAWABLE);
			startcall(display);
			if (isdeleted())
				return;
			XDrawLine(display, d, gc, x1, y1, x2, y2);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			if (compsavimage(A_DISPLAY, A_DRAWABLE, dosav))
				CHECK;
			else {
				report("dash offset of 0 not equivalent to sum of elements in dash list (%d) for linewidth %u", combinedlen, lw);
				FAIL;
			}

			dclear(A_DISPLAY, A_DRAWABLE);
		} /* odd and even widths */
	}

	CHECKPASS(2*(NELEM(dashoff)+1)*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/dash-offs.mc */

/* Including from file gc/dash-list.mc */
/*
 * Assertion XDrawLine-83.(A)
 * The initial and alternate elements of the dash-list within
 * the GC specify the lengths of the even dashes and the second
 * and alternate elements specify the lengths of the odd
 * dashes.
 */
static void
t083()
{
XVisualInfo	*vp;
static unsigned char	longdash[] = {
	255};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();
		setlinestyle(A_DISPLAY, A_GC, LineOnOffDash);

		trace("various length dashes, %d in list", NELEM(dashlist));
		XSetDashes(A_DISPLAY, A_GC, 0, dashlist, NELEM(dashlist));
		dclear(A_DISPLAY, A_DRAWABLE);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		trace("various length dashes, %d in list", NELEM(dashlist)-1);
		XSetDashes(A_DISPLAY, A_GC, 0, dashlist, NELEM(dashlist)-1);
		dclear(A_DISPLAY, A_DRAWABLE);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);

		trace("dash length of 255");
		XSetDashes(A_DISPLAY, A_GC, 0, (char *)longdash, NELEM(longdash));
		dclear(A_DISPLAY, A_DRAWABLE);
		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		PIXCHECK(A_DISPLAY, A_DRAWABLE);
	}

	CHECKPASS(3*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-84.(A)
 * The dashes component of the GC specifies the length of both
 * even and odd dashes.
 */
static void
t084()
{
XVisualInfo	*vp;
XGCValues	gcv;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAWABLE = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAWABLE);

		setfordash();
		setlinestyle(A_DISPLAY, A_GC, LineOnOffDash);

		gcv.dashes = 8;
		XChangeGC(A_DISPLAY, A_GC, GCDashList, &gcv);

		startcall(display);
		if (isdeleted())
			return;
		XDrawLine(display, d, gc, x1, y1, x2, y2);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(A_DISPLAY, A_DRAWABLE);
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDrawLine-85.(-)
 * When the line is horizontal, then the length of a dash is
 * measured along the x axis.
 */
static void
t085()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XDrawLine-86.(-)
 * When the line is vertical, then the length of a dash is
 * measured along the y axis.
 */
static void
t086()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* End of included file gc/dash-list.mc */

/* Including from file error/EDra.mc */
/*
 * Assertion XDrawLine-87.(A)
 * When a drawable argument does not name a valid Drawable,
 * then a BadDrawable error occurs.
 */
/* End of included file error/EDra.mc */

/* Including from file error/EDra.mc */
static void
t087()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_DRAWABLE = (Drawable)badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XDrawLine(display, d, gc, x1, y1, x2, y2);
	endcall(display);
	if (geterr() != BadDrawable) {
		report("Got %s, Expecting BadDrawable", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadDrawable)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EDra.mc */

/* Including from file error/EGC.mc */
/*
 * Assertion XDrawLine-88.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t088()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XDrawLine(display, d, gc, x1, y1, x2, y2);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/* Including from file error/EMat1.mc */
/*
 * Assertion XDrawLine-89.(A)
 * When a drawable argument is an InputOnly window then a
 * BadMatch error occurs.
 */
/* End of included file error/EMat1.mc */

/* Including from file error/EMat1.mc */
static void
t089()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_DRAWABLE = (Drawable)iponlywin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XDrawLine(display, d, gc, x1, y1, x2, y2);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat1.mc */

/* Including from file error/EMat2.mc */
/*
 * Assertion XDrawLine-90.(A)
 * When the graphics context and the drawable do not have the
 * same depth, then a BadMatch error occurs.
 */
/* End of included file error/EMat2.mc */

/* Including from file error/EMat2.mc */
static void
t090()
{
XVisualInfo	*vp;
Drawable	errpm;
int 	founddepth = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_PIX); nextvinf(&vp); ) {
		if (vp->depth != 1) {
			founddepth++;

			errpm = makewin(A_DISPLAY, vp);
			A_GC = makegc(A_DISPLAY, errpm);

			break;
		}
	}

	if (!founddepth) {
		report("Only one depth supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	vp->depth = 1;
	A_DRAWABLE = makewin(A_DISPLAY, vp);

	startcall(display);
	if (isdeleted())
		return;
	XDrawLine(display, d, gc, x1, y1, x2, y2);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat2.mc */

/* Including from file error/EMat3.mc */
/*
 * Assertion XDrawLine-91.(A)
 * When the graphics context and the drawable were not created
 * for the same root, then a BadMatch error occurs.
 */
/* End of included file error/EMat3.mc */

/* Including from file error/EMat3.mc */
static void
t091()
{
XVisualInfo	vi;
Pixmap	errpm;
int 	scr_num;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

        if (config.alt_screen == -1) {
                unsupported("No alternate root supported");
                return;
        }

	scr_num = config.alt_screen;
	if (scr_num == DefaultScreen(A_DISPLAY)) {
		delete("The alternate root was the same as the one under test");
		return;
	}
	if (scr_num >= ScreenCount(A_DISPLAY)) {
		delete("The alternate root could not be accessed");
		return;
	}

	vi.visual = NULL;
	vi.screen = DefaultScreen(A_DISPLAY);	/* XXX */
	vi.depth = 1;
	A_DRAWABLE = makepixm(A_DISPLAY, &vi);

	/*
	 * Create a 1x1 depth 1 pixmap on other screen
	 * and use it to create a gc
	 */
	errpm = XCreatePixmap(A_DISPLAY, RootWindow(A_DISPLAY, scr_num), 1, 1, 1);
	A_GC = makegc(A_DISPLAY, errpm);

	startcall(display);
	if (isdeleted())
		return;
	XDrawLine(display, d, gc, x1, y1, x2, y2);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	XFreePixmap(A_DISPLAY, errpm);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat3.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	t021, 21,
	t022, 22,
	t023, 23,
	t024, 24,
	t025, 25,
	t026, 26,
	t027, 27,
	t028, 28,
	t029, 29,
	t030, 30,
	t031, 31,
	t032, 32,
	t033, 33,
	t034, 34,
	t035, 35,
	t036, 36,
	t037, 37,
	t038, 38,
	t039, 39,
	t040, 40,
	t041, 41,
	t042, 42,
	t043, 43,
	t044, 44,
	t045, 45,
	t046, 46,
	t047, 47,
	t048, 48,
	t049, 49,
	t050, 50,
	t051, 51,
	t052, 52,
	t053, 53,
	t054, 54,
	t055, 55,
	t056, 56,
	t057, 57,
	t058, 58,
	t059, 59,
	t060, 60,
	t061, 61,
	t062, 62,
	t063, 63,
	t064, 64,
	t065, 65,
	t066, 66,
	t067, 67,
	t068, 68,
	t069, 69,
	t070, 70,
	t071, 71,
	t072, 72,
	t073, 73,
	t074, 74,
	t075, 75,
	t076, 76,
	t077, 77,
	t078, 78,
	t079, 79,
	t080, 80,
	t081, 81,
	t082, 82,
	t083, 83,
	t084, 84,
	t085, 85,
	t086, 86,
	t087, 87,
	t088, 88,
	t089, 89,
	t090, 90,
	t091, 91,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
