/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: frfnt.m,v 1.9 92/06/11 17:06:16 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XFreeFont(display, font_struct)
 *   Display	*display;
 *   XFontStruct *font_struct;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XFreeFont	1
char    *TestName = "XFreeFont";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XFreeFont function
 */
static Display	*display;
static XFontStruct *font_struct;


#define FNAME	"xtfont1"
#define	TEXTSTRING "AbyZ?@"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	font_struct = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XFreeFont-1.(A)
 * When another resource references the font with ID fid in
 * the font_struct argument, then a call to XFreeFont frees all
 * storage associated with the font_struct structure and
 * destroys the association between the Font ID and the font.
 */
static void
t001()
{
Font	font;
Drawable	d;
GC		gc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	font_struct = XLoadQueryFont(display, FNAME);
	if (isdeleted() || font_struct == NULL) {
		delete("Failed to load %s, check that xtest fonts are installed",FNAME);
		return;
	}
	font = font_struct->fid;

	startcall(display);
	if (isdeleted())
		return;
	XFreeFont(display, font_struct);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* Now try to use the font, should get BadFont */
	d = defdraw(display, VI_WIN_PIX);
	gc = makegc(display, d);

	if (isdeleted())
		return;

	/*
	 * Since gc's can be cached then error can occur anytime between the
	 * XSetFont and the XSync
	 */
	reseterr();
	XSetErrorHandler(error_status);
	XSetFont(display, gc, font);
	XDrawString(display, d, gc, 30, 30, TEXTSTRING, strlen(TEXTSTRING));
	XSync(display, False);
	XSetErrorHandler(unexp_err);

	if (geterr() == BadFont) {
		PASS;
	} else {
		report("Association between Font ID and font was not destroyed");
		FAIL;
	}

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeFont-2.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When no other resource ID references the font with ID fid
 * in the font_struct argument, then a call to XFreeFont frees
 * all storage associated with the font_struct structure,
 * destroys the association between the Font ID and the font
 * and that font is unloaded.
 */
static void
t002()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/* Including from file error/EFon1.mc */
/*
 * Assertion XFreeFont-3.(A)
 * When a font argument does not name a valid font, then a
 * BadFont error occurs.
 */
/* End of included file error/EFon1.mc */

static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	font_struct = XLoadQueryFont(display, FNAME);
	if (isdeleted() || font_struct == NULL) {
		delete("Failed to load %s, check that xtest fonts are installed",FNAME);
		return;
	}
	XUnloadFont(display, font_struct->fid);

	startcall(display);
	if (isdeleted())
		return;
	XFreeFont(display, font_struct);
	endcall(display);
	if (geterr() != BadFont) {
		report("Got %s, Expecting BadFont", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadFont)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
