/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ptimg.m,v 1.21 92/06/11 17:08:07 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height)
 *   Display *display;
 *   Drawable d;
 *   GC gc;
 *   XImage *image;
 *   int src_x;
 *   int src_y;
 *   int dest_x;
 *   int dest_y;
 *   unsigned int width;
 *   unsigned int height;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XPutImage	1
char    *TestName = "XPutImage";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE d
#define A_GC gc


/*
 * Arguments to the XPutImage function
 */
static Display *display;
static Drawable d;
static GC gc;
static XImage *image;
static int src_x;
static int src_y;
static int dest_x;
static int dest_y;
static unsigned int width;
static unsigned int height;


static struct area area;

/*
 * doxcall() -	set globals, remember dest and size, call xname
 */
static void
doxcall(win, sx, sy, dx, dy, w, h)
Window win;
int sx, sy, dx, dy;
unsigned int w, h;
{
	d = win;
	src_x = sx;
	src_y = sy;
	dest_x = dx;
	dest_y = dy;
	width = w;
	height = h;
	/* remember area */
	area.x = dest_x;
	area.y = dest_y;
	area.width = width;
	area.height = height;
}

#define	A_IMAGE	image


#define A_DRAW	A_DRAWABLE

static	int 	f_pix_x = -1;
static	int 	f_pix_y = -1;

static void
functest(functype)
int 	functype;
{
XVisualInfo	*vp;
unsigned long	srcpix;
unsigned long	destpix;
unsigned long	expected;
unsigned long	actualpix;
unsigned long	maxpix;
int 	pass = 0, fail = 0;

	trace("Function %s", gcfunctionname(functype));

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		/*
		 * If this is the first call find a pixel that is drawn
		 * that can be used for all the tests.
		 */
		if (f_pix_x == -1) {
			startcall(display);
			if (isdeleted())
				return;
			XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			setfuncpixel(A_DISPLAY, A_DRAW, &f_pix_x, &f_pix_y);
			dclear(A_DISPLAY, A_DRAW);
			if (f_pix_x == -1) {
				report("Nothing was drawn with a gc function of GXcopy");
				/*
				 * If we were actually testing GXcopy then the test has
				 * failed (in a way), so say FAIL.
				 */
				if (functype == GXcopy)
					FAIL;
				else
					delete("Setup error in functest");
				return;
			}
		}

		XSetFunction(A_DISPLAY, A_GC, functype);

		/* Try out a few more values without taking too long */
		maxpix = 3;

		for (destpix = 0; destpix <= maxpix; destpix++) {
			for (srcpix = 0; srcpix <= 1; srcpix++) {

				trace("src=%d, dest=%d", srcpix, destpix);

				/*
				 * The test on depth shouldn't be necessary, however a common
				 * bug exists where pixels are not truncated on mono screens.
				 * The bug is diagnosed in another test so causing failures here
				 * as well would be confusing.
				 */
				if (vp->depth == 1)
					dset(A_DISPLAY, A_DRAW, destpix&1);
				else
					dset(A_DISPLAY, A_DRAW, destpix);
				dsetimg(A_IMAGE, srcpix);
				startcall(display);
				if (isdeleted())
					return;
				XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}

				switch (functype) {
				case GXclear:
					expected = 0;
					break;
				case GXand:
					expected = srcpix & destpix;
					break;
				case GXandReverse:
					expected = srcpix & ~destpix;
					break;
				case GXcopy:
					expected = srcpix;
					break;
				case GXandInverted:
					expected = ~srcpix & destpix;
					break;
				case GXnoop:
					expected = destpix;
					break;
				case GXxor:
					expected = srcpix ^ destpix;
					break;
				case GXor:
					expected = srcpix | destpix;
					break;
				case GXnor:
					expected = ~srcpix & ~destpix;
					break;
				case GXequiv:
					expected = ~srcpix ^ destpix;
					break;
				case GXinvert:
					expected = ~destpix;
					break;
				case GXorReverse:
					expected = srcpix | ~destpix;
					break;
				case GXcopyInverted:
					expected = ~srcpix;
					break;
				case GXorInverted:
					expected = ~srcpix | destpix;
					break;
				case GXnand:
					expected = ~srcpix | ~destpix;
					break;
				case GXset:
					expected = ~0;
					break;
				default:
					report("Bad value given to functest");
					tet_result(TET_UNRESOLVED);
					break;
				}

				expected &= DEPTHMASK(vp->depth);
				actualpix = getpixel(A_DISPLAY, A_DRAW, f_pix_x, f_pix_y);

				debug(3, "src %d, dst %d, expect %d", srcpix, destpix,expected);

				if (actualpix == expected)
					CHECK;
				else {
					report("%s fail expected %d, got %d", gcfunctionname(functype), expected, actualpix);
					FAIL;
				}
			}
		}

	}

	CHECKPASS(2*(maxpix+1)*nvinf());
}


#define	PLANEFG	0x55555555
static	int 	planelist[] = {
	0, 0x1, 0x2, 0x23, 0xf8, 0x765, 0x3987, 0x129078, 0x23567193};



#if !defined(T_XDrawImageString) && !defined(T_XCopyPlane) && !defined(T_XPutImage) && !defined(T_XCopyArea)
#define NEED_STIPPLE
#endif



int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	d = 0;
	gc = 0;
	image = 0;
	src_x = 0;
	src_y = 0;
	dest_x = 0;
	dest_y = 0;
	width = 1;
	height = 1;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	d = ErrdefDrawable;
	gc = ErrdefGC;
}

/*
 * Assertion XPutImage-1.(A)
 * On a call to XPutImage the section of image defined by the
 * src_x, src_y, width and height is drawn on the specified
 * part of the drawable.
 */
static void
t001()
{
XVisualInfo	*vp;
Window	w;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
/* Create image in XYPixmap format. */
		/* do XYPixmap testing */
		trace("XYPixmap");
		image = makeimg(display, vp, XYPixmap);
		dsetimg(image, W_BG);
/* Create drawable. */
		w = makewin(display, vp);
		gc = makegc(display, w);
		dset(display, w, W_FG);

/* Call XPutImage. */
		doxcall(w, 0, 0, 0, 0, image->width, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

/* Verify results. */
		if (checkarea(display, w, &area, W_BG, W_FG, CHECK_ALL) == False) {
			report("Area was not properly put.");
			FAIL;
		}
		else
			CHECK;

/* Repeat for XYBitmap and ZPixmap. */
		/* do ZPixmap testing */
		trace("ZPixmap");
		image = makeimg(display, vp, ZPixmap);
		dsetimg(image, W_BG);
		w = makewin(display, vp);
		gc = makegc(display, w);
		dset(display, w, W_FG);

		doxcall(w, image->width/2, image->height/2, 0, 0, image->width/4, image->height/4);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (checkarea(display, w, &area, W_BG, W_FG, CHECK_ALL) == False) {
			report("Area was not properly put.");
			FAIL;
		}
		else
			CHECK;

		/* do XYBitmap testing */
		trace("XYBitmap");
		image = makeimg(display, vp, XYBitmap);
		dsetimg(image, 0L); /* so we'll fill with bg */
		w = makewin(display, vp);
		gc = makegc(display, w);
		dset(display, w, W_FG); /* so we can tell the difference */

		doxcall(w, 0, 0, 1, 1, image->width/2, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (checkarea(display, w, &area, W_BG, W_FG, CHECK_ALL) == False) {
			report("Area was not properly put.");
			FAIL;
		}
		else
			CHECK;
/* Repeat for each visual. */
	}

	CHECKPASS(3*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-2.(C)
 * When the image format is XYBitmap, then the foreground
 * pixel in gc defines the source for the one bits in the
 * image, and the background pixel defines the source for the
 * zero bits.
 */
static void
t002()
{
XVisualInfo	*vp;
Window	w;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {

/* Create image in XYBitmap format. */
		image = makeimg(display, vp, XYBitmap);

/* Create drawable. */
		w = makewin(display, vp);
		gc = makegc(display, w);

/* Set all bits in image to zero. */
		trace("Bits 0, drawable W_FG");
		dsetimg(image, 0L);

/* Set every pixel in drawable to W_FG. */
		dset(display, w, W_FG);

/* Call XPutImage. */
		doxcall(w, 0, 0, 0, 0, image->width, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

/* Verify results. */
		if (checkarea(display, w, &area, W_BG, W_BG, CHECK_ALL) == False) {
			report("Area was not properly put.");
			FAIL;
		}
		else
			CHECK;

/* Set every pixel in drawable to W_BG. */
		trace("Bits 0, drawable W_BG");
		dset(display, w, W_BG);

/* Call XPutImage. */
		doxcall(w, 0, 0, 0, 0, image->width, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

/* Verify results. */
		if (checkarea(display, w, &area, W_BG, W_BG, CHECK_ALL) == False) {
			report("Area was not properly put.");
			FAIL;
		}
		else
			CHECK;

/* Set all bits in image to one. */
		trace("Bits 1, drawable W_FG");
		dsetimg(image, 1);

/* Set every pixel in drawable to W_FG. */
		dset(display, w, W_FG);

/* Call XPutImage. */
		doxcall(w, 0, 0, 0, 0, image->width, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

/* Verify results. */
		if (checkarea(display, w, &area, W_FG, W_FG, CHECK_ALL) == False) {
			report("Area was not properly put.");
			FAIL;
		}
		else
			CHECK;

/* Set every pixel in drawable to W_BG. */
		trace("Bits 1, drawable W_BG");
		dset(display, w, W_BG);

/* Call XPutImage. */
		doxcall(w, 0, 0, 0, 0, image->width, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

/* Verify results. */
		if (checkarea(display, w, &area, W_FG, W_FG, CHECK_ALL) == False) {
			report("Area was not properly put.");
			FAIL;
		}
		else
			CHECK;
/* Repeat for each visual. */
	}

	CHECKPASS(4*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file gc/function.mc */
/*
 * Assertion XPutImage-3.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXclear, then the destination becomes (dst AND (NOT
 * plane_mask)).
 */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXclear);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-4.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXand, then the destination becomes ((src AND dst) AND
 * plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXand);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-5.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXandReverse, then the destination becomes ((src AND (NOT
 * dst)) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXandReverse);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-6.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXcopy, then the destination becomes (src AND plane_mask)
 * OR (dst AND (NOT plane_mask)).
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXcopy);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-7.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXandInverted, then the destination becomes (((NOT src) AND
 * dst) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXandInverted);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-8.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXnoop, then the destination becomes (dst AND plane_mask)
 * OR (dst AND (NOT plane_mask)).
 */
static void
t008()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXnoop);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-9.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXxor, then the destination becomes ((src XOR dst) AND
 * plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t009()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXxor);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-10.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is GXor,
 * then the destination becomes ((src OR dst) AND plane_mask)
 * OR (dst AND (NOT plane_mask)).
 */
static void
t010()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXor);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-11.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXnor, then the destination becomes (((NOT src) AND (NOT
 * dst)) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t011()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXnor);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-12.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXequiv, then the destination becomes (((NOT src) XOR dst)
 * AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t012()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXequiv);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-13.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXinvert, then the destination becomes ((NOT dst) AND
 * plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t013()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXinvert);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-14.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXorReverse, then the destination becomes ((src OR (NOT
 * dst)) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t014()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXorReverse);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-15.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXcopyInverted, then the destination becomes ((NOT src) AND
 * plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t015()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXcopyInverted);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-16.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXorInverted, then the destination becomes (((NOT src) OR
 * dst) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t016()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXorInverted);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-17.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXnand, then the destination becomes (((NOT src) OR (NOT
 * dst)) AND plane_mask) OR (dst AND (NOT plane_mask)).
 */
static void
t017()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXnand);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-18.(A)
 * When a graphics operation is performed and the source pixel
 * is src and the destination pixel is dst and function is
 * GXset, then the destination becomes ( plane_mask) OR (dst
 * AND (NOT plane_mask)).
 */
static void
t018()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	functest(GXset);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/function.mc */

/* Including from file gc/plane-mas.mc */
/*
 * Assertion XPutImage-19.(A)
 * The value for plane_mask is truncated to the depth of the
 * GC.
 */
static void
t019()
{
XVisualInfo	*vp;
int 	depthmask;
int 	*ip;
unsigned long	pix;
unsigned long	expected;
int 	pmx = -1, pmy;
int 	ntested;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	ntested = 0;
	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		if (pmx == -1) {
			/*
			 * Get a point that is set in the drawable.
			 */
			startcall(display);
			if (isdeleted())
				return;
			XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			setfuncpixel(A_DISPLAY, A_DRAW, &pmx, &pmy);
			dclear(A_DISPLAY, A_DRAW);
		}

		depthmask = DEPTHMASK(vp->depth);

		/* ZPixmap image so fg not used */
		dsetimg(A_IMAGE, PLANEFG);

		for (ip = planelist; ip < &planelist[NELEM(planelist)]; ip++) {
			if (*ip <= depthmask)
				continue;

			trace("plane-mask 0x%x", *ip);
			XSetPlaneMask(A_DISPLAY, A_GC, *ip);
			ntested++;

			dclear(A_DISPLAY, A_DRAW);
			startcall(display);
			if (isdeleted())
				return;
			XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			expected = (*ip & PLANEFG) & depthmask;
			pix = getpixel(A_DISPLAY, A_DRAW, pmx, pmy);

			if (pix == expected)
				CHECK;
			else {
				report("got pixel 0x%x, expecting 0x%x", pix, expected);
				FAIL;
			}
		}

	}

	CHECKPASS(ntested);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-20.(A)
 * The plane_mask specifies which planes of the destination
 * are to be modified, one bit per plane, with bits being
 * assigned to planes from the least significant bit of the
 * word to the most significant bit.
 */
static void
t020()
{
XVisualInfo	*vp;
int 	depthmask;
int 	*ip;
unsigned long	pix;
unsigned long	expected;
int 	pmx = -1, pmy;
int 	ntested = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		if (pmx == -1) {
			/*
			 * Get a point that is set in the drawable.
			 */
			startcall(display);
			if (isdeleted())
				return;
			XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			setfuncpixel(A_DISPLAY, A_DRAW, &pmx, &pmy);
			dclear(A_DISPLAY, A_DRAW);
		}

		depthmask = DEPTHMASK(vp->depth);

		/* ZPixmap image so fg not used */
		dsetimg(A_IMAGE, PLANEFG);
		for (ip = planelist; ip < &planelist[NELEM(planelist)]; ip++) {
			if (*ip > depthmask)
				break;

			trace("plane-mask 0x%x", *ip);
			XSetPlaneMask(A_DISPLAY, A_GC, *ip);
			ntested++;

			dclear(A_DISPLAY, A_DRAW);
			startcall(display);
			if (isdeleted())
				return;
			XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			expected = *ip & PLANEFG;
			pix = getpixel(A_DISPLAY, A_DRAW, pmx, pmy);

			if (pix == expected)
				CHECK;
			else {
				report("got pixel 0x%x, expecting 0x%x", pix, expected);
				FAIL;
			}
		}

	}

	CHECKPASS(ntested);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/plane-mas.mc */

/* Including from file gc/subwindow.mc */
/*
 * Assertion XPutImage-21.(A)
 * When subwindow_mode is ClipByChildren, then the
 * destination window and source (if it is a window) are
 * clipped by all viewable InputOutput children.
 */
static void
t021()
{
XVisualInfo	*vp;
Window	subwins[10];
unsigned int 	swmwidth, swmheight;
struct	area	area;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		XSetSubwindowMode(A_DISPLAY, A_GC, ClipByChildren);

		/*
		 * Place subwindows in strips across the window
		 * leaving spaces between them.
		 */
		getsize(A_DISPLAY, A_DRAW, &swmwidth, &swmheight);
		swmwidth /= 2*NELEM(subwins);

		for (i = 0; i < NELEM(subwins); i++) {
			setarea(&area, 2*i*swmwidth, 0, swmwidth, 0);
			subwins[i] = crechild(A_DISPLAY, A_DRAW, &area);
		}

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		for (i = 0; i < NELEM(subwins); i++) {
			if (checkclear(A_DISPLAY, subwins[i]))
				CHECK;
			else {
				report("Area of subwindow was not clear");
				FAIL;
			}
		}
	}

	CHECKPASS(NELEM(subwins)*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-22.(A)
 * When subwindow_mode is IncludeInferiors, then neither the
 * destination window nor source (if it is a window) is clipped
 * by inferiors.
 */
static void
t022()
{
XVisualInfo	*vp;
Window	subwins[5];
XImage	*swmsav;
unsigned int 	swmwidth, swmheight;
struct	area	area;
int 	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		swmsav = savimage(A_DISPLAY, A_DRAW);
		dclear(A_DISPLAY, A_DRAW);

		XSetSubwindowMode(A_DISPLAY, A_GC, IncludeInferiors);

		/*
		 * Place subwindows in strips across the window
		 * leaving spaces between them.
		 */
		getsize(A_DISPLAY, A_DRAW, &swmwidth, &swmheight);
		swmwidth /= 2*NELEM(subwins);

		for (i = 0; i < NELEM(subwins); i++) {
			setarea(&area, 2*i*swmwidth, 0, swmwidth, 0);
			subwins[i] = crechild(A_DISPLAY, A_DRAW, &area);
			for (j = 0; j < swmheight; j += 10) {
				setarea(&area, 0, j, swmwidth, 6);
				(void) crechild(A_DISPLAY, subwins[i], &area);
			}
		}

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (compsavimage(A_DISPLAY, A_DRAW, swmsav))
			CHECK;
		else {
			report("Inferiors affected result with IncludeInferiors");
			FAIL;
		}

		/*
		 * As a special test due to its importance, test that drawing
		 * on to the root window has the same effect.  Move the test
		 * window to the root window origin.
		 */
		{
		Drawable savdraw;

			dclear(A_DISPLAY, A_DRAW);
			XSetWindowBorderWidth(A_DISPLAY, A_DRAW, 0);
			XMoveWindow(A_DISPLAY, A_DRAW, 0, 0);

			savdraw = A_DRAW;
			A_DRAW = DRW(A_DISPLAY);

			startcall(display);
			if (isdeleted())
				return;
			XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			A_DRAW = savdraw;

			if (compsavimage(A_DISPLAY, A_DRAW, swmsav))
				CHECK;
			else {
				report("Drawing on root window with IncludeInferiors gave incorrect results");
				FAIL;
			}
		}
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-23.(A)
 * When subwindow_mode is IncludeInferiors on a window of one
 * depth with mapped inferiors of differing depth, then no
 * errors occur.
 */
static void
t023()
{
XVisualInfo	*vp1, *vp2;
XImage	*swmsav;
int 	found;
XSetWindowAttributes w_attr;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vp2 = NULL;
	found = 0;

	for (resetvinf(VI_WIN); nextvinf(&vp1); ) {
		if (vp2 == NULL)
			vp2 = vp1;
		if (vp1->depth != vp2->depth) {
			found++;
			break;
		}
	}

	if (!found) {
		report("Only one depth visual supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	A_DRAW = makewin(A_DISPLAY, vp1);
	A_GC = makegc(A_DISPLAY, A_DRAW);
	A_IMAGE = makeimg(A_DISPLAY, vp1, ZPixmap);
	dsetimg(A_IMAGE, W_FG);

	startcall(display);
	if (isdeleted())
		return;
	XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	swmsav = savimage(A_DISPLAY, A_DRAW);
	dclear(A_DISPLAY, A_DRAW);

        /*
         * Must explicitly set the colormap and border_pixel parameters
         * to ensure that a BadMatch error doesn't get generated, since
         * it is entire possible that the depth differs from that of the
         * parent window.
         */
        w_attr.colormap = makecolmap(A_DISPLAY, vp2->visual, AllocNone);
        w_attr.border_pixel = 0;
	XCreateWindow(A_DISPLAY
		, A_DRAW
		, 0
		, 0
		, 300
		, 300
		, 0
		, vp2->depth
		, InputOutput
		, vp2->visual
		, CWColormap | CWBorderPixel 
		, &w_attr
		);
	XSetWindowBackground(A_DISPLAY, A_DRAW, W_BG);
	XMapWindow(A_DISPLAY, A_DRAW);

	/* Wait for Expose? */
	/* No.... we're not a top level window so no wmgr interference */
		
	XSetSubwindowMode(A_DISPLAY, A_GC, IncludeInferiors);

	startcall(display);
	if (isdeleted())
		return;
	XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (compsavimage(A_DISPLAY, A_DRAW, swmsav))
		trace("Drawing through to different depth window produces same results");
	else
		trace("Drawing through to different depth window produces different results");
		
	if (geterr() == Success)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/subwindow.mc */

/* Including from file gc/clip-x-or.mc */
/*
 * Assertion XPutImage-24.(A)
 * The clip origin coordinates clip-x-origin and clip-y-origin
 * are interpreted relative to the origin of the destination
 * drawable specified in the graphics operation.
 */
static void
t024()
{
XVisualInfo	*vp;
Pixmap	cmopixmap;
struct	area	area;
unsigned int 	cmowidth;
unsigned int 	cmoheight;
int 	divsize;
int 	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	divsize = 3;

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		/*
		 * Get size of the pixmap.  It is divsize smaller on all sides
		 * than the window.
		 */
		getsize(A_DISPLAY, A_DRAW, &cmowidth, &cmoheight);
		cmowidth /= divsize;
		cmoheight /= divsize;
		debug(2, "clip-mask height=%d, width=%d", cmowidth, cmoheight);

		/*
		 * Create a pixmap that is about divsize^2 of the area of
		 * the window.
		 */
		cmopixmap = XCreatePixmap(A_DISPLAY, A_DRAW, cmowidth, cmoheight, 1);
		dset(A_DISPLAY, cmopixmap, 1L);

		XSetClipMask(A_DISPLAY, A_GC, cmopixmap);
		XFreePixmap(A_DISPLAY, cmopixmap);

		for (i = 0; i < divsize; i++) {
			for (j = 0; j < divsize; j++) {
				setarea(&area, i*cmowidth, j*cmoheight, cmowidth, cmoheight);
				debug(2, "Origin at (%d,%d)", area.x, area.y);
				XSetClipOrigin(A_DISPLAY, A_GC, area.x, area.y);
				startcall(display);
				if (isdeleted())
					return;
				XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
				endcall(display);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				if (checkarea(A_DISPLAY,A_DRAW,&area,W_BG,W_BG,CHECK_OUT))
					CHECK;
				else {
					report("Drawing occurred outside clip_mask");
					FAIL;
				}
				PIXCHECK(A_DISPLAY, A_DRAW);
				dclear(A_DISPLAY, A_DRAW);
			}
		}
	}

	CHECKPASS(2*divsize*divsize*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/clip-x-or.mc */

/* Including from file gc/clip-y-or.mc */
/* End of included file gc/clip-y-or.mc */

/* Including from file gc/clip-mask.mc */
/*
 * Assertion XPutImage-25.(A)
 * When the clip_mask is set to None, then the pixels are
 * always drawn regardless of the clip origin.
 */
static void
t025()
{
XVisualInfo	*vp;
XImage	*cmsav;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		cmsav = savimage(A_DISPLAY, A_DRAW);
		dclear(A_DISPLAY, A_DRAW);

		/*
		 * Not very interesting, because this should be the default
		 * anyway.
		 */
		XSetClipMask(A_DISPLAY, A_GC, None);

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (compsavimage(A_DISPLAY, A_DRAW, cmsav))
			CHECK;
		else {
			report("Clip mask of None changed graphics");
			FAIL;
		}
	}

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-26.(A)
 * When pixels are outside the area covered by the clip_mask,
 * then they are not drawn.
 */
static void
t026()
{
XVisualInfo	*vp;
Pixmap	cmpm;
struct	area	area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		setarea(&area, 0, 0, 5, 5);
		cmpm = XCreatePixmap(A_DISPLAY, A_DRAW, area.width, area.height, 1);
		dset(A_DISPLAY, cmpm, 1L);
		XSetClipMask(A_DISPLAY, A_GC, cmpm);
		XFreePixmap(A_DISPLAY, cmpm);

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (checkarea(A_DISPLAY, A_DRAW, &area, 0L, W_BG, CHECK_OUT))
			CHECK;
		else {
			report("Drawing outside clip mask");
			FAIL;
		}
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-27.(A)
 * When pixels have a clip_mask bit set to 1, and they would
 * be drawn, then they are drawn.
 */
static void
t027()
{
#define stipple_width 24
#define stipple_height 11
static unsigned char stipple_bits[] = {
   0xff, 0x0f, 0x00, 0x3f, 0xf0, 0xff, 0xcf, 0xff, 0x03, 0xf7, 0x0f, 0xfc,
   0xff, 0xf1, 0x0f, 0x78, 0x7e, 0xf0, 0x80, 0x8f, 0x1f, 0x2a, 0xf0, 0xe1,
   0x80, 0x07, 0x1e, 0xaa, 0xff, 0xe0, 0x80, 0xff, 0x0f};
XVisualInfo	*vp;
Pixmap	cmpixmap;
Pixmap	cmstip;
XImage	*cmsav;
GC		cmgc;
unsigned int 	cmwidth, cmheight;
int 	i, j;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		cmsav = savimage(A_DISPLAY, A_DRAW);
		if (isdeleted())
			return;
		/* Must clear the drawable - otherwise there is no way 
		of proving clip mask of all 1's allowed drawing */
		dclear(A_DISPLAY, A_DRAW);

		getsize(A_DISPLAY, A_DRAW, &cmwidth, &cmheight);
		cmpixmap = XCreatePixmap(A_DISPLAY, A_DRAW, cmwidth, cmheight, 1);
		if (isdeleted())
			return;
		dset(A_DISPLAY, cmpixmap, 1L);

		XSetClipMask(A_DISPLAY, A_GC, cmpixmap);

		trace("Clip mask of all 1's");
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (compsavimage(A_DISPLAY, A_DRAW, cmsav))
			CHECK;
		else {
			report("fail on clip_mask of all 1's");
			FAIL;
		}
		dclear(A_DISPLAY, A_DRAW);

		cmstip = XCreateBitmapFromData(A_DISPLAY, A_DRAW, (char*)stipple_bits,
			stipple_width, stipple_height);
		cmgc = makegc(A_DISPLAY, cmstip);
		if (isdeleted())
			return;

		for (i = 0; i < cmwidth; i += stipple_width) {
			for (j = 0; j < cmheight; j += stipple_height) {
				XCopyArea(A_DISPLAY, cmstip, cmpixmap, cmgc, 0, 0,
					stipple_width, stipple_height, i, j);
			}
		}
		XSetClipMask(A_DISPLAY, A_GC, cmpixmap);

		trace("Clip mask with pattern");
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		PIXCHECK(A_DISPLAY, A_DRAW);

		XFreePixmap(A_DISPLAY, cmpixmap);
		XFreePixmap(A_DISPLAY, cmstip);
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-28.(A)
 * When pixels have a clip_mask bit set to 0, then they are
 * not drawn.
 */
static void
t028()
{
XVisualInfo	*vp;
Pixmap	cmpixmap;
unsigned int 	cmwidth, cmheight;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
		dsetimg(A_IMAGE, W_FG);

		getsize(A_DISPLAY, A_DRAW, &cmwidth, &cmheight);
		cmpixmap = XCreatePixmap(A_DISPLAY, A_DRAW, cmwidth, cmheight, 1);
		if (isdeleted())
			return;
		dset(A_DISPLAY, cmpixmap, 0L);

		XSetClipMask(A_DISPLAY, A_GC, cmpixmap);

		trace("Clip mask of all 0's");
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (checkclear(A_DISPLAY, A_DRAW))
			CHECK;
		else {
			report("Pixels were drawn with clip_mask of all zeros");
			FAIL;
		}

		XFreePixmap(A_DISPLAY, cmpixmap);
	}

	CHECKPASS(nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/clip-mask.mc */

/* Including from file gc/foregroun.mc */
/*
 * Assertion XPutImage-29.(A)
 * The foregound component of the gc is used to determine the
 * foreground pixel value.
 */
static void
t029()
{
static unsigned int fglist[] = {
	0, 0x1, 0x2, 0x3, 0x4, 0x6, 0x7, 0x10, 0x33, 0x81,
	0xa3, 0xff, 256, 300, 1000,
	0x111111, 0x400200, 0x777777,
	};
XVisualInfo	*vp;
long	fgind;
unsigned long	fg;
int 	nchecks = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		/* XYBitmap full of 1's will do a foreground fill */
		A_IMAGE = makeimg(A_DISPLAY, vp, XYBitmap);
		dsetimg(A_IMAGE, 1L);

		for (fgind = 0; fgind < sizeof(fglist)/sizeof(int); fgind++) {

			fg = fglist[fgind];
			if (fg > DEPTHMASK(vp->depth))
				break;

			nchecks++;
			trace("Testing fg pixel of %d", fg);
			XSetForeground(A_DISPLAY, A_GC, fg);

			startcall(display);
			if (isdeleted())
				return;
			XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAW);

			dclear(A_DISPLAY, A_DRAW);
		}
	}

	CHECKPASS(nchecks);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-30.(A)
 * The value for foreground is truncated to the depth of the
 * GC.
 */
static void
t030()
{
XVisualInfo	*vp;
XImage	*imp;
unsigned long	pix;
unsigned long	fgfg;
int 	found;
int 	i_x, i_y;
unsigned int 	i_width, i_height;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		/* XYBitmap full of 1's will do a foreground fill */
		A_IMAGE = makeimg(A_DISPLAY, vp, XYBitmap);
		dsetimg(A_IMAGE, 1L);

		XSetForeground(A_DISPLAY, A_GC, ~0L);

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		imp = savimage(A_DISPLAY, A_DRAW);
		getsize(A_DISPLAY, A_DRAW, &i_width, &i_height);
		found = 0;
		/*
		 * Look for a non background pixel, if there is one then this
		 * should be the foreground truncated to the depth.
		 */
		for (i_y = 0; i_y < i_height; i_y++) {
			for (i_x = 0; i_x < i_width; i_x++) {
				pix = XGetPixel(imp, i_x, i_y);
				if (pix != W_BG) {
					found = 1;
					if (pix == DEPTHMASK(vp->depth)) {
						CHECK;
					} else {
						report("Pixel was set to %d, expecting %d", pix, DEPTHMASK(vp->depth));
						FAIL;
					}
					i_y = i_height;	/* So that we leave loop XXX */
					break;
				}
			}
		}
		if (found == 0) {
			if (W_BG == (~0 & DEPTHMASK(vp->depth))) {
				report("WARNING: W_BG != 0; this should not be the case");
				CHECK;
			} else {
				report("After setting foreground to (~0)");
				report("no non-background pixel was drawn");
				FAIL;
			}
		}

		dclear(A_DISPLAY, A_DRAW);

		/* Set all bits other than the bottom one */
		fgfg = (~0)^1;
		XSetForeground(A_DISPLAY, A_GC, fgfg);

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		imp = savimage(A_DISPLAY, A_DRAW);
		getsize(A_DISPLAY, A_DRAW, &i_width, &i_height);
		found = 0;
		/*
		 * Look for a non background pixel, if there is one then this
		 * should be the foreground truncated to the depth.
		 */
		for (i_y = 0; i_y < i_height; i_y++) {
			for (i_x = 0; i_x < i_width; i_x++) {
				pix = XGetPixel(imp, i_x, i_y);
				if (pix != W_BG) {
					found = 1;
					if (pix == (fgfg & DEPTHMASK(vp->depth))) {
						CHECK;
					} else {
						report("Pixel was set to %d, expecting %d", pix, (fgfg & DEPTHMASK(vp->depth)));
						FAIL;
					}
					i_y = i_height;	/* So that we leave loop XXX */
					break;
				}
			}
		}
		if (found == 0) {
			/*
			 * This should be the case on monochrome displays
			 */
			if (W_BG == (fgfg & DEPTHMASK(vp->depth))) {
				CHECK;
			} else {
				report("After setting foreground to (~0)^1");
				report("no non-background pixel was drawn");
				FAIL;
			}
		}
	}

	CHECKPASS(2*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/foregroun.mc */

/* Including from file gc/backgroun.mc */
/*
 * Assertion XPutImage-31.(A)
 * The background component of the gc is used to determine the
 * background pixel value.
 */
static void
t031()
{
static unsigned long bglist[] = {
	0, 0x1, 0x2, 0x3, 0x4, 0x6, 0x7, 0x10, 0x33, 0x81,
	0xa3, 0xff, 256, 300, 1000,
	0x111111, 0x400200, 0x777777,
	};
XVisualInfo	*vp;
#ifdef NEED_STIPPLE
Pixmap	stip;
char	*stipbit = "\0";
#endif
long	bgind;
unsigned long	bg;
int 	nchecks = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

#ifdef NEED_STIPPLE
	stip = XCreateBitmapFromData(A_DISPLAY, DRW(A_DISPLAY), stipbit, 1, 1);
#endif

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		/* Bitmap full of 0's will cause a bg fill */
		A_IMAGE = makeimg(A_DISPLAY, vp, XYBitmap);
		dsetimg(A_IMAGE, 0L);

		for (bgind = 0; bgind < NELEM(bglist); bgind++) {

			bg = bglist[bgind];
			if (bg > DEPTHMASK(vp->depth))
				break;

			trace("Testing bg pixel of %d", bg);
			nchecks++;

#ifdef NEED_STIPPLE
			/*
			 * Set a stipple to all zero's and set the fill style
			 * to be FillOpaqueStippled.  This will mean that everything
			 * should be drawn in the background colour.
			 * (different for XCopyPlane, which doesn't use fill style,
			 * nor does XPutImage or XCopyArea).
			 */
			XSetStipple(A_DISPLAY, A_GC, stip);
			XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
#endif
/* now set the background to bg and do the call. If it's XCopyPlane/XPutImage then
 * we have cleared the src drawable/image to zero so we will always find 0 in the
 * selected plane so we will fill with bg. If we're XDrawImageString then
 * we do the equivalent of the background fill anyway,  if we're
 * XCopyArea then we have to do a dset of src with bg otherwise we'll have
 * used FillOpaqueStippled and a stipple of a lone zero, only, to get the
 * same effect of a background fill.
 */
			XSetBackground(A_DISPLAY, A_GC, bg);

			startcall(display);
			if (isdeleted())
				return;
			XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			PIXCHECK(A_DISPLAY, A_DRAW);

			dclear(A_DISPLAY, A_DRAW);
		}
	}

#ifdef NEED_STIPPLE
	XFreePixmap(A_DISPLAY, stip);
#endif

	CHECKPASS(nchecks);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-32.(A)
 * The value for background is truncated to the depth of the
 * GC.
 */
static void
t032()
{
XVisualInfo	*vp;
XImage	*imp;
#ifdef NEED_STIPPLE
Pixmap	stip;
char	*stipbit = "\0\0\0\0";
#endif
unsigned long	pix;
int 	found;
int 	i_x, i_y;
unsigned int 	i_width, i_height;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

#ifdef NEED_STIPPLE
	stip = XCreateBitmapFromData(A_DISPLAY, DRW(A_DISPLAY), stipbit, 1, 1);
#endif

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		A_DRAW = makewin(A_DISPLAY, vp);
		A_GC = makegc(A_DISPLAY, A_DRAW);
		/* bitmap of all 0's will cause bg "fill" */
		A_IMAGE = makeimg(A_DISPLAY, vp, XYBitmap);
		dsetimg(A_IMAGE, 0L);

#ifdef NEED_STIPPLE
		/*
		 * Set a stipple to all zero's and set the fill style
		 * to be FillOpaqueStippled.  This will mean that everything
		 * should be drawn in the background colour.
		 */
		XSetStipple(A_DISPLAY, A_GC, stip);
		XSetFillStyle(A_DISPLAY, A_GC, FillOpaqueStippled);
#endif

/* now set the background to ~0 and do the call. If it's XCopyPlane/XPutImage then
 * we have cleared the src drawable/image to zero so we will always find 0 in the
 * selected plane so we will fill with ~0. If we're XDrawImageString then
 * we do the equivalent of the background fill anyway, if we're XCopyArea
 * then we need to fill the src will all 1's, otherwise we'll have
 * used FillOpaqueStippled and a stipple of a lone zero, only, to get the
 * same effect of a background fill.
 */
		XSetBackground(A_DISPLAY, A_GC, ~0L);

		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		imp = savimage(A_DISPLAY, A_DRAW);
		getsize(A_DISPLAY, A_DRAW, &i_width, &i_height);
		found = 0;
		for (i_y = 0; i_y < i_height; i_y++) {
			for (i_x = 0; i_x < i_width; i_x++) {
				pix = XGetPixel(imp, i_x, i_y);
				if (pix != W_BG) {
					found = 1;
					if (pix == DEPTHMASK(vp->depth)) {
						CHECK;
					} else {
						report("Pixel was set to %d, expecting %d", pix, DEPTHMASK(vp->depth));
#ifdef NEED_STIPPLE
						report("FillOpaqueStippled is also used in this test, and could affect the result");
#endif
						FAIL;
					}
					i_y = i_height;	/* So that we leave loop XXX */
					break;
				}
			}
		}
		if (found == 0) {
			report("Nothing was drawn");
#ifdef NEED_STIPPLE
			report("FillOpaqueStippled is also used in this test, and could affect the result");
#endif
			FAIL;
		}
	}
#ifdef NEED_STIPPLE
	XFreePixmap(A_DISPLAY, stip);
#endif

	CHECKPASS(nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file gc/backgroun.mc */

/*
 * Assertion XPutImage-33.(A)
 * If drawables with depth < 32 are supported: When the image
 * differs from the X server's format in bits-per-pixel, then
 * the image will be converted to that format before drawing on
 * the specified drawable.
 */
static void
t033()
{
XImage *im1, *im2;
XVisualInfo *vp;
Window w;
char *data;
int supported = 0;
int nbits;
int   i;
static int legal_bpps[] = {1, 4, 8, 16, 24, 32, 0};
#define ROUNDUP(nbytes, pad) ((((nbytes) + ((pad)-1)) / (pad)) * ((pad)>>3))
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		if (vp->depth == 32)
			continue;

		supported++;

/* Call makeimg to create image in ZPixmap format using */
/* server's format in bits-per-pixel, scanline-pad, byte-order, */
/* and bit-order. */
		trace("ZPixmap, standard");
		im1 = makeimg(display, vp, ZPixmap);

		for (i = 0; legal_bpps[i]; i++) {
			if (legal_bpps[i] >= im1->depth &&
			legal_bpps[i] != im1->bits_per_pixel)
				break;
		}
		if (!legal_bpps[i]) {
			report("Could not find differing bits_per_pixel");
			FAIL;
			continue;
		}

/* Call makeimg to create another image in ZPixmap format. */
		im2 = makeimg(display, vp, ZPixmap);

/* Set bits-per-pixel to be different from the server's format, but >= depth. */
		im2->bits_per_pixel = legal_bpps[i];
		trace("ZPixmap. Non-server normal: depth=%d, bits_per_pixel=%d",im2->depth, im2->bits_per_pixel);

/* Allocate memory for image data, discarding old data and clearing new. */
		/* discard old data as provided by makeimg() */
		if (im2->data != (char *) 0)
			XFree(im2->data);
		/* allocate a little extra just to be safe */
		im2->bytes_per_line = ROUNDUP((im2->bits_per_pixel * im2->width), im2->bitmap_pad);
		data = (char *) malloc(im2->bytes_per_line * im2->height);
		if (data == (char *) 0) {
			delete("malloc() error");
			return;
		} else
			CHECK;
		im2->data = data;
		/* clear new data to 0L */
		dsetimg(im2, 0L);
/* Write known pattern to both images, using min(normal b.p.p, abnormal b.p.p) bits. */
		nbits = (im1->bits_per_pixel < im2->bits_per_pixel) ?
				im1->bits_per_pixel : im2->bits_per_pixel;
		trace("patterning im1 with %x", DEPTHMASK(nbits));
		patternimg(im1, DEPTHMASK(nbits));
		trace("patterning im2 with %x", DEPTHMASK(nbits));
		patternimg(im2, DEPTHMASK(nbits));
/* Create drawable. */
		w = makewin(display, vp);
		gc = makegc(display, w);
		dset(display, w, W_FG);
/* Call XPutImage with server-normal image. */
		image = im1;
		doxcall(w, 0, 0, 0, 0, image->width, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
/* Compare server-normal image with drawable. */
		if (compsavimage(display, w, image) == False) {
			report("Area (server-normal bits-per-pixel=%d) was not properly put.", image->bits_per_pixel);
			FAIL;
		}
		else
			CHECK;
/* Clear drawable to W_FG. */
		dset(display, w, W_FG);
/* Call XPutImage with non-server-normal image. */
		image = im2;
		doxcall(w, 0, 0, 0, 0, image->width, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
/* Compare same image with drawable. */
		if (compsavimage(display, w, image) == False) {
			report("Area (non-server-normal bits-per-pixel=%d) was not properly put.", image->bits_per_pixel);
			FAIL;
		}
		else
			CHECK;
/* Compare other image with drawable. */
		if (compsavimage(display, w, im1) == False) {
			report("Server-normal image differs from drawable.");
			FAIL;
		}
		else
			CHECK;
/* Repeat for each visual. */
	}

	if (supported == 0) {
		report("Drawables with changeable bits_per_pixel not supported");
		tet_result(TET_UNSUPPORTED);
		return;
	} else
		CHECK;

	CHECKPASS(1+4*supported);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-34.(A)
 * When the image differs from the X server's format in
 * scanline-pad, byte-order, or bit-order, then the image will
 * be converted to that format before drawing on the specified
 * drawable.
 */
static void
t034()
{
XImage *im1, *im2;
XVisualInfo *vp;
Window w;
char *data;
int nbits;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {

/* Call makeimg to create image in ZPixmap format using */
/* server's format in bits-per-pixel, scanline-pad, byte-order, */
/* and bit-order. */
		trace("ZPixmap, standard");
		im1 = makeimg(display, vp, ZPixmap);

/* Call makeimg to create another image in ZPixmap format. */
		im2 = makeimg(display, vp, ZPixmap);

/* Set scanline-pad to 8 if it is 32, else set it to 32. */
		if (im2->bitmap_unit == 32)
			im2->bitmap_unit = 8;
		else if (im2->bitmap_pad == 32)
			im2->bitmap_unit = 32;
/* Toggle byte-order between LSBFirst and MSBFirst. */
		if (im2->byte_order == LSBFirst)
			im2->byte_order = MSBFirst;
		else
			im2->byte_order = LSBFirst;
/* Toggle bit-order between LSBFirst and MSBFirst. */
		if (im2->bitmap_bit_order == LSBFirst)
			im2->bitmap_bit_order = MSBFirst;
		else
			im2->bitmap_bit_order = LSBFirst;

/* Allocate memory for image data, discarding old data and clearing new. */
		/* discard old data as provided by makeimg() */
		if (im2->data != (char *) 0)
			XFree(im2->data);
		/* allocate a little extra just to be safe */
		data = (char *) malloc(2 * im2->bytes_per_line * im2->height);
		if (data == (char *) 0) {
			delete("malloc() error");
			return;
		}
		else
			CHECK;
		im2->data = data;
		/* clear new data to 0L */
		dsetimg(im2, 0L);
/* Write known pattern to both images, using as many bits as we can. */
		nbits = (im2->bitmap_unit < im1->bitmap_unit) ?
			im2->bitmap_unit : im1->bitmap_unit;
		trace("patterning im1 with %x", DEPTHMASK(nbits));
		patternimg(im1, DEPTHMASK(nbits));
		trace("patterning im2 with %x", DEPTHMASK(nbits));
		patternimg(im2, DEPTHMASK(nbits));
/* Create drawable. */
		w = makewin(display, vp);
		gc = makegc(display, w);
		dset(display, w, W_FG);
/* Call XPutImage with server-normal image. */
		image = im1;
		doxcall(w, 0, 0, 0, 0, image->width, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
/* Compare same image with drawable. */
		if (compsavimage(display, w, image) == False) {
			report("Area (server-normal bits-per-pixel=%d) was not properly put.", image->bits_per_pixel);
			FAIL;
		}
		else
			CHECK;
/* Clear drawable to W_FG. */
		dset(display, w, W_FG);
/* Call XPutImage with non-server-normal image. */
		image = im2;
		doxcall(w, 0, 0, 0, 0, image->width, image->height);
		startcall(display);
		if (isdeleted())
			return;
		XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
/* Compare same image with drawable. */
		if (compsavimage(display, w, image) == False) {
			report("Area (non-server-normal bits-per-pixel=%d) was not properly put.", image->bits_per_pixel);
			FAIL;
		}
		else
			CHECK;
/* Compare other image with drawable. */
		if (compsavimage(display, w, im1) == False) {
			report("Server-normal image differs from drawable.");
			FAIL;
		}
		else
			CHECK;

/* Repeat for each visual. */
	}

	CHECKPASS(4*nvinf());
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPutImage-35.(A)
 * When the image format is XYPixmap or ZPixmap and the image
 * depth does not match the drawable depth, then a BadMatch
 * error occurs.
 */
static void
t035()
{
XVisualInfo *vp;
Window w;
int image_depth;
char *data;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/* choose first visual: any will do */
	resetvinf(VI_WIN_PIX); nextvinf(&vp);

	if (nvinf() == 0) {
		unsupported("No usable visuals, check XT_DEBUG_WINDOWS_ONLY and XT_DEBUG_PIXMAPS_ONLY");
		return;
	} else
		CHECK;
	/* ensured that there is at least one drawable */

/* Create drawable. */
	w = makewin(display, vp);
	gc = makegc(display, w);

/* Create image in XYPixmap format with depth different from drawable. */
	if (vp->depth == 1)
		image_depth = 8;
	else
		image_depth = 1;
	image = XCreateImage(display, vp->visual, image_depth, XYPixmap, 0, NULL, I_STDWIDTH, I_STDHEIGHT, BitmapPad(display), 0);
	if (image == (XImage *) 0) {
		delete("XCreateImage failed");
		return;
	}
	else
		CHECK;

/* Allocate memory for image data. */
	data = (char *) malloc(image->height * image->bytes_per_line * image->depth);
	if (data == (char *) 0) {
		delete("malloc() error");
		return;
	}
	else
		CHECK;
	image->data = data;

/* Call XPutImage to write image to drawable. */
	doxcall(w, 0, 0, 0, 0, image->width, image->height);
	startcall(display);
	if (isdeleted())
		return;
	XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

/* Verify XPutImage generated BadMatch error. */
	if (geterr() != BadMatch)
		FAIL;
	else
		CHECK;

/* Destroy image with XDestroyImage. */
	XDestroyImage(image);

/* Create image in ZPixmap format with depth different from drawable. */
	image = XCreateImage(display, vp->visual, image_depth, ZPixmap, 0, NULL, I_STDWIDTH, I_STDHEIGHT, BitmapPad(display), 0);
	trace("Repeat for ZPixmap format image.");
	if (image == (XImage *) 0) {
		delete("XCreateImage failed");
		return;
	}
	else
		CHECK;

/* Allocate memory for image data. */
	data = (char *) malloc(image->bytes_per_line * image->height);
	if (data == (char *) 0) {
		delete("malloc() error");
		return;
	}
	else
		CHECK;
	image->data = data;

/* Call XPutImage to write image to drawable. */
	doxcall(w, 0, 0, 0, 0, image->width, image->height);
	startcall(display);
	if (isdeleted())
		return;
	XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

/* Verify XPutImage generated BadMatch error. */
	if (geterr() != BadMatch)
		FAIL;
	else
		CHECK;

/* Destroy image with XDestroyImage. */
	XDestroyImage(image);

	CHECKPASS(7);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EDra.mc */
/*
 * Assertion XPutImage-36.(A)
 * When a drawable argument does not name a valid Drawable,
 * then a BadDrawable error occurs.
 */
/* End of included file error/EDra.mc */

/* Including from file error/EDra.mc */
static void
t036()
{
	XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	resetvinf(VI_WIN_PIX); nextvinf(&vp);
	A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
	dsetimg(A_IMAGE, W_FG);
	A_DRAWABLE = (Drawable)badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
	endcall(display);
	if (geterr() != BadDrawable) {
		report("Got %s, Expecting BadDrawable", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadDrawable)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EDra.mc */

/* Including from file error/EGC.mc */
/*
 * Assertion XPutImage-37.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t037()
{
	XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	resetvinf(VI_WIN_PIX); nextvinf(&vp);
	A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
	dsetimg(A_IMAGE, W_FG);

	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/* Including from file error/EMat1.mc */
/*
 * Assertion XPutImage-38.(A)
 * When a drawable argument is an InputOnly window then a
 * BadMatch error occurs.
 */
/* End of included file error/EMat1.mc */

/* Including from file error/EMat1.mc */
static void
t038()
{
	XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	resetvinf(VI_WIN_PIX); nextvinf(&vp);
	A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
	dsetimg(A_IMAGE, W_FG);

	A_DRAWABLE = (Drawable)iponlywin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat1.mc */

/* Including from file error/EMat2.mc */
/*
 * Assertion XPutImage-39.(A)
 * When the graphics context and the drawable do not have the
 * same depth, then a BadMatch error occurs.
 */
/* End of included file error/EMat2.mc */

/* Including from file error/EMat2.mc */
static void
t039()
{
XVisualInfo	*vp;
Drawable	errpm;
int 	founddepth = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_PIX); nextvinf(&vp); ) {
		if (vp->depth != 1) {
			founddepth++;

			errpm = makewin(A_DISPLAY, vp);
			A_GC = makegc(A_DISPLAY, errpm);
			A_IMAGE = makeimg(A_DISPLAY, vp, ZPixmap);
			dsetimg(A_IMAGE, W_FG);

			break;
		}
	}

	if (!founddepth) {
		report("Only one depth supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	vp->depth = 1;
	A_DRAWABLE = makewin(A_DISPLAY, vp);

	startcall(display);
	if (isdeleted())
		return;
	XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat2.mc */

/* Including from file error/EMat3.mc */
/*
 * Assertion XPutImage-40.(A)
 * When the graphics context and the drawable were not created
 * for the same root, then a BadMatch error occurs.
 */
/* End of included file error/EMat3.mc */

/* Including from file error/EMat3.mc */
static void
t040()
{
XVisualInfo	vi;
Pixmap	errpm;
int 	scr_num;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

        if (config.alt_screen == -1) {
                unsupported("No alternate root supported");
                return;
        }

	scr_num = config.alt_screen;
	if (scr_num == DefaultScreen(A_DISPLAY)) {
		delete("The alternate root was the same as the one under test");
		return;
	}
	if (scr_num >= ScreenCount(A_DISPLAY)) {
		delete("The alternate root could not be accessed");
		return;
	}

	vi.visual = NULL;
	vi.screen = DefaultScreen(A_DISPLAY);	/* XXX */
	vi.depth = 1;
	A_DRAWABLE = makepixm(A_DISPLAY, &vi);
	A_IMAGE = makeimg(A_DISPLAY, &vi, ZPixmap);
	dsetimg(A_IMAGE, W_FG);

	/*
	 * Create a 1x1 depth 1 pixmap on other screen
	 * and use it to create a gc
	 */
	errpm = XCreatePixmap(A_DISPLAY, RootWindow(A_DISPLAY, scr_num), 1, 1, 1);
	A_GC = makegc(A_DISPLAY, errpm);

	startcall(display);
	if (isdeleted())
		return;
	XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	XFreePixmap(A_DISPLAY, errpm);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat3.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	t021, 21,
	t022, 22,
	t023, 23,
	t024, 24,
	t025, 25,
	t026, 26,
	t027, 27,
	t028, 28,
	t029, 29,
	t030, 30,
	t031, 31,
	t032, 32,
	t033, 33,
	t034, 34,
	t035, 35,
	t036, 36,
	t037, 37,
	t038, 38,
	t039, 39,
	t040, 40,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
