/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: qrybstcrsr.m,v 1.7 92/06/11 17:08:45 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XQueryBestCursor(display, d, width, height, rwidth, rheight)
 *   Display *display;
 *   Drawable d;
 *   unsigned int width;
 *   unsigned int height;
 *   unsigned int *rwidth;
 *   unsigned int *rheight;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XQueryBestCursor	1
char    *TestName = "XQueryBestCursor";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE d


/*
 * Arguments to the XQueryBestCursor function
 */
static Display *display;
static Drawable d;
static unsigned int width;
static unsigned int height;
static unsigned int *rwidth;
static unsigned int *rheight;


static int 	StatusReturn;

static unsigned int cheight, cwidth;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	d = 0;
	width = 0;
	height = 0;
	rwidth = &cwidth;
	rheight = &cheight;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	d = ErrdefDrawable;
}

/*
 * Assertion XQueryBestCursor-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XQueryBestCursor returns in the width_return and
 * height_return arguments the largest cursor height and width
 * to those specified in height and width that can be fully
 * displayed on the specified screen.
 */
static void
t001()
{
unsigned int swidth;
unsigned int sheight;
Status qstat;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	d = DRW(display);

/* Dynamically determine width and height values */
	swidth = DisplayWidth(display, DefaultScreen(display));
	sheight = DisplayHeight(display, DefaultScreen(display));
	width = 2 * swidth;
	height = 2 * sheight;

/* Call XQueryBestCursor with width twice that of DisplayWidth and */
/* twice that of DisplayHeight. */
	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestCursor(display, d, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XQueryBestCursor returns non-zero. */
	if (qstat == 0) {
		report("%s returned wrong value %ld", TestName, (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	width = cwidth;
	height = cheight;

	cwidth = cheight = 0;

/* Call XQueryBestCursor with returned values of width and height. */
	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestCursor(display, d, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that XQueryBestCursor returns non-zero. */
	if(qstat == 0) {
		report("%s returned wrong value %ld", TestName, (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

/* Verify that XQueryBestCursor returns the previously returned */
	if(width != cwidth) {
		report("%s returned best width %d", TestName, cwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != cheight) {
		report("%s returned best height %d", TestName, cheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;
/* Verify that returned values for width and height are small enough */
/* to be fully displayed on the specified screen. */
	if (cwidth > swidth) {
		report("%s returned non-fully displayable width %d",
			TestName, cwidth);
		FAIL;
	} else
		CHECK;
	if (cheight > sheight) {
		report("%s returned non-fully displayable height %d",
			TestName, cheight);
		FAIL;
	} else
		CHECK;

/* Repeat with initial width and height both set to zero. */
	width = 0;
	height = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestCursor(display, d, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("%s returned wrong value %ld", TestName, (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	width = cwidth;
	height = cheight;

	cwidth = cheight = 0;

	startcall(display);
	if (isdeleted())
		return;
	qstat = XQueryBestCursor(display, d, width, height, rwidth, rheight);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(qstat == 0) {
		report("%s returned wrong value %ld", TestName, (long) qstat);
		FAIL;
	} else
		CHECK;

	trace("Status returned was %d", qstat);
	trace("Best height (for %d ) = %d", height, cheight);
	trace("Best width (for %d)  = %d", width, cwidth);

	if(width != cwidth) {
		report("%s returned best width %d", TestName, cwidth);
		report("after previously returning best width %d", width);
		FAIL;
	} else
		CHECK;

	if(height != cheight) {
		report("%s returned best height %d", TestName, cheight);
		report("after previously returning best height %d", height);
		FAIL;
	} else
		CHECK;
	if (cwidth > swidth) {
		report("%s returned non-fully displayable width %d",
			TestName, cwidth);
		FAIL;
	} else
		CHECK;
	if (cheight > sheight) {
		report("%s returned non-fully displayable height %d",
			TestName, cheight);
		FAIL;
	} else
		CHECK;

	CHECKUNTESTED(12);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EDra.mc */
/*
 * Assertion XQueryBestCursor-2.(A)
 * When a drawable argument does not name a valid Drawable,
 * then a BadDrawable error occurs.
 */
/* End of included file error/EDra.mc */

/* Including from file error/EDra.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_DRAWABLE = (Drawable)badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XQueryBestCursor(display, d, width, height, rwidth, rheight);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadDrawable) {
		report("Got %s, Expecting BadDrawable", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadDrawable)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EDra.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
