/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: unldfnt.m,v 1.8 92/06/11 17:10:10 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XUnloadFont(display, font)
 *   Display	*display;
 *   Font	font;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XUnloadFont	1
char    *TestName = "XUnloadFont";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_FONT font


/*
 * Arguments to the XUnloadFont function
 */
static Display	*display;
static Font	font;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	font = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	font = ErrdefFont;
}

/*
 * Assertion XUnloadFont-1.(A)
 * When another resource or resource ID references the font,
 * then a call to XUnloadFont deletes the association between
 * the font resource ID and the specified font.
 */
static void
t001()
{
GC		gc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	gc = makegc(display, DRW(display));
	if (isdeleted())
		return;
	font = XLoadFont(display, "xtfont0");

	startcall(display);
	if (isdeleted())
		return;
	XUnloadFont(display, font);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* Try to use the font */
	XSetErrorHandler(error_status);
	reseterr();
	XSetFont(display, gc, font);
	XDrawString(display, DRW(display), gc, 30, 30, "abc", 3);
	XSync(display, 0);
	switch (geterr()) {
	case Success:
		report("font ID was still usable");
		FAIL;
		break;
	case BadFont:
		PASS;
		break;
	default:
		delete("Unexpected error in draw string");
		break;
	}
	XSetErrorHandler(unexp_err);
	
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUnloadFont-2.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When no other resource or resource ID references the font,
 * then a call to XUnloadFont deletes the association between
 * the font resource ID and the specified font and the font
 * itself will be freed.
 */
static void
t002()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/* Including from file error/EFon1.mc */
/*
 * Assertion XUnloadFont-3.(A)
 * When a font argument does not name a valid font, then a
 * BadFont error occurs.
 */
/* End of included file error/EFon1.mc */

/* Including from file error/EFon1.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_FONT = (Font) badfont(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XUnloadFont(display, font);
	endcall(display);
	if (geterr() != BadFont) {
		report("Got %s, Expecting BadFont", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadFont)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EFon1.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
