/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: addhst.m,v 1.5 92/06/11 17:10:40 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XAddHost(display, host)
 *   Display	*display;
 *   XHostAddress	*host;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XAddHost	1
char    *TestName = "XAddHost";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XAddHost function
 */
static Display	*display;
static XHostAddress	*host;



#include	"xthost.h"



static int
ishostinacl(host, acl, nhosts)
XHostAddress	*host;
XHostAddress	*acl;
int 	nhosts;
{
int 	i;

	for (i = 0; i < nhosts; i++) {
		if (samehost(host, &acl[i]))
			return True;
	}
	return False;
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	host = xthosts;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XAddHost-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XAddHost adds the specified host to the access
 * control list for the display.
 */
static void
t001()
{
XHostAddress	*oldhosts;
XHostAddress	*newhosts;
int 	nhosts;
Bool	state;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	oldhosts = XListHosts(display, &nhosts, &state);

	/*
	 * Try to set up the current list so as to exclude the host to
	 * be added.
	 */
	if (ishostinacl(host, oldhosts, nhosts)) {
		/* Have to remove it first, may get BadAccess */
		debug(1, "Removing host");
		CATCH_ERROR(display);
		XRemoveHost(display, host);
		RESTORE_ERROR(display);
		oldhosts = XListHosts(display, &nhosts, &state);
		if (ishostinacl(host, oldhosts, nhosts)) {
			delete("Could not set up host list to exclude host to be added");
			return;
		}
	}
	if (isdeleted())
		return;

	startcall(display);
	if (isdeleted())
		return;
	XAddHost(display, host);
	endcall(display);

	if (geterr() == BadAccess) {
		untested("This host does not have permission to change the list");
		report("  so this assertion cannot be tested");
		return;
	} else
		CHECK;

	newhosts = XListHosts(display, &nhosts, &state);
	if (ishostinacl(host, newhosts, nhosts))
		CHECK;
	else {
		report("Host was not added to access control list");
		FAIL;
	}

	CHECKPASS(2);

	XRemoveHost(display, host);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAcc4.mc */
/*
 * Assertion XAddHost-2.(A)
 * When an attempt is made to modify the access control list
 * from a client that is not authorised in a server-dependent
 * way to do so, then a BadAccess error occurs.
 */
/* End of included file error/EAcc4.mc */

static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	XAddHost(display, host);
	endcall(display);

	if (geterr() != Success) {
		if (geterr() == BadAccess)
			PASS;
		else {
			report("Received %s error, expecting BadAccess", errorname(geterr()));
			FAIL;
		}
	} else {
		untested("This host has permission to change the list");
		report("  so this assertion cannot be tested");
	}
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAddHost-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the host family, address length and address data do
 * not form a valid address, then a BadValue error occurs.
 */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	host = xtbadhosts;

	startcall(display);
	if (isdeleted())
		return;
	XAddHost(display, host);
	endcall(display);

	if (geterr() == BadAccess) {
		untested("This host does not have permission to change the list");
		report("  so this assertion cannot be tested");
		return;
	} else
		CHECK;

	if (geterr() == BadValue)
		CHECK;
	else {
		report("Got %s, expecting BadValue", errorname(geterr()));
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
