/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: bll.m,v 1.6 92/06/11 17:11:26 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XBell(display, percent)
 *   Display	*display;
 *   int 	percent;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XBell	1
char    *TestName = "XBell";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XBell function
 */
static Display	*display;
static int 	percent;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	percent = 50;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XBell-1.(D)
 *   Reason: There is no known reliable test method for this assertion
 * If it is possible to ring a bell on the display: When the
 * percent argument is nonnegative, then the volume at which
 * the bell rings, where base is the base volume of the bell,
 * is base - [(base*percent)/100]+ percent.
 */
static void
t001()
{
int 	i;
static int 	vals[] = {
	0, 2, 12, 30, 45, 60, 100};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (i = 0; i < NELEM(vals); i++) {
		percent = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XBell(display, percent);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	}

	untested("Test for bell functionality present only");

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XBell-2.(D)
 *   Reason: There is no known reliable test method for this assertion
 * If it is possible to ring a bell on the display: When the
 * percent argument is negative, then the volume at which the
 * bell rings, where base is the base volume of the bell, is
 * base+ [(base*percent)/100].
 */
static void
t002()
{
int 	i;
static int 	vals[] = {
	-2, -17, -30, -60, -100};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (i = 0; i < NELEM(vals); i++) {
		percent = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XBell(display, percent);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	}

	untested("Test for bell functionality present only");

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XBell-3.(A)
 * When the value for the percent argument is not in the range
 * -100 to 100 inclusive, then a BadValue error occurs.
 */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	percent = -101;
	startcall(display);
	if (isdeleted())
		return;
	XBell(display, percent);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		CHECK;

	percent = 141;
	startcall(display);
	if (isdeleted())
		return;
	XBell(display, percent);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
