/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: chngkybrdc.m,v 1.8 92/06/11 17:11:42 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XChangeKeyboardControl(display, value_mask, values)
 *   Display	*display;
 *   unsigned long	value_mask;
 *   XKeyboardControl	*values;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XChangeKeyboardControl	1
char    *TestName = "XChangeKeyboardControl";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XChangeKeyboardControl function
 */
static Display	*display;
static unsigned long	value_mask;
static XKeyboardControl	*values;



/* For the argument */
static XKeyboardControl Kcval;



/*
 * Functions to save and restore the old keyboard control values across TP's
 * This also makes the current values available for use in tests.
 */
static XKeyboardState	oldstate;

static void
savekey()
{
	startup();
	if(Dsp)
		XGetKeyboardControl(Dsp, &oldstate);
}

static void
restorekey()
{
XKeyboardControl	ctr;
int 	led;
int 	key;
int 	minkc, maxkc;

	if(!Dsp) {
		cleanup();
		return;
	}

	/*
	 * It is not easy to restore a previously saved keyboard state.
	 */
	ctr.key_click_percent = oldstate.key_click_percent;
	ctr.bell_percent = oldstate.bell_percent;
	ctr.bell_pitch = oldstate.bell_pitch;
	ctr.bell_duration = oldstate.bell_duration;
	XChangeKeyboardControl(Dsp, KBKeyClickPercent|KBBellPercent|KBBellPitch|KBBellDuration, &ctr);

	ctr.auto_repeat_mode = oldstate.global_auto_repeat;
	XChangeKeyboardControl(Dsp, KBAutoRepeatMode, &ctr);
	XFlush(Dsp);

	for (led = 1; led <= 32; led++) {
		ctr.led = led;
		ctr.led_mode = (oldstate.led_mask & (1<<(led-1)))? LedModeOn:LedModeOff;
		XChangeKeyboardControl(Dsp, KBLed|KBLedMode, &ctr);
	}
	XFlush(Dsp);

	XDisplayKeycodes(Dsp, &minkc, &maxkc);
	for (key = minkc; key < maxkc; key++) {
		ctr.key = key;
		ctr.auto_repeat_mode = (oldstate.auto_repeats[key/8] & (1<<(key%8)))?
			AutoRepeatModeOn: AutoRepeatModeOff;
		XChangeKeyboardControl(Dsp, KBAutoRepeatMode|KBKey, &ctr);
		XFlush(Dsp);
	}
	cleanup();
}


#define	NLEDS	32	/* Number of leds that the protocol allows */
#define	NLEDMASK 0xffffffff	/* Mask of all leds */

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	value_mask = KBBellPercent;
	values = &Kcval;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XChangeKeyboardControl-1.(A)
 * If the key click loudness can be set: When the
 * KBKeyClickPercent bit is set in value_mask, then the volume
 * of the key click is set to the value specified in
 * key_click_percent, which is between 0, signifying off, and
 * 100, signifying loud.
 */
static void
t001()
{
int 	i;
XKeyboardState	newks;
static int 	vals[] = {
	0, 100, 50};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBKeyClickPercent;
	for (i = 0; i < NELEM(vals); i++) {

		values->key_click_percent = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		XGetKeyboardControl(display, &newks);

		if (newks.key_click_percent == vals[i])
			CHECK;
		else {
			report("Key click percent was %d, expecting %d",
				newks.key_click_percent, vals[i]);
			FAIL;
		}
	}

	CHECKPASS(NELEM(vals));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-2.(B)
 *   Reason: The statement in the X11 specification is not specific enough to write a test
 * When the KBKeyClickPercent bit is set in value_mask and
 * key_click_percent is -1, then the default key click volume
 * is restored.
 */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * Since the default isn't specified there is no useful test that
	 * can be done.  However touch test with -1 to make sure that it
	 * doesn't blow up.
	 */
	value_mask = KBKeyClickPercent;
	values->key_click_percent = -1;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardControl(display, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	report("There is no complete test method, but a touch test was performed");

	UNTESTED;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-3.(A)
 * If the bell loudness can be set: When the KBBellPercent bit
 * is set in value_mask, then the base volume of the bell is
 * set to the value specified in bell_percent, which is
 * between 0, signifying off, to 100, signifying loud.
 */
static void
t003()
{
int 	i;
XKeyboardState	newks;
static int 	vals[] = {
	0, 12, 20, 32, 47, 61, 100};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBBellPercent;
	for (i = 0; i < NELEM(vals); i++) {

		values->bell_percent = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		/* Do the bell so that you can check audibly if you wish */
		XBell(display, 0);

		XGetKeyboardControl(display, &newks);

		if (newks.bell_percent == vals[i])
			CHECK;
		else {
			report("Key bell percent was %d, expecting %d",
				newks.key_click_percent, vals[i]);
			FAIL;
		}
	}

	/*
	 * Allow time for bells to happen before going on to next test.
	 */
	sleep(2);

	CHECKPASS(NELEM(vals));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-4.(B)
 *   Reason: The statement in the X11 specification is not specific enough to write a test
 * When the KBBellPercent bit is set in value_mask and
 * bell_percent is -1, then the base volume of the bell is set
 * to the default.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBBellPercent;
	values->bell_percent = -1;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardControl(display, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	report("There is no complete test method, but a touch test was performed");

	UNTESTED;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-5.(A)
 * If the bell pitch can be set: When the KBBellPitch bit is
 * set in value_mask, then the bell pitch in Hz is set to the
 * value specified in bell_pitch.
 */
static void
t005()
{
int 	i;
XKeyboardState	newks;
static int 	vals[] = {
	440, 528, 400, 294, 300, 800, 640};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBBellPitch;
	for (i = 0; i < NELEM(vals); i++) {

		values->bell_pitch = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		XBell(Dsp, 0);

		XGetKeyboardControl(display, &newks);

		if (newks.bell_pitch == vals[i])
			CHECK;
		else {
			report("Bell pitch was %d, expecting %d",
				newks.bell_pitch, vals[i]);
			FAIL;
		}
	}

	/*
	 * Allow time for bells to happen before going on to next test.
	 */
	sleep(2);

	CHECKPASS(NELEM(vals));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-6.(B)
 *   Reason: The statement in the X11 specification is not specific enough to write a test
 * When the KBBellPitch bit is set in value_mask and
 * bell_pitch is -1, then the bell pitch is set to the default.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBBellPitch;
	values->bell_pitch = -1;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardControl(display, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	report("There is no complete test method, but a touch test was performed");

	UNTESTED;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-7.(A)
 * If the bell duration can be set: When the KBBellDuration
 * bit is set in value_mask, then the bell duration in
 * milliseconds is set to the value specified in bell_duration.
 */
static void
t007()
{
int 	i;
XKeyboardState	newks;
static int 	vals[] = {
	10, 200, 600, 1000, 50};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBBellDuration;
	for (i = 0; i < NELEM(vals); i++) {

		values->bell_duration = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		XBell(display, 0);
		XGetKeyboardControl(display, &newks);

		if (newks.bell_duration == vals[i])
			CHECK;
		else {
			report("Bell duration was %d, expecting %d",
				newks.bell_duration, vals[i]);
			FAIL;
		}
	}

	/*
	 * Allow time for bells to happen before going on to next test.
	 */
	sleep(5);

	CHECKPASS(NELEM(vals));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-8.(B)
 *   Reason: The statement in the X11 specification is not specific enough to write a test
 * When the KBBellDuration bit is set in value_mask and
 * bell_duration is -1, then the bell duration is set to the
 * default.
 */
static void
t008()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBBellDuration;
	values->bell_duration = -1;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardControl(display, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	report("There is no complete test method, but a touch test was performed");

	UNTESTED;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-9.(A)
 * If LED s are supported: When both KBLed and KBLedMode are
 * specified, then the state of the LED specified by led is
 * changed to the state specified by led_mode.
 */
static void
t009()
{
int 	i, j;
unsigned long	onmask;
XKeyboardState	newks;
static int 	vals[] = {
	1, 3, 2, 4, 20, NLEDS};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBLed|KBLedMode;
	onmask = 0;
	for (i = 1; i <= NLEDS; i++) {

		values->led_mode = LedModeOff;
		for (j = 0; j < NELEM(vals); j++) {
			if (vals[j] == i) {
				onmask |= (1L<<(i-1));
				values->led_mode = LedModeOn;
				break;
			}
		}
		values->led = i;

		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	}

	XGetKeyboardControl(display, &newks);

	if (newks.led_mask == onmask)
		CHECK;
	else {
		report("Led mask was %ld, expecting %ld",
			newks.led_mask, onmask);
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-10.(A)
 * If LED s are supported: When KBLedMode is specified and
 * KBLed is not specified, then the state of all LED s is
 * changed to the mode specified by led_mode.
 */
static void
t010()
{
XKeyboardState	newks;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBLedMode;
	values->led_mode = LedModeOn;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardControl(display, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetKeyboardControl(display, &newks);

	if (newks.led_mask == (unsigned)NLEDMASK)
		CHECK;
	else {
		report("All leds were not set to correct value");
		report("  led_mask was 0x%x, expecting 0x%x", newks.led_mask, NLEDMASK);
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-11.(A)
 * When KBKey and KBAutoRepeatMode are specified, then the
 * auto repeat mode of key is changed to the mode specified by
 * auto_repeat_mode.
 */
static void
t011()
{
int 	i;
int 	minkc, maxkc;
int 	onoff;
XKeyboardState	newks;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

#define	NKEYSTOTRY	12

	value_mask = KBKey | KBAutoRepeatMode;
	XDisplayKeycodes(display, &minkc, &maxkc);
	/*
	 * Try out the first few keycodes.
	 */
	for (i = minkc; i < minkc+NKEYSTOTRY; i++) {

		values->key = i;
		onoff = (oldstate.auto_repeats[i/8] & (1<<(i%8)))?
			AutoRepeatModeOff: AutoRepeatModeOn;
		values->auto_repeat_mode = onoff;
		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		XGetKeyboardControl(display, &newks);

		if (((newks.auto_repeats[i/8] & (1<<(i%8))) != 0) == ((onoff==AutoRepeatModeOn)? 1: 0))
			CHECK;
		else {
			report("Key auto repeat was not set to %d", onoff);
			FAIL;
		}
	}

	CHECKPASS(NKEYSTOTRY);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-12.(A)
 * When KBAutoRepeatMode is specified and KBKey is not
 * specified, then the per-key settings are unchanged and the
 * auto repeat mode for the whole keyboard is changed to that
 * specified by auto_repeat_mode.
 */
static void
t012()
{
XKeyboardState	newks;
int 	mode;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBAutoRepeatMode;
	mode = (oldstate.global_auto_repeat==AutoRepeatModeOn)?
		AutoRepeatModeOff: AutoRepeatModeOn;
	values->auto_repeat_mode = mode;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardControl(display, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetKeyboardControl(display, &newks);

	if (newks.global_auto_repeat == mode)
		CHECK;
	else {
		report("global auto repeat mode was not set");
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-13.(A)
 * When the KBKeyClickPercent bit is set in value_mask and
 * key_click_percent is not between 0 and 100 inclusive or -1,
 * then a BadValue error occurs.
 */
static void
t013()
{
int 	i;
static	int 	badvals[] = {
	-100, -2, 101, 203};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBKeyClickPercent;
	for (i = 0; i < NELEM(badvals); i++) {
		values->key_click_percent = badvals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			report("No BadValue for key_click_percent of %d", badvals[i]);
			FAIL;
		}
	}
	CHECKPASS(NELEM(badvals));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-14.(A)
 * When the KBBellPercent bit is set in value_mask and
 * bell_percent is not between 0 and 100 inclusive or -1, then
 * a BadValue error occurs.
 */
static void
t014()
{
int 	i;
static	int 	badvals[] = {
	-100, -2, 101, 203};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBBellPercent;
	for (i = 0; i < NELEM(badvals); i++) {
		values->bell_percent = badvals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			report("No BadValue for bell_percent of %d", badvals[i]);
			FAIL;
		}
	}
	CHECKPASS(NELEM(badvals));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-15.(A)
 * When the KBBellPitch bit is set in value_mask and
 * bell_pitch is a negative number other than -1, then a
 * BadValue error occurs.
 */
static void
t015()
{
int 	i;
static	int 	badvals[] = {
	-9012, -100, -2};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBBellPitch;
	for (i = 0; i < NELEM(badvals); i++) {
		values->bell_pitch = badvals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			report("No BadValue for bell_pitch of %d", badvals[i]);
			FAIL;
		}
	}
	CHECKPASS(NELEM(badvals));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-16.(A)
 * When the KBBellDuration bit is set in value_mask and
 * bell_duration is a negative number other than -1, then a
 * BadValue error occurs.
 */
static void
t016()
{
int 	i;
static	int 	badvals[] = {
	-3456, -100, -2};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBBellDuration;
	for (i = 0; i < NELEM(badvals); i++) {
		values->bell_duration = badvals[i];
		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			report("No BadValue for bell_duration of %d", badvals[i]);
			FAIL;
		}
	}
	CHECKPASS(NELEM(badvals));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-17.(A)
 * When KBLed is specified and KBLedMode is not specified,
 * then a BadMatch error occurs.
 */
static void
t017()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBLed;
	values->led = 1;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardControl(display, value_mask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;	/* safe */
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-18.(A)
 * When KBKey is specified and KBAutoRepeatMode is not
 * specified, then a BadMatch error occurs.
 */
static void
t018()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	value_mask = KBKey;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardControl(display, value_mask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-19.(A)
 * When the value of led_mode is other than LedModeOn or
 * LedModeOff, then a BadValue error occurs.
 */
static void
t019()
{
static	int 	lmodes[] = {
	LedModeOn, LedModeOff};
long	badvals[NM_LEN];
int 	i, n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	n = notmember(lmodes, NELEM(lmodes), badvals);

	value_mask = KBLedMode;

	for (i = 0; i < n; i++) {
		values->led_mode = badvals[i];

		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			report("A led_mode of %d did not give BadValue", badvals[i]);
		}
	}
	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardControl-20.(A)
 * When the value of auto_repeat_mode is other than
 * AutoRepeatModeOff, AutoRepeatModeOn or
 * AutoRepeatModeDefault, then a BadValue error occurs.
 */
static void
t020()
{
static	int 	armodes[] = {
	AutoRepeatModeOff, AutoRepeatModeOn, AutoRepeatModeDefault};
long	badvals[NM_LEN];
int 	i, n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	n = notmember(armodes, NELEM(armodes), badvals);

	value_mask = KBAutoRepeatMode;

	for (i = 0; i < n; i++) {
		values->auto_repeat_mode = badvals[i];

		startcall(display);
		if (isdeleted())
			return;
		XChangeKeyboardControl(display, value_mask, values);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			report("A auto_repeat_mode of %d did not give a BadValue", badvals[i]);
		}
	}
	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = savekey;
void	(*tet_cleanup)() = restorekey;
