/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: grbsrvr.m,v 1.4 92/06/11 17:13:34 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XGrabServer(display)
 *   Display	*display;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGrabServer	1
char    *TestName = "XGrabServer";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XGrabServer function
 */
static Display	*display;



#include	"Xatom.h"



static Display	*client1;
static Display	*client2;

static Window	win;


static void
pproc()
{
int 	waittime;
XEvent	ev;
int 	pass = 0, fail = 0;

	/* Allow time for change prop to occur */
	waittime = 2*config.speedfactor + 5;
	sleep(waittime);

	if (XCheckWindowEvent(display, win, PropertyChangeMask, &ev)) {
		report("A request was processed for other than the grabbing client");
		FAIL;
	} else
		CHECK;

	/*
	 * Release the grab and wait a bit to allow the requests on the other
	 * connection to go ahead.
	 * NOTE: If the ungrab does not work then the child will time out
	 * and the tet_fork function will delete the test.  This isn't really
	 * what we want, but is not unreasonable in this case because we are
	 * testing Grab and not Ungrab.
	 */
	XUngrabServer(display);
	XSync(display, False);

	/* Allow time for change prop to occur */
	sleep(waittime);
	if (XCheckWindowEvent(display, win, PropertyChangeMask, &ev))
		CHECK;
	else {
		delete("Requests were not processed after server grab was released.");
		FAIL;
	}

	CHECKPASS(2);
}

/*
 * Perform operation on client2 while client1 has the server grabbed.
 */
static void
cproc()
{
long 	val;
Atom	name;

	val = 5;
	name = XInternAtom(client2, "name", False);
	XChangeProperty(client2, win, name, XA_INTEGER, 32, PropModeReplace,
		(unsigned char *)&val, 1);
	XFlush(client2);
	exit(0);
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGrabServer-1.(A)
 * A call to XGrabServer disables processing of requests on
 * all connections apart from the one issuing this request.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	client1 = opendisplay();
	client2 = opendisplay();

	win = defwin(Dsp);
	XSelectInput(client1, win, PropertyChangeMask);
	XSelectInput(client2, win, PropertyChangeMask);

	if (isdeleted())
		return;

	display = client1;
	startcall(display);
	if (isdeleted())
		return;
	XGrabServer(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/*
	 * The return value only tells us that the test has been deleted
	 * and reported as such.  There is nothing further that we can
	 * do with it.
	 */
	(void) tet_fork(cproc, pproc, 1, ~0);
	/* Test now continues in pproc */
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
