/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: staccsscnt.m,v 1.4 92/06/11 17:16:13 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetAccessControl(display, mode)
 *   Display	*display;
 *   int 	mode;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetAccessControl	1
char    *TestName = "XSetAccessControl";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XSetAccessControl function
 */
static Display	*display;
static int 	mode;



/* Value list for use in test t004 */
static int 	modevallist[] = {
	EnableAccess,
	DisableAccess,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	mode = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetAccessControl-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * When mode is EnableAccess, then the use of the access
 * control list is enabled at each connection setup.
 */
static void
t001()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSetAccessControl-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When mode is DisableAccess, then the use of the access
 * control list is disabled at each connection setup.
 */
static void
t002()
{
XHostAddress	*oldhosts;
XHostAddress	*list;
Display	*newclient;
int 	nhosts;
int 	njunk;
Bool	state;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	oldhosts = XListHosts(display, &nhosts, &state);

	CATCH_ERROR(display);
	XRemoveHosts(display, oldhosts, nhosts);
	RESTORE_ERROR(display);

	if (GET_ERROR(display) == BadAccess) {
		report("The client does not have permission to disable the acl");
		untested("  so this assertion cannot be tested.");
		return;
	}

	mode = DisableAccess;
	startcall(display);
	if (isdeleted())
		return;
	XSetAccessControl(display, mode);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	list = XListHosts(display, &njunk, &state);
	if (state == DisableAccess)
		CHECK;
	else {
		report("Access control state was not DisableAccess (was %d)", state);
		FAIL;
	}

	newclient = XOpenDisplay(config.display);
	if (newclient != (Display*)0)
		CHECK;
	else {
		report("Connections could not be made to host");
		FAIL;
	}

	CHECKPASS(2);

	XAddHosts(display, oldhosts, nhosts);
	XFree((char*)oldhosts);
	XFree((char*)list);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAcc4.mc */
/*
 * Assertion XSetAccessControl-3.(A)
 * When an attempt is made to modify the access control list
 * from a client that is not authorised in a server-dependent
 * way to do so, then a BadAccess error occurs.
 */
/* End of included file error/EAcc4.mc */

static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	XSetAccessControl(display, mode);
	endcall(display);

	if (geterr() != Success) {
		if (geterr() == BadAccess)
			PASS;
		else {
			report("Expecting BadAccess, was %s", errorname(geterr()));
			FAIL;
		}
	} else {
		untested("This client is authorised to disable the access list");
		untested("  so the assertion cannot be tested");
	}

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetAccessControl-4.(A)
 * When the value of mode is other than EnableAccess or
 * DisableAccess, then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t004()
{
#undef	VALUE_ARG
#define	VALUE_ARG mode
#undef	VALUE_LIST
#define	VALUE_LIST modevallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XSetAccessControl(display, mode);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
