/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stinptfcs.m,v 1.8 92/06/11 17:16:29 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetInputFocus(display, focus, revert_to, test_time)
 *   Display	*display;
 *   Window	focus;
 *   int 	revert_to;
 *   Time	test_time;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetInputFocus	1
char    *TestName = "XSetInputFocus";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW focus
#define A_DRAWABLE focus


/*
 * Arguments to the XSetInputFocus function
 */
static Display	*display;
static Window	focus;
static int 	revert_to;
static Time	test_time;


/*
 * NOTE: This test should not be run with a window manager as some
 * window managers interfere with the test.
 */


static char	*WindowTree[]= {
	"toplevel",
		"child1 toplevel (10,10) 30x30",
		"child2 toplevel (50,50) 30x30",
			"grandchild child2 (2,2) 20x20",
};

static int NWindowTree = NELEM(WindowTree);



/* Value list for use in test t012 */
static int 	revert_tovallist[] = {
	RevertToParent,
	RevertToPointerRoot,
	RevertToNone,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	focus = 0;
	revert_to = RevertToNone;
	test_time = CurrentTime;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	focus = ErrdefWindow;
}

/*
 * Assertion XSetInputFocus-1.(A)
 * A call to XSetInputFocus changes the input focus window to
 * that specified in the focus argument.
 */
static void
t001()
{
Window	newfocus;
int 	newrevert;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	focus = defwin(display);

	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetInputFocus(display, &newfocus, &newrevert);
	if (newfocus != focus) {
		report("Focus window was 0x%x, expecting 0x%x", newfocus, focus);
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-2.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the focus argument is None, then all keyboard events
 * are discarded until a new focus window is set.
 */
static void
t002()
{
Window	win;
XEvent	ev;
int	n;
long	mask;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;
	else
		CHECK;

	win = defwin(display);
	XSelectInput(display, win, mask = KeyPressMask|KeyReleaseMask);
	focus = None;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	(void) warppointer(display, win, 2, 2);
	XSync(display, True); /* clear out event queue */
	keypress(display, getkeycode(display));
	if (n=getevent(display, &ev)) /* assignment intentional */ {
		report("Got %d events instead of 0, first was type %s.",
			n, eventname(ev.type));
		FAIL;
	} else
		CHECK;
	focus = win;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XSync(display, True); /* clear out event queue */
	relalldev();
	if (!XCheckWindowEvent(display, win, mask, &ev)) {
		report("Normal event processing not restored.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-3.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the focus argument is a window, then keyboard events
 * that would normally be reported to the focus window or one
 * of its inferiors are reported as usual and all other
 * keyboard events are reported relative to the focus window.
 */
static void
t003()
{
int	keycode;
XEvent	ev;
int	n;
long	mask;
Window	parent,child2,gchild;
Window	windows[6]; /* root + 4 + None stopper */
Window	root;
Window	*wp;
struct buildtree *tree;
char	*wname;
char	*evwname;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;
	else
		CHECK;
	wp = windows;
	*wp++ = root = DRW(display);
	*wp++ = parent = defwin(display);
	tree = buildtree(display, parent, WindowTree, NWindowTree);
	*wp++ = btntow(tree, "child1");
	*wp++ = child2 = btntow(tree, "child2");
	*wp++ = gchild = btntow(tree, "grandchild");
	*wp = None;

	keycode=getkeycode(display);
	mask = KeyPressMask|KeyReleaseMask;
	for(wp=windows; *wp != None; wp++)
		XSelectInput(display, *wp, mask);

	focus = child2;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	for(wp=windows; *wp != None; wp++) { /* around 5 times */
		(void)warppointer(display, *wp, 0,0);
		/* use 0,0 as window making stuff keeps away from there on
		 * root. All of our tree windows are not at 0,0 either
		 */
		XSync(display, True); /* clear out event queue */
		keypress(display, keycode);
		relalldev();
		if (!(wname = btwton(tree,*wp)))
			wname = (*wp == root) ? "ROOT" : "<Unknown>";
		if (!(n=XCheckMaskEvent(display, mask, &ev))) {
			report("No event received after keypress/release in window %s.", (*wp==focus)?"focus":wname);
			FAIL;
		} else
			CHECK;
		if (n && !(evwname = btwton(tree,ev.xany.window)))
			evwname = (ev.xany.window == root) ? "ROOT" :
				((ev.xany.window == None) ? "None" : "<Unknown>");
		if (*wp == focus || *wp == gchild) {
			if (n && ev.xany.window != *wp) {
				report("Event window was %s instead of %s for focus window or child.", evwname, wname);
				FAIL;
			} else
				CHECK;
		} else {
			if (n && ev.xany.window != focus) {
				report("Event window was %s instead of focus window.", evwname);
				FAIL;
			} else
				CHECK;
		}
	}

	CHECKPASS(1+2*5);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-4.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the focus argument is PointerRoot, then the focus
 * window is taken to be the root window of the screen the
 * pointer is on at each keyboard event.
 */
static void
t004()
{
Window	win, root, altroot;
int	keycode;
XEvent	ev;
long	mask;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;
	else
		CHECK;

	win = defwin(display);
	XSelectInput(display, win, mask = KeyPressMask|KeyReleaseMask);
	XSelectInput(display, root = DRW(display), mask);
	trace("Test with toplevel window and PointerRoot.");
	focus = PointerRoot;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	(void) warppointer(display, win, 2, 2);
	XSync(display, True); /* clear out event queue */
	keypress(display, keycode=getkeycode(display));
	if (!getevent(display, &ev)) {
		report("No event received.");
		FAIL;
	} else if (ev.type != KeyPress && ev.type != KeyRelease) {
		report("First event was of unexpected type: %s.", eventname(ev.type));
		FAIL;
	} else if (ev.xkey.window != win) {
		report("First event had unexpected window: 0x%x instead of 0x%x.",
			(unsigned)ev.xkey.window, (unsigned)win);
		FAIL;
	} else
		CHECK;
	trace("Test with root and PointerRoot.");
	(void) warppointer(display, root, 0,0);
	XSync(display, True); /* clear out event queue */
	relalldev();
	if (!getevent(display, &ev)) {
		report("No event received.");
		FAIL;
	} else if (ev.type != KeyPress && ev.type != KeyRelease) {
		report("First event was of unexpected type: %s.", eventname(ev.type));
		FAIL;
	} else if (ev.xkey.window != root) {
		report("First event had unexpected window: 0x%x instead of 0x%x.",
			(unsigned)ev.xkey.window, (unsigned)root);
		FAIL;
	} else
		CHECK;

	if (config.alt_screen != -1) {
		altroot = RootWindow(display, config.alt_screen);
		trace("Testing with root of alternate screen as source (0x%x) and PointerRoot.",
				(unsigned)altroot);
		XSelectInput(display, altroot, mask);
		(void) warppointer(display, altroot, 0,0);
		XSync(display, True); /* clear out event queue */
		keypress(display, keycode);
		relalldev();

		if (!getevent(display, &ev)) {
			report("No event received.");
			FAIL;
		} else if (ev.type != KeyPress && ev.type != KeyRelease) {
			report("First event was of unexpected type: %s.", eventname(ev.type));
			FAIL;
		} else if (ev.xkey.window != altroot) {
			report("First event had unexpected window: 0x%x instead of focus (altroot) 0x%x.",
				(unsigned)ev.xkey.window, (unsigned)altroot);
			FAIL;
		} else if (!ev.xkey.same_screen) {
			report("same_screen unexpectedly False.");
			FAIL;
		} else if (ev.xkey.root != altroot) {
			report("First event had unexpected root window: 0x%x instead of 0x%x.",
				(unsigned)ev.xkey.root, (unsigned)altroot);
			FAIL;
		} else
			CHECK;
		CHECKPASS(4);
	} else {
		report("Tested as far as possible with just one screen.");
		CHECKUNTESTED(3);
	}

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-5.(A)
 * When the focus window later becomes not viewable and
 * revert_to is RevertToParent, then the focus reverts to the
 * closest viewable ancestor of the focus window, the revert_to
 * value is changed to RevertToNone and FocusIn and FocusOut
 * events are generated.
 */
static void
t005()
{
Window	base;
XFocusInEvent	figood;
XFocusOutEvent	fogood;
XEvent	ev;
Window	newfocus;
int 	newrevert;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	focus = crechild(display, base, (struct area *)0);
	revert_to = RevertToParent;

	if (isdeleted())
		return;

	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSelectInput(display, base, FocusChangeMask);
	XSelectInput(display, focus, FocusChangeMask);

	XUnmapWindow(display, focus);
	XGetInputFocus(display, &newfocus, &newrevert);
	if (isdeleted())
		return;

	if (newfocus != base) {
		report("Focus window was 0x%x, expecting 0x%x", newfocus, base);
		FAIL;
	} else
		CHECK;

	if (newrevert != RevertToNone) {
		report("New revert_to value was %s, expecting RevertToNone",
			reverttoname(newrevert));
		FAIL;
	} else
		CHECK;

	/*
	 * Event testing. In this case there should be a FocusOut on the
	 * focus window, followed by a FocusIn on the base window.
	 */
	defsetevent(fogood, display, FocusOut);
	fogood.window = focus;
	fogood.mode = NotifyNormal;
	fogood.detail = NotifyAncestor;

	if (getevent(display, &ev) == 0 || ev.type != FocusOut) {
		report("Was expecting a FocusOut event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&fogood, &ev))
		FAIL;
	else
		CHECK;

	defsetevent(figood, display, FocusIn);
	figood.window = base;
	figood.mode = NotifyNormal;
	figood.detail = NotifyInferior;

	if (getevent(display, &ev) == 0 || ev.type != FocusIn) {
		report("Was expecting a FocusIn event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&figood, &ev))
		FAIL;
	else
		CHECK;

	CHECKPASS(6);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-6.(A)
 * When the focus window later becomes not viewable and
 * revert_to is RevertToPointerRoot, then the focus window
 * reverts to PointerRoot and FocusIn and FocusOut events are
 * generated.
 */
static void
t006()
{
Window	base;
XFocusInEvent	figood;
XFocusOutEvent	fogood;
XEvent	ev;
Window	newfocus;
int 	newrevert;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * Use a non-default display, because we are setting the event mask
	 * on the root window, which would mess up things for the next tests.
	 */
	display = opendisplay();

	base = defwin(display);
	focus = crechild(display, base, (struct area *)0);
	revert_to = RevertToPointerRoot;

	(void) warppointer(display, DRW(display), 0, 0);

	if (isdeleted())
		return;

	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSelectInput(display, base, FocusChangeMask);
	XSelectInput(display, focus, FocusChangeMask);
	XSelectInput(display, DRW(display), FocusChangeMask);

	XUnmapWindow(display, focus);
	XGetInputFocus(display, &newfocus, &newrevert);
	if (isdeleted())
		return;

	if (newfocus != PointerRoot) {
		report("Focus window was 0x%x, expecting 0x%x", newfocus, PointerRoot);
		FAIL;
	} else
		CHECK;

	if (newrevert != RevertToPointerRoot) {
		report("New revert_to value was %s, expecting RevertToPointerRoot",
			reverttoname(newrevert));
		FAIL;
	} else
		CHECK;

	/* FocusOut from focus window */
	defsetevent(fogood, display, FocusOut);
	fogood.window = focus;
	fogood.mode = NotifyNormal;
	fogood.detail = NotifyNonlinear;

	if (getevent(display, &ev) == 0 || ev.type != FocusOut) {
		report("Was expecting a FocusOut event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&fogood, &ev))
		FAIL;
	else
		CHECK;

	/* FocusOut on the base window */
	fogood.window = base;
	fogood.detail = NotifyNonlinearVirtual;

	if (getevent(display, &ev) == 0 || ev.type != FocusOut) {
		report("Was expecting a FocusOut event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&fogood, &ev))
		FAIL;
	else
		CHECK;
	
	/* FocusOut on the root window */
	fogood.window = DRW(display);

	if (getevent(display, &ev) == 0 || ev.type != FocusOut) {
		report("Was expecting a FocusOut event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&fogood, &ev))
		FAIL;
	else
		CHECK;
	
	/* FocusIn on the root window */
	defsetevent(figood, display, FocusIn);
	figood.window = DRW(display);
	figood.mode = NotifyNormal;
	figood.detail = NotifyPointerRoot;

	if (getevent(display, &ev) == 0 || ev.type != FocusIn) {
		report("Was expecting a FocusIn event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&figood, &ev))
		FAIL;
	else
		CHECK;

	figood.window = DRW(display);
	figood.detail = NotifyPointer;

	/* FocusIn for the pointer */
	if (getevent(display, &ev) == 0 || ev.type != FocusIn) {
		report("Was expecting a FocusIn event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&figood, &ev))
		FAIL;
	else
		CHECK;
	
	CHECKPASS(12);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-7.(A)
 * When the focus window later becomes not viewable and
 * revert_to is RevertToNone, then the focus window reverts to
 * None and FocusIn and FocusOut events are generated.
 */
static void
t007()
{
Window	base;
XFocusInEvent	figood;
XFocusOutEvent	fogood;
XEvent	ev;
Window	newfocus;
int 	newrevert;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * Use a non-default display, because we are setting the event mask
	 * on the root window, which would mess up things for the next tests.
	 */
	display = opendisplay();

	base = defwin(display);
	focus = crechild(display, base, (struct area *)0);
	revert_to = RevertToNone;

	(void) warppointer(display, DRW(display), 0, 0);

	if (isdeleted())
		return;

	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSelectInput(display, base, FocusChangeMask);
	XSelectInput(display, focus, FocusChangeMask);
	XSelectInput(display, DRW(display), FocusChangeMask);

	XUnmapWindow(display, focus);
	XGetInputFocus(display, &newfocus, &newrevert);
	if (isdeleted())
		return;

	if (newfocus != None) {
		report("Focus window was 0x%x, expecting None", newfocus);
		FAIL;
	} else
		CHECK;

	if (newrevert != RevertToNone) {
		report("New revert_to value was %s, expecting RevertToNone",
			reverttoname(newrevert));
		FAIL;
	} else
		CHECK;

	/* FocusOut from focus window */
	defsetevent(fogood, display, FocusOut);
	fogood.window = focus;
	fogood.mode = NotifyNormal;
	fogood.detail = NotifyNonlinear;

	if (getevent(display, &ev) == 0 || ev.type != FocusOut) {
		report("Was expecting a FocusOut event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&fogood, &ev))
		FAIL;
	else
		CHECK;

	/* FocusOut on the base window */
	fogood.window = base;
	fogood.detail = NotifyNonlinearVirtual;

	if (getevent(display, &ev) == 0 || ev.type != FocusOut) {
		report("Was expecting a FocusOut event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&fogood, &ev))
		FAIL;
	else
		CHECK;
	
	/* FocusOut on the root window */
	fogood.window = DRW(display);

	if (getevent(display, &ev) == 0 || ev.type != FocusOut) {
		report("Was expecting a FocusOut event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&fogood, &ev))
		FAIL;
	else
		CHECK;
	
	/* FocusIn on the root window */
	defsetevent(figood, display, FocusIn);
	figood.window = DRW(display);
	figood.mode = NotifyNormal;
	figood.detail = NotifyDetailNone;

	if (getevent(display, &ev) == 0 || ev.type != FocusIn) {
		report("Was expecting a FocusIn event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&figood, &ev))
		FAIL;
	else
		CHECK;

	CHECKPASS(10);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-8.(A)
 * When the specified time is earlier than the current
 * last-focus-change time or is later than the current X server
 * time, then a call to XSetInputFocus has no effect.
 */
static void
t008()
{
Window	win;
Window	newfocus;
int 	newrevert;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	win = defwin(display);

	test_time = gettime(display);
	focus = None;

	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	test_time -= 12;
	focus = win;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetInputFocus(display, &newfocus, &newrevert);
	if (newfocus == None)
		CHECK;
	else {
		report("Focus was changed when time was earlier than last-focus-change time");
		FAIL;
	}

	test_time = gettime(display);
	test_time += ((config.speedfactor+1) * 1000000);

	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetInputFocus(display, &newfocus, &newrevert);
	if (newfocus == None)
		CHECK;
	else {
		report("Focus was changed when time was later than current X server time");
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-9.(A)
 * A successful call to XSetInputFocus sets the
 * last-focus-change time to the specified time with
 * CurrentTime being replaced by the current X server time.
 */
static void
t009()
{
Time	t1,t2;
Window	win;
Window	focus_return;
int	junk;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	win = defwin(display);
	XSync(display, True);
	t1 = gettime(display);
	if (t1 == CurrentTime) {
		delete("Could not get server time.");
		return;
	} else
		CHECK;

	test_time = t1;
	focus = win;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetInputFocus(display, &focus_return, &junk);
	if (focus_return != win) {
		report("Failed to change focus with time = 0x%lx.", (unsigned long)test_time);
		FAIL;
	} else
		CHECK;
	trace("Focus set at time 0x%lx.",(unsigned long)test_time);
	test_time--;
	focus = DRW(display);
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XGetInputFocus(display, &focus_return, &junk);
	if (focus_return != win) {
		report("Last focus change time set earlier than specified time.");
		FAIL;
	} else
		CHECK;
	test_time = t1;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XGetInputFocus(display, &focus_return, &junk);
	if (focus_return != focus) {
		report("Last focus change time set later than specified time.");
		FAIL;
	} else
		CHECK;

	/* last despairing attempt */
	test_time = CurrentTime;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XGetInputFocus(display, &focus_return, &junk);
	if (focus_return != focus) {
		delete("Cannot restore focus to perform CurrentTime tests.");
		return;
	} else
		CHECK;

	XSync(display, True);
	t1 = gettime(display);
	if (t1 == CurrentTime) {
		delete("Could not get earlier server time.");
		return;
	} else
		CHECK;

	test_time = CurrentTime;
	focus = win;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XGetInputFocus(display, &focus_return, &junk);
	if (focus_return != focus) {
		delete("Failed to change focus at CurrentTime.");
		return;
	} else
		CHECK;
	/* now set up OK */
	t2 = gettime(display);
	if (t2 == CurrentTime) {
		delete("Could not get later server time.");
		return;
	} else
		CHECK;

	trace("Focus changed at time between 0x%lx and 0x%lx (diff = %d).",t1,t2,t2-t1);
	test_time = t1;
	focus = DRW(display);
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XGetInputFocus(display, &focus_return, &junk);
	if (focus_return != win) {
		report("Last focus change time set earlier than 0x%lx.",t1);
		FAIL;
	} else
		CHECK;
	test_time = t2;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XGetInputFocus(display, &focus_return, &junk);
	if (focus_return != focus) {
		report("Last focus change time set later than 0x%lx.", t2);
		FAIL;
	} else
		CHECK;

	CHECKPASS(10);
	
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-10.(A)
 * When the focus changes, then FocusIn and FocusOut events
 * are generated.
 */
static void
t010()
{
Window	base;
Window	ch1;
Window	ch2;
struct	area	area;
XFocusInEvent	figood;
XFocusOutEvent	fogood;
XEvent	ev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	setarea(&area, 0, 0, 2, 2);
	ch1 = crechild(display, base, &area);
	setarea(&area, 20, 20, 2, 2);
	ch2 = crechild(display, base, &area);

	focus = ch1;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XSelectInput(display, ch1, FocusChangeMask);
	XSelectInput(display, ch2, FocusChangeMask);
	XSelectInput(display, base, FocusChangeMask);

	focus = ch2;
	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* FocusOut from old focus window, ch1 */
	defsetevent(fogood, display, FocusOut);
	fogood.window = ch1;
	fogood.mode = NotifyNormal;
	fogood.detail = NotifyNonlinear;

	if (getevent(display, &ev) == 0 || ev.type != FocusOut) {
		report("Was expecting a FocusOut event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&fogood, &ev))
		FAIL;
	else
		CHECK;

	/* FocusIn on the new focus window, ch2 */
	defsetevent(figood, display, FocusIn);
	figood.window = ch2;
	figood.mode = NotifyNormal;
	figood.detail = NotifyNonlinear;

	if (getevent(display, &ev) == 0 || ev.type != FocusIn) {
		report("Was expecting a FocusIn event");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&figood, &ev))
		FAIL;
	else
		CHECK;

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-11.(A)
 * When the specified focus window is not viewable, then a
 * BadMatch error occurs.
 */
static void
t011()
{
Window	base;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	base = defwin(display);
	focus = creunmapchild(display, base, (struct area *)0);

	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;	/* Already done */
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetInputFocus-12.(A)
 * When the value of revert_to is other than RevertToParent,
 * RevertToPointerRoot or RevertToNone, then a BadValue error
 * occurs.
 */
/* Including from file error/EVal.mc */
static void
t012()
{
#undef	VALUE_ARG
#define	VALUE_ARG revert_to
#undef	VALUE_LIST
#define	VALUE_LIST revert_tovallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XSetInputFocus(display, focus, revert_to, test_time);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XSetInputFocus-13.(A)
 * When a window argument does not name a valid Window,
 * PointerRoot or None, then a BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t013()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetInputFocus(display, focus, revert_to, test_time);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = focusstartup;
void	(*tet_cleanup)() = focuscleanup;
