/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stscrnsvr.m,v 1.5 92/06/11 17:16:55 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures)
 *   Display	*display;
 *   int 	timeout;
 *   int 	interval;
 *   int 	prefer_blanking;
 *   int 	allow_exposures;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetScreenSaver	1
char    *TestName = "XSetScreenSaver";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XSetScreenSaver function
 */
static Display	*display;
static int 	timeout;
static int 	interval;
static int 	prefer_blanking;
static int 	allow_exposures;



static	int 	origt;
static	int 	origi;
static	int 	origpb;
static	int 	origae;

static void
savesaver()
{
	startup();
	if(Dsp)
		XGetScreenSaver(Dsp, &origt, &origi, &origpb, &origae);
}

static void
resetsaver()
{
	if(Dsp)
		XSetScreenSaver(Dsp, origt, origi, origpb, origae);
	cleanup();
}



/* Value list for use in test t016 */
static int 	prefer_blankingvallist[] = {
	DontPreferBlanking,
	PreferBlanking,
	DefaultBlanking,
};



/* Value list for use in test t017 */
static int 	allow_exposuresvallist[] = {
	DontAllowExposures,
	AllowExposures,
	DefaultExposures,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	timeout = 34;
	interval = 12;
	prefer_blanking = PreferBlanking;
	allow_exposures = AllowExposures;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetScreenSaver-1.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the timeout argument is greater than zero, then the
 * screen saver is enabled and the value of timeout specifies
 * the time in seconds, until the screen saver is activated.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	timeout = 10;

	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test with timeout greater than zero");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-2.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When timeout is 0, then the screen saver is disabled.
 */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	timeout = 0;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test for timeout zero");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-3.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When timeout is 0 and the screen saver is activated, then a
 * call to XSetScreenSaver does not deactivate the screen
 * saver.
 */
static void
t003()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSetScreenSaver-4.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When timeout is -1, then the default timeout is restored.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	timeout = -1;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test for timeout of -1");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-5.(B)
 *   Reason: There is no known reliable test method for this assertion
 * If the server-dependent screen saver method uses periodic
 * change: The interval argument serves as a hint about how
 * long the change period should be, and zero hints that no
 * periodic change should be made.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	interval = 3;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	interval = 0;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test for interval, including zero");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-6.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When interval is -1, then the default interval is restored.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	interval = -1;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test for default interval");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-7.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When no input from devices is generated for the specified
 * number of timeout seconds once the screen saver is enabled,
 * then the screen saver is activated.
 */
static void
t007()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSetScreenSaver-8.(D)
 *   Reason: There is no known reliable test method for this assertion
 * If the hardware supports video blanking: When
 * prefer_blanking is PreferBlanking and the screen saver is
 * subsequently activated, then the screen goes blank.
 */
static void
t008()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	prefer_blanking = PreferBlanking;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test for PreferBlanking");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-9.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When prefer_blanking is DontPreferBlanking and
 * allow_exposures is AllowExposures or the screen can be
 * regenerated without sending Expose events and the screen
 * saver is subsequently activated, then the screen is altered
 * in an implementation defined way that avoids phosphor burn.
 */
static void
t009()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	prefer_blanking = DontPreferBlanking;
	allow_exposures = AllowExposures;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test for DontPreferBlanking");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-10.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When prefer_blanking is DefaultBlanking, then the default
 * value for the server is restored.
 */
static void
t010()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	prefer_blanking = DefaultBlanking;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test for DefaultBlanking");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-11.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When allow_exposures is DontAllowExposures, and the screen
 * can not be blanked or regenerated without sending Expose
 * events and the screen saver would subsequently be activated,
 * then the screen's state does not change and the screen saver
 * is not activated.
 */
static void
t011()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	allow_exposures = DontAllowExposures;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test for DontAllowExposures");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-12.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When allow_exposures is DefaultExposures, then the servers
 * default value is restored.
 */
static void
t012()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	allow_exposures = DefaultExposures;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	untested("Touch test for DefaultExposures");
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-13.(B)
 *   Reason: There is no known reliable test method for this assertion
 * The screen saver is deactivated, and all screen states are
 * restored at the next keyboard or pointer input or at the
 * next call to XForceScreenSaver with mode ScreenSaverReset.
 */
static void
t013()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSetScreenSaver-14.(A)
 * When timeout is a negative number other than -1, then a
 * BadValue error occurs.
 */
static void
t014()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	timeout = -2;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-15.(A)
 * When interval is a negative number other than -1, then a
 * BadValue error occurs.
 */
static void
t015()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	interval = -2;
	startcall(display);
	if (isdeleted())
		return;
	XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetScreenSaver-16.(A)
 * When the value of prefer_blanking is other than
 * DontPreferBlanking, PreferBlanking or DefaultBlanking,
 * then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t016()
{
#undef	VALUE_ARG
#define	VALUE_ARG prefer_blanking
#undef	VALUE_LIST
#define	VALUE_LIST prefer_blankingvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/*
 * Assertion XSetScreenSaver-17.(A)
 * When the value of allow_exposures is other than
 * DontAllowExposures, AllowExposures or DefaultExposures,
 * then a BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t017()
{
#undef	VALUE_ARG
#define	VALUE_ARG allow_exposures
#undef	VALUE_LIST
#define	VALUE_LIST allow_exposuresvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XSetScreenSaver(display, timeout, interval, prefer_blanking, allow_exposures);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = savesaver;
void	(*tet_cleanup)() = resetsaver;
