/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ungrbbttn.m,v 1.6 92/06/11 17:17:03 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XUngrabButton(display, button, modifiers, grab_window)
 *   Display	*display;
 *   unsigned int 	button;
 *   unsigned int 	modifiers;
 *   Window	grab_window;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XUngrabButton	1
char    *TestName = "XUngrabButton";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW grab_window
#define A_DRAWABLE grab_window


/*
 * Arguments to the XUngrabButton function
 */
static Display	*display;
static unsigned int 	button;
static unsigned int 	modifiers;
static Window	grab_window;



/*
 * Returns True if the pointer is grabbed.  This is not a general purpose
 * routine since it knows about the Grab Button args.
 * When the pointer is grabbed then pointer events are not reported to
 * none grabbing clients.
 */
static
pgrabbed()
{
Display	*client2;
XEvent	ev;

	client2 = opendisplay();
	
	XSelectInput(client2, grab_window, PointerMotionMask|EnterWindowMask);
	/* Flush events for client2 */
	XSync(client2, True);

	/*
	 * Ensure that pointer either enters or moves within grab_window.
	 */
	(void) warppointer(display, grab_window, 1, 1);
	(void) warppointer(display, grab_window, 1, 2);

	XSync(client2, False);
	if (getevent(client2, &ev)) {
		/*
		 * An event was reported - pointer isn't grabbed, do a sanity
		 * check on the type of event.
		 */
		if (ev.type != MotionNotify && ev.type != EnterNotify) {
			delete("Unexpected event received in pgrabbed()");
			delete("  event type %s", eventname(ev.type));
		}
		return(False);
	} else {
		return(True);
	}
}

#define	ACTPOSX	3
#define	ACTPOSY	6
/*
 * Warp to the grab_window and press the keys in modifiers and then press
 * the button in the 'button' arg.
 * (This activates the previously set up grab if the arg variables
 * have not been changed.)
 */
static
activate()
{
	(void) warppointer(display, grab_window, ACTPOSX, ACTPOSY);
	if (modifiers)
		modpress(display, modifiers);
	buttonpress(display, button);
}



/* Value list for use in test t005 */
static unsigned long	modifiersvallist[] = {
	ShiftMask,
	LockMask,
	ControlMask,
	Mod1Mask,
	Mod2Mask,
	Mod3Mask,
	Mod4Mask,
	Mod5Mask,
	AnyModifier,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	button = Button1;
	modifiers = 0;
	grab_window = defwin(display);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XUngrabButton-1.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When a passive grab for the specified button/key
 * combination on the specified window exists and it was
 * grabbed by this client, then a call to XUngrabButton
 * releases the grab.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	XGrabButton(display, button, modifiers, grab_window,
		False, PointerMotionMask, GrabModeAsync, GrabModeAsync,
		None, None);

	if (isdeleted())
		return;


	if (noext(1)) {
		startcall(display);
		if (isdeleted())
			return;
		XUngrabButton(display, button, modifiers, grab_window);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		untested("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;

	activate();
	if (pgrabbed())
		CHECK;
	else {
		delete("Could not set up grab");
		return;
	}

	relbuttons();

	startcall(display);
	if (isdeleted())
		return;
	XUngrabButton(display, button, modifiers, grab_window);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	activate();
	if (pgrabbed()) {
		report("Grab was not removed");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUngrabButton-2.(B)
 *   Reason: There is no known reliable test method for this assertion
 * A modifiers argument of AnyModifier is equivalent to
 * issuing the ungrab request for all possible modifier
 * combinations, including no modifiers.
 */
static void
t002()
{
unsigned int 	mods;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(1)) {
		modifiers = AnyModifier;
		startcall(display);
		if (isdeleted())
			return;
		XUngrabButton(display, button, modifiers, grab_window);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		untested("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;

	mods = wantmods(display, 3);

	/* Try a few modifiers */
	modifiers = mods;
	XGrabButton(display, button, modifiers, grab_window,
		False, PointerMotionMask, GrabModeAsync, GrabModeAsync,
		None, None);

	/* Try no modifiers */
	modifiers = 0;
	XGrabButton(display, button, modifiers, grab_window,
		False, PointerMotionMask, GrabModeAsync, GrabModeAsync,
		None, None);

	modifiers = AnyModifier;
	startcall(display);
	if (isdeleted())
		return;
	XUngrabButton(display, button, modifiers, grab_window);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* Try to activate with modifier combination */
	modifiers = mods;
	activate();
	if (pgrabbed()) {
		report("Grab with modifiers %s was not released", keymaskname(mods));
		FAIL;
	} else
		CHECK;

	/* Try to activate with no modifiers */
	modifiers = 0;
	activate();
	if (pgrabbed()) {
		report("Grab with no modifiers was not released");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUngrabButton-3.(B)
 *   Reason: There is no known reliable test method for this assertion
 * A button argument of AnyButton is equivalent to issuing the
 * request for all possible buttons.
 */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(1)) {
		button = AnyButton;
		startcall(display);
		if (isdeleted())
			return;
		XUngrabButton(display, button, modifiers, grab_window);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		report("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;

	button = Button1;
	XGrabButton(display, button, modifiers, grab_window,
		False, PointerMotionMask, GrabModeAsync, GrabModeAsync,
		None, None);

	if (nbuttons() > 1)
		button = Button2;
	XGrabButton(display, button, modifiers, grab_window,
		False, PointerMotionMask, GrabModeAsync, GrabModeAsync,
		None, None);

	button = AnyButton;
	startcall(display);
	if (isdeleted())
		return;
	XUngrabButton(display, button, modifiers, grab_window);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* Check the first grab */
	button = Button1;
	activate();

	if (pgrabbed()) {
		report("Grab on button1 was not released with AnyButton");
		FAIL;
	} else
		CHECK;

	if (nbuttons() > 1) {
		button = Button2;
		activate();
		if (pgrabbed()) {
			report("Grab on button2 was not released with AnyButton");
			FAIL;
		} else
			CHECK;
	} else
		CHECK;

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUngrabButton-4.(B)
 *   Reason: There is no known reliable test method for this assertion
 * A call to XUngrabButton has no effect on an active grab.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(1))
		return;

	XGrabButton(display, button, modifiers, grab_window,
		False, PointerMotionMask, GrabModeAsync, GrabModeAsync,
		None, None);

	activate();

	startcall(display);
	if (isdeleted())
		return;
	XUngrabButton(display, button, modifiers, grab_window);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (pgrabbed())
		CHECK;
	else {
		report("A call to %s released already active grab", TestName);
		FAIL;
	}

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUngrabButton-5.(A)
 * When the value of modifiers is not a bitwise combination of
 * ShiftMask, LockMask, ControlMask, Mod1Mask, Mod2Mask,
 * Mod3Mask, Mod4Mask, Mod5Mask or AnyModifier, then a
 * BadValue error occurs.
 */
/* Including from file error/EVal.mc */
static void
t005()
{
#undef	VALUE_ARG
#define	VALUE_ARG modifiers
#undef	VALUE_LIST
#define	VALUE_LIST modifiersvallist
#undef NOTMEMTYPE
#define NOTMEMTYPE unsigned
#undef	NOTMEMBER
#define	NOTMEMBER notmaskmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XUngrabButton(display, button, modifiers, grab_window);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XUngrabButton-6.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XUngrabButton(display, button, modifiers, grab_window);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
