/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: wrppntr.m,v 1.4 92/06/11 17:17:53 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y)
 *   Display	*display;
 *   Window	src_w;
 *   Window	dest_w;
 *   int 	src_x;
 *   int 	src_y;
 *   unsigned int 	src_width;
 *   unsigned int 	src_height;
 *   int 	dest_x;
 *   int 	dest_y;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XWarpPointer	1
char    *TestName = "XWarpPointer";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW src_w
#define A_DRAWABLE src_w
#define A_WINDOW2 dest_w
#define A_DRAWABLE2 dest_w


/*
 * Arguments to the XWarpPointer function
 */
static Display	*display;
static Window	src_w;
static Window	dest_w;
static int 	src_x;
static int 	src_y;
static unsigned int 	src_width;
static unsigned int 	src_height;
static int 	dest_x;
static int 	dest_y;



#define	SRC_X	7
#define	SRC_Y	10
#define	SRC_WIDTH	20
#define	SRC_HEIGHT	31
#define	DEST_X	11
#define	DEST_Y	13

#define	POS_ABS	1
#define	POS_REL	2

struct	coord {
	int 	x;
	int 	y;
};

/*
 * Get the current position of the pointer.  If type is POS_REL this
 * is relative to the given window.  For a type POS_ABS the coordinates
 * are absolute.
 */
static void
getpos(w, co, type)
Window	w;
struct	coord	*co;
int 	type;
{
Window	root, child;
unsigned int 	mask;
int 	xroot, yroot;
int 	status;

	if (w == None)
		w = DRW(Dsp);

	status = XQueryPointer(Dsp, w, &root, &child, &xroot, &yroot, &co->x, &co->y, &mask);
	if (status == False) {
		delete("Could not get pointer position");
		return;
	}

	if (type == POS_ABS) {
		co->x = xroot;
		co->y = yroot;
	}
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	src_w = 0;
	dest_w = 0;
	src_x = SRC_X;
	src_y = SRC_Y;
	src_width = SRC_WIDTH;
	src_height = SRC_HEIGHT;
	dest_x = DEST_X;
	dest_y = DEST_Y;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	src_w = ErrdefWindow;
	dest_w = ErrdefWindow;
}

/*
 * Assertion XWarpPointer-1.(A)
 * When dest_w is None, then a call to XWarpPointer moves the
 * pointer by the offsets dest_x, dest_y relative to the
 * current position of the pointer.
 */
static void
t001()
{
struct	coord	old;
struct	coord	new;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	(void) warppointer(display, DRW(display), 100, 100);

	dest_w = None;
	src_w = None;

	getpos(None, &old, POS_ABS);

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	getpos(None, &new, POS_ABS);

	if (dest_x == 0 || dest_y == 0)
		delete("Setup error of destination coordinates");

	if (new.x == old.x + dest_x)
		CHECK;
	else {
		report("X position of pointer incorrect - was %d, expecting %d",
			new.x, old.x + dest_x);
		FAIL;
	}

	if (new.y == old.y + dest_y)
		CHECK;
	else {
		report("Y position of pointer incorrect - was %d, expecting %d",
			new.y, old.y + dest_y);
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWarpPointer-2.(A)
 * When dest_w is a window and src_w is None, then a call to
 * XWarpPointer moves the pointer to the offsets dest_x,
 * dest_y relative to the origin of dest_w.
 */
static void
t002()
{
struct	coord	pos;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	(void) warppointer(display, DRW(display), 0, 0);

	dest_w = defwin(display);
	src_w = None;

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	getpos(dest_w, &pos, POS_REL);

	if (dest_x == pos.x && dest_y == pos.y)
		CHECK;
	else {
		report("Position relative to destination window was incorrect");
		report("  position was (%d, %d), expecting (%d, %d)",
			pos.x, pos.y, dest_x, dest_y);
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWarpPointer-3.(A)
 * When dest_w and src_w are windows and the pointer is within
 * the specified rectangle of src_w, then a call to
 * XWarpPointer moves the pointer to the offsets dest_x,
 * dest_y relative to the origin of dest_w.
 */
static void
t003()
{
struct	coord	pos;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	dest_w = defwin(display);
	src_w = defwin(display);

	(void) warppointer(display, src_w, src_x, src_y);

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	getpos(dest_w, &pos, POS_REL);

	if (pos.x == dest_x && pos.y == dest_y)
		CHECK;
	else {
		report("Pointer not positioned correctly when pointer in source rectangle");
		report("  position was (%d, %d), expecting (%d, %d)",
			pos.x, pos.y, dest_x, dest_y);
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWarpPointer-4.(A)
 * When dest_w and src_w are windows and the pointer is not
 * within the specified rectangle of src_w, then a call to
 * XWarpPointer does not move the pointer.
 */
static void
t004()
{
PointerPlace	*pplace;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	dest_w = defwin(display);
	src_w = defwin(display);

	pplace = warppointer(display, src_w, (int)(src_x+src_width+2), src_y);

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (pointermoved(display, pplace)) {
		report("Pointer was moved when outside the source rectangle");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWarpPointer-5.(A)
 * When src_height is zero, then it is replaced with the
 * current height of src_w minus src_y.
 */
static void
t005()
{
unsigned int 	width, height;
struct	coord	pos;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	src_w = defwin(display);
	dest_w = defwin(display);
	src_height = 0;

	getsize(display, src_w, &width, &height);

	(void) warppointer(display, src_w, src_x+3, (int)height-3);

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	getpos(dest_w, &pos, POS_REL);
	if (pos.x == dest_x && pos.y == dest_y)
		CHECK;
	else {
		report("Pointer not moved correctly when pointer in source rectangle");
		report("  and src_height 0 - position was (%d, %d), expecting (%d, %d)",
			pos.x, pos.y, dest_x, dest_y);
		FAIL;
	}

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWarpPointer-6.(A)
 * When src_width is zero, then it is replaced with the
 * current width of src_w minus src_x.
 */
static void
t006()
{
unsigned int 	width, height;
struct	coord	pos;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	src_w = defwin(display);
	dest_w = defwin(display);
	src_width = 0;

	getsize(display, src_w, &width, &height);

	(void) warppointer(display, src_w, (int)width-3, src_y+3);

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	getpos(dest_w, &pos, POS_REL);
	if (pos.x == dest_x && pos.y == dest_y)
		CHECK;
	else {
		report("Pointer not moved correctly when pointer in source rectangle");
		report("  and src_width 0 - position was (%d, %d), expecting (%d, %d)",
			pos.x, pos.y, dest_x, dest_y);
		FAIL;
	}

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWarpPointer-7.(A)
 * A call to XWarpPointer generates events as though the
 * pointer position had been instantaneously moved by the user.
 */
static void
t007()
{
XEvent	ev;
XMotionEvent	good;
struct	coord	pos;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	dest_w = defwin(display);
	src_w = None;

	(void) warppointer(display, dest_w, 0, 0);

	XSelectInput(display, dest_w, PointerMotionMask);

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	defsetevent(good, display, MotionNotify);
	good.window = dest_w;
	good.root = DRW(display);
	good.subwindow = None;
	good.time = 0;
	good.x = dest_x;
	good.y = dest_y;
	good.state = 0;
	good.is_hint = False;
	good.same_screen = True;

	getpos(None, &pos, POS_ABS);
	good.x_root = pos.x;
	good.y_root = pos.y;

	n = getevent(display, &ev);
	if (n > 1) {
		report("More than one event generated");
		FAIL;
	} else
		CHECK;

	if (checkevent((XEvent*)&good, &ev))
		FAIL;
	else
		CHECK;
	
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWarpPointer-8.(A)
 * When an active pointer grab is in progress and the pointer
 * is within the confine_to window, then a call to XWarpPointer
 * will only move the pointer as far as the closest edge of the
 * confine_to window.
 */
static void
t008()
{
Window	confine;
Window	grabwin;
struct	coord	pos;
unsigned int 	width, height;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	confine = defwin(display);
	dest_w = defwin(display);
	grabwin = defwin(display);
	src_w = None;

	/* Set border to zero so that containment size is just width and height */
	XSetWindowBorderWidth(display, confine, 0);
	getsize(display, confine, &width, &height);

	/* May as well put the pointer there to begin with */
	(void) warppointer(display, confine, 1, 1);

	XGrabPointer(display, grabwin, False, 0, GrabModeAsync, GrabModeAsync,
		confine, None, CurrentTime);

	if (isdeleted())
		return;

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XUngrabPointer(display, CurrentTime);
	getpos(confine, &pos, POS_REL);
	if (pos.x > 0 && pos.x < width && pos.y > 0 && pos.y < height)
		CHECK;
	else {
		report("Pointer did not remain within the confine_to window");
		FAIL;
	}

	/*
	 * We can also check that the pointer is on a edge.  (The windows
	 * do not overlap).
	 */
	if (pos.x == 0 || pos.y == 0 || pos.x == width-1 || pos.y == height-1)
		CHECK;
	else {
		report("Pointer was not on edge of confine_to window");
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XWarpPointer-9.(A)
 * When a window argument does not name a valid Window or None,
 * then a BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t009()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	seterrdef();
	A_WINDOW2 = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XWarpPointer(display, src_w, dest_w, src_x, src_y, src_width, src_height, dest_x, dest_y);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
