/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: chckifevnt.m,v 1.10 92/06/11 17:18:33 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Bool
 *   XCheckIfEvent(display, event_return, predicate, arg)
 *   Display *display;
 *   XEvent	*event_return;
 *   Predicate predicate;
 *   char *arg;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCheckIfEvent	1
char    *TestName = "XCheckIfEvent";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XCheckIfEvent function
 */
static Display *display;
static XEvent	*event_return;
static Predicate predicate;
static char *arg;


/*
 * Can not use "xcall" because it empties the event queue.
 */
#define	_xcall_(rvalue)	\
		_startcall(display);\
		rvalue = XCheckIfEvent(display, event_return, predicate, arg);\
		_endcall(display)
static XEvent _event;

/*
 * _predicate - a predicate procedure
 *
 * _predicate returns True only after it has been invoked the specified
 * number of times (_pred_max).  _pred_cnt is used to keep track of the
 * number of invocations.  _pred_retval always contains the previous
 * _predicate return value.  _pred_true is set to True if _predicate is
 * ever invoked while _pred_retval is set to True.  _pred_event contains
 * a copy of the event most recently passed to _predicate.
 *
 * These variables should be initialized by a call to PRED_SETUP() prior
 * to (indirectly!) invoking the predicate procedure.  PRED_SETUP takes
 * an argument which specifies at which invocation _predicate should
 * start returning True.
 */
static int _pred_max;
static int _pred_cnt;		/* _predicate invocation counter */
static XEvent _pred_event;	/* last event passed to _predicate */
static int _pred_retval;	/* last returnvalue from _predicate */
static int _pred_true;		/* True when True previously returned */

static int
_predicate (display, event, arg)
Display *display;
XEvent *event;
char *arg;
{
#ifdef	lint
	XCloseDisplay(display);
	*arg = '\0';
#endif
	_pred_event = *event;
	if (_pred_retval == True)
		_pred_true = True;
	_pred_retval = ((++_pred_cnt >= _pred_max) ? True : False);
	return(_pred_retval);
}

#define	PRED_SETUP(max)	\
		_pred_max = (max);\
		_pred_cnt = 0;\
		_pred_retval = False;\
		_pred_true = False


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	event_return = &_event;
	predicate = _predicate;
	arg = (char *) 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XCheckIfEvent-1.(A)
 * A call to XCheckIfEvent calls predicate once for each event
 * in the event queue until predicate returns True.
 */
static void
t001()
{
XEvent	event;
int	callcnt;
int	returnvalue;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = KeyPress;
	XPutBackEvent(display, &event);
	event.type = KeyRelease;
	XPutBackEvent(display, &event);
	event.type = ButtonPress;
	XPutBackEvent(display, &event);
/* Set up predicate procedure. */
	PRED_SETUP(callcnt = 2);
/* Call XCheckIfEvent. */
	_xcall_(returnvalue);
/* Verify that predicate was called the correct number of times. */
	if (_pred_cnt != callcnt) {
		report("predicate called %d times, expected %d", _pred_cnt, callcnt);
		FAIL;
	}
	else
		CHECK;
/* Verify that predicate returned True at most recent invocation. */
	if (_pred_retval != True) {
		report("predicate returned %d, expecting %d", _pred_retval, True);
		FAIL;
	}
	else
		CHECK;
/* Verify that XCheckIfEvent did not continue to call predicate */
/* after predicate returned True. */
	if (_pred_true == True) {	
		report("Did not return when predicate returned True.");
		FAIL;
	}
	else
		CHECK;
	/* empty event queue */
	XSync(display, True);
	
	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCheckIfEvent-2.(A)
 * When a call to XCheckIfEvent finds a matching event, then
 * XCheckIfEvent True.
 */
static void
t002()
{
XEvent	event;
int	callcnt;
int	returnvalue;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = KeyPress;
	XPutBackEvent(display, &event);
	event.type = KeyRelease;
	XPutBackEvent(display, &event);
	event.type = ButtonPress;
	XPutBackEvent(display, &event);
/* Set up predicate procedure. */
	PRED_SETUP(callcnt = 2);
/* Call XCheckIfEvent. */
	_xcall_(returnvalue);
/* Verify that predicate found a matching event. */
	if (_pred_cnt != callcnt) {
		report("predicate called %d times, expected %d", _pred_cnt, callcnt);
		FAIL;
	}
	else
		CHECK;
	if (_pred_retval != True) {
		report("predicate returned %d, expecting %d", _pred_retval, True);
		FAIL;
	}
	else
		CHECK;
/* Verify that XCheckIfEvent returned True. */
	if (returnvalue != True) {
		report("Returned %d, expecting %d", returnvalue, True);
		FAIL;
	}
	else
		CHECK;
	/* empty event queue */
	XSync(display, True);
	
	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCheckIfEvent-3.(A)
 * A call to XCheckIfEvent removes the returned event from the
 * event queue.
 */
static void
t003()
{
XEvent	event;
XEvent	nextevent;
int	returnvalue;
int	callcnt;
int	oldqsize;
int	newqsize;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = KeyPress;
	XPutBackEvent(display, &event);
	event.type = KeyRelease;
	XPutBackEvent(display, &event);
	event.type = ButtonPress;
	XPutBackEvent(display, &event);
/* Call XPending to get the current event queue size. */
	oldqsize = XPending(display);
/* Set up predicate procedure. */
	PRED_SETUP(callcnt = 1);
/* Call XCheckIfEvent. */
	_xcall_(returnvalue);
/* Verify that XCheckIfEvent returned True. */
	if (returnvalue != True) {	
		report("Returned %d, expecting %d", _pred_retval, True);
		FAIL;
	}
	else
		CHECK;
/* Call XPending to get the current event queue size. */
	newqsize = XPending(display);
/* Verify that size of the event queue has decreased by one. */
	if (newqsize != (oldqsize-1)) {
		report("Event queue size %d, expected %d", newqsize, oldqsize-1);
		FAIL;
	}
	else
		CHECK;
/* Verify that the returned event was removed from the event queue. */
	XNextEvent(display, &nextevent);
	if (event_return->type == nextevent.type) {
		report("Returned %s, expected %s", eventname(event_return->type), eventname(ButtonPress));
		FAIL;
	}
	else
		CHECK;

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCheckIfEvent-4.(A)
 * When on a call to XCheckIfEvent predicate returns True,
 * then XCheckIfEvent returns the event passed to predicate in
 * event_return.
 */
static void
t004()
{
XEvent	event;
int	callcnt;
int	returnvalue;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = KeyPress;
	XPutBackEvent(display, &event);
	event.type = KeyRelease;
	XPutBackEvent(display, &event);
	event.type = ButtonPress;
	XPutBackEvent(display, &event);
/* Set up predicate procedure. */
	PRED_SETUP(callcnt = 2);
/* Call XCheckIfEvent. */
	_xcall_(returnvalue);
/* Verify that predicate returned True at most recent invocation. */
	if (_pred_retval != True) {
		report("predicate returned %d, expecting %d", _pred_retval, True);
		FAIL;
	}
	else
		CHECK;
/* Verify that XCheckIfEvent returned True. */
	if (returnvalue != True) {	
		report("Returned %d, expecting %d", _pred_retval, True);
		FAIL;
	}
	else
		CHECK;
/* Verify that event_return is the same as the event passed to predicate. */
	if (event_return->type != _pred_event.type) {	
		report("Returned %s, expected %s", eventname(event_return->type), eventname(_pred_event.type));
		FAIL;
	}
	else
		CHECK;
/* Verify that event_return is the expected event. */
	if (event_return->type != KeyRelease) {	
		report("Returned %s, expected %s", eventname(event_return->type), eventname(KeyRelease));
		FAIL;
	}
	else
		CHECK;
/* Verify that XIfEvent did not continue to call predicate */
/* after predicate returned True. */
	if (_pred_true == True) {	
		report("Did not return when predicate returned True.");
		FAIL;
	}
	else
		CHECK;
	/* empty event queue */
	XSync(display, True);

	CHECKPASS(5);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCheckIfEvent-5.(A)
 * When on a call to XCheckIfEvent predicate has not returned
 * True after having been called once for each event in the
 * event queue, then XCheckIfEvent returns False.
 */
static void
t005()
{
XEvent	event;
int	callcnt;
int	returnvalue;
int	qsize;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = KeyPress;
	XPutBackEvent(display, &event);
	event.type = KeyRelease;
	XPutBackEvent(display, &event);
	event.type = ButtonPress;
	XPutBackEvent(display, &event);
/* Call XPending to get the current event queue size. */
	qsize = XPending(display);
/* Set up predicate procedure. */
	PRED_SETUP(callcnt = 100);
/* Call XCheckIfEvent. */
	_xcall_(returnvalue);
/* Verify that predicate never returned True. */
	if (_pred_retval == True) {
		report("Predicate erroneously returned True.");
		FAIL;
	}
	else
		CHECK;
	if (_pred_true == True) {
		report("Predicate erroneously returned True.");
		FAIL;
	}
	else
		CHECK;
/* Verify that XCheckIfEvent returned False. */
	if (returnvalue != False) {
		report("returned %d, expecting %d", returnvalue, False);
		FAIL;
	}
	else
		CHECK;
/* Verify that XCheckIfEvent called predicate once for */
/* each event in the event queue. */
	if (_pred_cnt != qsize) {	
		report("predicate called %d times, expected %d", _pred_cnt, qsize);
		FAIL;
	}
	else
		CHECK;
	/* empty event queue */
	XSync(display, True);
	
	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCheckIfEvent-6.(A)
 * When on a call to XCheckIfEvent predicate has not returned
 * True after having been called once for each event in the
 * event queue, then XCheckIfEvent flushes the output buffer.
 */
static void
t006()
{
XEvent	event;
int	callcnt;
int	returnvalue;
int	qsize;
Pixmap	pm;
Display *client2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create client2. */
	client2 = opendisplay();
	if (client2 == (Display *) NULL) {
		delete("Can not open display");
		return;
	}
	else
		CHECK;
/* Discard all events on the event queue. */
	XSync(display, True);
/* Create pixmap. */
	/* avoid using makepixm() */
	pm = XCreatePixmap(display, DRW(display), 10, 10, 1);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = KeyPress;
	XPutBackEvent(display, &event);
	event.type = KeyRelease;
	XPutBackEvent(display, &event);
	event.type = ButtonPress;
	XPutBackEvent(display, &event);
/* Call XPending to get the current event queue size. */
	qsize = XPending(display);
/* Set up predicate procedure. */
	PRED_SETUP(callcnt = 100);
/* Call XCheckIfEvent. */
	_xcall_(returnvalue);
/* Empty the buffer. */
	(void)XTestDiscard(display);
/* Ensure the server has dealt with anything flushed to it: do XSync() */
	XSync(display, False);
/* Verify that the output buffer was flushed by effect on server. */
	_startcall(client2);
	XFreePixmap(client2, pm);
	XSync(client2, True);
	_endcall(client2);
	if (geterr() != Success) {
		report("The output buffer was not flushed.");
		XFreePixmap(display, pm);
		FAIL;
	}
	else
		CHECK;
/* Verify that predicate never returned True. */
	if (_pred_retval == True) {
		report("Predicate erroneously returned True.");
		FAIL;
	}
	else
		CHECK;
	if (_pred_true == True) {
		report("Predicate erroneously returned True.");
		FAIL;
	}
	else
		CHECK;
/* Verify that XCheckIfEvent returned False. */
	if (returnvalue != False) {
		report("returned %d, expecting %d", returnvalue, False);
		FAIL;
	}
	else
		CHECK;
/* Verify that XCheckIfEvent called predicate once for */
/* each event in the event queue. */
	if (_pred_cnt != qsize) {	
		report("predicate called %d times, expected %d", _pred_cnt, qsize);
		FAIL;
	}
	else
		CHECK;
	/* empty event queue */
	XSync(display, True);
	
	CHECKPASS(6);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
