/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: grvtyntfy.m,v 1.8 92/06/11 17:21:38 rws Exp $
 * 
 */
/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_GravityNotify	1
char    *TestName = "GravityNotify";



#define	EVENT		GravityNotify
#define	MASK		StructureNotifyMask
#define	MASKP		SubstructureNotifyMask

int 	tet_thistest;

/*
 * Assertion GravityNotify-1.(A)
 * When a GravityNotify event is generated, then all clients
 * having set StructureNotifyMask event mask bits on the moved
 * window are delivered a GravityNotify event.
 */
static void
t001()
{
Display	*display = Dsp;
Display	*client2;
Display	*client3;
Winh	*parent, *eventw;
XSetWindowAttributes attrs;
Winhg	winhg;
int	status;
XEvent	event;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create clients client2 and client3. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
	if ((client3 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client3.");
		return;
	}
	else
		CHECK;
/* Build and create window hierarchy with child having */
/* win-gravity SouthEastGravity. */
	/* can't use winh() because the windows need to overlap */
	winhg.border_width = 1;
	winhg.area.x = 1;
	winhg.area.y = 1;
	winhg.area.width = 100;
	winhg.area.height = 100;
	parent = winh_adopt(display, (Winh *) NULL, 0L, (XSetWindowAttributes *) NULL, &winhg, WINH_NOMASK);
	if (parent == (Winh *) NULL) {
		report("Could not create parent");
		return;
	}
	else
		CHECK;
	winhg.area.x = 1;
	winhg.area.y = 1;
	winhg.area.width /= 2;
	winhg.area.height /= 2;
	attrs.win_gravity = SouthEastGravity;
	eventw = winh_adopt(display, parent, CWWinGravity, &attrs, &winhg, WINH_NOMASK);
	if (eventw == (Winh *) NULL) {
		report("Could not create child");
		return;
	}
	else
		CHECK;
	if (winh_create(display, (Winh *) NULL, WINH_MAP))
		return;
	else
		CHECK;
/* Select for GravityNotify events using StructureNotifyMask. */
	if (winh_selectinput(display, (Winh *) NULL, MASK)) {
		report("Selection with first client failed.");
		return;
	}
	else
		CHECK;
/* Select for GravityNotify events using StructureNotifyMask with client2. */
	if (winh_selectinput(client2, (Winh *) NULL, MASK)) {
		report("Selection with client2 failed.");
		return;
	}
	else
		CHECK;
/* Select for no events with client3. */
	if (winh_selectinput(client3, (Winh *) NULL, NoEventMask)) {
		report("Selection with client3 failed.");
		return;
	}
	else
		CHECK;
/* Generate GravityNotify event. */
	XSync(display, True);
	XSync(client2, True);
	XSync(client3, True);
	XResizeWindow(display, parent->window, ++(parent->winhg.area.width), ++(parent->winhg.area.height));
	event.xany.type = EVENT;
	event.xany.window = eventw->window;
	if (winh_plant(eventw, &event, MASK, WINH_NOMASK)) {
		report("Could not plant events.");
		return;
	}
	else
		CHECK;
	XSync(display, False);
	XSync(client2, False);
	XSync(client3, False);
/* Verify that a GravityNotify event is delivered. */
/* Verify that a GravityNotify event is delivered to client2. */
/* Verify that no events are delivered to client3. */
	if (winh_harvest(display, (Winh *) NULL)) {
		report("Could not harvest events for display");
		return;
	}
	else
		CHECK;
	if (winh_harvest(client2, (Winh *) NULL)) {
		report("Could not harvest events for client2");
		return;
	}
	else
		CHECK;
	if (winh_harvest(client3, (Winh *) NULL)) {
		report("Could not harvest events for client3");
		return;
	}
	else
		CHECK;
	/* not interested in resulting ConfigureNotify events */
	if (winh_ignore_event((Winh *) NULL, ConfigureNotify, WINH_NOMASK))
		return;
	else
		CHECK;
	status = winh_weed((Winh *) NULL, -1, WINH_WEED_IDENTITY);
	if (status == -1)
		return;
	else if (status) {
		report("Event delivery not as expected");
		FAIL;
	}
	else {
		XGravityEvent	good;
		Winhe	*winhe;

/* Verify delivered event structure members. */
		/* ignore non-GravityNotify events */
		for (winhe=winh_qdel; winhe!=(Winhe *) NULL; winhe=winhe->next){
			if (winhe->event->type == EVENT) {
				CHECK;
				break;
			}
		}
		if (winhe == (Winhe *) NULL) {
			delete("Inconsistencies in delivered event queue");
			FAIL;
		}
		else
		{
			good = winhe->event->xgravity;
			good.type = EVENT;
			good.event = eventw->window;
			good.window = eventw->window;
			good.x = eventw->winhg.area.x + 1;
			good.y = eventw->winhg.area.y + 1;
			if (checkevent((XEvent *) &good, winhe->event)) {
				report("Unexpected event member values");
				FAIL;
			}
			else
				CHECK;
		}
	}

	CHECKPASS(15);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion GravityNotify-2.(A)
 * When a GravityNotify event is generated, then all clients
 * having set SubstructureNotifyMask event mask bits on the
 * parent of the moved window are delivered a GravityNotify
 * event.
 */
static void
t002()
{
Display	*display = Dsp;
Display	*client2;
Display	*client3;
Winh	*parent, *eventw;
XSetWindowAttributes attrs;
Winhg	winhg;
int	status;
XEvent	event;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create clients client2 and client3. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
	if ((client3 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client3.");
		return;
	}
	else
		CHECK;
/* Build and create window hierarchy with child having */
/* win-gravity SouthEastGravity. */
	/* can't use winh() because the windows need to overlap */
	winhg.border_width = 1;
	winhg.area.x = 1;
	winhg.area.y = 1;
	winhg.area.width = 100;
	winhg.area.height = 100;
	parent = winh_adopt(display, (Winh *) NULL, 0L, (XSetWindowAttributes *) NULL, &winhg, WINH_NOMASK);
	if (parent == (Winh *) NULL) {
		report("Could not create parent");
		return;
	}
	else
		CHECK;
	winhg.area.x = 1;
	winhg.area.y = 1;
	winhg.area.width /= 2;
	winhg.area.height /= 2;
	attrs.win_gravity = SouthEastGravity;
	eventw = winh_adopt(display, parent, CWWinGravity, &attrs, &winhg, WINH_NOMASK);
	if (eventw == (Winh *) NULL) {
		report("Could not create child");
		return;
	}
	else
		CHECK;
	if (winh_create(display, (Winh *) NULL, WINH_MAP))
		return;
	else
		CHECK;
/* Select for GravityNotify events using SubstructureNotifyMask. */
	if (winh_selectinput(display, (Winh *) NULL, MASKP)) {
		report("Selection with first client failed.");
		return;
	}
	else
		CHECK;
/* Select for GravityNotify events using SubstructureNotifyMask with client2. */
	if (winh_selectinput(client2, (Winh *) NULL, MASKP)) {
		report("Selection with client2 failed.");
		return;
	}
	else
		CHECK;
/* Select for no events with client3. */
	if (winh_selectinput(client3, (Winh *) NULL, NoEventMask)) {
		report("Selection with client3 failed.");
		return;
	}
	else
		CHECK;
/* Generate GravityNotify event. */
	XSync(display, True);
	XSync(client2, True);
	XSync(client3, True);
	XResizeWindow(display, parent->window, ++(parent->winhg.area.width), ++(parent->winhg.area.height));
	event.xany.type = EVENT;
	event.xany.window = parent->window;
	if (winh_plant(parent, &event, MASKP, WINH_NOMASK)) {
		report("Could not plant events");
		return;
	}
	else
		CHECK;
	XSync(display, False);
	XSync(client2, False);
	XSync(client3, False);
/* Verify that a GravityNotify event is delivered. */
/* Verify that a GravityNotify event is delivered to client2. */
/* Verify that no events are delivered to client3. */
	if (winh_harvest(display, (Winh *) NULL)) {
		report("Could not harvest events for display");
		return;
	}
	else
		CHECK;
	if (winh_harvest(client2, (Winh *) NULL)) {
		report("Could not harvest events for client2");
		return;
	}
	else
		CHECK;
	if (winh_harvest(client3, (Winh *) NULL)) {
		report("Could not harvest events for client3");
		return;
	}
	else
		CHECK;
	/* not interested in resulting ConfigureNotify events */
	if (winh_ignore_event((Winh *) NULL, ConfigureNotify, WINH_NOMASK))
		return;
	else
		CHECK;
	status = winh_weed((Winh *) NULL, -1, WINH_WEED_IDENTITY);
	if (status == -1)
		return;
	else if (status) {
		report("Event delivery not as expected");
		FAIL;
	}
	else {
		XGravityEvent	good;
		Winhe	*winhe;

/* Verify delivered event structure members. */
		/* ignore non-GravityNotify events */
		for (winhe=winh_qdel; winhe!=(Winhe *) NULL; winhe=winhe->next){
			if (winhe->event->type == EVENT) {
				CHECK;
				break;
			}
		}
		if (winhe == (Winhe *) NULL) {
			delete("Inconsistencies in delivered event queue");
			FAIL;
		}
		else
		{
			good = winhe->event->xgravity;
			good.type = EVENT;
			good.event = parent->window;
			good.window = eventw->window;
			good.x = eventw->winhg.area.x + 1;
			good.y = eventw->winhg.area.y + 1;
			if (checkevent((XEvent *) &good, winhe->event)) {
				report("Unexpected event member values");
				FAIL;
			}
			else
				CHECK;
		}
	}

	CHECKPASS(15);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion GravityNotify-3.(-)
 * When a GravityNotify event is generated, then clients not
 * having set StructureNotifyMask event mask bits on the moved
 * window and also not having set SubstructureNotifyMask event
 * mask bits on the parent of the moved window are not
 * delivered a GravityNotify event.
 */
static void
t003()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
