/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: nxtevnt.m,v 1.9 92/06/11 17:23:43 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XNextEvent(display, event_return)
 *   Display *display;
 *   XEvent	*event_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XNextEvent	1
char    *TestName = "XNextEvent";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XNextEvent function
 */
static Display *display;
static XEvent	*event_return;


static XEvent _event;
/*
 * Can not use "xcall" because it empties the event queue.
 */
#define	_xcall_()	\
		_startcall(display);\
		XNextEvent(display, event_return);\
		_endcall(display)

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	event_return = &_event;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XNextEvent-1.(A)
 * When the event queue is not empty, the a call to XNextEvent
 * returns the first event from the event queue in event_return.
 */
static void
t001()
{
XEvent	event;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put events on the event queue. */
	event.type = KeyPress;
	XPutBackEvent(display, &event);
	event.type = KeyRelease;
	XPutBackEvent(display, &event);
	event.type = ButtonPress;
	XPutBackEvent(display, &event);
/* Call XNextEvent. */
	_xcall_();
/* Verify that XNextEvent returned the correct event. */
	if (event_return->type != event.type) {
		report("Returned %s, expected %s", eventname(event_return->type), eventname(event.type));
		FAIL;
	}
	else
		CHECK;
	/* empty event queue */
	XSync(display, True);

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XNextEvent-2.(A)
 * A call to XNextEvent removes the returned event from the
 * event queue.
 */
static void
t002()
{
XEvent	event;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Discard all events on the event queue. */
	XSync(display, True);
/* Call XPutBackEvent to put a three events on the event queue. */
	event.type = KeyPress;
	XPutBackEvent(display, &event);
	XPutBackEvent(display, &event);
	XPutBackEvent(display, &event);
/* Call XNextEvent. */
	_xcall_();
/* Verify that XNextEvent returned the correct event. */
	if (event_return->type != event.type) {
		report("Returned %s, expected %s", eventname(event_return->type), eventname(event.type));
		FAIL;
	}
	else
		CHECK;
/* Call XNextEvent. */
	_xcall_();
/* Verify that XNextEvent returned the correct event. */
	if (event_return->type != event.type) {
		report("Returned %s, expected %s", eventname(event_return->type), eventname(event.type));
		FAIL;
	}
	else
		CHECK;
/* Call XNextEvent. */
	_xcall_();
/* Verify that XNextEvent returned the correct event. */
	if (event_return->type != event.type) {
		report("Returned %s, expected %s", eventname(event_return->type), eventname(event.type));
		FAIL;
	}
	else
		CHECK;
/* Verify that the event queue is now empty. */
	if (XPending(display) != 0) {
		report("Events not removed from the event queue.");
		FAIL;
	}
	else
		CHECK;
	/* empty event queue */
	XSync(display, True);

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XNextEvent-3.(A)
 * When the event queue is empty, then a call to XNextEvent
 * flushes the output buffer and blocks until an event is
 * received and returns the event in event_return.
 */
static void
t003()
{
Block_Info info;
int	block_status;
Pixmap	pm;
Display *client2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create client2. */
	client2 = opendisplay();
	if (client2 == (Display *) NULL) {
		delete("Can not open display");
		return;
	}
	else
		CHECK;
/* Discard all events on the event queue. */
	XSync(display, True);
/* Create pixmap. */
	/* avoid using makepixm() */
	pm = XCreatePixmap(display, DRW(display), 10, 10, 1);
/* Call XNextEvent and verify that blocking did occur. */
	XNextEvent_Type(info);
	block_status = block(display, (XEvent *) NULL, &info);
	if (block_status == -1)
		return;
	else
		CHECK;
	if (block_status == 0) {
		report("Blocking did not occur.");
		FAIL;
	}
	else
		CHECK;
/* Verify that the output buffer was flushed. */
	_startcall(client2);
	XFreePixmap(client2, pm);
	XSync(client2, True);
	_endcall(client2);
	if (geterr() != Success) {
		report("The output buffer was not flushed.");
		XFreePixmap(display, pm);
		FAIL;
	}
	else
		CHECK;

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
