/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: staftrfnct.m,v 1.7 92/06/11 17:25:34 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int ((*)())()
 *   XSetAfterFunction(display, procedure)
 *   Display *display;
 *   int (*procedure)();
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetAfterFunction	1
char    *TestName = "XSetAfterFunction";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XSetAfterFunction function
 */
static Display *display;
static int (*procedure)();


static int 	ValueReturn;

static	int	counter = 0;

static int
afterfunction(display)
Display *display;
{
	trace("In afterfunction(), screen number %d", XDefaultScreen(display));
	return(counter++);
}
static int
_afterfunction(display)
Display *display;
{
	trace("In _afterfunction(), screen number %d", XDefaultScreen(display));
	return(--counter);
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	procedure = afterfunction;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetAfterFunction-1.(A)
 * A call to XSetAfterFunction sets the after function to
 * procedure.
 */
static void
t001()
{
int	oldcounter;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	oldcounter = counter;
/* Create window. */
	(void) mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Verify that afterfunction was not called. */
	if (counter != oldcounter) {
		delete("After function already set to afterfunction.");
		return;
	}
	else
		CHECK;
/* Call XSetAfterFunction to set the after function to afterfunction. */
	startcall(display);
	if (isdeleted())
		return;
	(void) XSetAfterFunction(display, procedure);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Create window. */
	(void) mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Verify that the after function was called. */
	if (counter == oldcounter) {
		report("After function not called.");
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetAfterFunction-2.(A)
 * A call to XSetAfterFunction returns the previous after
 * function.
 */
static void
t002()
{
int	(*proc)();
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call XSetAfterFunction to set after function to afterfunction. */
	procedure = afterfunction;
	startcall(display);
	if (isdeleted())
		return;
	(void) XSetAfterFunction(display, procedure);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Call XSetAfterFunction to set after function to _afterfunction. */
	procedure = _afterfunction;
	startcall(display);
	if (isdeleted())
		return;
	proc = XSetAfterFunction(display, procedure);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that XSetAfterFunction returned afterfunction. */
	if (proc != afterfunction) {
		report("Returned 0x%x, expected 0x%x", proc, afterfunction);
		FAIL;
	}
	else
		CHECK;
/* Call XSetAfterFunction to set after function to afterfunction. */
	procedure = afterfunction;
	startcall(display);
	if (isdeleted())
		return;
	proc = XSetAfterFunction(display, procedure);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that XSetAfterFunction returned _afterfunction. */
	if (proc != _afterfunction) {
		report("Returned 0x%x, expected 0x%x", proc, _afterfunction);
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetAfterFunction-3.(B)
 *   Reason: Creating a test would require an unreasonable amount of test development time.
 * After a call to XSetAfterFunction, all Xlib functions that
 * generate protocol requests call procedure before returning.
 */
static void
t003()
{
	report("Creating a test would require an unreasonable amount of test development time.");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
