/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stioerrrhn.m,v 1.12 92/06/11 17:25:51 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int ((*)())()
 *   XSetIOErrorHandler(handler)
 *   int (*handler)();
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetIOErrorHandler	1
char    *TestName = "XSetIOErrorHandler";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XSetIOErrorHandler function
 */
static int (*handler)();


static int 	ValueReturn;

#include <signal.h>
#include <sys/wait.h>

#define	_xcall_(rvalue)		rvalue = XSetIOErrorHandler(handler)

static	Display	*client;
static	int	exit_status = -1;
static	int	counter = 0;

/*
 * various exit values are used because we do not know the exit status
 * used by the default handler
 */
static	int	exit_statuses[] = {
	2, 3, 4, 8, 16
};

static int
errorhandler(display)
Display *display;
{
	/* Handler exits with exit status of exit_status. */
	exit(exit_status);
}

static int
_errorhandler(display)
Display *display;
{
	/* Handler returns first time called. */
	/* Handler exits upon second and subsequent invocations */
	/* with exit status of exit_status. */
	if (counter++)
		exit(exit_status);
	return(0);
}

/*
 * Child exits with exit_status on success.
 * Child exits with exit_status+1 to indicate TET_DELETE.
 * Child exits with exit_status-1 to indicate failure.
 */
static	void
child_proc1()
{
	Window	w;
	int 	fail = 0;

	/* Child process closes connection number. */
	if (close(ConnectionNumber(client))) {
		report("Close failed on ConnectionNumber number.");
		exit(exit_status + 1);
	}
	/* Child process attempts to communicate to server, */
	/* causing handler to be invoked. */
	w = XCreateSimpleWindow(client, DefaultRootWindow(client), 1, 1, 50, 50, 1, 0, 0);
	XFlush(client);

	report("Handler not invoked or did not exit");
	FAIL;
	exit(exit_status - 1);
}



#include <stdio.h>
#define	MESSBUF	55


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	handler = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetIOErrorHandler-1.(A)
 * A call to XSetIOErrorHandler sets the fatal I/O error
 * handler to handler.
 */
static void
t001()
{
int	(*proc)();
int	child_exit;
int	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call XSetIOErrorHandler to set the handler to errorhandler. */
	handler = errorhandler;
	_xcall_(proc);
	for (i=0; i<NELEM(exit_statuses); i++) {
/* Open display. */
		client = opendisplay();
		if (client == (Display *) NULL) {
			delete("Can not open display");
			return;
		}
		else
			CHECK;
/* Set exit_status. */
		exit_status = exit_statuses[i];
/* Create child process. */
		child_exit = tet_fork(child_proc1, (void (*)()) NULL, 20, ~0);
/* Child process closes connection number. */
/* Child process attempts to communicate to server, */
/* causing handler to be invoked. */
/* Handler exits with exit status of exit_status. */
/* Verify that child's exit status was exit_status. */
		if (child_exit == (exit_statuses[i]+1)) {
			delete("Child process experienced unexpected problem.");
			return;
		}
		else
			CHECK;
		if (child_exit != exit_statuses[i]) {
			report("Handler not invoked (got %d exit status, expected %d.", child_exit, exit_statuses[i]);
			FAIL;
		}
		else
			CHECK;
/* Repeat for various other exit_status values. */
	}
	CHECKPASS(3*NELEM(exit_statuses));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetIOErrorHandler-2.(A)
 * A call to XSetIOErrorHandler returns the previous fatal I/O
 * error handler.
 */
static void
t002()
{
int	(*proc)();
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call XSetIOErrorHandler to set error handler to errorhandler. */
	handler = errorhandler;
	_xcall_(proc);
/* Call XSetIOErrorHandler to set error handler to _errorhandler. */
	handler = _errorhandler;
	_xcall_(proc);
/* Verify that errorhandler was returned. */
	if (proc != errorhandler) {
		report("Returned 0x%x, expected 0x%x", proc, errorhandler);
		FAIL;
	}
	else
		CHECK;
/* Call XSetIOErrorHandler to set error handler to errorhandler. */
	handler = errorhandler;
	_xcall_(proc);
/* Verify that _errorhandler was returned. */
	if (proc != _errorhandler) {
		report("Returned 0x%x, expected 0x%x", proc, _errorhandler);
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetIOErrorHandler-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * On a call to XSetIOErrorHandler with handler set to NULL
 * the fatal I/O error handler is set to the default fatal I/O
 * error handler.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSetIOErrorHandler-4.(A)
 * The default fatal I/O error handler prints a message and
 * exits.
 */
static void
t004()
{
int	(*proc)();
pid_t	child;
int	stat_loc;
int	waitstatus;
int     p[2];
int     gotmessage = 0;
char    buf[MESSBUF];
FILE    *fp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	client = Dsp;
/* Get default error handler. */
	handler = (int (*)()) NULL;
	_xcall_(proc);
	/* requires two calls! */
	_xcall_(proc);

	if (pipe(p) == -1) {
		delete("Could not create pipe");
		return;
	}

/* Create child process. */
	child = fork();
	if (!child) {

		close(p[0]);
		/*
		 * Capture both stdout and stderr into the pipe.
		 */
		dup2(p[1], 1);
		dup2(p[1], 2);

/* Child calls default error handler and loops forever. */
		(*proc)(client);
		/*
		 * Now close the pipe to make sure that the parent will not hang.
		 */
		close(p[1]);
		close(1);
		close(2);

		for (;;)
			continue;
	}
	else
		CHECK;

	close(p[1]);

/* Parent sleeps for 10 seconds. */
	sleep(10);
    /*
     * Read message in reasonable size chunks.
     */
    fp = fdopen(p[0], "r");
    if (fp == NULL) {
        delete("Could not fdopen pipe");
        return;
    }
    trace("The message produced by the default handler:");
    gotmessage = 0;
    while (fgets(buf, MESSBUF-1, fp)) {
        gotmessage = 1;
        buf[MESSBUF-1] = '\0';
        trace("Message: %s", buf);
    }

    if (!gotmessage) {
        report("No message was printed");
        FAIL;
    } else
        CHECK;

/* Parent verifies that child no longer exists. */
	waitstatus = waitpid(child, &stat_loc, WNOHANG);
	if (waitstatus != child) {
		report("Child did not exit.");
		FAIL;
		(void) kill(child, SIGKILL);
		(void) waitpid(child, &stat_loc, WNOHANG);
	}
	else
		CHECK;
	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetIOErrorHandler-5.(B)
 *   Reason: Testing the assertion would require an unreasonable amount of time or resources on most systems
 * There is no limit to the number of times XSetIOErrorHandler
 * may be called.
 */
static void
t005()
{
int	(*proc)();
int	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Set handler to errorhandler. */
	handler = errorhandler;
/* Call XSetIOErrorHandler 1000 times. */
	for (i=0; i<1000; i++) {
		if (i == 0)
			CHECK;
		startcall(Dsp);
		if (isdeleted())
			return;
		proc = XSetIOErrorHandler(handler);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	}
/* Report untested. */
	CHECKUNTESTED(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetIOErrorHandler-6.(-)
 * When a system call error occurs, then Xlib calls handler.
 */
static void
t006()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XSetIOErrorHandler-7.(A)
 * When handler returns, then the client process exits.
 */
static void
t007()
{
int	(*proc)();
int	child_exit;
char	*server;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if ((server = config.display) == (char *) NULL) {
		delete("XT_DISPLAY not set");
		return;
	}
	else
		CHECK;
/* Call XSetIOErrorHandler to set the handler to _errorhandler. */
	handler = _errorhandler;
	_xcall_(proc);
/* Open display. */

	client = XOpenDisplay(server);
	if (client == (Display *) NULL) {
		delete("Can not open display: %s", server);
		return;
	}
	else
		CHECK;

/* Set exit_status. */
	exit_status = exit_statuses[0];

	counter = 0;
/* Create child process. */
/* Child process closes connection number. */
/* Child process attempts to communicate to server, */
/* causing handler to be invoked. */
/* Handler returns first time called. */
	child_exit = tet_fork(child_proc1, (void (*)()) NULL, 10, ~0);

	/*
	 * Since the exit status when the handler returns is not known, then
	 * no checks can be made on it.
	 * If the handler did not exit then this will have been reported in the
	 * child.  In this case the pass here will be spurious, however the
	 * TET will give precedence to the fail, so a false pass will not result.
	 */
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
