/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtclsshnt.m,v 1.6 92/06/11 17:27:44 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XGetClassHint(display, w, class_hints_return)
 *   Display		*display;
 *   Window		w;
 *   XClassHint	*class_hints_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetClassHint	1
char    *TestName = "XGetClassHint";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XGetClassHint function
 */
static Display		*display;
static Window		w;
static XClassHint	*class_hints_return;


static int 	StatusReturn;

#include	"Xatom.h"
XClassHint	chints;
XClassHint	chints_ret = { "<Unset String>", "<UnsetString>" };

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	class_hints_return = &chints;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetClassHint-1.(A)
 * When the WM_CLASS property is set for the window w, and
 * has format 8 and type STRING, then a call to XGetClassHint
 * returns the class hint, which can be freed with XFree, in
 * the XClassHint structure named by the class_hints_return
 * argument and returns non-zero.
 */
static void
t001()
{
char		*s;
Window		win;
Status		status;
XVisualInfo	*vp;
XClassHint	chints;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);

	chints.res_name = "Xtest_res_name";
	chints.res_class = "Xtest_res_class";

	XSetClassHint(display, win, &chints);

	w = win;
	class_hints_return = &chints_ret;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetClassHint(display, w, class_hints_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == False) {
		report("XGetClassHint() returned False.");
		FAIL;
	} else
		CHECK;

	if( strcmp((s=chints_ret.res_name), chints.res_name) != 0) {
		report("The res_name component of the XClassHint structure was \"%s\" instead of %s.",
			s == (char *)NULL ? "NULL" : s,
			chints.res_name);
		FAIL;
	} else
		CHECK;

	if(strcmp((s=chints_ret.res_class), chints.res_class) != 0) {
		report("The res_class component of the XClassHint structure was \"%s\" instead of %s.",
			s == (char *)NULL ? "NULL" : s,
			chints.res_class);
		FAIL;
	} else
		CHECK;


	XFree(chints_ret.res_name);
	XFree(chints_ret.res_class);

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetClassHint-2.(A)
 * When the WM_CLASS property is not set for the window w, or
 * has a format other than 8 or a type other than STRING, then
 * a call to XGetClassHint returns zero.
 */
static void
t002()
{
Status		status;
XVisualInfo	*vp;
char		*s1 = "XTestResName";
char		*s2 = "XTestResClass";
unsigned 	lenname, lenclass;
char		*hints, *hp;
XClassHint	chints_ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);

/* property not set */
	if( (char *) (hints = malloc( (unsigned int) ((unsigned)2 + (lenname = strlen(s1)) + (lenclass = strlen(s2)))) ) == (char *) 0) {
		delete("malloc() failed.");
		return;	
	} else 
		CHECK;

	hp = hints;
	strcpy( hp, s1);
	hp += lenname + 1;
	strcpy( hp, s2);

	class_hints_return = &chints_ret;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetClassHint(display, w, class_hints_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("XGetClassHint() did not return False with the WM_CLASS property unset.");
		FAIL;
	} else
		CHECK;

/* format not 8 */

	w = makewin(display, vp);
 	
	XChangeProperty(display, w, XA_WM_CLASS, XA_STRING, 16,
			PropModeReplace, (unsigned char *) &hints, 2 + lenclass +lenname);

	class_hints_return = &chints_ret;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetClassHint(display, w, class_hints_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("XGetClassHint() did not return False with the WM_CLASS property format set to 16.");
		FAIL;
	} else
		CHECK;


/* type wrong */

	w = makewin(display, vp);
 	
	XChangeProperty(display, w, XA_WM_CLASS, XA_ATOM, 8,
			PropModeReplace, (unsigned char *) &hints, 2 + lenclass + lenname);

	class_hints_return = &chints_ret;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetClassHint(display, w, class_hints_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("XGetClassHint() did not return False with the WM_CLASS property type set to Atom.");
		FAIL;
	} else
		CHECK;

	free(hints);

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XGetClassHint-3.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XGetClassHint(display, w, class_hints_return);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
