/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtcmmnd.m,v 1.7 92/06/11 17:27:51 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XGetCommand(display, w, argv_return, argc_return)
 *   Display		*display;
 *   Window		w;
 *   char		***argv_return;
 *   int		*argc_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetCommand	1
char    *TestName = "XGetCommand";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XGetCommand function
 */
static Display		*display;
static Window		w;
static char		***argv_return;
static int		*argc_return;


static int 	StatusReturn;

#include	"Xatom.h"
char		**argvdef;
int		argcdef;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	argv_return = &argvdef;
	argc_return = &argcdef;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetCommand-1.(A)
 * When the WM_COMMAND property is set for the window w, is
 * of type STRING, and is of format 8, then a call to
 * XGetCommand returns the property string list, which can be
 * freed with XFree, in the argv_return argument and the number
 * of strings in the argc_return argument and returns non-zero.
 */
static void
t001()
{
XVisualInfo	*vp;
Status	status;
char	*nullstr = "<NULL>";
char	**strpp, *strp;
char	*str1 = "XTest string 1____";
char	*str2 = "XTest string 2__";
char	*str3 = "XTest string 3___";
int	nstrs = 3;
char	*prop[3];
char	**rstrings = (char**) NULL;
int	rcount = 0;
int	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	prop[0] = str1;
	prop[1] = str2;
	prop[2] = str3;

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);

	XSetCommand(display, w, prop, nstrs);

	argv_return = &rstrings;
	argc_return = &rcount;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetCommand(display, w, argv_return, argc_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == False) {
		delete("XGetCommand() returned False.");
		return;
	} else
		CHECK;

	if(rstrings == (char **) NULL) {
		report("Returned list of strings was NULL");
		FAIL;
	} else {
		CHECK;

		if(rcount != nstrs) {
			report("%d strings were returned instead of %d", rcount, nstrs);
			FAIL;
		} else {
			CHECK;

			for(i=0, strpp = rstrings; i< nstrs; i++, strpp++) {
				strp = (*strpp == NULL ? nullstr : *strpp);

				if(strcmp(strp, prop[i]) != 0) {
					report("String %d was \"%s\" instead of \"%s\"", i, strp, prop[i]);
					FAIL;
				} else
					CHECK;
			}

		}
		XFreeStringList(rstrings);
	}

	CHECKPASS(nstrs + 3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetCommand-2.(A)
 * When the WM_COMMAND property is not set for the window w,
 * or is not of type STRING, or is not of format 8, then a call
 * to XGetCommand returns zero.
 */
static void
t002()
{
Status		status;
char		*s = "XTestString1";
XVisualInfo	*vp;
char		**rstrings = (char **) NULL;
int		rcount = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	argv_return = &rstrings;
	argc_return = &rcount;

	w = makewin(display, vp);

/* unset property */

	startcall(display);
	if (isdeleted())
		return;
	status = XGetCommand(display, w, argv_return, argc_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when the property was unset.", TestName);
		FAIL;
	} else
		CHECK;


	w = makewin(display, vp);

/* format 16 */
	XChangeProperty(display, w, XA_WM_COMMAND, XA_STRING, 16, PropModeReplace, (unsigned char *) s, strlen(s) );

	startcall(display);
	if (isdeleted())
		return;
	status = XGetCommand(display, w, argv_return, argc_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when property was of format 16.", TestName);
		FAIL;
	} else
		CHECK;


	w = makewin(display, vp);

/* type ATOM */
	XChangeProperty(display, w, XA_WM_COMMAND, XA_ATOM, 8, PropModeReplace, (unsigned char *) s, strlen(s) );

	startcall(display);
	if (isdeleted())
		return;
	status = XGetCommand(display, w, argv_return, argc_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when property was of type Atom.", TestName);
		FAIL;
	} else
		CHECK;


	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetCommand-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When insufficient memory is available to contain the string
 * list, then a call to XGetCommand returns a zero status.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
