/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtrgbclrmp.m,v 1.10 92/06/11 17:28:14 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XGetRGBColormaps(display, w, std_colormap_return, count_return, property)
 *   Display			*display;
 *   Window			w;
 *   XStandardColormap	**std_colormap_return;
 *   int			*count_return;
 *   Atom			property;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetRGBColormaps	1
char    *TestName = "XGetRGBColormaps";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w
#define A_ATOM property


/*
 * Arguments to the XGetRGBColormaps function
 */
static Display			*display;
static Window			w;
static XStandardColormap	**std_colormap_return;
static int			*count_return;
static Atom			property;


static int 	StatusReturn;


#include		"Xatom.h"
static XStandardColormap	*scolormap;
static int			cntret;
static XStandardColormap	scmp1 = { (Colormap)  1,  2L,  3L,  4L,  5L,  6L,  7L,  8L, (VisualID) -1, (XID) -2 };
static XStandardColormap	scmp2 = { (Colormap) 11, 12L, 13L, 14L, 15L, 16L, 17L, 18L, (VisualID) 19, (XID) 20 };
static XStandardColormap	scmp3 = { (Colormap) 21, 22L, 23L, 24L, 25L, 26L, 27L, 28L, (VisualID) 21, (XID) 22 };


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	std_colormap_return = &scolormap;
	count_return = &cntret;
	property = XA_ATOM;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetRGBColormaps-1.(A)
 * When the property named by the property argument is set on
 * the window named by the w argument, is of type
 * RGB_COLOR_MAP, is of format 32, and is of length n1*8+ n2*9+
 * n3*10 elements (where n1,n2,n3 (>= 0 and n1+n2+n3 (>= 1),
 * then a call to XGetRGBColormaps returns in the
 * std_colormap_return argument the RBG colormap definitions,
 * which can be freed with XFree, and in count_return the
 * number of structures and returns non-zero.
 */
static void
t001()
{
Status			status;
int			ncmp = 3;
XStandardColormap	scmp[3];
XStandardColormap	*rscmp = (XStandardColormap *) NULL;
XStandardColormap	*cmpp;
int			rncmp = 0;
int			i;
XVisualInfo		*vp;
Atom			prop = XA_RGB_DEFAULT_MAP;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	scmp[0] = scmp1;
	scmp[1] = scmp2;
	scmp[2] = scmp3;

	resetvinf(VI_WIN);	
	nextvinf(&vp);

	w = makewin(display, vp);
	XSetRGBColormaps(display, w, scmp, ncmp, prop);

	std_colormap_return = &rscmp;
	count_return = &rncmp;
	property = prop;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetRGBColormaps(display, w, std_colormap_return, count_return, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == False) {
		delete("%s returned False.", TestName);
		return;
	} else
		CHECK;

	if(rscmp == (XStandardColormap *) NULL) {	
		report("The returned list of XStandardColormap structures was NULL.");
		FAIL;
	} else {

		CHECK;
		if( rncmp != ncmp) {
			report("%d XStandardColormap structures were returned instead of %d.", rncmp, ncmp);
			FAIL;
		} else {

			CHECK;
			
			for(i = 0, cmpp = rscmp; i < ncmp; i++, cmpp++) {

				if(cmpp->colormap != scmp[i].colormap) {
					report("The colormap component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
				if(cmpp->red_max != scmp[i].red_max)  {
					report("The red_max component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
				if(cmpp->red_mult != scmp[i].red_mult) {
					report("The red_mult component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
				if(cmpp->green_max !=scmp[i].green_max) {
					report("The green_max component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
				if(cmpp->green_mult != scmp[i].green_mult) {
					report("The green_mult component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
				if(cmpp->blue_max != scmp[i].blue_max) {
					report("The blue_max component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
				if(cmpp->blue_mult != scmp[i].blue_mult) {
					report("The blue_mult component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
				if(cmpp->base_pixel != scmp[i].base_pixel) {
					report("The base_pixel component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
				if(cmpp->visualid != scmp[i].visualid) {
					report("The visualid component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
				if(cmpp->killid != scmp[i].killid ) {
					report("The killid component of the XStandardColormap structure %d was incorrect.", i);
					FAIL;
				} else
					CHECK;
			
			}
		}
		XFree((char*)rscmp);
	}

	CHECKPASS(ncmp * 10 + 3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetRGBColormaps-2.(A)
 * When the property named by the property argument is not set
 * on the window named by the w argument, or is not of type
 * RGB_COLOR_MAP, or is not of format 32, or is not of length
 * n1*8+ n2*9+ n3*10 elements (where n1,n2,n3 (>= 0 and
 * n1+n2+n3 (>= 1), then a call to XGetRGBColormaps does not
 * set the std_colormap_return or count_return arguments and
 * returns zero.
 */
static void
t002()
{
Status			status;
char			*when;
XStandardColormap	*rscmp = (XStandardColormap *) NULL;
int			rncmp = 0;
XVisualInfo		*vp;
Atom			prop = XA_RGB_BEST_MAP;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	std_colormap_return = &rscmp;
	count_return = &rncmp;
	property = prop;

	w = makewin(display, vp);
/* Property unset */

	rscmp = (XStandardColormap *) -1;
	rncmp = -1;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetRGBColormaps(display, w, std_colormap_return, count_return, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	when = "when the RGB_BEST_MAP property was not set.";

	if( status != False ) {
		report("%s() did not return False %s", TestName, when);
		FAIL;
	} else
		CHECK;

	if(rscmp != (XStandardColormap *) -1) {
		report("The std_colormap_return argument was updated %s", when);
		FAIL;
	} else
		CHECK;

	if(rncmp != -1) {
		report("The count_return argument was updated %s", when);
		FAIL;
	} else
		CHECK;

	when = "when the RGB_BEST_MAP property format was 8.";

	w = makewin(display, vp);
/* format 8 */
 	XChangeProperty(display, w, prop,  XA_RGB_COLOR_MAP, 8, PropModeReplace, (unsigned char *) &scmp1, 10);

	rscmp = (XStandardColormap *) -1;
	rncmp = -1;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetRGBColormaps(display, w, std_colormap_return, count_return, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( status != False ) {
		report("%s() did not return False %s", TestName, when);
		FAIL; 
	} else
		CHECK;

	if(rscmp != (XStandardColormap *) -1) {
		report("The std_colormap_return argument was updated %s", when);
		FAIL;
	} else
		CHECK;

	if(rncmp != -1) {
		report("The count_return argument was updated %s", when);
		FAIL;
	} else
		CHECK;

	when = "when the RGB_BEST_MAP property type was ATOM.";

	w = makewin(display, vp);
/* type ATOM */
 	XChangeProperty(display, w, prop,  XA_ATOM, 32, PropModeReplace, (unsigned char *) &scmp1, 10);

	rscmp = (XStandardColormap *) -1;
	rncmp = -1;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetRGBColormaps(display, w, std_colormap_return, count_return, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( status != False ) {
		report("%s() did not return False %s.", TestName, when);
		FAIL;
	} else
		CHECK;

	if(rscmp != (XStandardColormap *) -1) {
		report("The std_colormap_return argument was updated %s", when);
		FAIL;
	} else
		CHECK;

	if(rncmp != -1) {
		report("The count_return argument was updated %s", when);
		FAIL;
	} else
		CHECK;

	when = "when the RGB_BEST_MAP property number of elements was 7.";

	w = makewin(display, vp);
/* Bad number of elements */
 	XChangeProperty(display, w, prop,  XA_RGB_COLOR_MAP, 32, PropModeReplace, (unsigned char *) &scmp1, 7);

	rscmp = (XStandardColormap *) -1;
	rncmp = -1;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetRGBColormaps(display, w, std_colormap_return, count_return, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( status != False ) {
		report("%s() did not return False %s.", TestName, when);
		FAIL;
	} else
		CHECK;

	if(rscmp != (XStandardColormap *) -1) {
		report("std_colormap_return argument was updated %s", when);
		FAIL;
	} else
		CHECK;

	if(rncmp != -1) {
		report("The count_return argument was updated %s", when);
		FAIL;
	} else
		CHECK;

	CHECKPASS(12);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetRGBColormaps-3.(A)
 * When the visualid is not specified in the property
 * colormaps, then the visualid of the returned structures is
 * set to the default visual for the screen of the window w.
 */
static void
t003()
{
Status			status;
XStandardColormap	*rscmp = (XStandardColormap *) NULL;
int			rncmp = 0;
XVisualInfo		*vp;
Visual			*vi;
VisualID		vid;
Atom			prop = XA_RGB_GRAY_MAP;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	std_colormap_return = &rscmp;
	count_return = &rncmp;
	property = prop;

	w = makewin(display, vp);
 	XChangeProperty(display, w, prop,  XA_RGB_COLOR_MAP, 32, PropModeReplace, (unsigned char *) &scmp1, 8);

	startcall(display);
	if (isdeleted())
		return;
	status = XGetRGBColormaps(display, w, std_colormap_return, count_return, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	vi = XDefaultVisual(display, XDefaultScreen(display));
	vid = XVisualIDFromVisual(vi);

	if(status == False) {
		delete("%s() returned False.", TestName);
		return;
	} else
		CHECK;

	if(vid != rscmp->visualid) {
		report("%s() did not report the default visual in the returned visualid component.", TestName);
		FAIL;
	} else
		CHECK;

	XFree((char*)rscmp);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetRGBColormaps-4.(A)
 * When the killid is not specified in the property colormaps,
 * then the killid of the returned structures is set to None.
 */
static void
t004()
{
Status			status;
XStandardColormap	*rscmp = (XStandardColormap *) NULL;
int			rncmp = 0;
XVisualInfo		*vp;
Atom			prop = XA_RGB_GRAY_MAP;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	std_colormap_return = &rscmp;
	count_return = &rncmp;
	property = prop;

	w = makewin(display, vp);
 	XChangeProperty(display, w, prop,  XA_RGB_COLOR_MAP, 32, PropModeReplace, (unsigned char *) &scmp1, 9);

	startcall(display);
	if (isdeleted())
		return;
	status = XGetRGBColormaps(display, w, std_colormap_return, count_return, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == False) {
		delete("%s() returned False.", TestName);
		return;
	} else
		CHECK;

	if(rscmp->killid != None) {
		report("%s() did not set the returned killid component to None.", TestName);
		FAIL;
	} else
		CHECK;

	XFree((char*)rscmp);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAto.mc */
/*
 * Assertion XGetRGBColormaps-5.(A)
 * When an atom argument does not name a valid Atom, then a
 * BadAtom error occurs.
 */
/* End of included file error/EAto.mc */

/* Including from file error/EAto.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_ATOM = ~0L;
	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XGetRGBColormaps(display, w, std_colormap_return, count_return, property);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadAtom) {
		report("Got %s, Expecting BadAtom", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadAtom)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAto.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XGetRGBColormaps-6.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XGetRGBColormaps(display, w, std_colormap_return, count_return, property);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
