/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: icnfywdw.m,v 1.11 92/06/11 17:29:36 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XIconifyWindow(display, w, screen_number)
 *   Display	*display;
 *   Window	w;
 *   int	screen_number;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XIconifyWindow	1
char    *TestName = "XIconifyWindow";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XIconifyWindow function
 */
static Display	*display;
static Window	w;
static int	screen_number;


static int 	StatusReturn;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	screen_number = DefaultScreen(Dsp);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XIconifyWindow-1.(A)
 * A call to XIconifyWindow sends a WM_CHANGE_STATE
 * ClientMessage event with a window of w, a format of 32 and
 * a first data element of IconicState to the root window of
 * the screen specified by the screen_number argument using an
 * event mask of SubstructureRedirectMask|
 * SubstructureNotifyMask and returns non-zero.
 */
static void
t001()
{
Status			status;
Atom			wm_change_state;
XEvent			ev, rev;
int			i;
int			nevents = 0;
XVisualInfo		*vp;
XWindowAttributes	watts;
unsigned long		event_mask[2];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);
	screen_number = DefaultScreen(display);

	event_mask[0] = SubstructureNotifyMask;
	event_mask[1] = SubstructureRedirectMask;

	if( (wm_change_state = XInternAtom(display, "WM_CHANGE_STATE", False)) == None) {
		delete("The string \"WM_CHANGE_STATE\" could not be interned.");
		return;
	} else
		CHECK;

	ev.type = ClientMessage;
	ev.xany.display = display;


	ev.xclient.window = w;
	ev.xclient.message_type = wm_change_state;
	ev.xclient.format = 32;
	ev.xclient.data.l[0] = IconicState;
	ev.xclient.data.l[1] = 0;
	ev.xclient.data.l[2] = 0;
	ev.xclient.data.l[3] = 0;
	ev.xclient.data.l[4] = 0;

	for(i=0; i < 2; i++) {

		startcall(display);
		XSelectInput(display, DRW(display), event_mask[i]);
		endcall(display);
	
		if(geterr() != Success) {
			delete("XSelectInput() failed with an event mask of 0x%lx.", event_mask[i]);
			return;
		} else
			CHECK;
	
		startcall(display);
		if (isdeleted())
			return;
		status = XIconifyWindow(display, w, screen_number);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	
		if(status == 0) {
			delete("%s() returned zero.", TestName);
			return;
		} else
			CHECK;
	
		rev.type = -1;
		if( (nevents = getevent(display, &rev)) == 0 ) {
			report("No event was generated.");
			FAIL;
		} else {
			CHECK;
			if(nevents != 1) {
				delete("There were %d events generated instead of 1.", nevents);
				return;
			} else	{
				CHECK;
	
				rev.xclient.data.l[1] = 0;
				rev.xclient.data.l[2] = 0;
				rev.xclient.data.l[3] = 0;
				rev.xclient.data.l[4] = 0;
	
				if( checkevent(&ev, &rev) != 0 ) {
					FAIL;
				} else
					CHECK;
			}
		}
	}

	CHECKPASS(11);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XIconifyWindow-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the atom name (lqWM_CHANGE_STATE(rq cannot be
 * interned, then a call to XIconifyWindow does not send a
 * message and returns zero.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XIconifyWindow-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the ClientMessage event cannot be sent, then a call to
 * XIconifyWindow returns zero.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
