/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rcnfgrwmwd.m,v 1.11 92/06/11 17:29:43 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XReconfigureWMWindow(display, w, screen_number, value_mask, values)
 *   Display		*display;
 *   Window		w;
 *   int		screen_number;
 *   unsigned int	value_mask;
 *   XWindowChanges	*values;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XReconfigureWMWindow	1
char    *TestName = "XReconfigureWMWindow";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XReconfigureWMWindow function
 */
static Display		*display;
static Window		w;
static int		screen_number;
static unsigned int	value_mask;
static XWindowChanges	*values;


static int 	StatusReturn;

XWindowChanges	wchanges;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	screen_number = DefaultScreen(Dsp);
	value_mask = 0;
	values = &wchanges;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XReconfigureWMWindow-1.(A)
 * A call to XReconfigureWMWindow issues a ConfigureWindow
 * request on the specified top-level window named by the w
 * argument.
 */
static void
t001()
{
Status			status;
XEvent			ev;
int			nevents = 0;
XVisualInfo		*vp;
XWindowChanges		changes;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);
	XSelectInput(display, w, StructureNotifyMask);

	screen_number = DefaultScreen(display);
	value_mask = CWBorderWidth;
	changes.border_width = 5;
	values = &changes;

	startcall(display);
	if (isdeleted())
		return;
	status = XReconfigureWMWindow(display, w, screen_number, value_mask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == 0) {
		report("%s() returned zero.", TestName);
		FAIL;
	} else
		CHECK;

	if( (nevents = getevent(display, &ev)) == 0 ) {
		report("No event was generated.");
		FAIL;
		
	} else {

		CHECK;
		if(nevents != 1) {
			delete("There were %d events generated instead of 1.", nevents);
			return;
		} else	
			CHECK;

		if(ev.type != ConfigureNotify) {
			report("The type component of the generated event was %d instead of ConfigureWindow (%d)",
				ev.xclient.type, ConfigureNotify);
			FAIL;		
		} else
			CHECK;
	}

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReconfigureWMWindow-2.(A)
 * When the ConfigureWindow request fails with a BadMatch
 * error, then a call to XReconfigureWMWindow sends a synthetic
 * ConfigureRequestEvent to the root of the window specified by
 * the w argument, with the event containing the same
 * configuration parameters specified by the values and
 * valuemask arguments and having a window element of w and a
 * parent element equal to the receiving root window, using an
 * event mask of SubstructureRedirectMask|
 * SubstructureNotifyMask and returns non-zero.
 */
static void
t002()
{
Status			status;
XEvent			rev, expectev;
int			nevents = 0;
int			i;
XVisualInfo		*vp;
XWindowChanges		changes;
unsigned long		event_mask[2];
unsigned long		valuemask = CWSibling /**/| CWStackMode/**/ | CWX | CWY | CWWidth | CWHeight | CWBorderWidth;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mask[0] = SubstructureNotifyMask;
	event_mask[1] = SubstructureRedirectMask;

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);
	
	expectev.type = ConfigureRequest;
	expectev.xany.display = display;
	
	expectev.xconfigurerequest.type = ConfigureRequest;
	expectev.xconfigurerequest.send_event = True;
	expectev.xconfigurerequest.display = display;
	expectev.xconfigurerequest.window = w;
	expectev.xconfigurerequest.parent = DRW(display);
	expectev.xconfigurerequest.x = 1;
	expectev.xconfigurerequest.y = 2;
	expectev.xconfigurerequest.width = 3;
	expectev.xconfigurerequest.height = 4;
	expectev.xconfigurerequest.border_width = 5;
	expectev.xconfigurerequest.above = DRW(display);
	expectev.xconfigurerequest.detail = Above;
	expectev.xconfigurerequest.value_mask = valuemask;
	
	screen_number = DefaultScreen(display);
	value_mask = valuemask; 
	changes.x = 1;
	changes.y = 2;
	changes.width = 3;
	changes.height = 4;
	changes.border_width = 5;
	changes.sibling = DRW(display);
	changes.stack_mode = Above;
	values = &changes;
	

	for(i=0; i<2; i++) {

		startcall(display);
		XSelectInput(display, DRW(display), event_mask[i]);
		endcall(display);

		if(geterr() != Success) {
			delete("XSelectInput() failed with an event mask of 0x%lx.", event_mask[i]);
			return;
		} else
			CHECK;

		startcall(display);
		if (isdeleted())
			return;
		status = XReconfigureWMWindow(display, w, screen_number, value_mask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
	
		if(status == 0) {
			report("%s() returned zero.", TestName);
			FAIL;
		} else
			CHECK;
	
		rev.type = -1;
		if( (nevents = getevent(display, &rev)) == 0 ) {
			report("No event was generated.");
			FAIL;
	
		} else {
	
			CHECK;
			if(nevents != 1){
				delete("There were %d events generated instead of 1.", nevents);
				return;
			} else	
				CHECK;
	
			if( checkevent( &expectev, &rev) != 0)
				FAIL;
			else
				CHECK;
	
		}
	}

	CHECKPASS(10);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReconfigureWMWindow-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the ConfigureRequestEvent is not successfully sent,
 * then a call to XReconfigureWMWindow returns zero.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XReconfigureWMWindow-4.(A)
 * When the value_mask argument contains the bits CWWidth or
 * CWHeight and either of the width or height components of the
 * XWindowChanges structure named by the values argument is
 * zero, then a BadValue error occurs.
 */
static void
t004()
{
Status			status;
XEvent			ev;
int			nevents = 0;
XVisualInfo		*vp;
XWindowChanges		changes;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);

	screen_number = DefaultScreen(display);
	value_mask = CWHeight ;
	changes.height = 0;
	values = &changes;
	startcall(display);
	if (isdeleted())
		return;
	status = XReconfigureWMWindow(display, w, screen_number, value_mask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadValue)
		CHECK;

	changes.width = 0;
	value_mask = CWWidth;
	startcall(display);
	if (isdeleted())
		return;
	status = XReconfigureWMWindow(display, w, screen_number, value_mask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadValue)
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XReconfigureWMWindow-5.(A)
 * When the window argument does not name a valid window, then
 * a BadWindow error occurs.
 */
static void
t005()
{
Status status;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

        A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	status = XReconfigureWMWindow(display, w, screen_number, value_mask, values);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

        if (geterr() == BadWindow)
                PASS;
        else
                FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
