/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: sticnszs.m,v 1.5 92/06/11 17:30:14 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XSetIconSizes(display, w, size_list, count)
 *   Display		*display;
 *   Window		w;
 *   XIconSize	*size_list;
 *   int		count;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetIconSizes	1
char    *TestName = "XSetIconSizes";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XSetIconSizes function
 */
static Display		*display;
static Window		w;
static XIconSize	*size_list;
static int		count;


#include	"Xatom.h"
XIconSize	sizelist;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	size_list = &sizelist;
	count = 1;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetIconSizes-1.(A)
 * A call to XSetIconSizes sets the WM_ICON_SIZE property for
 * the window w to be of type WM_ICON_SIZE, format 32 and to
 * have value set to the count XIconSize structures named by
 * the size_list argument.
 */
static void
t001()
{
#define		NumPropIconSizeElements 6
XVisualInfo	*vp;
unsigned long	leftover, nitems, len;
int		actual_format;
Atom		actual_type;
unsigned char	*prop = (unsigned char *) NULL;
long		*rsizelist = (long *) NULL;
XIconSize	sizelist[7], *sp, rsize;
int		cnt = 7;
int		i, v;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);

	for(i=0, sp=sizelist, v=0; i<cnt; i++, sp++) {
		sp->min_width = v++;
		sp->min_height = v++;
		sp->max_width = v++;
		sp->max_height = v++;
		sp->width_inc = v++;
		sp->height_inc = v++;
	}

	count = cnt;
	size_list = sizelist;
	startcall(display);
	if (isdeleted())
		return;
	XSetIconSizes(display, w, size_list, count);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	len = cnt * NumPropIconSizeElements;
	if(XGetWindowProperty(display, w, XA_WM_ICON_SIZE,
                            0L, len, False,
                            AnyPropertyType, &actual_type, &actual_format,
                            &nitems, &leftover, &prop) != Success) {
		delete("XGetWindowProperty() did not return Success.");
		return;
	} else
		CHECK;

	if(leftover != 0) {
		report("The leftover elements numbered %lu instead of 0", leftover);
		FAIL;
	} else
		CHECK;

	if(actual_format != 32) {
		report("The format of the WM_ICON_SIZE property was %lu instead of 32", actual_format);
		FAIL;
	} else
		CHECK;

	if(actual_type != XA_WM_ICON_SIZE) {
		report("The type of the WM_ICON_SIZE property was %lu instead of WM_ICON_SIZE (%lu)", actual_type, (long) XA_WM_ICON_SIZE);
		FAIL;
	} else
		CHECK;

	rsizelist = (long *) prop;
	if( rsizelist == (long *) NULL) {
		report("No value was obtained for the WM_ICON_SIZES property.");
		FAIL;
	} else {

		CHECK;
		if( nitems != len) {
			report("The WM_ICON_SIZES property comprised %ul elements instead of %d", nitems, cnt);
			FAIL;
		} else 
			if(actual_format == 32) {

				CHECK;
				/* If the returned format is 32, the returned data is represented as a long array. */
				for(i=0, sp=&rsize, v=0; i<cnt; i++) {
					rsize.min_width = *rsizelist++;
					rsize.min_height = *rsizelist++;
					rsize.max_width = *rsizelist++;
					rsize.max_height = *rsizelist++;
					rsize.width_inc = *rsizelist++;
					rsize.height_inc = *rsizelist++;
				
					if(sp->min_width != v++) {
						report("The min_width component of the XIconSize structure %d was %d instead of %d.", i, sp->min_width, v-1);
						FAIL;
					} else
						CHECK;
				
					if(sp->min_height != v++) {
						report("The min_height component of the XIconSize structure %d was %d instead of %d.", i, sp->min_height, v-1);
						FAIL;
					} else
						CHECK;
				
					if(sp-> max_width != v++) {
						report("The max_width component of the XIconSize structure %d was %d instead of %d.", i, sp-> max_width, v-1);
						FAIL;
					} else
						CHECK;
				
					if(sp->max_height != v++) {
						report("The max_height component of the XIconSize structure %d was %d instead of %d.", i, sp->max_height, v-1);
						FAIL;
					} else
						CHECK;
				
					if(sp->width_inc != v++) {
						report("The width_inc component of the XIconSize structure %d was %d instead of %d.", i, sp->width_inc, v-1);
						FAIL;
					} else
						CHECK;
				
					if(sp-> height_inc != v++) {
						report("The height_inc component of the XIconSize structure %d was %d instead of %d.", i, sp->height_inc, v-1);
						FAIL;
					} else
						CHECK;
				}
				XFree( prop);			
			}
	}

	CHECKPASS(6*cnt+6);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetIconSizes-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XSetIconSizes-3.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetIconSizes(display, w, size_list, count);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
