/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: txtprprtyt.m,v 1.8 92/06/11 17:32:23 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XTextPropertyToStringList(text_prop, list_return, count_return)
 *   XTextProperty	*text_prop;
 *   char		***list_return;
 *   int		*count_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XTextPropertyToStringList	1
char    *TestName = "XTextPropertyToStringList";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XTextPropertyToStringList function
 */
static XTextProperty	*text_prop;
static char		***list_return;
static int		*count_return;


static int 	StatusReturn;

#include	"Xatom.h"
char		*nullstr = "<NULL>";
char		*str1 = "Xtest_._.";
char		*str2 = "test._";
char		*str3 = "string._..";

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	text_prop = 0;
	list_return = 0;
	count_return = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XTextPropertyToStringList-1.(A)
 * When the encoding component of the text_prop argument is
 * STRING and the format component is 8, then a call to
 * XTextPropertyToStringList returns in the list_string
 * argument the list of strings representing the null-separated
 * elements of the XTextProperty structure named by the
 * text_prop argument, returns in count_return the number of
 * strings and returns non-zero.
 */
static void
t001()
{
Status		status;
XTextProperty	tp;
char		**list;
int		count;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	tp.value = (unsigned char *) "Xtest_._.\0test._\0string._..";
	tp.nitems = 27;
	tp.format = 8;
	tp.encoding = XA_STRING;
	text_prop = &tp;
	list_return = &list;
	count_return = &count;
	startcall(Dsp);
	if (isdeleted())
		return;
	status = XTextPropertyToStringList(text_prop, list_return, count_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == 0) {
		delete("%s() returned zero.", TestName);
		return;
	} else
		CHECK;

	if(count != 3) {
		report("The returned number of strings was %d instead of %d", count, 3);
		FAIL;
	} else {
		CHECK;

		if(strcmp(list[0], str1) != 0 ||strcmp(list[1], str2) != 0 || strcmp(list[2], str3)) {
			report("The returned strings were:");
			report("\"%s\"", list[0] == NULL ? nullstr : list[0]);
			report("\"%s\"", list[1] == NULL ? nullstr : list[1]);
			report("\"%s\"", list[2] == NULL ? nullstr : list[2]);
			report("instead of:");
			report("\"%s\"", str1);
			report("\"%s\"", str2);
			report("\"%s\"", str3);
		} else
			CHECK;

	}

	XFreeStringList(list);
	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XTextPropertyToStringList-2.(A)
 * When the encoding component of the text_prop argument is
 * not STRING, then a call to XTextPropertyToStringList sets no
 * return values and returns zero.
 */
static void
t002()
{
Status		status;
XTextProperty	tp;
char		**list = (char **) -1;
int		count = -1;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	tp.value = (unsigned char *) "Xtest_._.\0test._\0string._..";
	tp.nitems = 27;
	tp.format = 8;
	tp.encoding = XA_WINDOW;
	text_prop = &tp;
	list_return = &list;
	count_return = &count;
	startcall(Dsp);
	if (isdeleted())
		return;
	status = XTextPropertyToStringList(text_prop, list_return, count_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != 0) {
		report("%s() did not return zero.", TestName);
		FAIL;
	} else
		CHECK;

	if(list != (char**) -1) {
		report("The list_return argument was altered.");
		FAIL;
	} else
		CHECK;

	if(count != -1) {
		report("The count_return argument was altered.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XTextPropertyToStringList-3.(A)
 * When the format component of the text_prop argument is not
 * 8, then a call to XTextPropertyToStringList sets no return
 * values and returns zero.
 */
static void
t003()
{
Status		status;
XTextProperty	tp;
char		**list = (char **) -1;
int		count = -1;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	tp.value = (unsigned char *) "Xtest_._.\0test._\0string._..";
	tp.nitems = 27;
	tp.format = 16;
	tp.encoding = XA_STRING;
	text_prop = &tp;
	list_return = &list;
	count_return = &count;
	startcall(Dsp);
	if (isdeleted())
		return;
	status = XTextPropertyToStringList(text_prop, list_return, count_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != 0) {
		report("%s() did not return zero.", TestName);
		FAIL;
	} else
		CHECK;

	if(list != (char**) -1) {
		report("The list_return argument was altered.");
		FAIL;
	} else
		CHECK;

	if(count != -1) {
		report("The count_return argument was altered.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XTextPropertyToStringList-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When insufficient memory is available for the list and its
 * elements, then a call to XTextPropertyToStringList sets no
 * return values and returns zero.
 */
static void
t004()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
