/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: addpxl.m,v 1.5 92/06/11 17:33:00 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XAddPixel(ximage, value)
 *   XImage	*ximage;
 *   long	value;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XAddPixel	1
char    *TestName = "XAddPixel";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XAddPixel function
 */
static XImage	*ximage;
static long	value;


mpattern(display, d, w, h, dep)
Display		*display;
Drawable	d;
unsigned int	w;
unsigned int	h;
int		dep;
{
int		i;
int		j;
int		mod;
GC		gc;
unsigned long	val;

	gc = makegc(display, d);
	mod = 1<<dep;
	val = 0;
	for(j=0; j<h; j++)
		for(i=0; i<w; i++) {
			XSetForeground(display, gc, val);
			XDrawPoint(display, d, gc, i, j);
			val = (val + 1) % mod;
		}
}

int
mcheck(xi, w, h, dep)
XImage		*xi;
unsigned int	w;
unsigned int	h;
int		dep;
{
int		i;
int		j;
int		mod;
unsigned long	val;

	mod = 1<<dep;
	val = value;
	for(j=0; j<h; j++)
		for(i=0; i<w; i++) {
			if(XGetPixel(ximage, i, j) != (val & 0xffffffff)){
				return 0;
			}
			val = (val + 1) % mod;
		}
	return 1;
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	ximage = 0;
	value = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XAddPixel-1.(A)
 * A call to XAddPixel adds the value argument to every pixel
 * in the ximage argument.
 */
static void
t001()
{
XVisualInfo	*vi;
int		npix;
unsigned int	width;
unsigned int	height;
Pixmap		pm;
Window		win;
int		i;
static int	fmats[2] = { XYPixmap, ZPixmap };
	
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(resetvinf(VI_PIX); nextvinf(&vi);) {

		pm = makepixm(Dsp, vi);
		getsize(Dsp, pm, &width, &height);
		width = width > 17 ? 17 : width;
		height = height > 19  ? 19 : height;
		mpattern(Dsp, pm, width, height, vi->depth);

		for(i=0; i<2; i++) {
			ximage = XGetImage(Dsp, pm, 0,0, width, height, AllPlanes, fmats[i]);
			if( ximage == (XImage *) NULL ) {
				delete("XGetImage() returned NULL.");
				return;
			} else {

				value = (1 << vi->depth) - 1;
				startcall(Dsp);
				if (isdeleted())
					return;
				XAddPixel(ximage, value);
				endcall(Dsp);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				if(mcheck(ximage, width, height, vi->depth) == 0) {
					report("XImage structure was not correct.");
					FAIL;
				} else
					CHECK;

				XDestroyImage(ximage);
			}

		}
	}
	npix = nvinf();

	for(resetvinf(VI_WIN); nextvinf(&vi);) {

		win = makewin(Dsp, vi);
		getsize(Dsp, win, &width, &height);
		width = width > 17 ? 17 : width;
		height = height > 19  ? 19 : height;
		mpattern(Dsp, win, width, height, vi->depth);

		for(i=0; i<2; i++) {
			ximage = XGetImage(Dsp, win, 0,0, width, height, AllPlanes, fmats[i]);
			if(ximage == (XImage *) NULL) {
				delete("XGetImage() returned NULL.");
				return;
			} else {
				value = (1 << vi->depth) - 1;
				startcall(Dsp);
				if (isdeleted())
					return;
				XAddPixel(ximage, value);
				endcall(Dsp);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				if(mcheck(ximage, width, height, vi->depth) == 0) {
					report("XImage structure was not correct.");
					FAIL;
				} else
					CHECK;

				XDestroyImage(ximage);
			}
		}

	}

	CHECKPASS(2 * (npix + nvinf()));
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
