/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: clpbx.m,v 1.8 92/06/11 17:33:08 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XClipBox(r, rect_return)
 *   Region		r;
 *   XRectangle	*rect_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XClipBox	1
char    *TestName = "XClipBox";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XClipBox function
 */
static Region		r;
static XRectangle	*rect_return;


Status
checknotclear(disp, d, fg)
Display		*disp;
Drawable	d;
unsigned long	fg;
{
XImage			*im;
unsigned long		pix;
unsigned int		width;
unsigned int		height;
int			x;
int			y;

	getsize(Dsp, d, &width, &height);
	
        if ((im = XGetImage(disp, d, 0, 0, width, height, AllPlanes, ZPixmap)) == (XImage*) NULL) {
                delete("XGetImage failed");
                return(False);
        }

	for(y=0; y<height; y++)		
		for(x=0; x<width; x++)
			if(XGetPixel(im, x, y) == fg)
				return(True);
	return(False);
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	r = 0;
	rect_return = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XClipBox-1.(A)
 * A call to XClipBox returns in the rect_return argument the
 * smallest rectangle enclosing the region r.
 */
static void
t001()
{
static XPoint   points[] = { {10, 10},  {74, 10},  {75, 30},  {65, 60},  {65, 30}, {75, 60},
                         {30, 50},  {30, 74},  {60, 74},   {60, 5},   {70, 5},  {70, 75}, {10, 85} };
Region		reg;
XRectangle	rect;
XVisualInfo	*vi;
Window		win;
GC		gc;
GC		gc2;
unsigned int		width;
unsigned int		height;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	reg = XPolygonRegion( points, NELEM(points), WindingRule);
	resetvinf(VI_WIN);
	nextvinf(&vi);
	win = makewin(Dsp, vi);
	gc = makegc(Dsp, win);
	gc2 = makegc(Dsp, win);
	XSetRegion(Dsp, gc, reg);
	XSetForeground(Dsp, gc, W_FG);
	XSetForeground(Dsp, gc2, W_BG);
	getsize(Dsp, win, &width, &height);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;

	r = reg;
	rect_return = &rect;
	startcall(Dsp);
	if (isdeleted())
		return;
	XClipBox(r, rect_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDestroyRegion(reg);

	XFillRectangle(Dsp, win, gc2, rect.x, rect.y, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) != 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently large rectangle.", TestName);
		FAIL;
		return;
	} else
		CHECK;

	win = makewin(Dsp, vi);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;
	XFillRectangle(Dsp, win, gc2, rect.x+1, rect.y, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) == 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently small rectangle.in the x-direction.", TestName);
		FAIL;
	} else
		CHECK;

	win = makewin(Dsp, vi);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;
	XFillRectangle(Dsp, win, gc2, rect.x-1, rect.y, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) == 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently small rectangle in the x-direction.", TestName);
		FAIL;
	} else
		CHECK;

	win = makewin(Dsp, vi);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;

	XFillRectangle(Dsp, win, gc2, rect.x, rect.y+1, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) == 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently small rectangle in the y-direction.", TestName);
		FAIL;
	} else
		CHECK;

	win = makewin(Dsp, vi);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;

	XFillRectangle(Dsp, win, gc2, rect.x, rect.y-1, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) == 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently small rectangle in the y-direction.", TestName);
		FAIL;
	} else
		CHECK;

	CHECKPASS(10);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
