/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: eqlrgn.m,v 1.5 92/06/11 17:34:18 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Bool
 *   XEqualRegion(r1, r2)
 *   Region	r1;
 *   Region	r2;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XEqualRegion	1
char    *TestName = "XEqualRegion";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XEqualRegion function
 */
static Region	r1;
static Region	r2;


static	XRectangle	rect1 = {0,0, 7,13 };
static	XRectangle	rect2 = {5,5, 10,15 };
static	XRectangle	rect3 = {0,0, 17,20 };
static	XRectangle	rect4 = {-1,-1, 1,1 };

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	r1 = 0;
	r2 = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XEqualRegion-1.(A)
 * When the regions r1 and r2 have the same offset, size and
 * shape, then a call to XEqualRegion returns True.
 */
static void
t001()
{
Bool	res;
Region	R1;
Region	R2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R1 = makeregion();
	R2 = makeregion();
	if(isdeleted()) return;

	XUnionRectWithRegion(&rect1, R1, R1);
	XUnionRectWithRegion(&rect3, R1, R1);
	XUnionRectWithRegion(&rect2, R1, R1);
	XUnionRectWithRegion(&rect3, R2, R2);

	r1 = R1;
	r2 = R2;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XEqualRegion(r1, r2);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != True) {
		report("%s() did not return True for two equal regions.", TestName);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XEqualRegion-2.(A)
 * When the regions r1 and r2 do not have the same offset,
 * then a call to XEqualRegion returns False.
 */
static void
t002()
{
Bool	res;
Region	R1;
Region	R2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R1 = makeregion();
	R2 = makeregion();

	if(isdeleted()) return;

	XUnionRectWithRegion(&rect1, R1, R1);
	XUnionRectWithRegion(&rect1, R2, R2);
	XOffsetRegion(R2,1,0);

	r1 = R1;
	r2 = R2;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XEqualRegion(r1, r2);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != False) {
		report("%s() did not return False for regions with a different offset.", TestName);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XEqualRegion-3.(A)
 * When the regions r1 and r2 do not have the same size, then
 * a call to XEqualRegion returns False.
 */
static void
t003()
{
Bool	res;
Region	R1;
Region	R2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R1 = makeregion();
	R2 = makeregion();

	if(isdeleted()) return;

	XUnionRectWithRegion(&rect1, R1, R1);
	XUnionRectWithRegion(&rect1, R2, R2);
	XShrinkRegion(R2, 1, 0);

	r1 = R1;
	r2 = R2;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XEqualRegion(r1, r2);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != False) {
		report("%s() did not return False for regions with a different size.", TestName);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XEqualRegion-4.(A)
 * When the regions r1 and r2 do not have the same shape, then
 * a call to XEqualRegion returns False.
 */
static void
t004()
{
Bool	res;
Region	R1;
Region	R2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R1 = makeregion();
	R2 = makeregion();

	if(isdeleted()) return;

	XUnionRectWithRegion(&rect1, R1, R1);
	XUnionRectWithRegion(&rect1, R2, R2);
	XUnionRectWithRegion(&rect4, R2, R2);

	r1 = R1;
	r2 = R2;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XEqualRegion(r1, r2);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != False) {
		report("%s() did not return False for two different regions.", TestName);
		FAIL;
	} else
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
