/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ftchbyts.m,v 1.5 92/06/11 17:34:41 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char *
 *   XFetchBytes(display, nbytes_return)
 *   Display	*display;
 *   int	*nbytes_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XFetchBytes	1
char    *TestName = "XFetchBytes";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XFetchBytes function
 */
static Display	*display;
static int	*nbytes_return;


#include	"Xatom.h"
char		buff[17];

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	nbytes_return = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XFetchBytes-1.(A)
 * When cut buffer zero contains data, then a call to
 * XFetchBytes returns in the nbytes_return argument the number
 * of bytes stored in the buffer and returns a pointer to
 * storage allocated for the data, which can be freed with
 * XFree.
 */
static void
t001()
{
char	*bp;
char	*bpr = (char *) NULL;
int	i;
int	br;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(i=NELEM(buff), bp = buff; i>0; *bp++ = (char) (i & 255), i--);

	XStoreBuffer(display, buff, NELEM(buff), 0);

	nbytes_return = &br;
	startcall(display);
	if (isdeleted())
		return;
	bp = XFetchBytes(display, nbytes_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(bp == (char *) NULL) {
		report("%s() did return the contents of cut buffer 0.", TestName);
		FAIL;
	} else {
		CHECK;

		if(br != NELEM(buff)) {
			report("%s() returned %d bytes from cut buffer 0 instead of %d.", TestName, br, NELEM(buff));
			FAIL;
		} else {
			CHECK;

			if(memcmp(bp, buff, NELEM(buff)) != 0) {
				report("%s() did not return the contents of cut buffer 0 correctly.", TestName);
				FAIL;
			} else
				CHECK;
		}

		XFree(bp);
	}

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFetchBytes-2.(A)
 * When cut buffer zero does not contain any data, then a call
 * to XFetchBytes sets the nbytes_return argument to zero, and
 * returns NULL.
 */
static void
t002()
{
char	*tstr = "XTest cut buffer 0 string";
char	*bp;
int	br;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	XStoreBytes(display, tstr, 1 + strlen(tstr));
	XDeleteProperty(display, RootWindow(display, 0), XA_CUT_BUFFER0);
	nbytes_return = &br;
	startcall(display);
	if (isdeleted())
		return;
	bp = XFetchBytes(display, nbytes_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(bp != (char *) NULL) {
		report("%s() did not return NULL.", TestName);
		FAIL;
	} else 
		CHECK;

	if(br != 0) {
		report("%s() returned %d bytes from cut buffer 0 instead of 0.", TestName, br);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
