/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: isfnctnky.m,v 1.4 92/06/11 17:36:19 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   IsFunctionKey(keysym)
 *   KeySym	keysym;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_IsFunctionKey	1
char    *TestName = "IsFunctionKey";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the IsFunctionKey function
 */
static KeySym	keysym;


#define XK_LATIN1
#define XK_MISCELLANY
#include	"keysymdef.h"
#undef	XK_MISCELLANY
#undef XK_LATIN1

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	keysym = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion IsFunctionKey-1.(A)
 * When the keysym argument is a function key, then invocation
 * of the IsFunctionKey macro returns True.
 */
static void
t001()
{
static KeySym ks[] = {
			XK_F1,
			XK_F2,
			XK_F3,
			XK_F4,
			XK_F5,
			XK_F6,
			XK_F7,
			XK_F8,
			XK_F9,
			XK_F10,
			XK_F11,
			XK_L1,
			XK_F12,
			XK_L2,
			XK_F13,
			XK_L3,
			XK_F14,
			XK_L4,
			XK_F15,
			XK_L5,
			XK_F16,
			XK_L6,
			XK_F17,
			XK_L7,
			XK_F18,
			XK_L8,
			XK_F19,
			XK_L9,
			XK_F20,
			XK_L10,
			XK_F21,
			XK_R1,
			XK_F22,
			XK_R2,
			XK_F23,
			XK_R3,
			XK_F24,
			XK_R4,
			XK_F25,
			XK_R5,
			XK_F26,
			XK_R6,
			XK_F27,
			XK_R7,
			XK_F28,
			XK_R8,
			XK_F29,
			XK_R9,
			XK_F30,
			XK_R10,
			XK_F31,
			XK_R11,
			XK_F32,
			XK_R12,
			XK_R13,
			XK_F33,
			XK_F34,
			XK_R14,
			XK_F35,
			XK_R15,
			0 };
KeySym	*ksp;
Bool	res;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(ksp = ks; *ksp; ksp++) {
		keysym = *ksp;
		startcall(Dsp);
		if (isdeleted())
			return;
		res = IsFunctionKey(keysym);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if(res != True) {
			char	*kstr = XKeysymToString(*ksp);

			report("%s() did not return True for KeySym XK_%s (value %lu).",
				TestName, kstr != (char *) NULL ? kstr : "<KeySym Undefined>", *ksp);
			FAIL;
		} else
			CHECK;

	}

	CHECKPASS(NELEM(ks) - 1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion IsFunctionKey-2.(A)
 * When the keysym argument is not a function key, then
 * invocation of the IsFunctionKey macro returns False.
 */
static void
t002()
{
Bool	res;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	keysym = XK_A;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = IsFunctionKey(keysym);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(res != False) {
		char	*kstr = XKeysymToString(keysym);

		report("%s() did not return False for KeySym XK_%s (value %lu).",
			TestName, kstr != (char *) NULL ? kstr : "<KeySym Undefined>", keysym);
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
