/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ismdfrky.m,v 1.4 92/06/11 17:36:34 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   IsModifierKey(keysym)
 *   KeySym	keysym;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_IsModifierKey	1
char    *TestName = "IsModifierKey";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the IsModifierKey function
 */
static KeySym	keysym;


#include "keysym.h"
#define XK_LATIN1
#define XK_MISCELLANY
#include	"keysymdef.h"
#undef	XK_MISCELLANY
#undef XK_LATIN1

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	keysym = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion IsModifierKey-1.(A)
 * When the keysym argument is a modifier key, then invocation
 * of the IsModifierKey macro returns True.
 */
static void
t001()
{
static KeySym ks[] = {
			XK_Shift_L,
			XK_Shift_R,
			XK_Control_L,
			XK_Control_R,
			XK_Caps_Lock,
			XK_Shift_Lock,
			XK_Meta_L,
			XK_Meta_R,
			XK_Alt_L,
			XK_Alt_R,
			XK_Super_L,
			XK_Super_R,
			XK_Hyper_L,
			XK_Hyper_R,
			0 };
KeySym	*ksp;
Bool	res;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(ksp = ks; *ksp; ksp++) {
		keysym = *ksp;
		startcall(Dsp);
		if (isdeleted())
			return;
		res = IsModifierKey(keysym);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if(res != True) {
			char	*kstr = XKeysymToString(*ksp);

			report("%s() did not return True for KeySym XK_%s (value %lu).",
				TestName, kstr != (char *) NULL ? kstr : "<KeySym Undefined>", *ksp);
			FAIL;
		} else
			CHECK;

	}

	CHECKPASS(NELEM(ks) - 1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion IsModifierKey-2.(A)
 * When the keysym argument is not a modifier key, then
 * invocation of the IsModifierKey macro returns False.
 */
static void
t002()
{
Bool	res;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	keysym = XK_A;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = IsModifierKey(keysym);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(res != False) {
		char	*kstr = XKeysymToString(keysym);

		report("%s() did not return False for KeySym XK_%s (value %lu).",
			TestName, kstr != (char *) NULL ? kstr : "<KeySym Undefined>", keysym);
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
