/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: kycdtkysym.m,v 1.6 92/06/11 17:36:58 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   KeySym
 *   XKeycodeToKeysym(display, keycode, index)
 *   Display	*display;
 *   KeyCode	keycode;
 *   int	index;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XKeycodeToKeysym	1
char    *TestName = "XKeycodeToKeysym";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XKeycodeToKeysym function
 */
static Display	*display;
static KeyCode	keycode;
static int	index;


#define XK_LATIN1
#include	"keysymdef.h"
#undef XK_LATIN1

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	keycode = 0;
	index = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XKeycodeToKeysym-1.(A)
 * A call to XKeycodeToKeysym returns the KeySym specified by
 * element index of the KeyCode vector specified by the keycode
 * argument.
 */
static void
t001()
{
KeyCode	kc;
KeySym	ks;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	kc = XKeysymToKeycode(display, XK_a);

	if(kc == 0) {
		delete("XKeysymToKeycode() returned 0 for KeySym XK_a");
		return;
	} else
		CHECK;

	keycode = kc;
	index = 0;
	startcall(display);
	if (isdeleted())
		return;
	ks = XKeycodeToKeysym(display, keycode, index);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != XK_a) {
		report("%s() returned KeySym %lu instead of %lu for KeyCode %lu and index %d.",
			TestName, (long) ks, (long) XK_a, (long) kc, index);
		FAIL;
	} else
		CHECK;


	index = 1;
	startcall(display);
	if (isdeleted())
		return;
	ks = XKeycodeToKeysym(display, keycode, index);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != XK_A) {
		report("%s() returned KeySym %lu instead of %lu for KeyCode %lu and index %d.",
			TestName, (long) ks, (long) XK_A, (long) kc, index);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XKeycodeToKeysym-2.(A)
 * When the symbol specified by element index of the KeyCode
 * vector specified by the keycode argument is not defined,
 * then a call to XKeycodeToKeysym returns NoSymbol.
 */
static void
t002()
{
int	maxi,mini;
KeyCode	maxkc, minkc;
KeySym	ks;
int	kspkc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	XDisplayKeycodes(display, &mini, &maxi);
	minkc = (KeyCode)mini;
	maxkc = (KeyCode)maxi;

	index = 0;		
	keycode = maxkc + 1;
	startcall(display);
	if (isdeleted())
		return;
	ks = XKeycodeToKeysym(display, keycode, index);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != NoSymbol) {
		report("%s() returned KeySym value %ul instead of NoSymbol (%lu) for KeyCode %lu and index %d.",
			TestName, (long) ks, (long) NoSymbol, (long) keycode, index);
		FAIL;
	} else
		CHECK;

	keycode = minkc;
	index = 9; /* only 0-8 are valid. */
	startcall(display);
	if (isdeleted())
		return;
	ks = XKeycodeToKeysym(display, keycode, index);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != NoSymbol) {
		report("%s() returned KeySym value %ul instead of NoSymbol (%lu) for KeyCode %lu and index %d.",
			TestName, (long) ks, (long) NoSymbol, (long) keycode, index);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
