/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: kysymtstr.m,v 1.4 92/06/11 17:37:17 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char *
 *   XKeysymToString(keysym)
 *   KeySym	keysym;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XKeysymToString	1
char    *TestName = "XKeysymToString";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XKeysymToString function
 */
static KeySym	keysym;


#define XK_MISCELLANY
#include "keysymdef.h"
#undef  XK_MISCELLANY

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	keysym = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XKeysymToString-1.(A)
 * A call to XKeysymToString returns as a null-terminated
 * string the name of the KeySym specified by the keysym
 * argument.
 */
static void
t001()
{
KeySym	ks = XK_BackSpace;
char	*value = "BackSpace";
char	*res;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	keysym = ks;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XKeysymToString(keysym);	
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( res == (char *) NULL) {
		report("%s() returned NULL for KeySym XK_BackSpace.", TestName);
		FAIL;
	} else {
		CHECK;
		if(strcmp(res, value) != 0) {
			report("%s() returned \"%s\" instead of \"%s\" for KeySym XK_Backspace.",
				TestName, res, value);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XKeysymToString-2.(A)
 * When the KeySym specified by the keysym argument is not
 * defined, then a call to XKeysymToString returns NULL.
 */
static void
t002()
{
KeySym	ks = NoSymbol;
char	*res;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	keysym = ks;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XKeysymToString(keysym);	
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if( res != (char *) NULL) {
		report("%s() returned \"%s\" instead of NULL for KeySym NoSymbol.",
			TestName, res);
		FAIL;
	} else
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
