/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: lkpstr.m,v 1.10 92/06/11 17:37:34 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XLookupString(event_struct, buffer_return, bytes_buffer, keysym_return, status_in_out)
 *   XKeyEvent	*event_struct;
 *   char		*buffer_return;
 *   int		bytes_buffer;
 *   KeySym		*keysym_return;
 *   XComposeStatus	*status_in_out;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XLookupString	1
char    *TestName = "XLookupString";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XLookupString function
 */
static XKeyEvent	*event_struct;
static char		*buffer_return;
static int		bytes_buffer;
static KeySym		*keysym_return;
static XComposeStatus	*status_in_out;


static int 	ValueReturn;

#define XK_LATIN1
#define XK_MISCELLANY
#include	"keysymdef.h"
#undef XK_MISCELLANY
#undef XK_LATIN1

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	event_struct = 0;
	buffer_return = 0;
	bytes_buffer = 0;
	keysym_return = 0;
	status_in_out = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XLookupString-1.(A)
 * A call to XLookupString returns in the keysym_return
 * argument the KeySym and in the buffer_return the string of
 * maximum length bytes_buffer specified by the keycode and
 * state of the event_struct argument, using the standard shift
 * modifier computations as defined in the X protocol
 * specification, and returns the length of the returned
 * string.
 */
static void
t001()
{
KeySym		ks = 0;
KeyCode		kc;
static char	buf[]="Xtest uninitialised string.";
int		res;
int		cmplen;
int		mlen = strlen(buf);
XKeyEvent	ke;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	kc = XKeysymToKeycode(Dsp, XK_b);

	ke.type = KeyPress;
	ke.display = Dsp;
	ke.keycode = kc;
	ke.state = 0;

	event_struct = &ke;
	buffer_return = (char *) buf;
	bytes_buffer = mlen;
	keysym_return = &ks;
	status_in_out = (XComposeStatus *) NULL;

	startcall(Dsp);
	if (isdeleted())
		return;
	res = XLookupString(event_struct, buffer_return, bytes_buffer, keysym_return, status_in_out);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != XK_b) {
		report("%s() returned keysym %d instead of %d.", TestName, (int) ks, (int) XK_b);
		FAIL;
	} else
		CHECK;

	if(res != 1) {
		report("%s() returned %d instead of 1.", TestName, res);
		FAIL;
	} else
		CHECK;

	cmplen = mlen;
	if(res>0 && res<mlen)
		cmplen = res;
	buf[cmplen] = '\0';

	if(strncmp(buffer_return, "b", cmplen) != 0) {
		report("%s() returned string \"%s\" instead of \"b\".", TestName, buffer_return);
		FAIL;
	} else
		CHECK;

	ke.state = ShiftMask;
	event_struct = &ke;
	startcall(Dsp);
	if (isdeleted())
		return;
	res = XLookupString(event_struct, buffer_return, bytes_buffer, keysym_return, status_in_out);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != XK_B) {
		report("%s() returned keysym %d instead of %d.", TestName, (int) ks, (int) XK_B);
		FAIL;
	} else
		CHECK;

	cmplen = mlen;
	if(res>0 && res<mlen)
		cmplen = res;
	buf[cmplen] = '\0';

	if(strncmp(buffer_return, "B", cmplen) != 0) {
		report("%s() returned string \"%s\" instead of \"B\".", TestName, buffer_return);
		FAIL;
	} else
		CHECK;

	CHECKPASS(5);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XLookupString-2.(A)
 * When the KeySym corresponding to the event_struct argument
 * has been rebound, then the bound string is returned,
 * truncated to bytes_buffer, in the buffer_return argument.
 */
static void
t002()
{
KeySym		ks = 0;
KeyCode		kc;
static char	buf[]="XtestRebound";
static char	buf_ret[4];
int		res;
int		cmplen;
int		mlen = 3;
XKeyEvent	ke;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	kc = XKeysymToKeycode(Dsp, XK_c);
	XRebindKeysym(Dsp, XK_c, (KeySym *) NULL, 0, (unsigned char *) buf, strlen(buf));

	ke.type = KeyPress;
	ke.display = Dsp;
	ke.keycode = kc;
	ke.state = 0;

	event_struct = &ke;
	buffer_return = (char *) buf_ret;
	bytes_buffer = mlen;
	keysym_return = &ks;
	status_in_out = (XComposeStatus *) NULL;

	startcall(Dsp);
	if (isdeleted())
		return;
	res = XLookupString(event_struct, buffer_return, bytes_buffer, keysym_return, status_in_out);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != XK_c) {
		report("%s() returned keysym %d instead of %d.", TestName, (int) ks, (int) XK_b);
		FAIL;
	} else
		CHECK;

	if(res != mlen) {
		report("%s() returned %d instead of %d.", TestName, res, mlen);
		FAIL;
	} else
		CHECK;

	cmplen = mlen;
	if(res>0 && res<mlen)
		cmplen = res;
	buf[cmplen] = '\0';
	buffer_return[cmplen] = '\0';

	if(strncmp(buffer_return, buf, cmplen) != 0) {
		report("%s() returned string \"%s\" instead of \"%s\".", TestName, buffer_return, buf);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XLookupString-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the status_in_out argument is not NULL, then a call to
 * XLookupString records in it the state of compose processing.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
