/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: mtchvslinf.m,v 1.6 92/06/11 17:37:42 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XMatchVisualInfo(display, screen, depth, class, vinfo)
 *   Display	*display;
 *   int 	screen;
 *   int 	depth;
 *   int 	class;
 *   XVisualInfo	*vinfo;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XMatchVisualInfo	1
char    *TestName = "XMatchVisualInfo";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XMatchVisualInfo function
 */
static Display	*display;
static int 	screen;
static int 	depth;
static int 	class;
static XVisualInfo	*vinfo;


static int 	StatusReturn;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	screen = DefaultScreen(Dsp);
	depth = 0;
	class = 0;
	vinfo = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XMatchVisualInfo-1.(A)
 * When a visual exists on screen screen of depth depth and of
 * class class, then a call to XMatchVisualInfo returns the
 * associated XVisualInfo structure in the vinfo_return
 * argument, and returns non-zero.
 */
static void
t001()
{
Status	s;
XVisualInfo	vi;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Initialise the list of class/depth pairs from parameter XT_VISUAL_CLASSES. */
	if(initvclass() < 0) {
		delete("The list of expected visual classes could not be initialised.");
		return;
	}
	vinfo = &vi;
/* For each class/depth pair: */
	for (resetvclass(); nextvclass(&class, &depth); ) {

/*   Call xname. */
		startcall(display);
		if (isdeleted())
			return;
		s = XMatchVisualInfo(display, screen, depth, class, vinfo);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		trace("Matching depth=%d, class=%s", depth, displayclassname(class));

/*   Verify that xname does not return zero. */
		if (s == 0) {
			report("%s returned 0 with depth=%d, class=%s", 
				TestName, depth, displayclassname(class));
			FAIL;
/*   Verify that class and depth of returned structure are correct. */
		} else if (vi.class != class || vi.depth != depth) {
			/*
			 * Since the actual visual returned is not defined then
			 * all we can do is check that class and depth are OK
			 * and output all the other fields for regression test purposes.
			 */
			report(" Incorrect class or depth found");
			report(" Got %s, depth %d", displayclassname(vi.class), vi.depth);
			report(" Expecting %s, depth", displayclassname(class),
				depth);
			FAIL;
		} else {
			trace(" visualid=0x%x, screen=%d, depth=%u, class=%s",
				vi.visualid, vi.screen, vi.depth,
				displayclassname(vi.class));
			trace(" red_mask=0x%x, green_mask=0x%x, blue_mask=0x%x",
				vi.red_mask,
				vi.green_mask,
				vi.blue_mask);
			trace(" colormap_size=%d, bits_per_rgb=%d",
				vi.colormap_size,
				vi.bits_per_rgb);

			CHECK;
		}
	}

	CHECKPASS(nvclass());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XMatchVisualInfo-2.(A)
 * When a visual does not exist on screen screen of depth
 * depth and of class class, then a call to XMatchVisualInfo
 * returns zero.
 */
static void
t002()
{
Status	s;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * Try with a depth of 0 which is always invalid.
	 */
	depth = 0;
	class = StaticColor;

	startcall(display);
	if (isdeleted())
		return;
	s = XMatchVisualInfo(display, screen, depth, class, vinfo);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (s == 0)
		CHECK;
	else {
		report("Return value was %d", s);
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
