/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: plygnrgn.m,v 1.5 92/06/11 17:37:59 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Region
 *   XPolygonRegion(points, n, fill_rule)
 *   XPoint	*points;
 *   int	n;
 *   int	fill_rule;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XPolygonRegion	1
char    *TestName = "XPolygonRegion";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XPolygonRegion function
 */
static XPoint	*points;
static int	n;
static int	fill_rule;


XPoint	poly1[] = { {20,20}, {35,10}, {55,10}, {80,20}, {90,35}, {85,50}, {75,65}, {50,70}, {30,65}, {20,60}, {15,40} };
XPoint	poly2[] = { {75,60}, {60,45}, {55,60}, {30,55}, {35,35}, {15,40}, {20,18}, {45,10}, {70,15}, {55,30}, {80,45} };
XPoint	poly3[] = { {85,40}, {30,20}, {20,55}, {30,69}, {55,35}, {85,44}, {10,43}, {14,66} };
/*
 * The same polygon used in XFillPolygon tests.
 */
XPoint   compshape[] = { {10, 10},  {80, 10},  {100, 30},  {65, 60},  {65, 30}, {85, 60},
                         {30, 50},  {30, 80},  {60, 80},   {60, 5},   {70, 5},  {70, 100}, {10, 100},
};

struct prec {
	XPoint	*points;
	int	size;
} poly[] = { 	{poly1, NELEM(poly1)},
		{poly2, NELEM(poly2)},
		{compshape, NELEM(compshape)}	};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	points = 0;
	n = 99;
	fill_rule = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XPolygonRegion-1.(A)
 * A call to XPolygonRegion returns a region comprising the
 * interior of the filled polygon as described by the n points
 * specified by the points argument, filled with the fill_rule
 * argument.
 */
static void
t001()
{
int     loop;
XImage		im;
XVisualInfo	*vi;
Region	reg;
Window	win;
GC	gc;
unsigned int    width;
unsigned int    height;
int x;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vi);

	for(loop=0; loop < NELEM(poly); loop++) {
	
		win = makewin(Dsp, vi);
		gc = makegc(Dsp, win);
		fill_rule = EvenOddRule;
		XSetFillRule(Dsp, gc, fill_rule);
		XSetFillStyle(Dsp, gc, FillSolid);
		XSetFunction(Dsp, gc, GXxor);
		XSetForeground(Dsp, gc, W_BG^W_FG);
		points = poly[loop].points;
		n = poly[loop].size;
		startcall(Dsp);
		if (isdeleted())
			return;
		reg = XPolygonRegion(points, n, fill_rule);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		getsize(Dsp, win, &width, &height);
		XFillPolygon(Dsp, win, gc, points, n, Complex, CoordModeOrigin);
		if( checkarea(Dsp, win, (struct area *) NULL, W_BG, W_BG, CHECK_ALL|CHECK_DIFFER)){
			delete("XFillPolygon() did set any pixels to non-background values.");
			return;
		} else
			CHECK;
		XSetRegion(Dsp, gc, reg);
		XFillRectangle(Dsp, win, gc, 0,0, width, height);	

		if( checkarea(Dsp, win, (struct area *) NULL, W_BG, W_BG, CHECK_ALL) == 0){
			report("When used as a clip-mask in a GC,");
			report("the polygon region did not produce the expected clipping.");
			report("Results depend on the correctness of XFillPolygon().");
			FAIL;
		} else
			CHECK;
		XDestroyRegion(reg);

		win = makewin(Dsp, vi);
		gc = makegc(Dsp, win);
		fill_rule = WindingRule;
		XSetFillRule(Dsp, gc, fill_rule);
		XSetFillStyle(Dsp, gc, FillSolid);
		XSetFunction(Dsp, gc, GXxor);
		XSetForeground(Dsp, gc, W_BG^W_FG);
		startcall(Dsp);
		if (isdeleted())
			return;
		reg = XPolygonRegion(points, n, fill_rule);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		getsize(Dsp, win, &width, &height);
		XFillPolygon(Dsp, win, gc, points, n, Complex, CoordModeOrigin);
		if( checkarea(Dsp, win, (struct area *) NULL, W_BG, W_BG, CHECK_ALL|CHECK_DIFFER)){
			delete("XFillPolygon() did set any pixels to non-background values.");
			return;
		} else
			CHECK;
		XSetRegion(Dsp, gc, reg);
		XFillRectangle(Dsp, win, gc, 0,0, width, height);	

		if( checkarea(Dsp, win, (struct area *) NULL, W_BG, W_BG, CHECK_ALL) == 0){
			report("When used as a clip-mask in a GC,");
			report("the polygon region did not produce the expected clipping.");
			report("Results depend on the correctness of XFillPolygon().");
			FAIL;
		} else
			CHECK;
		XDestroyRegion(reg);
	}

	CHECKPASS(NELEM(poly) * 4);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
