/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rctinrgn.m,v 1.3 92/06/11 17:38:51 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XRectInRegion(r, x, y, width, height)
 *   Region		r;
 *   int		x;
 *   int		y;
 *   unsigned int	width;
 *   unsigned int	height;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XRectInRegion	1
char    *TestName = "XRectInRegion";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XRectInRegion function
 */
static Region		r;
static int		x;
static int		y;
static unsigned int	width;
static unsigned int	height;


static int 	ValueReturn;

static XRectangle	medrect = { 10,10, 15,5 };

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	r = 0;
	x = 0;
	y = 0;
	width = 0;
	height = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XRectInRegion-1.(A)
 * When the rectangle specified by the x, y, width, and
 * height arguments is entirely in the region r, then a call
 * to XRectInRegion returns RectangleIn.
 */
static void
t001()
{
Region			R;
int			res;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R = makeregion();
	if(isdeleted()) return;

	XUnionRectWithRegion(&medrect, R, R);
	r = R;
	x=10;	
	y=10;
	width=15;
	height=5;

	startcall(Dsp);
	if (isdeleted())
		return;
	res = XRectInRegion(r, x, y, width, height);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != RectangleIn) {
		report("%s() returned %d instead of RectangleIn (%d).", TestName, res, RectangleIn);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XRectInRegion-2.(A)
 * When the rectangle specified by the x, y, width, and
 * height arguments is entirely out of the region r, then a
 * call to XRectInRegion returns RectangleOut.
 */
static void
t002()
{
Region			R;
int			res;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R = makeregion();
	if(isdeleted()) return;

	XUnionRectWithRegion(&medrect, R, R);
	r = R;
	x=35;	
	y=15;
	width=5;
	height=2;

	startcall(Dsp);
	if (isdeleted())
		return;
	res = XRectInRegion(r, x, y, width, height);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != RectangleOut) {
		report("%s() returned %d instead of RectangleOut (%d).", TestName, res, RectangleOut);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XRectInRegion-3.(A)
 * When the rectangle specified by the x, y, width, and
 * height arguments is partly in the region r, then a call to
 * XRectInRegion returns RectanglePart.
 */
static void
t003()
{
Region			R;
int			res;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R = makeregion();
	if(isdeleted()) return;

	XUnionRectWithRegion(&medrect, R, R);
	r = R;
	x=20;	
	y=5;
	width=4;
	height=10;

	startcall(Dsp);
	if (isdeleted())
		return;
	res = XRectInRegion(r, x, y, width, height);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != RectanglePart) {
		report("%s() returned %d instead of RectanglePart (%d).", TestName, res, RectanglePart);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
