/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rmmrgdtbss.m,v 1.4 92/06/11 17:40:04 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XrmMergeDatabases(source_db, target_db)
 *   XrmDatabase source_db;
 *   XrmDatabase *target_db;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XrmMergeDatabases	1
char    *TestName = "XrmMergeDatabases";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XrmMergeDatabases function
 */
static XrmDatabase source_db;
static XrmDatabase *target_db;



static XrmDatabase
xrm_create_database(data)
char *data;
{
	/* Create a new database for testing with */
	return(XrmGetStringDatabase(data));
}

static void
xrm_fill_value(value, data)
XrmValue *value;
char *data;
{
	value->addr = (caddr_t)data;
	value->size = (unsigned int)(strlen(data)+1);
}

static int
xrm_check_entry(dbase, fullspec, fullclass, type, val)
XrmDatabase dbase;
char *fullspec;
char *fullclass;
char *type;
char *val;
{
	int ret_val;
	char	*type_ret;
	XrmValue	value_ret;

	ret_val=0;
	type_ret=(char *)NULL;
	value_ret.size=0;
	value_ret.addr=(caddr_t)NULL;

	if(XrmGetResource(dbase, fullspec, fullclass, &type_ret, &value_ret)
		==False) {
		report("XrmGetResource failed to find database entry");
		report("Specifier was: %s", fullspec);
		ret_val++;
	} else {
		if (type_ret==NULL || strcmp(type_ret, type)) {
			report("XrmGetResource returned unexpected type information.");
			report("Specifier was: %s", fullspec);
			report("Expected type: '%s'", type);
			report("Returned type: '%s'",
				(type_ret==NULL)?"<NULL POINTER>":type_ret);
			ret_val++;
		}

		if (( value_ret.addr==(caddr_t)NULL)
			|| (strncmp((char *)value_ret.addr, val, strlen(val)))) {
			report("XrmGetResource returned unexpected value information.");
			report("Specifier was: %s", fullspec);
			report("Expected value: '%s'", val);
			if(value_ret.addr == (caddr_t)NULL) {
				report("Returned value: <NULL POINTER>");
			} else {
				report("Returned value: '%.*s' (%u bytes)",
					value_ret.size, (char *)value_ret.addr, value_ret.size);
			}
			ret_val++;
		}
	}

	return(ret_val);
}

static int
xrm_tabulate(from, into)
char *from, *into;
{
	int i,j;

	j = strlen(from);
	for(i=0; i<j; i++) {
		if(from[i]=='T') {
			into[i]='\t';
		} else {
			into[i]=from[i];
		}
	}
	return(j);
}

static XrmDatabase target;

#define XMD_SPEC 0
#define XMD_TYPE 1
#define XMD_VALUE 2
#define XMD_FSPEC 3
#define XMD_FCLASS 4
#define XMD_XTYPE 5
#define XMD_XVALUE 6

/* Test database data */
#define XMD_SIZE 3
static char	*d1_data[XMD_SIZE][5] = {
	/* spec, type, value, fullspec, fullclass */
	{ "one.data", "String", "one", "one.data", "One.Data" },
	{ "one.Misc", "Nylon",  "two", "one.misc", "One.Misc" },
	{ "one*type", "Cotton", "three", "one.type", "One.Type" } };

static char	*d2_data[XMD_SIZE][5] = {
	/* spec, type, value, fullspec, fullclass */
	{ "two.data", "Thread", "four", "two.data", "Two.Data" },
	{ "Two.star", "Lycra",  "five", "two.star", "Two.Star" },
	{ "two*Halt", "Silk",   "six", "two.halt", "Two.Halt" } };

static char *d3_data[XMD_SIZE][7] = {
	/* spec, type, value, fullspec, fullclass, exptype, expvalue */
/* These should replace test data in the target */
	{ "one.data", "Thread", "seven", "one.data", "One.Data", "Thread", "seven" },
	{ "one.Misc", "Lycra",  "eight", "one.misc", "One.Misc", "Lycra",  "eight" },
/* These is new data from the source. */
	{ "two.odd",	"Silk", 	"nine",	"two.odd",	"Two.Odd",	"Silk", "nine" } };

/* This defines from which point in d1_data entries the data is expected to
	remain in the target after the merge in test 3. */
#define XMD_UNREPLACED 2


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	source_db = 0;
	target_db = &target;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/* Including from file ../rmptrsrc/fn.mc */
/* End of included file ../rmptrsrc/fn.mc */

/*
 * Assertion XrmMergeDatabases-1.(A)
 * A call to XrmMergeDatabases merges the contents of
 * source_db into target_db.
 */
static void
t001()
{
int a;
XrmValue val;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create two test databases. */
	source_db = xrm_create_database("");
	if(source_db == (XrmDatabase)NULL) {
		delete("Could not create source database.");
		return;
	} else
		CHECK;

	target = xrm_create_database("");
	if(target == (XrmDatabase)NULL) {
		delete("Could not create target database.");
		return;
	} else
		CHECK;

/* Add test data to the test databases. */
	for(a=0; a<XMD_SIZE; a++) {
		CHECK;
		xrm_fill_value( &val, d1_data[a][XMD_VALUE] );
		XrmPutResource(&source_db,
			d1_data[a][XMD_SPEC],
			d1_data[a][XMD_TYPE],
			&val);
		
		xrm_fill_value( &val, d2_data[a][XMD_VALUE] );
		XrmPutResource(target_db,
			d2_data[a][XMD_SPEC],
			d2_data[a][XMD_TYPE],
			&val);
	}

/* Call xname to merge the databases. */
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmMergeDatabases(source_db, target_db);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that the target database contains all the test data. */
	for(a=0; a<XMD_SIZE; a++) {

		if (xrm_check_entry(target,
			d1_data[a][XMD_FSPEC],
			d1_data[a][XMD_FCLASS],
			d1_data[a][XMD_TYPE],
			d1_data[a][XMD_VALUE])) {
			FAIL;
			report("%s did not merge in the source database correctly",
				TestName);
		} else
			CHECK;

		if (xrm_check_entry(target,
			d2_data[a][XMD_FSPEC],
			d2_data[a][XMD_FCLASS],
			d2_data[a][XMD_TYPE],
			d2_data[a][XMD_VALUE])) {
			FAIL;
			report("%s did not preserve the target database correctly",
				TestName);
		} else
			CHECK;
	}

	CHECKPASS(2 + XMD_SIZE + XMD_SIZE*2);

	XrmDestroyDatabase(target);
	
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmMergeDatabases-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * On a call to XrmMergeDatabases, the source database
 * source_db is destroyed.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XrmMergeDatabases-3.(A)
 * When a resource is a member of both target_db and source_db,
 * then the value and type of the resource in target_db is
 * overwritten with the corresponding value and type from the
 * resource in source_db.
 */
static void
t003()
{
int a;
XrmValue val;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create two test databases. */
	source_db = xrm_create_database("");
	if(source_db == (XrmDatabase)NULL) {
		delete("Could not create source database.");
		return;
	} else
		CHECK;

	target = xrm_create_database("");
	if(target == (XrmDatabase)NULL) {
		delete("Could not create target database.");
		return;
	} else
		CHECK;

/* Add test data to the test databases. */
	for(a=0; a<XMD_SIZE; a++) {
		CHECK;
		
		xrm_fill_value( &val, d3_data[a][XMD_VALUE] );
		XrmPutResource(&source_db,
			d3_data[a][XMD_SPEC],
			d3_data[a][XMD_TYPE],
			&val);

		xrm_fill_value( &val, d1_data[a][XMD_VALUE] );
		XrmPutResource(target_db,
			d1_data[a][XMD_SPEC],
			d1_data[a][XMD_TYPE],
			&val);
	}

/* Call xname to merge the databases. */
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmMergeDatabases(source_db, target_db);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that the overlapping source database entries replaced the */
/* 	corresponding target database entries, and that unique source */
/* 	database entries were merged correctly. */
	for(a=0; a<XMD_SIZE; a++) {
		if (xrm_check_entry(target,
			d3_data[a][XMD_FSPEC],
			d3_data[a][XMD_FCLASS],
			d3_data[a][XMD_XTYPE],
			d3_data[a][XMD_XVALUE])) {
			FAIL;
			report("%s did not merge the databases correctly",
				TestName);
		} else
			CHECK;
	}

/* Verify that unique target database entries remained following the merge. */
	for(a=XMD_UNREPLACED; a<XMD_SIZE; a++) {
		if (xrm_check_entry(target,
			d1_data[a][XMD_FSPEC],
			d1_data[a][XMD_FCLASS],
			d1_data[a][XMD_TYPE],
			d1_data[a][XMD_VALUE])) {
			FAIL;
			report("%s did not preserve the target database correctly",
				TestName);
		} else
			CHECK;
	}

	CHECKPASS(2 + XMD_SIZE + XMD_SIZE*2 - XMD_UNREPLACED) ;

	XrmDestroyDatabase(target);
	
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = rmstartup;
void	(*tet_cleanup)() = cleanup;
