/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: sbimg.m,v 1.5 92/06/11 17:42:37 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XImage *
 *   XSubImage(ximage, x, y, subimage_width, subimage_height)
 *   XImage		*ximage;
 *   int		x;
 *   int		y;
 *   unsigned int	subimage_width;
 *   unsigned int	subimage_height;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSubImage	1
char    *TestName = "XSubImage";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XSubImage function
 */
static XImage		*ximage;
static int		x;
static int		y;
static unsigned int	subimage_width;
static unsigned int	subimage_height;


static int mpattern(display, d, w, h, dep)
Display		*display;
Drawable	d;
unsigned int	w;
unsigned int	h;
int		dep;
{
int		i;
int		j;
int		mod;
GC		gc;
unsigned long	val;

	gc = makegc(display, d);
	mod = 1<<dep;
	for(j=0; j<h; j++)
		for(i=0; i<w; i++) {
			val = (i + j) % mod;
			XSetForeground(display, gc, val);
			XDrawPoint(display, d, gc, i, j);
		}
}

static int checksubimage(subim, x,y, w,h, dep)
XImage		*subim;
int		x;
int		y;
unsigned int	w;
unsigned int	h;
int 		dep;
{
int	i;
int	j;
int	a;
int	b;
int	mod;

	mod = 1<<dep;
	for(j=0, b=y; j<h; j++, b++)
		for(i=0, a=x; i<w; i++, a++)
			if(XGetPixel(subim, i,j) != (a+b) % mod) {
				return(0);
			}
	return 1;
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	ximage = 0;
	x = 0;
	y = 0;
	subimage_width = 0;
	subimage_height = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSubImage-1.(A)
 * When the rectangle specified by the x, y, subimage_width
 * and subimage_height arguments is contained in the image
 * ximage, then a call to XSubImage creates a copy of the
 * subimage and returns a pointer to the newly created
 * subimage.
 */
static void
t001()
{
XImage		*sxi;
XVisualInfo	*vi;
int		npix;
unsigned int	width;
unsigned int	height;
Pixmap		pm;
Window		win;
int		i;
static int	fmats[2] = { XYPixmap, ZPixmap };
	
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(resetvinf(VI_PIX); nextvinf(&vi);) {

		pm = makepixm(Dsp, vi);
		getsize(Dsp, pm, &width, &height);
		width = width > 17 ? 17 : width;
		height = height > 19  ? 19 : height;
		x = 3;
		y = 5;
		subimage_width = width - 2*x;
		subimage_height = height - 2*y;
		mpattern(Dsp, pm, width, height, vi->depth);

		for(i=0; i<2; i++) {
			ximage = XGetImage(Dsp, pm, 0,0, width, height, AllPlanes, fmats[i]);

			if(ximage == (XImage *) NULL) {
				delete("XGetImage() returned NULL.");
				fail++; /* Avoid path checking in CHECKPASS */
			} else {
				startcall(Dsp);
				if (isdeleted())
					return;
				sxi = XSubImage(ximage, x, y, subimage_width, subimage_height);
				endcall(Dsp);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				if(sxi == (XImage *) NULL) {
					delete("%s() returned NULL.", TestName);
					fail++; /* Avoid path checking */
				} else {
					if(checksubimage(sxi, x,y, subimage_width, subimage_height, vi->depth) == 0) {
						report("Subimage structure was not correct.");
						FAIL;
					} else
						CHECK;

					XDestroyImage(sxi);
				}

				XDestroyImage(ximage);
			}
		}
	}

	npix = nvinf();

	for(resetvinf(VI_WIN); nextvinf(&vi);) {

		win = makewin(Dsp, vi);
		getsize(Dsp, win, &width, &height);
		width = width > 17 ? 17 : width;
		height = height > 19  ? 19 : height;
		x = 3;
		y = 5;
		subimage_width = width - 2*x;
		subimage_height = height - 2*y;
		mpattern(Dsp, win, width, height, vi->depth);

		for(i=0; i<2; i++) {
			ximage = XGetImage(Dsp, win, 0,0, width, height, AllPlanes, fmats[i]);
			if(ximage == (XImage *) NULL) {
				delete("XGetImage() returned NULL.");
				fail++; /* Avoid path checking in CHECKPASS */
			} else {

				startcall(Dsp);
				if (isdeleted())
					return;
				sxi = XSubImage(ximage, x, y, subimage_width, subimage_height);
				endcall(Dsp);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					FAIL;
				}
				if(sxi == (XImage *) NULL) {
					delete("%s() returned NULL.", TestName);
					fail++; /* Avoid path checking */
				} else {
					if(checksubimage(sxi, x,y, subimage_width, subimage_height, vi->depth) == 0) {
						report("Subimage structure was not correct.");
						FAIL;
					} else
						CHECK;

					XDestroyImage(sxi);
				}

				XDestroyImage(ximage);
			}
		}
	
	}

	CHECKPASS(2 * (npix + nvinf()));
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
