/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: strtkysym.m,v 1.5 92/06/11 17:43:22 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   KeySym
 *   XStringToKeysym(string)
 *   char	*string;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XStringToKeysym	1
char    *TestName = "XStringToKeysym";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XStringToKeysym function
 */
static char	*string;


#define XK_MISCELLANY
#include "keysymdef.h"
#undef  XK_MISCELLANY

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	string = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XStringToKeysym-1.(A)
 * A call to XStringToKeysym returns the KeySym value for the
 * KeySym name specified by the null-terminated string
 * argument.
 */
static void
t001()
{
KeySym	ks;
char	*tstr = "Escape";
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	string = tstr;
	startcall(Dsp);
	if (isdeleted())
		return;
	ks = XStringToKeysym(string);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(ks != XK_Escape) {
		report("%s() returned KeySym %lu instead of XK_Escape (%lu) for \"%s\".",
			TestName, (long) ks, (long) XK_Escape, tstr);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XStringToKeysym-2.(A)
 * When the specified string does not match a valid KeySym,
 * then a call to XStringToKeysym returns NoSymbol.
 */
static void
t002()
{
KeySym	ks;
char	*tstr = "XTest_No_Sym";
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	string = tstr;
	startcall(Dsp);
	if (isdeleted())
		return;
	ks = XStringToKeysym(string);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(ks != NoSymbol) {
		report("%s() returned KeySym %lu instead of NoSymbol (%lu) for \"%s\".",
			TestName, (long) ks, (long) NoSymbol, tstr);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
