/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: cpygc.m,v 1.4 92/06/11 17:47:26 rws Exp $
 * 
 */
/* $XConsortium: mcxpinc.mc,v 1.2 92/06/29 19:42:05 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"tet_api.h"


#define T_CopyGC	1
char    *TestName = "CopyGC";



/* Touch test for CopyGC request */

/****************************************************************************
 * Copyright 1988 by Sequent Computer Systems, Inc., Portland, Oregon       *
 *                                                                          *
 *                                                                          *
 *                         All Rights Reserved                              *
 *                                                                          *
 * Permission to use, copy, modify, and distribute this software and its    *
 * documentation for any purpose and without fee is hereby granted,         *
 * provided that the above copyright notice appears in all copies and that  *
 * both that copyright notice and this permission notice appear in          *
 * supporting documentation, and that the name of Sequent not be used       *
 * in advertising or publicity pertaining to distribution or use of the     *
 * software without specific, written prior permission.                     *
 *                                                                          *
 * SEQUENT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING *
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL *
 * SEQUENT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR  *
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,      *
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,   *
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS      *
 * SOFTWARE.                                                                *
 ****************************************************************************/

#include "Xstlib.h"


#define CLIENT 0
static TestType test_type = SETUP;
GContext gc1, gc2;
xChangeGCReq *cgcr;
xCopyGCReq *req;

static
void
tester()
{
	Create_Client(CLIENT);

	(void) Create_Default_Window(CLIENT);
	gc1 = Create_Default_GContext(CLIENT);
	gc2 = Create_GContext(CLIENT);

	cgcr = (xChangeGCReq *) Make_Req(CLIENT, X_ChangeGC);
	Send_Req(CLIENT, (xReq *) cgcr);
	Log_Trace("client %d sent default ChangeGC request\n", CLIENT);
	(void) Expect_Nothing(CLIENT);
	Free_Req(cgcr);

	Set_Test_Type(CLIENT, test_type);
	req = (xCopyGCReq *) Make_Req(CLIENT, X_CopyGC);
	req->dstGC = gc2;
	if(test_type == BAD_VALUE) {
		req->mask |= 1<<28;	/* an unused mask bit */
		req = (xCopyGCReq *) Xstrealloc((char *) req, req->length<<2);
	}
	Send_Req(CLIENT, (xReq *) req);
	Set_Test_Type(CLIENT, GOOD);
	switch(test_type) {
	case GOOD:
		Log_Trace("client %d sent default CopyGC request\n", CLIENT);
		(void) Expect_Nothing(CLIENT);
		break;
	case BAD_VALUE:
		Log_Trace("client %d sent CopyGC request with illegal mask\n", CLIENT);
		(void) Expect_BadValue(CLIENT);
		(void) Expect_Nothing(CLIENT);
		break;
	case BAD_LENGTH:
		Log_Trace("client %d sent CopyGC request with bad length (%d)\n", CLIENT, req->length);
		(void) Expect_BadLength(CLIENT);
		(void) Expect_Nothing(CLIENT);
		break;
	case TOO_LONG:
	case JUST_TOO_LONG:
		Log_Trace("client %d sent overlong CopyGC request (%d)\n", CLIENT, req->length);
		(void) Expect_BadLength(CLIENT);
		(void) Expect_Nothing(CLIENT);
		break;
	default:
		Log_Err("INTERNAL ERROR: test_type %d not one of GOOD(%d), BAD_LENGTH(%d), TOO_LONG(%d) or JUST_TOO_LONG(%d)\n",
			test_type, GOOD, BAD_LENGTH, TOO_LONG, JUST_TOO_LONG);
		Abort();
		/*NOTREACHED*/
		break;
	}
	Free_Req(req);
	Exit_OK();
}

int 	tet_thistest;

/*
 * Assertion CopyGC-1.(A)
 * When a client sends a valid CopyGC protocol request to the
 * X server, then the X server does not send back an error,
 * event or reply to the client.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();

	test_type = GOOD;

	/* Call a library function to exercise the test code */
	testfunc(tester);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion CopyGC-2.(A)
 * When a client sends an invalid CopyGC protocol request to
 * the X server, in which the length field of the request is
 * not the minimum length required to contain the request, then
 * the X server sends back a BadLength error to the client.
 */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();

	test_type = BAD_LENGTH; /* < minimum */

	/* Call a library function to exercise the test code */
	testfunc(tester);

	test_type = JUST_TOO_LONG; /* > minimum */

	/* Call a library function to exercise the test code */
	testfunc(tester);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion CopyGC-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a client sends an invalid CopyGC protocol request to
 * the X server, in which the length field of the request
 * exceeds the maximum length accepted by the X server, then
 * the X server sends back a BadLength error to the client.
 */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();

	test_type = TOO_LONG;

	/* Call a library function to exercise the test code */
	testfunc(tester);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion CopyGC-4.(A)
 * When a client sends an invalid CopyGC protocol request to
 * the X server, in which the mask value contains an unused
 * bit, then the X server sends back a BadValue error to the
 * client.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();

	test_type = BAD_VALUE;

	/* Call a library function to exercise the test code */
	testfunc(tester);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = protostartup;
void	(*tet_cleanup)() = protocleanup;
